using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Data;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;
using System.ComponentModel;
#if !PocketPC
using System.Drawing.Text;
#endif

namespace GeoFramework.Controls
{
    /// <summary>
    /// Represents a base class for designing flicker-free, multithreaded user
    /// controls.
    /// </summary>
    /// <remarks>
    /// 	<para>This powerful and versatile class provides a framework upon which to create
    ///     high-performance owner-drawn controls. Common rendering challenges such as
    ///     multithreading, thread synchronization, double-buffering, performance tuning,
    ///     platform tuning and animation are all handled by this class.</para>
    /// 	<para>Controls which inherit from this class perform all paint operations by
    ///     overriding the <strong>OnPaintOffScreen</strong> method, and optionally the
    ///     <strong>OnPaintOffScreenBackground</strong> and
    ///     <strong>OnPaintOffScreenAdornments</strong> methods. Controls which demand higher
    ///     performance should perform all rendering on a separate thread. This is done by
    ///     setting the <strong>IsPaintingOnSeparateThread</strong> property to
    ///     <strong>True</strong>. Since the actual painting of the control is handled by this
    ///     class, the <strong>OnPaint</strong> method should not be overridden.</para>
    /// 	<para>When all off-screen paint operations have completed, this class will copy the
    ///     contents of the off-screen bitmap to the on-screen bitmap which is visibly
    ///     displayed on the control. By deferring all rendering operations to another thread,
    ///     the user interface remains very responsive even during time-consuming paint
    ///     operations.</para>
    /// 	<para>Performance tuning is another major feature of this class. The
    ///     <strong>OptimizationMode</strong> property gives developers the ability to tune
    ///     rendering performance for animation speed, rendering quality, low CPU usage, or a
    ///     balance between all three.</para>
    /// 	<para>While thread synchronization has been implemented wherever possible in this
    ///     class, and the class is almost entirely thread-safe, some care should be taken when
    ///     accessing base <strong>Control</strong> properties from a separate thread. Even
    ///     basic properties like <strong>Visible</strong> can fail, especially on the Compact
    ///     Framework. For minimal threading issues, avoid reading control properties during
    ///     paint events.</para>
    /// 	<para>This class has been tuned to deliver the best performance on all versions of
    ///     the .NET Framework (1.0, 1.1 and 2.0) as well as the .NET Compact Framework (1.0
    ///     and 2.0). Compatibility is also managed internally, which simplifies the process of
    ///     porting controls to the Compact Framework.</para>
    /// </remarks>
    /// <example>
    ///     This example demonstrates how little code is required to use features like
    ///     double-buffering and multithreading. <strong>IsPaintingOnSeparateThread</strong>
    ///     enables multithreading, and all paint operations take place in the
    ///     <strong>OnPaintOffScreen</strong> method instead of <strong>OnPaint</strong>. To
    ///     prevent memory leaks, all GDI objects are disposed of during the
    ///     <strong>Dispose</strong> method.
    ///     <code lang="VB" title="[New Example]">
    /// Public Class MyControl 
    ///     Inherits DoubleBufferedControl
    ///     Dim MyBrush As New SolidBrush(Color.Blue)
    ///     
    ///     Sub New()
    ///         IsPaintingOnSeparateThread = True
    ///     End Sub
    ///     
    ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
    ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50))
    ///     End Sub
    ///     
    ///     Public Overrides Sub Dispose(ByVal disposing As Boolean)
    ///         MyBrush.Dispose()
    ///     End Sub
    /// End Class
    ///     </code>
    /// 	<code lang="CS" title="[New Example]">
    /// public class MyControl : DoubleBufferedControl
    /// {
    ///     SolidBrush MyBrush = new SolidBrush(Color.Blue);
    ///     
    ///     MyControl()
    ///     {
    ///         IsPaintingOnSeparateThread = true;
    ///     }
    ///  
    ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
    ///     {
    ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50));
    ///     }
    ///     
    ///     public override void Dispose(bool disposing)
    ///     {
    ///         MyBrush.Dispose();
    ///     }
    /// }
    ///     </code>
    /// </example>
#if !PocketPC || DesignTime
	[Serializable()]
	[RefreshProperties(RefreshProperties.All)]
#endif    
#if PocketPC     
    #if DesignTime
	[ToolboxItemFilter("System.CF.Windows.Forms", ToolboxItemFilterType.Custom)]
	[ToolboxItemFilter("GeoFramework.Controls", ToolboxItemFilterType.Allow)]
#endif
#if Framework20
#if !PocketPC
    [ToolboxItem(true)]
#endif
#endif
#endif
	[CLSCompliant(false)]
    public abstract class DoubleBufferedControl : Control
#if !PocketPC
, ISupportInitialize
#endif
    {
        private Bitmap pOffScreenBitmap = new Bitmap(1, 1);
        [NonSerialized]
        internal Graphics OffScreenGraphics;
        private Size pOffScreenBitmapSize = new Size(1, 1);
        private Bitmap pOnScreenBitmap = new Bitmap(1, 1);
		private object OffScreenSyncRoot = new object();
		private object OnScreenSyncRoot = new object();
		private bool pIsExceptionTextAllowed = true;
#if !PocketPC
        // Default to the highest quality


        private InterpolationMode pInterpolationMode = InterpolationMode.Low;
        private SmoothingMode pSmoothingMode = SmoothingMode.HighQuality;
        private TextRenderingHint pTextRenderingHint = TextRenderingHint.AntiAlias;
        private CompositingMode pCompositingMode = CompositingMode.SourceOver;
        private CompositingQuality pCompositingQuality = CompositingQuality.AssumeLinear;
        private PixelOffsetMode pPixelOffsetMode = PixelOffsetMode.HighSpeed;
        private ControlOptimizationMode pOptimizationMode = ControlOptimizationMode.Balanced;
        private PixelFormat pPixelFormat = PixelFormat.Format32bppArgb;
        //private int pTextContrast = 4;
        private int pTargetFramesPerSecond;
#endif
#if PocketPC
        private Color pBackgroundColor = SystemColors.Window;
#elif Framework20
        private Color pBackgroundColor = Color.Transparent;
#else
		private Color pBackgroundColor = SystemColors.Control;
#endif
        internal int ControlWidth;
        internal int ControlHeight;
        private bool IsRefreshAllowed = true;
        private bool pIsTransparent;
        private bool pIsPaintingSuspended;
        private bool pIsDisposed;
        private bool AllowPainting;
        private bool pIsPaintingOnSeparateThread;
        private bool pIsInitialized;
		[NonSerialized()]
		private ManualResetEvent RenderRequestWaitHandle = new ManualResetEvent(false);
		[NonSerialized()]
		private Thread PaintingThread;
		[NonSerialized()]
		private ManualResetEvent PaintingWaitHandle;
		private bool IsPaintingActive;
		private bool NeedNewBitmaps = true;
		private int RenderRequestIteration;
		private int RenderCurrentIteration;
		private int RenderCompleteIteration;
		private string pThreadName;
		private ThreadPriority pThreadPriority = ThreadPriority.Normal;
#if !PocketPC || DesignTime
		private const int MaximumGracefulShutdownTime = 500;
#elif PocketPC && Framework20
		private const int MaximumGracefulShutdownTime = 500;
#endif


	#if PocketPC
		private bool pIsUpperLeftCornerAdjusted;

#if PocketPC && !DesignTime && !Framework20
		private bool pIsPaintingThreadAlive;
#endif
#endif



        /// <summary>Occurs when an exception is thrown during off-screen painting operations.</summary>
        /// <remarks>
        /// 	<para>When the control is rendering on a separate thread, exceptions cannot be
        ///     caught by a regular <strong>Try..Catch</strong> statement. Exceptions are instead
        ///     channeled through this event. The control will also attempt to display exception
        ///     information on-screen to inform developers of the code which failed.</para>
        /// 	<para>It is important to capture this event or override the
        ///     <strong>OnPaintException</strong> method in order to be properly notified of
        ///     problems. Without doing this, the control could fail to paint properly yet give no
        ///     indication that there is a problem.</para>
        /// </remarks>
        /// <example>
        ///     This example hooks into the <strong>ExceptionOccurred</strong> event of a control
        ///     in order to handle painting exceptions. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl
        ///     Inherits DoubleBufferedControl
        ///     
        ///     Sub New()
        ///         ' Receive notifications of paint problems
        ///         AddHandler ExceptionOccurred, AddressOf HandleExceptions
        ///     End Sub
        ///     
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         ' Try to paint with a null Pen
        ///         e.Graphics.DrawRectangle(Nothing, Rectangle.Empty)
        ///     End Sub
        ///     
        ///     Private Sub HandleExceptions(ByVal sender As Object, ByVal e As ExceptionEventArgs)
        ///         ' Write the error to the Debug window
        ///         Debug.WriteLine(e.Exception.ToString())
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {   
        ///     MyControl()
        ///     {
        ///         // Receive notifications of paint problems
        ///         ExceptionOccurred += new ExceptionEventHandler(HandleExceptions);
        ///     }
        ///     
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         // Try to paint with a null Pen. 
        ///         e.Graphics.DrawRectangle(null, Rectangle.Empty);
        ///     }
        ///     
        ///     private sub HandleExceptions(object sender, ExceptionEventArgs e)
        ///     {
        ///         // Write the error to the Console
        ///         Console.WriteLine(e.Exception.ToString());
        ///     }
        /// }
        ///     </code>
        /// </example>
        public event ExceptionEventHandler ExceptionOccurred;

        /// <summary>
        /// Occurs when painting operations are temporarily suspended.
        /// </summary>
        public event EventHandler PaintingSuspended;
        /// <summary>
        /// Occurs when painting operations are allowed to continue after being suspended.
        /// </summary>
        public event EventHandler PaintingResumed;
		/// <summary>
		/// Occurs when the control is to be redrawn in the background.
		/// </summary>
		/// <remarks>In the DoubleBufferedControl class, all painting occurs off-screen.  Then, the off-screen bitmap is quickly
		/// drawn on-screen when painting completes.  This event is called immediately after the PaintOffScreenBackground event.</remarks>
		[CLSCompliant(false)]
		public event CancelablePaintEventHandler PaintOffScreen;
		/// <summary>
		/// Occurs when the control's background is to be redrawn before the main graphics of the control.
		/// </summary>
		/// <remarks>In the DoubleBufferedControl class, all painting occurs off-screen.  Then, the off-screen bitmap is quickly
		/// drawn on-screen when painting completes.   This event is called to paint any background graphics before the main elements of the
		/// control are drawn.  Some painting effects, such as glass or plastic, use this event along with PaintOffScreenAdornments
		/// to add annotations to the control.  This event is called immediately before the PaintOffScreen event.</remarks>
        [CLSCompliant(false)]
		public event CancelablePaintEventHandler PaintOffScreenBackground;
		/// <summary>
		/// Occurs when additional graphics or annotations must be added to the control.
		/// </summary>
		/// <remarks>In the DoubleBufferedControl class, all painting occurs off-screen.  Then, the off-screen bitmap is quickly
		/// drawn on-screen when painting completes.   This event is called to paint any additional graphics after the main elements of the
		/// control are drawn.  Some painting effects, such as glass, adding text or logos, use this event to draw on the control.  This event is called immediately after the PaintOffScreen event.</remarks>
        [CLSCompliant(false)]
		public event CancelablePaintEventHandler PaintOffScreenAdornments;

        /// <summary>Creates a new instance.</summary>        
        protected DoubleBufferedControl()
            : this("GeoFramework Multithreaded Control (http://www.geoframeworks.com)")
        {}     

        /// <summary>Creates a new instance using the specified thread name.</summary>
        /// <param name="threadName">
        /// The name associated with the rendering thread when rendering is multithreaded.
        /// This name appears in the Output window when the thread exits and can be useful during
        /// debugging. The name should also contain a company or support URL.
        /// </param>
        protected DoubleBufferedControl(string threadName)
        {
            // Set the name of the thread to help with debugging
            pThreadName = threadName;

			// Finally, move painting to its own thread
			IsPaintingOnSeparateThread = true;

#if !PocketPC
            // Set control rendering options for best performance
            SetStyle(ControlStyles.AllPaintingInWmPaint
                | ControlStyles.UserPaint
                | ControlStyles.SupportsTransparentBackColor
                | ControlStyles.Selectable
                | ControlStyles.ResizeRedraw
                | ControlStyles.CacheText, true);
            SetStyle(ControlStyles.Opaque, false);

            // Use a high frames-per-second setting
			TargetFramesPerSecond = 90;

#if Framework20
            // Use "optimized" double-buffering on DF2 platform
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
#else
            // Use "regular" double-buffering on DF1.1 and DF1.0
			SetStyle(ControlStyles.DoubleBuffer, true);
#endif
            // Default to a transparent background
			BackColor = Color.Transparent;

            // Default to a square control, 200x200px
			Size = new Size(200, 200);
#else
            // Default to a smaller control, 100x100px
            Size = new Size(100, 100);

            // Is there a parent control?
            if (Parent != null)
            {
                // Yes.  Inherit the parent's background color
                BackColor = Parent.BackColor;
            }
            else
            {
                // Set the background color depending on the platform
                switch (Platform.HostPlatformID)
                {
                    case HostPlatformID.Desktop:
                        
                        // Use the "Window" system color
                        BackColor = SystemColors.Window;

                        // We don't need to adjust the upper-left corner of controls
                        pIsUpperLeftCornerAdjusted = false;
                        break;

                    case HostPlatformID.WindowsCE:

                        // Use the grey color
                        BackColor = Color.Silver;

                        // We need to adjust the upper-left corner of controls at run-time
                        pIsUpperLeftCornerAdjusted = !IsDesignMode;
                        break;

                    case HostPlatformID.PocketPC:
                    
                        // Use the "Window" system color
                        BackColor = SystemColors.Window;

                        // We need to adjust the upper-left corner of controls at run-time
                        pIsUpperLeftCornerAdjusted = !IsDesignMode;
                        break;

                    case HostPlatformID.Smartphone:
                        
                        // Use the "Window" system color
                        BackColor = SystemColors.Window;

                        // We don't need to adjust the upper-left corner of controls
                        pIsUpperLeftCornerAdjusted = false;
                        break;
                }
            }          
#endif

#if !PocketPC
            /* The following event sink is very important.  It appears that .NET
			 * has trouble trying to resolve assemblies, especially when TypeConverters
			 * are involved.  This event sink assists .NET with locating the proper
			 * assembly.
			 * 
			 * Without this event, strange "Invalid cast." and other design-time errors
			 * will happen!  So please leave this in.
			 */
            AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(CurrentDomain_AssemblyResolve);
#endif
        }

        /// <summary>
        /// Indicates whether the control is currently running inside of a Windows Forms designer.
        /// </summary>
        public bool IsDesignMode
        {
            get
            {
#if PocketPC && !DesignTime
				return Licensing.LicenseManager.CurrentContext.UsageMode == GeoFramework.Licensing.LicenseUsageMode.Designtime;				
#else
                return System.ComponentModel.LicenseManager.CurrentContext.UsageMode == System.ComponentModel.LicenseUsageMode.Designtime;
#endif
            }
        }

        /// <summary>Indicates if the current rendering iteration has been cancelled.</summary>
        /// <value>
        /// A <strong>Boolean</strong>, <strong>True</strong> if the current rendering
        /// iteration has been cancelled.
        /// </value>
        /// <remarks>
        /// 	<para>When rendering on a separate thread, a new request to render the control may
        ///     occur while a rendering iteration is still in progress. This property indicates
        ///     that the current rendering iteration should be cancelled so that the next one may
        ///     begin. Each control, however, may decide whether or not to abort a rendering
        ///     iteration. This property is always <strong>False</strong> when rendering does not
        ///     take place on a separate thread.</para>
        /// 	<para>If a control decides to abort a rendering iteration, it must set the
        ///     <strong>IsCanceled</strong> property of the
        ///     <strong>CancelablePaintEventArgs</strong> object to <strong>True</strong> during
        ///     the <strong>OnPaintOffScreen</strong> method.</para>
        /// </remarks>
        /// <example>
        ///     This example demonstrates how a control might choose to abort rendering if a new
        ///     rendering request is made. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///  
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         ' Do some rendering
        ///         ' ...
        ///         
        ///         ' Check if rendering is aborted before doing an
        ///         ' intense operation such as a loop
        ///         If IsPaintingAborted Then
        ///             e.IsCanceled = True
        ///             Exit Sub
        ///         End If
        ///           
        ///         ' Continue rendering normally    
        ///         ' ...
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         // Do some rendering
        ///         // ...
        ///         
        ///         // Check if rendering is aborted before doing an
        ///         // intense operation such as a loop
        ///         if(IsPaintingAborted)
        ///         {
        ///             e.IsCanceled = true;
        ///             return;
        ///         }
        ///           
        ///         // Continue rendering normally    
        ///         // ...
        ///     }
        /// }
        ///     </code>
        /// </example>
#if !PocketPC || DesignTime
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [EditorBrowsable(EditorBrowsableState.Never)]
#endif
        protected bool IsPaintingAborted
        {
            get
            {
                if (IsPaintingOnSeparateThread)
                    return RenderRequestIteration > RenderCurrentIteration;
                else
                    return false;
            }
        }

#if (!PocketPC || DesignTime)
        /// <summary>Indicates if the control and its resources have been disposed.</summary>
        public new bool IsDisposed
        {
            get
            {
                return pIsDisposed;
            }
        }
#elif PocketPC 
        /// <summary>Indicates if the control and its resources have been disposed.</summary>
        public new bool IsDisposed
        {
            get
            {
                return pIsDisposed;
            }
        }
#endif


        /// <summary>Indicates if the control has been initialized.</summary>
#if !PocketPC || DesignTime
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [EditorBrowsable(EditorBrowsableState.Never)]
#endif
        protected bool IsInitialized
        {
            get
            {
                return pIsInitialized;
            }
        }

        /// <summary>
        /// Allows painting operations to be 
        /// </summary>
        public void ResumePainting()
        {
            if (!pIsPaintingSuspended)
                return;

            pIsPaintingSuspended = false;

            // Broadcast that painting is allowed
            OnPaintingResumed();

            // Lastly, redraw the control
            InvokePaintOffScreen();
        }

        protected virtual void OnPaintingResumed()
        {
            if (PaintingResumed != null)
#if PocketPC && !Framework20
                PaintingResumed(this, EventArgs.Empty);
#else
                DynamicEvent.BeginInvoke(PaintingResumed, this);
#endif
        }

        public void SuspendPainting()
        {
            if (pIsPaintingSuspended)
                return;
            pIsPaintingSuspended = true;

            OnPaintingSuspended();
        }

        protected virtual void OnPaintingSuspended()
        {
            if (PaintingSuspended != null)
#if PocketPC && !Framework20
                PaintingSuspended(this, EventArgs.Empty);
#else
                DynamicEvent.BeginInvoke(PaintingSuspended, this);
#endif
        }

        /// <summary>
        /// Indicates if all off-screen rendering takes place on a separate thread.
        /// </summary>
        /// <remarks>
        /// 	<para>This powerful property controls whether or not rendering operations are
        ///     multithreaded. When set to <strong>True</strong>, a new thread is launched and all
        ///     subsequent calls to <strong>OnPaintOffScreen</strong>,
        ///     <strong>OnPaintOffScreenBackground</strong> and
        ///     <strong>OnPaintOffScreenAdornments</strong> occur on that thread. Thread
        ///     synchronization features are enabled so that painting operations never interfere
        ///     with rendering operations. The priority of the rendering thread is controlled via
        ///     the <strong>ThreadPriority</strong> property.</para>
        /// 	<para>When this property is <strong>False</strong>, the rendering thread is torn
        ///     down and all rendering occurs on the owner's thread. Controls which perform
        ///     significant painting operations should enable this property to allow the user
        ///     interface to be more responsive. As a general rule, any intense processing should
        ///     be moved away from the user interface thread.</para>
        /// </remarks>
        /// <example>
        ///     This example instructs the control to perform all rendering on a separate thread.
        ///     Note that all thread management is handled automatically -- the only operation
        ///     required is enabling the property. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     
        ///     Sub New()
        ///         ' Enable multithreading
        ///         IsPaintingOnSeparateThread = True
        ///     End Sub
        ///     
        ///     ' This method is now called from another thread
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         Dim MyBrush As New SolidBrush(Color.Blue)
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50))
        ///         MyBrush.Dispose()
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {    
        ///     MyControl()
        ///     {
        ///         // Enable multithreading
        ///         IsPaintingOnSeparateThread = true;
        ///     }
        ///     
        ///     // This method is now called from another thread
        ///     protected overrides void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         SolidBrush MyBrush = new SolidBrush(Color.Blue);
        ///         e.Graphics.FillRectangle(MyBrush, new Rectangle(50, 50, 50, 50));
        ///         MyBrush.Dispose();
        ///     }
        /// }
        ///     </code>
        /// </example>
#if !PocketPC || DesignTime
        [Category("Performance")]
        [Description("Controls whether off-screen rendering takes place on a separate thread.  Good for demanding controls.")]
        [DefaultValue(typeof(bool), "True")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
        [EditorBrowsable(EditorBrowsableState.Advanced)]
#endif
        public bool IsPaintingOnSeparateThread
        {
            get
            {
                return pIsPaintingOnSeparateThread;
            }
            set
            {
                if (pIsPaintingOnSeparateThread == value) return;

                // If we were using a separate thread, tear it down
                if (pIsPaintingOnSeparateThread)
                {
                    // We're no longer painting on another thread
                    pIsPaintingOnSeparateThread = false;

                    try
                    {
                        // Wake up the painting thread
                        RenderRequestWaitHandle.Set();
                    }
                    catch (ObjectDisposedException)
                    {
                        // Can't wait because we're tearing down. Ignore
                        return;  // ?
                    }   
                
#if !PocketPC || DesignTime || (PocketPC && Framework20)
					/* Compact Framework 2.0 supports the ability to
					 * forcefully abort a thread.
					 */
                    try
                    {
                        if(PaintingThread != null && !PaintingThread.Join(MaximumGracefulShutdownTime))
                            PaintingThread.Abort();
                    }
                    catch
                    {
                    }
#else
                    /* Compact Framework 1.0 does not support the ability to
					 * forcefully shut down a thread, so we must let it exit
					 * on its own.
					 */
					//System.Diagnostics.Debugger.Break();
#endif
                }
                else
                {
                    try
                    {
                        // We're painting on another thread
                        pIsPaintingOnSeparateThread = true;

                        // Otherwise, start it up
                        PaintingThread = new Thread(new ThreadStart(PaintingThreadProc));

#if PocketPC || !Framework20
                        // To make everything blow up, change this line here
                        PaintingThread.Priority = pThreadPriority;
#endif

#if !PocketPC || Framework20
                        // Set the control name to assist with debugging
                        PaintingThread.Name = pThreadName;
                        PaintingThread.IsBackground = true;
#endif

                        // Initialize wait handles which signal that we need rendering
                        if (PaintingWaitHandle == null)
                            PaintingWaitHandle = new ManualResetEvent(false);
                        else
                            PaintingWaitHandle.Reset();

                        if (RenderRequestWaitHandle == null)
                            RenderRequestWaitHandle = new ManualResetEvent(false);
                        else
                            RenderRequestWaitHandle.Reset();

                        // Start the rendering thread
                        IsPaintingActive = true;
                        PaintingThread.Start();

						// Let it start up
						Thread.Sleep(0);
					}
#if !PocketPC || Framework20
                    catch (ThreadAbortException)
                    {
                        // Ignore this error and tear down     
                    }
#endif
                    catch
                    {
                        throw;  // (?)
                    }
                }
            }
        }

        //        /// <summary>Indicates if a rendering iteration can be ignored.</summary>
        //        /// <value>
        //        /// A <strong>Boolean</strong>, <strong>True</strong> if rendering iterations can be
        //        /// aborted as needed by the control.
        //        /// </value>
        //        /// <remarks>
        //        /// 	<para>When rendering operations are multithreaded, a request to render the control
        //        ///     may occur while a rendering iteration is already in progress. To obtain better
        //        ///     performance, a control may abort rendering of an iteration and restart in order to
        //        ///     display the most recent data. When this property is True, rendering iterations may
        //        ///     be aborted before they can be displayed on-screen in order to restart.</para>
        //        /// 	<para>Controls which require high-performance animation should set this property to
        //        ///     <strong>False</strong> for smoother results. Controls which take a long time to
        //        ///     render, however, should set this property to <strong>True</strong> in order to
        //        ///     deliver updated frames as fast as possible.</para>
        //        /// </remarks>
        //#if !PocketPC || DesignTime
        //        [Browsable(true)]
        //        [Category("Performance")]
        //        [Description("Indicates if a rendering operation can be interrupted once it starts.  Enabling this feature makes the control more responsive, but appear jerky if it is refreshed several times per second.")]
        //        [DefaultValue(typeof(bool), "false")]
        //#else
        //        [DefaultValue(false)]
        //#endif
        //        public bool IsPaintingInterruptable
        //        {
        //            get
        //            {
        //                return pIsPaintingInterruptable;
        //            }
        //            set
        //            {
        //                pIsPaintingInterruptable = value;
        //            }
        //        }

        /// <summary>
        /// Returns the size of the off-screen bitmap.
        /// </summary>
        /// <remarks>
        /// 	<para>This property allows a control to paint a bitmap that is larger or smaller
        ///     than the visible size of the control. For most situations, this property is not
        ///     overridden and the size of the off-screen bitmap is the same as the size of the
        ///     on-screen bitmap.</para>
        /// 	<para>More advanced controls such as GIS.NET's mapping control uses this size to
        ///     render the entire map whever possible in order to speed up panning
        ///     operations.</para>
        /// </remarks>
        /// <value>
        /// A <strong>Size</strong> structure indicating the size of the off-screen bitmap.
        /// Default is the same as the controls own size.
        /// </value>
#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Returns the size of the off-screen bitmap.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
#endif
        protected virtual Size OffScreenBitmapSize
        {
            get
            {
                return new Size(ControlWidth, ControlHeight);
            }
        }

        /// <summary>
        /// Controls the upper-left portion of the off-screen bitmap to paint
        /// on-screen.
        /// </summary>
        /// <value>
        /// A <strong>Point</strong> structure indicating the corner of the off-screen bitmap
        /// to draw on-screen. Default is <strong>Empty</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>If the size of the off-screen bitmap is different than the on-screen bitmap,
        ///     a control may need to draw different portions of the off-screen bitmap. For
        ///     example, if an off-screen bitmap is 200x200 pixels but the visible portion of the
        ///     control is only 50x50 pixels, an offset of (10,10) instructs the control to paint
        ///     the off-screen bitmap from (10,10)-(60,60).</para>
        /// 	<para>for most controls, this property does not need to be overridden. Controls
        ///     which override this property also override the <strong>OffScreenBitmapSize</strong>
        ///     property to specify a size defferent than the visible portion of the
        ///     control.</para>
        /// </remarks>
#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Controls the upper-left portion of the off-screen bitmap to paint on-screen.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
#endif
        protected virtual Point OffScreenBitmapOffset
        {
            get
            {
                return Point.Empty;
            }
        }

        /// <summary>Returns the bitmap used for off-screen painting operations.</summary>
        /// <value>A <strong>Bitmap</strong> containing off-screen painted data.</value>
        /// <remarks>
        /// 	<para>This control maintains two separate bitmaps: an "off-screen" bitmap, where
        ///     all painting operations take place, and an "on-screen" bitmap which is displayed
        ///     visually to the user. When an off-screen painting operation completes successfully,
        ///     the off-screen bitmap is copies to the on-screen bitmap, then painted on the
        ///     display. This property returns the off-screen bitmap created during the most recent
        ///     paint iteration.</para>
        /// </remarks>
#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Returns the bitmap used for off-screen painting operations.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(true)]
        [EditorBrowsable(EditorBrowsableState.Advanced)]
#endif
        public Bitmap OffScreenBitmap
        {
            get
            {
                return pOffScreenBitmap;
            }
        }

        /// <summary>Controls the relative priority of multithreaded painting operations.</summary>
        /// <value>
        /// A <strong>ThreadPriority</strong> value. Default is
        /// <strong>Normal</strong>.
        /// </value>
        /// <remarks>
        /// Painting operations may require more CPU time if they represent the majority of
        /// a user interface, or if painting operations are more complicated. Performance can be
        /// improved by increasing the priority of the rendering thread. Likewise, if a control is
        /// of minor importance to an application, a lower thread priority can improve performance
        /// in more important areas of the application.
        /// </remarks>
        /// <example>
        ///     This example enables multithreaded painting then changes the priority of the
        ///     rendering thread to <strong>Lowest</strong> to give the rest of the application
        ///     more CPU time. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     
        ///     Sub New()
        ///         ' Enable multithreading
        ///         IsPaintingOnSeparateThread = True
        ///         ' Set a low thread priority
        ///         ThreadPriority = ThreadPriority.Lowest
        ///     End Sub
        ///     
        ///     ' This method is now called from another thread
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///        ' ...etc.
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     MyControl()
        ///     {
        ///         // Enable multithreading
        ///         IsPaintingOnSeparateThread = true;
        ///         // Set a low thread priority
        ///         ThreadPriority = ThreadPriority.Lowest;
        ///     }
        ///     
        ///     // This method is now called from another thread
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///        // ...etc.
        ///     }
        /// }
        ///     </code>
        /// </example>
#if !PocketPC || DesignTime
		[Category("Performance")]
		[Description("Controls the relative priority of multithreaded painting operations.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		[DefaultValue(ThreadPriority.Normal)]        
#endif        
#if !PocketPC && Framework20
		[Obsolete("Changing thread priorty on Desktop Framework 2.0 can cause instability.  A setting of Normal is recommended for all threads.")]
#endif
		public ThreadPriority ThreadPriority
        {
            get
            {
                return pThreadPriority;
            }
            set
            {
                if (pThreadPriority == value) 
                    return;
                pThreadPriority = value;
            }
        }

        //public new bool Visible
        //{
        //    get
        //    {
        //    }
        //    set
        //    {
        //    }
        //}

        //        public new int Width
        //        {
        //            get
        //            {
        //                return ControlWidth;
        //            }
        //			set
        //			{
        //				base.Width = value;
        //				ControlWidth = value;
        //			}
        //        }
        //
        //#if !PocketPC
        //        [Category("Appearance")]
        //        [Description("Controls the width and height of the control.")]
        //        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        //#endif
        //		public new Size Size
        //		{
        //			get
        //			{
        //				return new Size(ControlWidth, ControlHeight);
        //			}
        //			set
        //			{
        //				ControlWidth = value.Width;
        //				ControlHeight = value.Height;
        //				base.Size = value;
        //			}
        //		}
        //
        //        public new Rectangle ClientRectangle
        //        {
        //            get
        //            {
        //                return new Rectangle(0, 0, Width, Height);
        //            }
        //        }
        //
        //        public new Size ClientSize
        //        {
        //            get
        //            {
        //                return new Size(Width, Height);
        //            }
        //        }
        //
        //        public new int Height
        //        {
        //            get
        //            {
        //                return ControlHeight;
        //            }
        //			set
        //			{
        //				base.Height = value;
        //				ControlHeight = value;
        //			}
        //        }

        /// <summary>Controls the background color of the control.</summary>
        /// <value>A <strong>Color</strong> structure representing the background color.</value>
        /// <remarks>
        /// The default <strong>BackColor</strong> property of the <strong>Control</strong>
        /// class cannot be accessed from a thread other than the UI thread. As a result, this
        /// property was shadowed in order to make it thread-safe.
        /// </remarks>
#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Controls the color of any pixel not included in other rendering operations.")]
		#if PocketPC
				[DefaultValue(typeof(Color), "Window")]
		#elif Framework20
				[DefaultValue(typeof(Color), "Transparent")]
		#else
				[DefaultValue(typeof(Color), "Control")]
		#endif
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
        [NotifyParentProperty(true)]
#endif
        public override Color BackColor
        {
            get
            {
                return pBackgroundColor;
            }
            set
            {
                if (pBackgroundColor.Equals(value)) 
                    return;

                pBackgroundColor = value;
                pIsTransparent = pBackgroundColor.Equals(Color.Transparent);
                
#if !PocketPC
                // Notify of the change
                OnBackColorChanged(EventArgs.Empty);
#endif                
                //#if PocketPC
                InvokePaintOffScreen();
                //#endif
            }
        }

        private void InvokePaintOffScreenInternal(object state)
        {
            InvokePaintOffScreenInternal();
        }

		private bool InvokePaintOffScreenInternal()
		{
			try
			{
				// If we're not initialized, exit.  Initialization happens when the Paint method
				// is called for the first time
				if (!pIsInitialized)
					return false;

				// Cache the size of the bitmap
				Size BitmapSize = OffScreenBitmapSize;

#if PocketPC
				// Is the bitmap size too small?
				if (BitmapSize.Width == 0 || BitmapSize.Height == 0)
					return false;
#else
				// Is the bitmap size too small?
				if (BitmapSize.Width <= 0 || BitmapSize.Height <= 0 || !Visible)
					//					|| OffScreenBitmapSize.Width > 3000 || OffScreenBitmapSize.Height > 3000)
					return false;
#endif
				//                // Is it too big?
				//                if (OffScreenBitmapSize.Width > 3000 || OffScreenBitmapSize.Height > 3000)
				//                    return false; //throw new OverflowException("The size of the bitmap to render is greater than 3000x3000, the maximum size allowed.");
				// If the size has changed, we need a new bitmap
				if ((pOffScreenBitmap != null &&
					(pOffScreenBitmapSize.Width != BitmapSize.Width
					|| pOffScreenBitmapSize.Height != BitmapSize.Height))
					|| pIsTransparent)
				{
					NeedNewBitmaps = true;
				}
				
				// If the background is Transparent, we need a new bitmap
				//if (pBackgroundColor.Equals(Color.Transparent))
				//NeedNewBitmaps = true;
				// Do we need a new bitmap?
				lock(OffScreenSyncRoot)
				{
					if (NeedNewBitmaps)
					{


						// No.  Keep going
						if (pOffScreenBitmap != null)
						{
							// Remember the size
							if (OffScreenGraphics != null)
								OffScreenGraphics.Dispose();

							// ** Console.WriteLine("Before dispose off-screen buffer: " + GC.GetTotalMemory(true).ToString());

							pOffScreenBitmap.Dispose();

							// ** Console.WriteLine("After dispose off-screen buffer: " + GC.GetTotalMemory(true).ToString());
						}

#if PocketPC
						pOffScreenBitmap = new Bitmap(BitmapSize.Width, BitmapSize.Height);
#else
						pOffScreenBitmap = new Bitmap(BitmapSize.Width, BitmapSize.Height, pPixelFormat);
#endif

						OffScreenGraphics = Graphics.FromImage(pOffScreenBitmap);

					
						pOffScreenBitmapSize = BitmapSize;
					}
				
#if !PocketPC
						// Set the rendering quality
						OffScreenGraphics.InterpolationMode = pInterpolationMode;
						OffScreenGraphics.SmoothingMode = pSmoothingMode;
						OffScreenGraphics.TextRenderingHint = pTextRenderingHint;
						OffScreenGraphics.CompositingMode = pCompositingMode;
						OffScreenGraphics.CompositingQuality = pCompositingQuality;
						OffScreenGraphics.PixelOffsetMode = pPixelOffsetMode;

#elif DesignTime
					OffScreenGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
					OffScreenGraphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
					OffScreenGraphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
					OffScreenGraphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
#endif
#if PocketPC
					// Now call the rendering function
					CancelablePaintEventArgs f = new CancelablePaintEventArgs(OffScreenGraphics, new Rectangle(0, 0, ControlWidth, ControlHeight));
#else
					// Now call the rendering function
					CancelablePaintEventArgs f = new CancelablePaintEventArgs(OffScreenGraphics, ClientRectangle);
#endif

					// On CF2 we can tell if the control is disposed
					if (IsDisposed)
						return false;

					// No.  Render the background
					OnPaintOffScreenBackground(f);

					// We'll allow cancelling unless we're 5+ frames behind
					if (f.Canceled)
						return false;

					// ** Console.WriteLine("After render off-screen background: " + GC.GetTotalMemory(true).ToString());

					// Did they cancel?
					//if(f.IsCanceled) return;
					// No. Render the main contents
					OnPaintOffScreen(f);

					// ** Console.WriteLine("After render off-screen main: " + GC.GetTotalMemory(true).ToString());

					// Did they cancel?
					if (f.Canceled)
						return false;

					//if(f.IsCanceled) 
					//    return;
					// No, render adornments
					OnPaintOffScreenAdornments(f);

					//f.Graphics.DrawString(RenderCompleteIteration.ToString(), Font, Brushes.Black, 5, 5);
					// ** Console.WriteLine("After render off-screen adornments: " + GC.GetTotalMemory(true).ToString());

					// Did they cancel?
					//if(f.IsCanceled) return;
					//}

					lock(OnScreenSyncRoot)
					{
						if (NeedNewBitmaps)
						{

							pOnScreenBitmap.Dispose();
							pOnScreenBitmap = new Bitmap(ControlWidth, ControlHeight);
							// Bitmaps are now updated
							NeedNewBitmaps = false;
						}

						// If we're disposing, skip all this
						if(IsDisposed)
							return true;

						using (Graphics d = Graphics.FromImage(pOnScreenBitmap))
						{
							d.Clear(pBackgroundColor);
						
							lock(OffScreenSyncRoot)
							{
#if PocketPC
								d.DrawImage(pOffScreenBitmap, OffScreenBitmapOffset.X, OffScreenBitmapOffset.Y);
#else
							d.DrawImageUnscaled(pOffScreenBitmap, OffScreenBitmapOffset);
#endif
							}
						}
					}
				}
				
				// And update the control
				InvokeRefresh();
				return true;
			}
#if !PocketPC
			catch (ThreadAbortException)
			{
				// Painting has been aborted. Shut down
				return false;
			}
#endif
//			catch(MissingMethodException ex)
//			{
//				if(DesignMode)
//				{
//					// The VS IDE is being retarded.  Indicate Wait for a while, then paint again
//					Thread.Sleep(1500);
//					return false;
//				}
//				else
//				{
//					OnPaintException(ex);
//					return false;
//				}
//			}
			catch (ObjectDisposedException)
			{
				// Painting is cancelled.  Shut down.
				return false;
			}
			catch (Exception ex)
			{
				OnPaintException(ex);
				return false;
			}
		}

        /// <summary>
        /// Controls whether exception messages are displayed on the control.
        /// </summary>
        /// <remarks>In some situations, an exception can occur during a paint operation.  To notify
        /// developers of the problem, the error text is written directly to the control.  This behavior
        /// may not be suitable for some developers, however, who want to trap errors more gracefully.
        /// Setting this property to False causes error messages to never be drawn on the control.  The
        /// ExceptionOccurred event should instead be used to gracefully handle errors when they occur.</remarks>
#if !PocketPC || DesignTime
		[Category("Performance")]
		[Description("Controls whether exception messages are drawn in the control, or suppressed.")]
		[DefaultValue(typeof(bool), "True")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Advanced)]
#endif
		public bool IsExceptionTextAllowed
        {
            get
            {
                return pIsExceptionTextAllowed;
            }
            set
            {
                pIsExceptionTextAllowed = value;
            }
        }

        /// <summary>Occurs when an exception occurs during a multithreaded paint operation.</summary>
        /// <remarks>
        /// 	<para>Exceptions caught during rendering iterations are channeled through this
        ///     method instead of being re-thrown. This allows developers the ability to silently
        ///     handle rendering problems without letting them interrupt the user interface. This
        ///     method invokes the <strong>ExceptionOccurred</strong> event.</para>
        /// </remarks>
        /// <example>
        ///     This example demonstrates how to be notified of rendering failures using the
        ///     <strong>OnPaintException</strong> method. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         ' Cause an error by using a null pen
        ///         e.Graphics.DrawRectangle(Nothing, Rectangle.Empty)
        ///     End Sub
        ///     
        ///     Protected Overrides Sub OnPaintException(ByVal e As ExceptionEventArgs)
        ///         Throw e.Exception
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         // Cause an error by using a null pen
        ///         e.Graphics.DrawRectangle(null, Rectangle.Empty);
        ///     }
        ///     
        ///     protected override void OnPaintException(ExceptionEventArgs e)
        ///     {
        ///         throw e.Exception;
        ///     }
        /// }
        ///     </code>
        /// </example>
        /// <param name="exception">An <strong>Exception</strong> object describing the rendering error.</param>
        protected virtual void OnPaintException(Exception exception)
        {
            try
            {
                // No.  If we have an existing one, release it
                if (pIsDisposed)
                    return;

                // Are we allowed to render the error?
                if (pIsExceptionTextAllowed)
                {
                    // No.  Keep going
                    lock (OffScreenSyncRoot)
                    {
                        pOffScreenBitmap.Dispose();
                        pOffScreenBitmap = new Bitmap(ControlWidth, ControlHeight);
                        OffScreenGraphics = Graphics.FromImage(pOffScreenBitmap);


                        // Set the rendering quality
#if !PocketPC
                        // Set the rendering quality
                        OffScreenGraphics.InterpolationMode = pInterpolationMode;
                        OffScreenGraphics.SmoothingMode = pSmoothingMode;
                        OffScreenGraphics.TextRenderingHint = pTextRenderingHint;
                        OffScreenGraphics.CompositingMode = pCompositingMode;
                        OffScreenGraphics.CompositingQuality = pCompositingQuality;
                        OffScreenGraphics.PixelOffsetMode = pPixelOffsetMode;

#elif DesignTime
					OffScreenGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
					OffScreenGraphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
					OffScreenGraphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
					OffScreenGraphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
#endif
                        OffScreenGraphics.Clear(SystemColors.Control);
#if PocketPC
                            SolidBrush ControlBrush = new SolidBrush(SystemColors.ControlText);
                            using (Font ErrorFont = new Font("Tahoma", 7.0f, FontStyle.Regular))
                            {
                                OffScreenGraphics.DrawString(exception.ToString(), ErrorFont, ControlBrush,
                                    new Rectangle(0, 0, ControlWidth, ControlHeight));
                            }
                            ControlBrush.Dispose();
#else
                        OffScreenGraphics.DrawString(exception.ToString(), Font, SystemBrushes.ControlText, ClientRectangle);
#endif


                        lock (OnScreenSyncRoot)
                        {
                            // Release the old bitmap
                            pOnScreenBitmap.Dispose();

#if PocketPC
                        pOnScreenBitmap = new Bitmap(OffScreenBitmapSize.Width, OffScreenBitmapSize.Height);
                        using (Graphics d = Graphics.FromImage(pOnScreenBitmap))
                        {
                            d.DrawImage(pOffScreenBitmap, 0, 0);
                        }
#else

                            // And commit the new one
                            pOnScreenBitmap = pOffScreenBitmap.Clone() as Bitmap;
#endif
                        }
                    }
                }

                // And update the control
                InvokeRefresh();

                if (ExceptionOccurred != null)
#if PocketPC && !Framework20
				ExceptionOccurred(this, new ExceptionEventArgs(exception));
#else
                    DynamicEvent.Invoke(ExceptionOccurred, this, new ExceptionEventArgs(exception));
#endif

            }
            catch
            {
                // We must ignore subsequent errors to maintain stability
            }
        }

#if PocketPC
        private void DoInvalidate(object sender, EventArgs e)
        {
            Invalidate();
        }
#endif

        /// <summary>Causes the control to be repainted.</summary>
        /// <remarks>
        /// 	<para>This method profives a thread-safe way to refresh the control. A call to the
        ///     <strong>Refresh</strong> method is marshalled to the control's own thread. This
        ///     method is called internally when an off-screen painting operation has completed.
        ///     Since this class manages the task of repainting the control and the latest
        ///     off-screen bitmap is rendered as soon as possible, it is typically not necessary to
        ///     call this method.</para>
        /// </remarks>
        /// <example>
        /// An example was excluded because <strong>InvokeRefresh</strong> and
        /// <strong>Refresh</strong> are not necessary when using this class.
        /// </example>
        public void InvokeRefresh()
        {
            // If we're disposing or disposed, skip this
            if (pIsDisposed || !pIsInitialized)
                return;

            try
            {
#if PocketPC
#if Framework20
                GeoFramework.Threading.ThreadPool.Queue(new WaitCallback(InvokeRefreshInternal));
                //BeginInvoke(new EventHandler(InvokeRefreshInternal));
#else

				#if !PocketPC || Framework20
				GeoFramework.Threading.ThreadPool.Queue(new WaitCallback(InvokeRefreshInternal));
#else
				System.Threading.ThreadPool.QueueUserWorkItem(new WaitCallback(InvokeRefreshInternal));
#endif
                
#endif
#else
                if (InvokeRequired)
                    BeginInvoke(new EventHandler(InvokeRefreshInternal), new object[] { null, null });
                else
                    InvokeRefreshInternal(null, null);
#endif
            }
#if !PocketPC
            catch (ThreadAbortException)
            {
                // Do nothing!  Just shut down
            }
#endif
            catch (ObjectDisposedException)
            {
                // Ignore!
            }
            catch
            {
                throw;
            }
        }

#if PocketPC
		private void InvokeRefreshInternal(object sender)
		{
            try
            {
                Invoke(new EventHandler(InvokeRefreshInternal));
            }
            catch (ObjectDisposedException)
            {
                // The control was disposed, just exit
            }
            //catch (InvalidOperationException)
            //{
            //    // The control is not yet initialized.  Jut exit
            //}
            catch
            {
                throw;
            }
		}
#endif

        private void InvokeRefreshInternal(object sender, EventArgs e)
        {
            try
            {
                // If the control is not visible, it's not critical to get a refresh right away,
                // so do an Invalidate instead of a Refresh
                if (Visible)
                {
                    if (IsRefreshAllowed)
                    {
                        IsRefreshAllowed = false;
                        Refresh();
                    }
                    //else
                    //{
                    //    //Application.DoEvents();
                    //    //Console.WriteLine("Waiting for paint to complete...");
                    //    //Invalidate();
                    //}
                }
                else
                    Invalidate();
            }
#if !PocketPC
            catch (ThreadAbortException)
            {
                // Do nothing!
            }
#endif
            catch
            {
                throw;
            }
        }

        /// <summary>Causes the control to be repainted off-screen.</summary>
        /// <remarks>
        /// 	<para>This method makes a new request to repaint the control off-screen. If a
        ///     separate rendering thread is not used, the control is repainted and redrawn. If
        ///     rendering on a separate thread, the request is queued and this method returns
        ///     immedately.</para>
        /// 	<para>For multithreaded repainting, this method produces the same result as calling
        ///     <strong>InvokePaintOffScreen</strong> because all repaint requests are queued
        ///     instead of executed immediately.</para>
        /// 	<para>This method should be called in place of the <strong>Invalidate</strong>
        ///     method.</para>
        /// </remarks>
        /// <example>
        ///     This example causes a control to repaint off-screen. 
        ///     <code lang="VB" title="[New Example]">
        /// ' Has any data changed?
        /// If DataHasChanged Then
        ///     ' Yes.  Tell the control to repaint
        ///     MyControl.InvokePaintOffScreen()
        /// End If
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// 	</code>
        /// </example>
        public virtual void InvokePaintOffScreen()
        {
            InvokePaintOffScreenImpl(false);
        }

#if PocketPC && !Framework20
        private void InvokePaintOffScreen(object state)
        {
            InvokePaintOffScreenImpl(false);
        }
#endif

        /// <summary>Causes an asynchronous request to repaint the control off-screen.</summary>
        /// <remarks>
        /// 	<para>This method makes a new request to repaint the control off-screen. If a
        ///     separate rendering thread is not used, the control is repainted and redrawn. If
        ///     rendering on a separate thread, the request is queued and this method returns
        ///     immedately.</para>
        /// 	<para>For multithreaded repainting, this method produces the same result as calling
        ///     <strong>InvokePaintOffScreen</strong> because all repaint requests are queued
        ///     instead of executed immediately.</para>
        /// 	<para>This method should be called in place of the <strong>Invalidate</strong>
        ///     method.</para>
        /// </remarks>
        /// <example>
        ///     This example causes a control to repaint off-screen. 
        ///     <code lang="VB" title="[New Example]">
        /// ' Has any data changed?
        /// If DataHasChanged Then
        ///     ' Yes.  Tell the control to repaint
        ///     MyControl.InvokePaintOffScreen()
        /// End If
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// 	</code>
        /// </example>
        public void BeginInvokePaintOffScreen()
        {
            InvokePaintOffScreenImpl(true);
        }

#if PocketPC && !Framework20
        private void DoPaint(object state)
        {
            InvokePaintOffScreenInternal();
        }
#endif

        private void InvokePaintOffScreenImpl(bool invokeAsynchronously)
        {
            // If the control is not yet created, exit
            if (!AllowPainting || pIsDisposed || pIsPaintingSuspended || !pIsInitialized)
                return;

            if (pIsPaintingOnSeparateThread)
            {
                // Bump up the request iteration
                Interlocked.Increment(ref RenderRequestIteration);
                // Signal a new rendering operation
                RenderRequestWaitHandle.Set();
            }
            else
            {
                if (invokeAsynchronously)
#if !PocketPC || Framework20
                    GeoFramework.Threading.ThreadPool.Queue(new WaitCallback(InvokePaintOffScreenInternal));
#else
					System.Threading.ThreadPool.QueueUserWorkItem(new WaitCallback(InvokePaintOffScreenInternal));
#endif
                else
                    InvokePaintOffScreenInternal();
            }
        }

#if PocketPC && !Framework20
        private void InvokePaintOffScreenInternalImpl(object sender, EventArgs e)
        {
            InvokePaintOffScreenInternal();
        }
#endif

        /// <summary>Returns the width of the control in pixels.</summary>
        /// <value>An <strong>Integer</strong> indicating the width of the control in pixels.</value>
        /// <remarks>
        /// The default <strong>Width</strong> property of the <strong>Control</strong> class
        /// cannot be accessed from a thread other than the UI thread. As a result, this property
        /// was shadowed in order to make it thread-safe.
        /// </remarks>
#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Returns the horizontal length of the control in pixels.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
        [EditorBrowsable(EditorBrowsableState.Always)]
#endif
        public new int Width
        {
            get
            {
                return ControlWidth;
            }
            set
            {
                if (ControlWidth.Equals(value)) return;
                ControlWidth = value;
                base.Width = value;
            }
        }

#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Returns the horizontal and vertical length of the control in pixels.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
        [EditorBrowsable(EditorBrowsableState.Always)]    
#endif
        public new Size Size
        {
            get
            {
                return new Size(ControlWidth, ControlHeight);
            }
            set
            {
                if (ControlWidth.Equals(value.Width) && ControlHeight.Equals(value.Height))
                    return;

                ControlWidth = value.Width;
                ControlHeight = value.Height;

                base.Size = value;
            }
        }

        /// <remarks>
        /// The default <strong>Height</strong> property of the <strong>Control</strong>
        /// class cannot be accessed from a thread other than the UI thread. As a result, this
        /// property was shadowed in order to make it thread-safe.
        /// </remarks>
        /// <summary>Returns the height of the control in pixels.</summary>
        /// <value>An <strong>Integer</strong> indicating the width of the control in pixels.</value>
#if !PocketPC || DesignTime
        [Category("Appearance")]
        [Description("Returns the vertical length of the control in pixels.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
        public new int Height
        {
            get
            {
                return ControlHeight;
            }
            set
            {
                if (ControlHeight.Equals(value)) return;
                ControlHeight = value;
                base.Height = value;
            }
        }

        private void PaintingThreadProc()
        {
            try
            {
                try
                {
                    // Signal that start-up is complete
                    PaintingWaitHandle.Set();

                    // And reset so people can WaitOne during shutdown
                    PaintingWaitHandle.Reset();
                }
                catch (ObjectDisposedException)
                {
                    // Premature shutdown!  Exit.
                    return;
                }

#if PocketPC && !DesignTime && !Framework20
                // Signal that we're alive now
                pIsPaintingThreadAlive = true;
#endif

                // Keep rendering until we're stopping
                while (IsPaintingActive && pIsPaintingOnSeparateThread)
                {
                    try
                    {
#if PocketPC || !Framework20
                        // Update the thread priority
                        if (PaintingThread.Priority != pThreadPriority)
                            PaintingThread.Priority = pThreadPriority;
#endif

                        // Wait for rendering to become needed
                        if (RenderRequestIteration == RenderCompleteIteration)
                        {
                            // There's no Suspend method, so use a reset event instead
                            RenderRequestWaitHandle.WaitOne();
                        }

                        // If we've stopped rendering, exit
                        if (!IsPaintingActive)
                            break;

                        // If painting hasn't been suspended or is just disallowed for any reason, continue rendering
                        if (AllowPainting)
                        {
                            // Begin rendering
                            RenderCurrentIteration = RenderRequestIteration;
                            if (InvokePaintOffScreenInternal())
                            {
                                // Flag that we've caught up to the current request number
                                RenderCompleteIteration = RenderCurrentIteration;
                            }
                            else
                            {
                                if (!pIsDisposed)
                                    Thread.Sleep(500);
                            }
                        }
                    }
                    catch (ObjectDisposedException)
                    {
                        // Exit!
                        break;
                    }
#if !PocketPC || Framework20
                    catch (ThreadAbortException)
                    {
                        // Just exit!
                        break;
                    }
#endif
                    catch (Exception ex)
                    {
                        OnPaintException(ex);
                    }
                    finally
                    {
                        // Flag that rendering is complete
                        if (RenderCompleteIteration == RenderCurrentIteration
                            && !IsDisposed 
#if Framework20 && !PocketPC
                            && !RenderRequestWaitHandle.SafeWaitHandle.IsInvalid)
#else
                            && RenderRequestWaitHandle.Handle != new IntPtr(-1))
#endif
                        {
                            try
                            {
								RenderRequestWaitHandle.Reset();
                            }
                            catch
                            {
                                // Ignore
                            }
                        }

						// KEEP THIS IN.  It prevents the CPU from going to 100%
						// when multiple controls are on a form.
                        Thread.Sleep(10);
                        Thread.Sleep(0);
                    }
                }
            }
            catch
            {
                // Ignore since we're on a separate thread
            }
            finally
            {
                // Flag that the thread is dead
#if Framework20 && !PocketPC
                if (!PaintingWaitHandle.SafeWaitHandle.IsInvalid)
#else
                if (PaintingWaitHandle.Handle != new IntPtr(-1))
#endif
                {
                    try
                    {
                        PaintingWaitHandle.Set();
                    }
                    catch
                    {
                        // Do nothing, we're shutting down
                    }
                }

#if PocketPC && !DesignTime && !Framework20
                // Signal that we're alive now
                pIsPaintingThreadAlive = false;
#endif
            }
        }

#if !PocketPC || Framework20 || DesignTime
        protected override void OnHandleCreated(EventArgs e)
        {
            try
            {
                base.OnHandleCreated(e);
                // Are we initialized?
                if (!pIsInitialized)
                {
                    try
                    {
                        OnInitialize();
                    }
                    catch
                    {
                    }
                    finally
                    {
                        pIsInitialized = true;
                        AllowPainting = true;
                        NeedNewBitmaps = true;
                    }
                }
                // And render
                BeginInvokePaintOffScreen();
            }
#if !PocketPC || Framework20 || DesignTime
            catch (ThreadAbortException)
            {
                // Just exit!
            }
#endif
            catch (Exception ex)
            {
                OnPaintException(ex);
            }
        }

        protected override void OnHandleDestroyed(EventArgs e)
        {
            try
            {
                // Shut down rendering
                AllowPainting = false;
                IsPaintingOnSeparateThread = false;
            }
#if !PocketPC || Framework20 || DesignTime
            catch (ThreadAbortException)
            {
                // Just exit!
            }
#endif
            catch (Exception ex)
            {
                OnPaintException(ex);
            }
            finally
            {
                base.OnHandleDestroyed(e);
            }
        }
#endif

        /// <summary>Returns the bitmap used to paint the visible portion of the control.</summary>
        /// <value>A <strong>Bitmap</strong> containing data to be painted on the display.</value>
        /// <remarks>
        /// 	<para>This control maintains two separate bitmaps: an "off-screen" bitmap, where
        ///     all painting operations take place, and an "on-screen" bitmap which is displayed
        ///     visually to the user. When an off-screen painting operation completes successfully,
        ///     the off-screen bitmap is copies to the on-screen bitmap, then painted on the
        ///     display. This property returns the on-screen bitmap matching what is actually seen
        ///     in the control.</para>
        /// </remarks>
#if !PocketPC || DesignTime
		[Category("Appearance")]
        [Description("Returns the bitmap used to paint the visible area of the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Advanced)]
#endif
        public Bitmap OnScreenBitmap
        {
            get
            {
                return pOnScreenBitmap;
            }
        }

#if !PocketPC
        /// <summary>Controls the amount of smoothing applied to lines and shapes.</summary>
        /// <value>
        /// A <strong>SmoothingMode</strong> value. Default is
        /// <strong>HighQuality</strong>.
        /// </value>
        /// <remarks>
        /// This property can drastically improve the look of a control by enabling smoothing
        /// effects such as anti-aliasing. While smoothing effects improve appearance, they also
        /// slow down painting operations.
        /// </remarks>
        [Category("Quality")]
        [DefaultValue(typeof(SmoothingMode), "HighQuality")]
        [Description("Controls the amount of smoothing applied to all graphics drawn on the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public SmoothingMode SmoothingMode
        {
            get
            {
                return pSmoothingMode;
            }
            set
            {
                if (pSmoothingMode == value) return;
                pSmoothingMode = value;

				lock(OffScreenSyncRoot)
				{
					if(OffScreenGraphics != null)
						OffScreenGraphics.SmoothingMode = pSmoothingMode;
				}

                InvokePaintOffScreen();
            }
        }

#if !PocketPC
        /// <summary>Controls the desired frame rate for rendering operations.</summary>
        /// <value>
        /// An <strong>Integer</strong> indicating the desired frame rate. Default is
        /// <strong>30</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>This property is used by controls which make use of animation effects.
        ///     Typically, this property tells a control how long to delay between repaint
        ///     operations, or how many frames to allocate for a certain time period. Controls
        ///     which do not do any custom animation effects ignore this property and it has no
        ///     effect. When this property changes, the <strong>OnTargetFrameRateChanged</strong>
        ///     virtual method is called.</para>
        /// 	<para>If a control is able to repaint itself very quickly, smooth animation effects
        ///     can be achieved. A refresh rate of 30 frames per second or above is necessary to
        ///     give the human eye the illusion of motion. Refresh rates of 60 or even 120 are
        ///     possible for extremely fast controls and can result in very nice, smooth animation
        ///     effects.</para>
        /// 	<para>This property can be a bit confusing for developers because it does not
        ///     control the rate at which the control is actually repainted. (The control is only
        ///     repainted when necessary, whenever that occurs.) This property is used only to tune
        ///     custom effects implemented by controls inheriting from this class.</para>
        /// </remarks>
        /// <example>
        ///     This example animates a green rectangle. The default target frame rate is set to 60
        ///     frames per second, which causes the interval of a timer to change. The timer
        ///     changes the position of the rectangle. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     Dim AnimatedRectangle As Rectangle
        ///     
        ///     Sub New()
        ///         ' Set a 60 frames per second target
        ///         TargetFramesPerSecond = 60
        ///     End Sub
        ///     
        ///     Protected Overrides Sub OnTargetFrameRateChanged()
        ///         ' Change the timer to fire 60 times per second
        ///         MyTimer.Interval = 1000 / TargetFramesPerSecond
        ///     End Sub
        ///     
        ///     Private Sub MyTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        ///         ' Change the location of an animated thing
        ///         AnimatedRectangle = New Rectangle((AnimatedRectangle.X + 1) Mod Width, 
        ///                                           (AnimatedRectangle.Y + 1) Mod Height, 
        ///                                           50, 50)
        ///         ' And cause the control to repaint
        ///         InvokePaintOffScreen()
        ///     End Sub
        ///  
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         Dim MyBrush As New SolidBrush(Color.Green)
        ///         e.Graphics.FillRectangle(MyBrush, AnimatedRectangle)
        ///         MyBrush.Dispose()
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        ///     Rectangle AnimatedRectangle;
        ///     
        ///     MyControl()
        ///     {
        ///         // Set a 60 frames per second target
        ///         TargetFramesPerSecond = 60;
        ///     }
        ///     


        ///     protected override void OnTargetFrameRateChanged()
        ///     {
        ///         // Change the timer to fire 60 times per second
        ///         MyTimer.Interval = 1000 / TargetFramesPerSecond
        ///     }
        ///     
        ///     private void MyTimer_Tick(object sender, EventArgs e)
        ///     {
        ///         // Change the location of an animated thing
        ///         AnimatedRectangle = New Rectangle((AnimatedRectangle.X + 1) ^ Width, 
        ///                                           (AnimatedRectangle.Y + 1) ^ Height, 
        ///                                           50, 50);
        ///         // And cause the control to repaint
        ///         InvokePaintOffScreen();
        ///     }
        ///  
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         SolidBrush MyBrush = new SolidBrush(Color.Green);
        ///         e.Graphics.FillRectangle(MyBrush, AnimatedRectangle);
        ///         MyBrush.Dispose();
        ///     }
        /// }
        ///     </code>
        /// </example>
        [Category("Performance")]
        [DefaultValue(typeof(int), "90")]
        [Description("Controls the preferred frames per second refresh rate for the control.  Animations at 30FPS or above will appear more fluid.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public int TargetFramesPerSecond
        {
            get
            {
                return pTargetFramesPerSecond;
            }
            set
            {
                if (pTargetFramesPerSecond == value) return;
                if (value <= 0)
                    throw new ArgumentException(Resources.DoubleBufferedControl_InvalidFramesPerSecond);
                pTargetFramesPerSecond = value;
                OnTargetFrameRateChanged(pTargetFramesPerSecond);
                InvokePaintOffScreen();
            }
        }

        /// <summary>Controls the goal of all painting operations.</summary>
        /// <value>
        /// A <strong>ControlOptimizationMode</strong> value. Default is
        /// <strong>Balanced</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>This powerful property provides a way to modify properties in bulk in order
        ///     to achieve different levels of performance. For example, changing this property to
        ///     <strong>BestRefreshRate</strong> disables all anti-aliasing and compositing effects
        ///     to that rendering is as fast as possible at the cost of quality. On the other hand,
        ///     changing this property to <strong>BestQuality</strong> enables all smoothing
        ///     effects at the cost of speed. When this property changes, the
        ///     <strong>OnOptimizationModeChanged</strong> virtual method is called.</para>
        /// </remarks>
        [Category("Performance")]
        [DefaultValue(typeof(ControlOptimizationMode), "Balanced")]
        [Description("Controls rendering bias towards either performance or quality.  Changing this setting alters other properties in the Quality category.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public ControlOptimizationMode OptimizationMode
        {
            get
            {
                return pOptimizationMode;
            }
            set
            {
                if (pOptimizationMode == value) return;
                pOptimizationMode = value;
                // Don't render for a moment
                SuspendLayout();
                // Modify all settings
                OnOptimizationModeChanged(pOptimizationMode);
                // And resume rendering
                ResumeLayout();
            }
        }

        /// <summary>Occurs when the desired frame rate has changed.</summary>
        /// <remarks>
        /// This virtual method is called whenever the <strong>TargetFrameRate</strong>
        /// property has changed. This method gives controls the opportunity to adjust animation
        /// effects to achieve the frame rate as closely as possible.
        /// </remarks>
        /// <param name="framesPerSecond">
        /// An <strong>Integer</strong> specifying the ideal or desired frame rate for the
        /// control.
        /// </param>
        protected virtual void OnTargetFrameRateChanged(int framesPerSecond)
        {
        }

        //		public int TargetFramesPerSecond
        //		{
        //			get
        //			{
        //				return pTargetFramesPerSecond;
        //			}
        //			set
        //			{
        //				if(pTargetFramesPerSecond == value) return;
        //				pTargetFramesPerSecond == value;
        //			}
        //		}
#endif

        /// <summary>Occurs when the optimization mode has changed.</summary>
        /// <remarks>
        /// This virtual method gives control the opportunity to enable or disable features
        /// that affect rendering performance. This method is called whenever the
        /// <strong>OptimizationMode</strong> property has changed.
        /// </remarks>
        /// <param name="optimizationMode">
        /// A <strong>ControlOptimizationMode</strong> specifying the rendering goal for the
        /// control.
        /// </param>
        protected virtual void OnOptimizationModeChanged(ControlOptimizationMode optimizationMode)
        {
            // Now adjust all of the settings
            switch (pOptimizationMode)
            {
                case ControlOptimizationMode.Balanced:
                    {
#if PocketPC || !Framework20
					ThreadPriority = ThreadPriority.Normal;
#endif
                        InterpolationMode = InterpolationMode.Low;
                        SmoothingMode = SmoothingMode.HighQuality;
                        PixelOffsetMode = PixelOffsetMode.HighSpeed;
                        TextRenderingHint = TextRenderingHint.AntiAlias;
                        CompositingQuality = CompositingQuality.AssumeLinear;
                        PixelFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                case ControlOptimizationMode.MostDetail:
                    {
#if PocketPC || !Framework20
					ThreadPriority = ThreadPriority.Highest;
#endif
                        InterpolationMode = InterpolationMode.HighQualityBicubic;
                        SmoothingMode = SmoothingMode.HighQuality;
                        PixelOffsetMode = PixelOffsetMode.HighQuality;
                        TextRenderingHint = TextRenderingHint.AntiAlias;
                        CompositingQuality = CompositingQuality.HighQuality;
                        PixelFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
                case ControlOptimizationMode.Custom:
                    {
                        break;
                    }
                case ControlOptimizationMode.BestRefreshRate:
                    {
#if PocketPC || !Framework20
					ThreadPriority = ThreadPriority.AboveNormal;
#endif
                        InterpolationMode = InterpolationMode.Low;
                        SmoothingMode = SmoothingMode.None;
                        PixelOffsetMode = PixelOffsetMode.None;
                        TextRenderingHint = TextRenderingHint.SingleBitPerPixel;
                        CompositingQuality = CompositingQuality.HighSpeed;
                        PixelFormat = PixelFormat.Format16bppRgb555;
                        break;
                    }
                case ControlOptimizationMode.LeastProcessorUsage:
                    {
#if PocketPC || !Framework20
					ThreadPriority = ThreadPriority.Lowest;
#endif
                        InterpolationMode = InterpolationMode.Low;
                        SmoothingMode = SmoothingMode.None;
                        PixelOffsetMode = PixelOffsetMode.None;
                        TextRenderingHint = TextRenderingHint.SingleBitPerPixel;
                        CompositingQuality = CompositingQuality.HighSpeed;
                        PixelFormat = PixelFormat.Format16bppRgb555;
                        break;
                    }
                case ControlOptimizationMode.BestQuality:
                    {
#if PocketPC || !Framework20
					ThreadPriority = ThreadPriority.Highest;
#endif
                        InterpolationMode = InterpolationMode.HighQualityBicubic;
                        SmoothingMode = SmoothingMode.HighQuality;
                        PixelOffsetMode = PixelOffsetMode.HighQuality;
                        TextRenderingHint = TextRenderingHint.AntiAlias;
                        CompositingQuality = CompositingQuality.HighQuality;
                        PixelFormat = PixelFormat.Format32bppArgb;
                        break;
                    }
            }
        }

        /// <summary>Controls the level of quality applied to text painting operations.</summary>
        /// <value>
        /// A <strong>TextRenderingHint</strong> value. Default is
        /// <strong>AntiAlias</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>This property controls the amount of smoothing applied to text when it is
        ///     drawn on-screen. When set to <strong>AntiAlias</strong>, pixels nearby lines drawn
        ///     in text are interpolated, giving the appearance of smoother lines.</para>
        /// 	<para>After researching this property, GeoFrameworks believes that a setting of
        ///     <strong>AntiAlias</strong> can produce better results than
        ///     <strong>ClearType</strong> or <strong>ClearTypeGridFit</strong> settings because
        ///     ClearType is not used on rotated text, whereas <strong>AntiAlias</strong>
        ///     functions well on all text operations.</para>
        /// </remarks>
        [Category("Quality")]
        [DefaultValue(typeof(TextRenderingHint), "AntiAlias")]
        [Description("Controls the amount of smoothing applied to all graphics drawn on the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public TextRenderingHint TextRenderingHint
        {
            get
            {
                return pTextRenderingHint;
            }
            set
            {
                if (pTextRenderingHint == value) return;
                pTextRenderingHint = value;

				lock(OffScreenSyncRoot)
				{
					if(OffScreenGraphics != null)
						OffScreenGraphics.TextRenderingHint = pTextRenderingHint;
				}

                InvokePaintOffScreen();
            }
        }

        /// <summary>Controls the technique used to merge pixels with the background.</summary>
        /// <remarks>
        /// The compositing mode determines whether pixels from a source image overwrite or
        /// are combined with background pixels.
        /// </remarks>
        /// <value>
        /// A <strong>CompositingMode</strong> value. Default is
        /// <strong>SourceOver</strong>.
        /// </value>
        [Category("Quality")]
        [DefaultValue(typeof(CompositingMode), "SourceOver")]
        [Description("Controls the technique used to blend painted information with pixels of the background.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
		public CompositingMode CompositingMode
        {
            get
            {
                return pCompositingMode;
            }
            set
            {
                if (pCompositingMode == value) return;
                pCompositingMode = value;

				lock(OffScreenSyncRoot)
				{
					if(OffScreenGraphics != null)
						OffScreenGraphics.CompositingMode = pCompositingMode;
				}

                InvokePaintOffScreen();
            }
        }

        /// <summary>Controls the technique used to blend overlapping pixels.</summary>
        /// <value>
        /// A <strong>CompositingQuality</strong> value. Default is
        /// <strong>AssumeLinear</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>Compositing is done during rendering when the source pixels are combined with
        ///     the destination pixels to produce the resultant pixels. The quality of compositing
        ///     directly relates to the visual quality of the output and is inversely proportional
        ///     to the render time. The higher the quality, the slower the render time. This is
        ///     because the higher the quality level, the more surrounding pixels need to be taken
        ///     into account during compositing. The linear quality setting <b>AssumeLinear</b>
        ///     compromises by providing better quality than the default quality at a slightly
        ///     lower speed.</para>
        /// </remarks>
        [Category("Quality")]
        [DefaultValue(typeof(CompositingQuality), "AssumeLinear")]
        [Description("Controls the technique used to blend overlapping pixels.")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
        [NotifyParentProperty(true)]
        public CompositingQuality CompositingQuality
        {
            get
            {
                return pCompositingQuality;
            }
            set
            {
                if (pCompositingQuality == value) return;
                pCompositingQuality = value;

				lock(OffScreenSyncRoot)
				{
					if(OffScreenGraphics != null)
						OffScreenGraphics.CompositingQuality = pCompositingQuality;
				}

                InvokePaintOffScreen();
            }
        }

        /// <summary>
        /// Controls the technique used to calculate points in between other known
        /// points.
        /// </summary>
        /// <value>
        /// An <strong>InterpolationMode</strong> value. Default is
        /// <strong>Low</strong>.
        /// </value>
        /// <remarks>
        /// Interpolation is the process of calculating values between two other values. This
        /// technique is frequently used for the drawing of splines and curves, where only a few
        /// "control points" are known. The highest quality is achieved using bicubic
        /// interpolation, whereas more performance can be seen when using a setting of
        /// <strong>Low</strong>.
        /// </remarks>
        [Category("Quality")]
        [DefaultValue(typeof(InterpolationMode), "Low")]
        [Description("Controls the technique used to calculate intermediate positions for curved shapes.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public InterpolationMode InterpolationMode
        {
            get
            {
                return pInterpolationMode;
            }
            set
            {
                if (pInterpolationMode == value) return;
                pInterpolationMode = value;

				lock(OffScreenSyncRoot)
				{
					if(OffScreenGraphics != null)
						OffScreenGraphics.InterpolationMode = pInterpolationMode;
				}

                InvokePaintOffScreen();
            }
        }

        /// <summary>Controls the color depth of bitmaps used by the control.</summary>
        /// <value>
        /// A <strong>PixelFormat</strong> value. Default is
        /// <strong>Format32bppArgb</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>The pixel format controls the bits-per-pixel of bitmaps. Having more bits per
        ///     pixel allows for greater ranges of color but requires more memory. Likewise, less
        ///     bits reduces the number of colors per pixel but uses less memory. This property is
        ///     typically modified via the <strong>OptimizationMode</strong> property to use bitmap
        ///     formats that improve rendering speed or reduce the memory requirements of the
        ///     control.</para>
        /// 	<para>This property is for advanced users only. Several of the possible values in
        ///     the <strong>PixelFormat</strong> enumeration will produce a black screen or other
        ///     undesirable results.</para>
        /// </remarks>
        [DefaultValue(typeof(PixelFormat), "Format32bppArgb")]
        [Category("Quality")]
        [Description("Controls the number of bits per pixel for the bitmaps rendered by the control.  A smaller value uses less memory at the cost of quality.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public PixelFormat PixelFormat
        {
            get
            {
                return pPixelFormat;
            }
            set
            {
                if (pPixelFormat == value) return;
                pPixelFormat = value;
                InvokePaintOffScreen();
            }
        }

        /// <summary>Controls the offset applied to all pixels during painting operations.</summary>
        /// <value>
        /// A <strong>PixelOffsetMode</strong> value. Default is
        /// <strong>HighSpeed</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>This property was intended to improve anti-aliasing effects by shifting all
        ///     pixels by slight amount, such as half a pixel. A value of
        ///     <strong>HighQuality</strong> causes pixels to be shifted slightly, whereas a value
        ///     of <strong>HighSpeed</strong> ignores pixel shifting.</para>
        /// 	<para>Author's Note: "During development of this class, we tested the
        ///     <strong>HighQuality</strong> setting and didn't see any improvement in quality that
        ///     was worth the extra CPU time, so <strong>HighSpeed</strong> should be favored." --
        ///     Jon Persion</para>
        /// </remarks>
        [Category("Quality")]
        [DefaultValue(typeof(PixelOffsetMode), "HighSpeed")]
        [Description("Controls the offset amount applied to pixels in order to improve anti-aliasing.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		[Browsable(true)]
		[NotifyParentProperty(true)]
        public PixelOffsetMode PixelOffsetMode
        {
            get
            {
                return pPixelOffsetMode;
            }
            set
            {
                if (pPixelOffsetMode == value) return;
                pPixelOffsetMode = value;

				lock(OffScreenSyncRoot)
				{
					if(OffScreenGraphics != null)
						OffScreenGraphics.PixelOffsetMode = pPixelOffsetMode;
				}

                InvokePaintOffScreen();
            }
        }

        //		[Category("Quality")]
        //		[DefaultValue(typeof(int), "4")] 
        //		[Description("Controls the contract level applied to rendered text.")]
        //		public int TextContrast
        //		{
        //			get
        //			{
        //				return pTextContrast;
        //			}
        //			set
        //			{
        //				if(pTextContrast == value) return;
        //				pTextContrast = value;
        //				InvokePaintOffScreen();
        //			}
        //		}
#endif

        /// <summary>Performs all major painting operations for the control.</summary>
        /// <remarks>
        /// 	<para>This method must be overridden. All painting operations for the control take
        ///     place during this method. After this method completes, the
        ///     <strong>OnPaintOffScreenAdornments</strong> method is called. When all painting
        ///     operations have completed, the task of repainting the control visually takes place
        ///     automatically.</para>
        /// 	<para>Developers seeking to upgrade their existing controls to use this control
        ///     must move all code from <strong>OnPaint</strong> into
        ///     <strong>OnPaintOffScreen</strong>. For best performance, the
        ///     <strong>OnPaint</strong> method would not be overridden at all.</para>
        /// </remarks>
        /// <param name="e">
        /// A <strong>CancelablePaintEventArgs</strong> object used for all painting
        /// operations, as well as to signal when a rendering iteration has been aborted.
        /// </param>
        protected virtual void OnPaintOffScreen(CancelablePaintEventArgs e)
        {
			if (PaintOffScreen != null)
#if PocketPC && !Framework20
				PaintOffScreen(this, e);
#else
				DynamicEvent.Invoke(PaintOffScreen, this, e);
#endif
        }

        /// <summary>
        /// Optional. Performs painting operations which draw the control's
        /// background.
        /// </summary>
        /// <remarks>
        /// 	<para>This method provides the ability to prepare the background of the control
        ///     before major painting operations take place. By default, this method calls the
        ///     <strong>Graphics.Clear</strong> method to erase the control to the background
        ///     color.</para>
        /// 	<para>This method can be overridden to perform certain visual effects. For example,
        ///     the <strong>PolarControl</strong> class, which inherits from this class, uses this
        ///     method to apply a fading circular gradient to create a 3D illusion. Then, the
        ///     <strong>OnPaintOffScreenAdornments</strong> method is also overridden to apply a
        ///     second effect which gives the appearance of glass over the control. For more
        ///     information on glass effects, see the <strong>Effect</strong> property of the
        ///     <strong>PolarControl</strong> class.</para>
        /// </remarks>
        /// <param name="e">
        /// A <strong>CancelablePaintEventArgs</strong> object used for all painting
        /// operations, as well as to signal when a rendering iteration has been aborted.
        /// </param>
        protected virtual void OnPaintOffScreenBackground(CancelablePaintEventArgs e)
        {
            // Clear its background
            e.Graphics.Clear(pBackgroundColor);

			if (PaintOffScreenBackground != null)
#if PocketPC && !Framework20
				PaintOffScreenBackground(this, e);
#else
				DynamicEvent.Invoke(PaintOffScreenBackground, this, e);
#endif
        }

        /// <summary>
        /// Optional. Performs any additional painting operations after the main portions of
        /// the control are painted.
        /// </summary>
        /// <remarks>
        /// By default, this method does nothing. This method can be overridden to apply
        /// certain details, however, such as a watermark, company logo, or glass. For example, the
        /// <strong>PolarControl</strong> class overrides this method to apply a glass effect on
        /// top of anything that has already been painted. For more information on glass effects,
        /// see the <strong>Effect</strong> property of the <strong>PolarControl</strong>
        /// class.
        /// </remarks>
        /// <param name="e">
        /// A <strong>CancelablePaintEventArgs</strong> object used for all painting
        /// operations, as well as to signal when a rendering iteration has been aborted.
        /// </param>
        protected virtual void OnPaintOffScreenAdornments(CancelablePaintEventArgs e)
        {
			if (PaintOffScreenAdornments != null)
#if PocketPC && !Framework20
				PaintOffScreenAdornments(this, e);
#else
				DynamicEvent.Invoke(PaintOffScreenAdornments, this, e);
#endif
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            // Update the control width (used for rendering on a separate thread)
            // For some reason, .NET barfs when trying to access Width or Height from a seaprate
            // thread.  Using these variables makes the property thread-safe.
            ControlWidth = base.Width;
            ControlHeight = base.Height;
            NeedNewBitmaps = true;

            // Refresh the control
            InvokePaintOffScreen();
        }

#if !PocketPC
        protected override void OnVisibleChanged(EventArgs e)
        {
            base.OnVisibleChanged(e);
            // Suspend or resume rendering.  There's no need to keep
            // rendering if the control is invisible anyway
            if (!Visible)
                SuspendLayout();
            else
                ResumeLayout();
        }
#endif

        //#if !PocketPC		
        //        protected override void OnBackColorChanged(EventArgs e)
        //        {
        //            base.OnBackColorChanged (e);
        //            InvokePaintOffScreen();
        //        }
        //#endif

#if PocketPC
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //Do nothing
        }
#endif

        /// <summary>
        /// Occurs when the control is about to be rendered for the first time.
        /// </summary>
        /// <remarks>
        /// 	<para>This virtual method provides the ability to prepare any local variables
        ///     before the control is painted for the first time. This method is typically used to
        ///     create GDI objects such as <strong>Pen</strong>, <strong>Brush</strong>, and
        ///     <strong>Font</strong> immediately before they are needed. It is recommended that
        ///     this event be used to create any such GDI objects. Additionally, it is also
        ///     recommended that GDI objects get created outside of the
        ///     <strong>OnPaintOffScreen</strong> method if they are used repeatedly.</para>
        /// 	<para>For desktop framework applications, this method is called when the control's
        ///     handle is created. For Compact Framework 1.0 applications, there is no handle
        ///     creation event, so this method is called when the first call to
        ///     <strong>OnPaint</strong> occurs.</para>
        /// </remarks>
        protected virtual void OnInitialize()
        {
        }

        /// <summary>Occurs when the control is to be redrawn on-screen.</summary>
        /// <remarks>
        /// 	<para>In the <strong>DoubleBufferedControl</strong> class, the process of painting
        ///     the control on-screen is handled automatically. As a result, this method does not
        ///     have to be overloaded to paint the control. In fact, this method should not be
        ///     overridden to perform any painting operations because it would defeat the purpose
        ///     of the control and re-introduce flickering problems.</para>
        /// 	<para>Ideally, well-written controls will move any and all painting operations from
        ///     the <strong>OnPaint</strong> method into the <strong>OnPaintOffScreen</strong>
        ///     method, and avoid overriding <strong>OnPaint</strong> entirely. By keeping the
        ///     <strong>OnPaint</strong> method as lightweight as possible, user interfaces remain
        ///     responsive and free from flickering problems.</para>
        /// </remarks>
        /// <example>
        /// 	<para>This example demonstrates how a user control is upgraded from Control to the
        ///     <strong>DoubleBufferedControl</strong> class. Upgrading is straightforward: All
        ///     painting operations are moved from <strong>OnPaint</strong> to
        ///     <strong>OnPaintOffScreen</strong> and <strong>OnPaint</strong> is no longer
        ///     overridden.</para>
        /// 	<code lang="VB" title="[New Example]" description="Before: A control's paint operations before being upgraded to use the DoubleBufferedControl class.">
        /// Public Class MyControl 
        ///     Inherits Control
        ///     Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        ///         Dim MyBrush As New SolidBrush(Color.Blue)
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50))
        ///         MyBrush.Dispose();
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]" description="Before: A control's paint operations before being upgraded to use the DoubleBufferedControl class.">
        /// public class MyControl : Control
        /// {
        ///     protected override void OnPaint(PaintEventArgs e)
        ///     {
        ///         SolidBrush MyBrush = new SolidBrush(Color.Blue);
        ///         e.Graphics.FillRectangle(MyBrush, new Rectangle(50, 50, 50, 50));
        ///         MyBrush.Dispose();
        ///     }
        /// }
        ///     </code>
        /// 	<code lang="VB" title="[New Example]" description="After: A control's paint operations after upgrading to DoubleBufferedControl.">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         Dim MyBrush As New SolidBrush(Color.Blue)
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50))
        ///         MyBrush.Dispose();
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         SolidBrush MyBrush = new SolidBrush(Color.Blue);
        ///         e.Graphics.FillRectangle(MyBrush, new Rectangle(50, 50, 50, 50));
        ///         MyBrush.Dispose();
        ///     }
        /// }
        ///     </code>
        /// </example>
        protected override void OnPaint(PaintEventArgs e)
        {
            try
            {
#if PocketPC && !Framework20
				// Are we initialized?
				if(!pIsInitialized)
				{
					OnInitialize();
					pIsInitialized = true;
					AllowPainting = true;
#if !PocketPC || Framework20
					GeoFramework.Threading.ThreadPool.Queue(new WaitCallback(InvokePaintOffScreen));
#else
					ThreadPool.QueueUserWorkItem(new WaitCallback(InvokePaintOffScreen));
#endif
				}
#elif !PocketPC && !DesignTime
                // Set the rendering quality
                //				e.Graphics.InterpolationMode = pInterpolationMode;
                //				e.Graphics.SmoothingMode = pSmoothingMode;
                //				e.Graphics.TextRenderingHint = pTextRenderingHint;
                //				e.Graphics.CompositingMode = pCompositingMode;
                //				e.Graphics.CompositingQuality = pCompositingQuality;
                //				e.Graphics.PixelOffsetMode = pPixelOffsetMode;
                //				e.Graphics.TextContrast = pTextContrast;
#elif PocketPC && DesignTime
                e.Graphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Low;
				e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;
				e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.SingleBitPerPixelGridFit;
				e.Graphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighSpeed;
#endif

                // Draw the final bitmap on-screen if there is one
                lock (OnScreenSyncRoot)
                {
#if PocketPC 
					if(pIsUpperLeftCornerAdjusted)
						e.Graphics.DrawImage(pOnScreenBitmap, new Rectangle(-1, -1, ControlWidth + 1, ControlHeight + 1), new Rectangle(0, 0, ControlWidth, ControlHeight), GraphicsUnit.Pixel);
					else
						e.Graphics.DrawImage(pOnScreenBitmap, 0, 0);
#else

                    e.Graphics.DrawImageUnscaled(pOnScreenBitmap, 0, 0, ControlWidth, ControlHeight);
#endif
                }
				//Thread.Sleep(0);
            }
            catch (ArgumentException)
            {
                // Ignore!  This exception happens very rarely, when a bitmap
                // is halfway-created or destroyed.  It occurs in about 1 per 
                // every 1,000 rendering iterations.  
            }
            catch (Exception ex)
            {
                OnPaintException(ex);
            }
            finally
            {
                // Flag that we're able to refresh again
                IsRefreshAllowed = true;
            }
        }

#if PocketPC
		/// <summary>
		/// Controls whether bitmaps rendered on-screen are shifted slightly to correct display problems.
		/// </summary>
		/// <remarks>When Compact Framework 1.0 applications are run on newer PocketPC devices which
		/// support higher resolutions, a rendering bug can occur.  This bug, caused by the Compact Framework,
		/// causes bitmaps to be drawn in the wrong location, inaccurate by one pixel.  The end result is a 
		/// black "corner" in the upper-left of the control.  This property, when enabled, will correct
		/// the problem by scaling the bitmap one pixel larger during rendering.</remarks>
#if !PocketPC || DesignTime
		[Category("Layout")]
		[Description("Controls whether bitmaps rendered on-screen are shifted slightly to correct display problems on newer devices.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Advanced)]
		#if PocketPC
			#if Smartphone
							[DefaultValue(typeof(bool), "False")]
			#elif Framework20
								[DefaultValue(typeof(bool), "False")]
			#else
								[DefaultValue(typeof(bool), "True")]
			#endif
		#endif
#endif
        public bool IsUpperLeftCornerAdjusted
		{
			get
			{
				return pIsUpperLeftCornerAdjusted;
			}
			set
			{
				if(pIsUpperLeftCornerAdjusted == value)
					return;
				pIsUpperLeftCornerAdjusted = value;
				InvokeRefresh();
			}
		}
#endif

        /// <summary>
        /// Indicates the point at the center of the control.
        /// </summary>
        /// <remarks>This property is typically used for centering items in the control.  This property
        /// is updated automatically as the control is resized.</remarks>
        /// <value>
        /// A <strong>Point</strong> structure representing the pixel at the center of the
        /// control.
        /// </value>
        /// <example>
        ///     This example uses the <strong>Center</strong> property to center a rectangle in the
        ///     middle of the control. 
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///  
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         ' Center a rectangle in the middle of the control
        ///         Dim MyShape As New Rectangle(Center.X - 25, Center.Y - 25, 50, 50)
        ///         ' Now paint it        
        ///         Dim MyBrush As New SolidBrush(Color.Green)
        ///         e.Graphics.FillRectangle(MyBrush, MyShape)
        ///         MyBrush.Dispose()
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         // Center a rectangle in the middle of the control
        ///         Rectangle MyShape = new Rectangle(Center.X - 25, Center.Y - 25, 50, 50);
        ///         // Now paint it        
        ///         SolidBrush MyBrush = new SolidBrush(Color.Green);
        ///         e.Graphics.FillRectangle(MyBrush, MyShape);
        ///         MyBrush.Dispose();
        ///     }
        /// }
        ///     </code>
        /// </example>
#if !PocketPC || DesignTime
		[Category("Behavior")]
		[Description("Returns the pixel coordinate located at the center of the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
        public Point Center
        {
            get
            {
                return new Point((int)(ControlWidth * 0.5), (int)(ControlHeight * 0.5));
            }
        }

		~DoubleBufferedControl()
		{
			Dispose(true);
		}

        /// <summary>Releases memory and GDI resources created by the control.</summary>
        /// <remarks>
        /// 	<para>This method is very important to implement properly when working with
        ///     controls. Any <strong>Brush</strong>, <strong>Pen</strong>, <strong>Font</strong>,
        ///     <strong>Matrix</strong>, <strong>GraphicsPath</strong>, <strong>Region</strong> or
        ///     other GDI+ object containing a <strong>Dispose</strong> method must be disposed of
        ///     before they are destructed by the garbage collector.</para>
        /// 	<para>Well-written controls will create unmanaged GDI+ objects outside of the
        ///     <strong>OnPaintOffScreen</strong> method, then dispose of them during the
        ///     <strong>Dispose</strong> method. This practice improves performance by creating as
        ///     new new objects and resources as possible while minimizing problems which may occur
        ///     due to resources which are not properly released.</para>
        /// 	<para>Failure to call the <strong>Dispose</strong> method on GDI+ objects will
        ///     cause memory leaks which will slowly eat up available memory until the application
        ///     can no longer function. Use the "GDI Objects" column of the Windows Task Manager to
        ///     monitor the number of GDI objects allocated. Memory leaks will cause the GDI Object
        ///     count to increase continuously, whereas well-written controls will experience a GDI
        ///     Object count that remains fairly constant over a longer period of time.</para>
        /// 	<para>To view the GDI Objects column in the Windows Task Manager:</para>
        /// 	<para class="xmldocnumberedlist"></para>
        /// 	<list type="bullet">
        /// 		<item>Open the Windows Task Manager</item>
        /// 		<item>Select the "Processes" tab.</item>
        /// 		<item>Select "Choose Columns..." from the View menu.</item>
        /// 		<item>Check the "GDI Objects" box and click OK.</item>
        /// 	</list>
        /// </remarks>
        /// <example>
        ///     This example demonstrates how a control might create subtle problems when the
        ///     <strong>Dispose</strong> method is not used on every GDI+ object. 
        ///     <code lang="VB" title="[New Example]" description="An example of a poorly written control: A new SolidBrush is created on every paint iteration when it only needs to be created once. The brush is never disposed of, creating a memory leak.">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         Dim MyBrush As New SolidBrush(Color.Blue)
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50))
        ///     End Sub
        ///     
        ///     ' NOTE: MyBrush is never disposed.  A memory leak
        ///     ' will occur!
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]" description="An example of a poorly written control: A new SolidBrush is created on every paint iteration when it only needs to be created once. The brush is never disposed of, creating a memory leak.">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         SolidBrush MyBrush = new SolidBrush(Color.Blue);
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50));
        ///     }
        ///     
        ///     // NOTE: MyBrush is never disposed.  A memory leak
        ///     // will occur!
        /// }
        ///     </code>
        /// 	<code lang="VB" title="[New Example]" description="Problems are solved by properly implementing the Dispose method. Performance is improved by moving the SolidBrush declaration out of OnPaintOffScreen.">
        /// Public Class MyControl 
        ///     Inherits DoubleBufferedControl
        ///     ' 1. GDI objects are created outside of the OnPaintOffScreen
        ///     '    methods whenever possible.
        ///     Dim MyBrush As New SolidBrush(Color.Blue)
        ///     
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         ' 2. The paint method is as lightweight as possible,
        ///         '    improving rendering performance
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50))
        ///     End Sub
        ///     
        ///     Public Overrides Sub Dispose(ByVal disposing As Boolean)
        ///         ' 3. Any GDI+ objects are disposed of properly    
        ///         MyBrush.Dispose()
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]" description="Problems are solved by properly implementing the Dispose method. Performance is improved by moving the SolidBrush declaration out of OnPaintOffScreen.">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     // 1. GDI objects are created outside of the OnPaintOffScreen
        ///     //    methods whenever possible.
        ///     SolidBrush MyBrush = new SolidBrush(Color.Blue);
        ///     
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         // 2. The paint method is as lightweight as possible,
        ///         //    improving rendering performance
        ///         e.Graphics.FillRectangle(MyBrush, New Rectangle(50, 50, 50, 50));
        ///     }
        ///     
        ///     public override void Dispose(bool disposing)
        ///     {
        ///         // 3. Any GDI+ objects are disposed of properly    
        ///         MyBrush.Dispose();
        ///     }
        /// }
        ///     </code>
        /// </example>
        protected override void Dispose(bool disposing)
        {
            if (pIsDisposed)
                return;

            pIsDisposed = true;
            
			// No longer need to finalize
			GC.SuppressFinalize(this);

#if !PocketPC 
            // Unhook the event
            AppDomain.CurrentDomain.AssemblyResolve -= new ResolveEventHandler(CurrentDomain_AssemblyResolve);
#endif


            try
            {
                // If we're rendering, close down the thread
                IsPaintingOnSeparateThread = false;
            }
            catch
            {
            }          

            if (pOffScreenBitmap != null)
            {
                try
                {
                    pOffScreenBitmap.Dispose();
                }
                catch
                {
                    // Ignore
                }
            }

			if (pOnScreenBitmap != null)
			{
				try
				{
					pOnScreenBitmap.Dispose();
				}
				catch
				{
					// Ignore
				}
			}

            // And shut down wait handles
#if Framework20 && !PocketPC
            if (!PaintingWaitHandle.SafeWaitHandle.IsClosed)
#else
            if (PaintingWaitHandle.Handle != new IntPtr(-1))
#endif
            {
                try
                {
                    PaintingWaitHandle.Close();
                }
                catch (ObjectDisposedException)
                {
                    // Ignore
                }
            }

#if PocketPC || !Framework20
			if (RenderRequestWaitHandle.Handle != new IntPtr(-1))
			{
				try
				{
					RenderRequestWaitHandle.Close();
				}
				catch (ObjectDisposedException)
				{
					// Ignore
				}
			}
#endif


            try
            {
                base.Dispose(disposing);
            }
            catch
            {
            }
        }

#if !PocketPC
        /// <summary>
        /// Converts a color from HSB (hue, saturation, brightness) to RGB.
        /// </summary>
        /// <param name="a"></param>
        /// <param name="h"></param>
        /// <param name="s"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static Color ColorFromAhsb(int a, float h, float s, float b)
        {

            //if (0 > a || 255 < a)
            //{
            //    return Color.Magenta;
            //    throw new ArgumentOutOfRangeException("a", a,
            //      Resources.InvalidAlpha);
            //}
            //if (0f > h || 360f < h)
            //{
            //    throw new ArgumentOutOfRangeException("h", h,
            //      Resources.InvalidHue);
            //}
            //if (0f > s || 1f < s)
            //{
            //    throw new ArgumentOutOfRangeException("s", s,
            //      Resources.InvalidSaturation);
            //}
            //if (0f > b || 1f < b)
            //{
            //    throw new ArgumentOutOfRangeException("b", b,
            //      Resources.InvalidBrightness);
            //}

            if (0 == s)
            {
                return Color.FromArgb(a, Convert.ToInt32(b * 255),
                  Convert.ToInt32(b * 255), Convert.ToInt32(b * 255));
            }

            float fMax, fMid, fMin;
            int iSextant, iMax, iMid, iMin;

            if (0.5 < b)
            {
                fMax = b - (b * s) + s;
                fMin = b + (b * s) - s;
            }
            else
            {
                fMax = b + (b * s);
                fMin = b - (b * s);
            }

            iSextant = (int)Math.Floor(h / 60f);
            if (300f <= h)
            {
                h -= 360f;
            }
            h /= 60f;
            h -= 2f * (float)Math.Floor(((iSextant + 1f) % 6f) / 2f);
            if (0 == iSextant % 2)
            {
                fMid = h * (fMax - fMin) + fMin;
            }
            else
            {
                fMid = fMin - h * (fMax - fMin);
            }

            iMax = Convert.ToInt32(fMax * 255);
            iMid = Convert.ToInt32(fMid * 255);
            iMin = Convert.ToInt32(fMin * 255);

            switch (iSextant)
            {
                case 1:
                    return Color.FromArgb(a, iMid, iMax, iMin);
                case 2:
                    return Color.FromArgb(a, iMin, iMax, iMid);
                case 3:
                    return Color.FromArgb(a, iMin, iMid, iMax);
                case 4:
                    return Color.FromArgb(a, iMid, iMin, iMax);
                case 5:
                    return Color.FromArgb(a, iMax, iMin, iMid);
                default:
                    return Color.FromArgb(a, iMax, iMid, iMin);
            }
        }
#endif

        //#region Component Designer generated code

        ///// <summary> 
        ///// Required designer variable.
        ///// </summary>
        //[NonSerialized]
        //private System.ComponentModel.Container components = null;

        ///// <summary> 
        ///// Required method for Designer support - do not modify 
        ///// the contents of this method with the code editor.
        ///// </summary>
        //private void InitializeComponent()
        //{
        //}
        //#endregion

#if !PocketPC
        #region ISupportInitialize Members

        protected virtual void OnBeginInit()
        {
            // Don't do any rendering
            AllowPainting = false;
        }

        void ISupportInitialize.BeginInit()
        {
            OnBeginInit();
        }

        protected virtual void OnEndInit()
        {
            // Enable rendering
            AllowPainting = true;
            // And render
            InvokePaintOffScreen();
        }

        void ISupportInitialize.EndInit()
        {
            OnEndInit();
        }

        #endregion
#endif

#if !PocketPC 
        private Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
        {
            AppDomain domain = (AppDomain)sender;

            foreach (Assembly asm in domain.GetAssemblies())
            {
                if (asm.FullName == args.Name)
                    return asm;
            }
            return null;
        }
#endif

//		private void DoubleBufferedControl_Disposed(object sender, EventArgs e)
//		{
//			Dispose();
//		}
	}

    /// <summary>Represents information about a cancelable paint iteration.</summary>
    /// <remarks>
    /// This class is used primarily by the <strong>OnPaintOffScreen</strong> method of
    /// the <strong>DoubleBufferedControl</strong> class when paint operations need to be
    /// performed. This class behaves the same as <strong>PaintEventArgs</strong>, but includes
    /// an extra <strong>IsCanceled</strong> property to indicate when a rendering iteration
    /// should be aborted.
    /// </remarks>
    [CLSCompliant(false)]
    public sealed class CancelablePaintEventArgs : PaintEventArgs
    {
        private bool pCanceled;

        /// <summary>
        /// Creates a new instance using the specified <strong>Graphics</strong> object and
        /// clipping rectangle.
        /// </summary>
        /// <param name="graphics">
        /// A <strong>Graphics</strong> object used for all painting within the
        /// control.
        /// </param>
        /// <param name="clipRectangle">
        /// A <strong>Rectangle</strong> that defines the area that should be painted.
        /// Typically the size of the entire control.
        /// </param>
        public CancelablePaintEventArgs(Graphics graphics, Rectangle clipRectangle)
            : base(graphics, clipRectangle)
        {
        }

        /// <summary>Indicates if the painting operation should be completely aborted.</summary>
        /// <value>
        /// A <strong>Boolean</strong>, <strong>True</strong> if painting was aborted.
        /// Default is <strong>False</strong>.
        /// </value>
        /// <remarks>
        /// 	<para>This property is used by controls which allow their paint operations to be
        ///     cancelled. When set to True, the entire painting iteration is stopped and
        ///     restarted. This property is useful if a control always needs to display the very
        ///     latest information.</para>
        /// 	<para>Setting this property to <strong>True</strong> can have some undesirable
        ///     affects. For example, if a paint iteration is cancelled repeatedly, the control
        ///     will never get far enough in a paint operation to paint on-screen. Some care should
        ///     be taken when using this property.</para>
        /// </remarks>
        /// <example>
        ///     This example demonstrates how to write a cancelable paint operation. It's typically
        ///     a good idea to check for conditions which should cause a paint to cancel before
        ///     beginning a time-consuming painting task. In this case, the
        ///     <strong>IsPaintingAborted</strong> property is examined before entering a large
        ///     loop. <strong>IsPaintingAborted</strong> becomes <strong>True</strong> when a new
        ///     request to paint the control is made after starting the current paint iteration.
        ///     <code lang="VB" title="[New Example]">
        /// Public Class MyControl
        ///     Inherits DoubleBufferedControl
        ///     
        ///     Sub New()
        ///         IsPaintingOnSeparateThread = True
        ///     End Sub
        ///     
        ///     Protected Overrides Sub OnPaintOffScreen(ByVal e As CancelablePaintEventArgs)
        ///         ' Should painting be cancelled?
        ///         If IsPaintingAborted
        ///             ' Yes.  Abort all painting
        ///             e.IsCanceled = True
        ///             Exit Sub
        ///         End If
        ///             
        ///         ' Otherwise, A big paint operation begins
        ///         Dim Count As Integer
        ///         For Count = 1 To 20000
        ///             Dim MyBrush As New SolidBrush(Color.Green)
        ///             e.Graphics.DrawRectangle(MyBrush, New Rectangle(Count, Count, 5, 5))
        ///             MyBrush.Dispose()
        ///         Next Count
        ///     End Sub
        /// End Class
        ///     </code>
        /// 	<code lang="CS" title="[New Example]">
        /// public class MyControl : DoubleBufferedControl
        /// {
        ///     MyControl()
        ///     {
        ///         IsPaintingOnSeparateThread = true;
        ///     }
        ///     
        ///     protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
        ///     {
        ///         // Should painting be cancelled?
        ///         if(IsPaintingAborted)
        ///         {
        ///             // Yes.  Abort all painting
        ///             e.IsCanceled = true;
        ///             return;
        ///         }
        ///             
        ///         // Otherwise, A big paint operation begins
        ///         for(int Count = 1; Count &lt;= 20000; Count++)
        ///         {
        ///             SolidBrush MyBrush = new SolidBrush(Color.Green);
        ///             e.Graphics.DrawRectangle(MyBrush, new Rectangle(Count, Count, 5, 5));
        ///             MyBrush.Dispose();
        ///         }
        ///     }
        /// }
        ///     </code>
        /// </example>
        public bool Canceled
        {
            get
            {
                return pCanceled;
            }
            set
            {
                pCanceled = value;
            }
        }

 
    }

    /// <summary>
    /// Indicates the rendering goal for a double-buffered user control.
    /// </summary>
    /// <remarks>
    /// This enumeration is used primarily by the <strong>OptimizationMode</strong>
    /// property of the <strong>DoubleBufferedControl</strong> class to modify several
    /// properties at once to obtain a specific rendering goal.
    /// </remarks>
    public enum ControlOptimizationMode
    {
        /// <summary>
        /// Default. All but minor quality improvements are turned off to yield the best
        /// balance between rendering speed and visual quality. The rendering thrad is given Normal
        /// priority.
        /// </summary>
        Balanced,
        /// <summary>All quality improvements are turned off in to maximize rendering speed.</summary>
        BestRefreshRate,
        /// <summary>
        /// All quality improvements are turned on in an attempt to maximize rendering
        /// quality. The rendering thread is given higher priority to account for the higher demand
        /// on the CPU.
        /// </summary>
        BestQuality,
        /// <summary>
        /// Similar to BestRefreshRate, plus the rendering thread is given lowest priority.
        /// This setting is recommended if your application needs more CPU time.
        /// </summary>
        LeastProcessorUsage,
        /// <summary>
        /// All quality improvements are enabled to make finer detail easier to discern, and
        /// information which might otherwise be hidden is made visible. This setting is
        /// recommended if the control is about to be printed.
        /// </summary>
        MostDetail,
        /// <summary>
        /// Indicates that optimization settings are being modified to custom
        /// settings.
        /// </summary>
        Custom
    }

	[CLSCompliant(false)]
    public delegate void CancelablePaintEventHandler(object sender, CancelablePaintEventArgs e);
}
