using System;

namespace GeoFramework
{
	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Angle</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	///     This example declares a custom event using this delegate.
	///     <code lang="VB" title="[New Example]">
	/// ' Declare a new event
	/// Dim MyAzimuthEvent As EventHandler&lt;AzimuthEventArgs&gt;
	///  
	/// Sub Main()
	///     ' Create an Azimuth of 90
	///     Dim MyAzimuth As New Azimuth(90)
	///     ' Raise our custom event
	///     RaiseEvent MyAzimuthEvent(Me, New AzimuthEventArgs(MyAzimuth))
	/// End Sub
	///     </code>
	/// 	<code lang="CS" title="[New Example]">
	/// // Declare a new event
	/// EventHandler&lt;AzimuthEventArgs&gt; MyAzimuthEvent;
	///  
	/// void Main()
	/// {
	///     // Create an Azimuth of 90
	///     Azimuth MyAzimuth = new Azimuth(90);
	///     // Raise our custom event
	///     if(MyAzimuthEvent != null)
	///         MyAzimuthEvent(this, new AzimuthEventArgs(MyAzimuth));
	/// }
	///     </code>
	/// </example>
	/// <param name="sender">An <strong>Object</strong> which initiated the call to the delegate.</param>
	/// <param name="e">
	/// An <strong>AngleEventArgs</strong> object containing an <strong>Azimuth</strong>
	/// involved in the event.
	/// </param>
	public delegate void AngleEventHandler(object sender, AngleEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Area</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	///     This example declares a custom event using this delegate. 
	///     <code lang="VB">
	/// Dim MyAreaEvent As AreaEventHandler
	///     </code>
	/// 	<code lang="CS">
	/// AreaEventHandler MyAreaEvent;
	///     </code>
	/// </example>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	/// <param name="e">
	/// An <strong>AreaEventArgs</strong> object containing an <strong>Area</strong>
	/// object involved with the event.
	/// </param>
	public delegate void AreaEventHandler(object sender, AreaEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Azimuth</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	///     This example declares a custom event using the <strong>EventHandler&lt;AzimuthEventArgs&gt;</strong>
	///     delegate.
	///     <code lang="VB">
	/// ' Declare a new event
    /// Dim MyAzimuthEvent As EventHandler&lt;AzimuthEventArgs&gt;
	///  
	/// Sub Main()
	///     ' Create an Azimuth of 90
	///     Dim MyAzimuth As New Azimuth(90)
	///     ' Raise our custom event
	///     RaiseEvent MyAzimuthEvent(Me, New AzimuthEventArgs(MyAzimuth))
	/// End Sub
	///     </code>
	/// 	<code lang="CS">
	/// // Declare a new event
	/// EventHandler&lt;AzimuthEventArgs&gt; MyAzimuthEvent;
	///  
	/// void Main()
	/// {
	///     // Create an Azimuth of 90
	///     Azimuth MyAzimuth = new Azimuth(90);
	///     // Raise our custom event
	///     if(MyAzimuthEvent != null)
	///         MyAzimuthEvent(this, new AzimuthEventArgs(MyAzimuth));
	/// }
	///     </code>
	/// </example>
	/// <seealso cref="AzimuthEventArgs">AzimuthEventArgs Class</seealso>
	/// <seealso cref="Azimuth">Azimuth Class</seealso>
	/// <param name="sender">An <strong>Object</strong> which initiated the call to the delegate.</param>
	/// <param name="e">
	/// An <strong>AzimuthEventArgs</strong> object containing an
	/// <strong>Azimuth</strong> involved in the event.
	/// </param>
	public delegate void AzimuthEventHandler(object sender, AzimuthEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Distance</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyDistanceEvent As DistanceEventHandler
	/// </code>
	/// <code lang="C#">
	/// DistanceEventHandler MyDistanceEvent;
	/// </code>
	/// </example>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	/// <param name="e">A <see cref="DistanceEventArgs">DistanceEventArgs</see> object containing a <see cref="Distance">Distance</see> involved in the event.</param>
	public delegate void DistanceEventHandler(object sender, DistanceEventArgs e);

	public delegate void CancelableEventHandler(object sender, CancelableEventArgs e);

	/// <example>
	///     This example demonstrates how to use <strong>ByteArrayEventArgs</strong> when
	///     raising an event. 
	///     <code lang="VB" title="[New Example]">
	/// ' Declare a new event
	/// Dim MyByteArrayEvent As ByteArrayEventHandler
	///  
	/// Sub Main()
	///     ' Create a byte array
	///     Dim MyArray As New Byte(50);
	///     ' Raise our custom Event
	///     RaiseEvent MyByteArrayEvent(Me, New ByteArrayEventArgs(MyByteArray));
	/// End Sub
	///     </code>
	/// 	<code lang="CS" title="[New Example]">
	/// // Declare a new event
	/// ByteArrayEventHandler MyByteArrayEvent;
	///  
	/// void Main()
	/// {
	///     // Create a byte array
	///     byte[] MyByteArray = new byte(90);
	///     // Raise our custom event
	///     if(MyByteArrayEvent != null)
	///         MyByteArrayEvent(this, new ByteArrayEventArgs(MyByteArray));
	/// }
	///     </code>
	/// </example>
	/// <summary>
	/// Facilitates events and asynchronous calls which use a byte array object as a
	/// parameter.
	/// </summary>
	/// <param name="sender">An <strong>Object</strong> which initiated the call to the delegate.</param>
	/// <param name="e">
	/// A <strong>ByteArrayEventArgs</strong> object containing a byte array involved in
	/// the event.
	/// </param>
	public delegate void ByteArrayEventHandler(object sender, ByteArrayEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use a <strong>DateTime</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyDateTimeEvent As DateTimeEventHandler
	/// </code>
	/// <code lang="C#">
	/// DateTimeEventHandler MyDateTimeEvent;
	/// </code>
	/// </example>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	/// <param name="e">A <see cref="DateTimeEventArgs">DateTimeEventArgs</see> object containing an <see cref="System.DateTime">DateTime</see> involved in the event.</param>
	public delegate void DateTimeEventHandler(object sender, DateTimeEventArgs e);

	/// <remarks>
	/// This example demonstrates how to use a custom event which involves an
	/// <strong>Elevation</strong>.
	/// </remarks>
	/// <example>
	///     This example declares a custom event using the
	///     <strong>ElevationEventHandler</strong> delegate. 
	///     <code lang="VB" title="[New Example]">
	/// ' Declare a new event
	/// Dim MyElevationEvent As ElevationEventHandler
	///  
	/// Sub Main()
	///     ' Create an Elevation of 90
	///     Dim MyElevation As New Elevation(90)
	///     ' Raise our custom event
	///     RaiseEvent MyElevationEvent(Me, New ElevationEventArgs(MyElevation))
	/// End Sub
	///     </code>
	/// 	<code lang="CS" title="[New Example]">
	/// // Declare a new event
	/// ElevationEventHandler MyElevationEvent;
	///  
	/// void Main()
	/// {
	///     // Create an Elevation of 90
	///     Elevation MyElevation = new Elevation(90);
	///     // Raise our custom event
	///     if(MyElevationEvent != null)
	///         MyElevationEvent(this, new ElevationEventArgs(MyElevation));
	/// }
	///     </code>
	/// </example>
	/// <param name="sender">An <strong>Object</strong> which initiated the call to the delegate.</param>
	/// <param name="e">
	/// An <strong>ElevationEventArgs</strong> object containing an
	/// <strong>Elevation</strong> involved in the event.
	/// </param>
	public delegate void ElevationEventHandler(object sender, ElevationEventArgs e);

	public delegate void GeographicRectangleEventHandler(object sender, GeographicRectangleEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Latitude</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyLatitudeEvent As LatitudeEventHandler
	/// </code>
	/// <code lang="C#">
	/// LatitudeEventHandler MyLatitudeEvent;
	/// </code>
	/// </example>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	/// <param name="e">A <see cref="LatitudeEventArgs">LatitudeEventArgs</see> object containing a <see cref="Latitude">Latitude</see> object involved in the event.</param>
	public delegate void LatitudeEventHandler(object sender, LatitudeEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Longitude</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyLongitudeEvent As LongitudeEventHandler
	/// </code>
	/// <code lang="C#">
	/// LongitudeEventHandler MyLongitudeEvent;
	/// </code>
	/// </example>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	/// <param name="e">A <see cref="LongitudeEventArgs">LongitudeEventArgs</see> object containing a <see cref="Longitude">Longitude</see> object involved in the event.</param>
	public delegate void LongitudeEventHandler(object sender, LongitudeEventArgs e);

	public delegate void PolarCoordinateOrientationEventHandler(object sender, PolarCoordinateOrientationEventArgs e);

	public delegate void PositionCollectionEventHandler(object sender, PositionCollectionEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Position</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyPositionEvent As PositionEventHandler
	/// </code>
	/// <code lang="C#">
	/// PositionEventHandler MyPositionEvent;
	/// </code>
	/// </example>
	/// <param name="e">A <see cref="PositionEventArgs">PositionEventArgs</see> object containing a <see cref="Position">Position</see> object involved in the event.</param>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	public delegate void PositionEventHandler(object sender, PositionEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use a
	/// <strong>GeoException</strong> object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Public Event MyErrorEvent As ExceptionEventHandler
	/// </code>
	/// <code lang="C#">
	/// public event ExceptionEventHandler MyErrorEvent;
	/// </code>
	/// </example>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	/// <param name="e">A <see cref="ExceptionEventArgs">ExceptionEventArgs</see> object containing an exception involved in the event.</param>
	public delegate void ExceptionEventHandler(object sender, ExceptionEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Radian</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyRadianEvent As RadianEventHandler
	/// </code>
	/// <code lang="C#">
	/// RadianEventHandler MyRadianEvent;
	/// </code>
	/// </example>
	/// <param name="e">A <see cref="RadianEventArgs">RadianEventArgs</see> object containing a <see cref="Radian">Radian</see> object involved with the event.</param>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	public delegate void RadianEventHandler(Radian sender, EventArgs e);

	public delegate void RectangleDEventHandler(object sender, RectangleDEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>Speed</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MySpeedEvent As SpeedEventHandler
	/// </code>
	/// <code lang="C#">
	/// SpeedEventHandler MySpeedEvent;
	/// </code>
	/// </example>
	/// <param name="e">A <see cref="SpeedEventArgs">SpeedEventArgs</see> object containing a <see cref="Speed">Speed</see> object involved with the event.</param>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	public delegate void SpeedEventHandler(object sender, SpeedEventArgs e);

    /// <summary>
    /// Facilitates events and asynchronous calls which use a <strong>Waypoint</strong>
    /// object as a parameter.
    /// </summary>
    /// <example>
    /// This example declares a custom event using this delegate.
    /// <code lang="VB">
    /// Dim MyWaypointEvent As WaypointEventHandler
    /// </code>
    /// <code lang="C#">
    /// WaypointEventHandler MyWaypointEvent;
    /// </code>
    /// </example>
    /// <param name="e">A <see cref="WaypointEventArgs">WaypointEventArgs</see> object containing a <see cref="Waypoint">Waypoint</see> object involved with the event.</param>
    /// <param name="sender">The object which initiated the call to the delegate.</param>
    public delegate void WaypointEventHandler(object sender, WaypointEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which report updates during an
	/// iterative task.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyProgressEvent As ProgressEventHandler
	/// </code>
	/// <code lang="C#">
	/// ProgressEventHandler MyProgressEvent;
	/// </code>
	/// </example>
	/// <param name="e">A <see cref="ProgressEventArgs">ProgressEventArgs</see> object containing information describing the minimum, maximum and value for a <see cref="System.Windows.Forms.ProgressBar">ProgressBar</see> control.</param>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	public delegate void ProgressEventHandler(object sender, ProgressEventArgs e);

	/// <summary>
	/// Facilitates events and asynchronous calls which use an <strong>TimeSpan</strong>
	/// object as a parameter.
	/// </summary>
	/// <example>
	/// This example declares a custom event using this delegate.
	/// <code lang="VB">
	/// Dim MyTimeSpanEvent As TimeSpanEventHandler
	/// </code>
	/// <code lang="C#">
	/// TimeSpanEventHandler MyTimeSpanEvent;
	/// </code>
	/// </example>
	/// <param name="e">A <see cref="TimeSpanEventArgs">TimeSpanEventArgs</see> object containing a <see cref="System.TimeSpan">TimeSpan</see> object involved with the event.</param>
	/// <param name="sender">The object which initiated the call to the delegate.</param>
	public delegate void TimeSpanEventHandler(object sender, TimeSpanEventArgs e);

	public delegate void TimeoutEventHandler(object sender, TimeoutExceptionEventArgs e);

}