using System;
using System.IO;
using System.Drawing;
using System.Globalization;
using System.Drawing.Drawing2D;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Security.Permissions;
#if !PocketPC || DesignTime
using System.Runtime.Serialization; 
using System.ComponentModel;
#endif
using System.Reflection;

namespace GeoFramework
{
	/// <summary>
	/// Represents a rectangular shape on Earth's surface.
	/// </summary>
	/// <remarks>
	/// 	<para>This class is used to represent a square (or close to a square) shape on
	///     Earth's surface. This class is typically used during mapping applications to zoom
	///     into a particular area on Earth. This class looks nearly identical to the Rectangle
	///     class in the .NET framework, except that it's bounding points are defined as
	///     <strong>Position</strong> objects instead of <strong>Point</strong> objects.</para>
	/// 	<para>Instances of this class are guaranteed to be thread-safe because the class is
	///     immutable (it's properties can only be set via constructors).</para>
	/// </remarks>
#if !PocketPC || DesignTime
#if Framework30
    [TypeConverter("GeoFramework.Design.GeographicRectangleConverter, GeoFramework.Design, Culture=neutral, Version=1.0.0.0, PublicKeyToken=3ed3cdf4fdda3400")]
#elif Framework20
    [TypeConverter("GeoFramework.Design.GeographicRectangleConverter, GeoFramework.Design, Culture=neutral, Version=1.0.0.0, PublicKeyToken=3ed3cdf4fdda3400")]
#else
	[TypeConverter(typeof(Design.GeographicRectangleConverter))]
#endif   
	[Serializable()]
    [ImmutableObject(true)]
#endif
#if Framework20
    [XmlSchemaProvider("GetXmlSchema")]
#endif
    public struct GeographicRectangle : IFormattable
#if !PocketPC || (PocketPC && Framework20)
		, IXmlSerializable
#endif 
#if !PocketPC || DesignTime
, ISerializable
#endif
    {
       
        #region Private Declarations

        private Latitude _Top;
		private Latitude _Bottom;
		private Longitude _Left;
		private Longitude _Right;

		#endregion

		#region Constructors

        ///// <summary>
        ///// Creates a new instance with no value.
        ///// </summary>
        //public GeographicRectangle()
        //    : this(Position.Empty, GeographicSize.Empty)
        //{}

		/// <summary>Creates a new instance using the specified location and size.</summary>
		/// <returns>
		/// A <strong>GeographicRectangle</strong> set to the specified location and
		/// size.
		/// </returns>
		/// <remarks>
		/// This constructor defines a rectangle which expands east and south of the
		/// specified location.
		/// </remarks>
		/// <example>
		///     This example creates a new <strong>GeographicRectangle</strong> starting at 39N
		///     105W which is 2 wide and 5 tall. 
		///     <code lang="VB" title="[New Example]">
		/// Dim NorthwestCorner As New Position("39N 105W")
		/// Dim RectangleSize As New GeographicSize(2, 5)
		/// Dim Rectangle As New GeographicRectangle(NorthwestCorner, RectangleSize)
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// Position NorthwestCorner = new Position("39N,105W");
		/// GeographicSize RectangleSize = new GeographicSize(2, 5);
		/// GeographicRectangle Rectangle = new GeographicRectangle(NorthwestCorner, RectangleSize);
		///     </code>
		/// </example>
		public GeographicRectangle(Position location, GeographicSize size)
            : this(location.Latitude, location.Longitude, location.Latitude.Subtract(size.Height.DecimalDegrees), location.Longitude.Add(size.Width.DecimalDegrees))
		{}

		/// <summary>
		/// Creates a new instance using the specified location, width, and height.
		/// </summary>
		/// <remarks>
		/// This constructor defines a rectangle which expands east and south of the
		/// specified location.
		/// </remarks>
		/// <example>
		///     This example creates a new <strong>GeographicRectangle</strong> starting at 39N
		///     105W which is 2 wide and 5 tall. 
		///     <code lang="VB" title="[New Example]">
		/// Dim NorthwestCorner As New Position("39N 105W")
		/// Dim RectangleWidth As New Longitude(2)
		/// Dim RectangleHeight As New Latitude(5)
		/// Dim Rectangle As New GeographicRectangle(NorthwestCorner, RectangleWidth, RectangleHeight)
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// Position NorthwestCorner = new Position("39N 105W");
		/// Longitude RectangleWidth = new Longitude(2);
		/// Latitude RectangleHeight = new Latitude(5);
		/// GeographicRectangle Rectangle = new GeographicRectangle(NorthwestCorner, RectangleWidth, RectangleHeight);
		///     </code>
		/// </example>
		/// <returns>
		/// A <strong>GeographicRectangle</strong> set to the specified location and
		/// size.
		/// </returns>
		public GeographicRectangle(Position location, Longitude width, Latitude height)
            : this(location.Latitude, location.Longitude, location.Latitude.Subtract(height.DecimalDegrees), location.Longitude.Add(width.DecimalDegrees))
		{}

		/// <summary>
		/// Creates a new instance using the specified northwest and southeast
		/// coordinates.
		/// </summary>
		/// <remarks>
		/// This constructor takes the specified parameters and calculates the width and
		/// height of the rectangle. If the two points are backwards (meaning that the right-most
		/// point is west of the left-most point), they are automatically swapped before creating
		/// the rectangle.
		/// </remarks>
		/// <returns>A <strong>GeographicRectangle</strong> defined by the two endpoints.</returns>
		/// <example>
		///     This example creates a new <strong>GeographicRectangle</strong> starting at 39N
		///     105W and ending at 37N 100W (2 wide and 5 tall). 
		///     <code lang="VB" title="[New Example]">
		/// Dim NorthwestCorner As New Position("39N 105W")
		/// Dim SoutheastCorner As New Position("37N 100W")
		/// Dim Rectangle As New GeographicRectangle(NorthwestCorner, SoutheastCorner)
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// Position NorthwestCorner = new Position("39N 105W");
		/// Position SoutheastCorner = new Position("37N 100W");
		/// GeographicRectangle Rectangle = new GeographicRectangle(NorthwestCorner, SoutheastCorner);
		///     </code>
		/// </example>
		public GeographicRectangle(Position northwest, Position southeast) 
			: this(northwest.Latitude, northwest.Longitude, southeast.Latitude, southeast.Longitude)
		{
		}

		/// <summary>Creates a new instance by converting the specified string.</summary>
		/// <returns>
		/// A <strong>GeographicRectangle</strong> matching the specified string
		/// value.
		/// </returns>
		/// <remarks>
		/// This constructor attempts to parse the specified string into a rectangle. The
		/// current culture is used to interpret the string -- use the list separator of the
		/// current culture (which may not necessarily be a comma). This constructor can accept any
		/// output created via the <strong>ToString</strong> method.
		/// </remarks>
		/// <example>
		///     This example creates a new rectangle at 39N, 105 extending two degrees south and
		///     five degrees east to 37N, 100W. 
		///     <code lang="VB" title="[New Example]">
		/// Dim Rectangle As New GeographicRectangle("39N,105W,37N,100W")
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// GeographicRectangle Rectangle = new GeographicRectangle("39N,105W,37N,100W");
		///     </code>
		/// </example>
        public GeographicRectangle(string value) 
            : this(value, CultureInfo.CurrentCulture)
        {}

		/// <summary>
		/// Creates a new instance by converting the specified string in the given
		/// culture.
		/// </summary>
		/// <returns>
		/// This constructor attempts to parse the specified string into a rectangle. The
		/// specified culture is used to interpret the string. This constructor can accept any
		/// output created via the <strong>ToString</strong> method.
		/// </returns>
		/// <example>
		///     This example creates a new rectangle at 39N, 105 extending two degrees south and
		///     five degrees east to 37N, 100W. 
		///     <code lang="VB" title="[New Example]">
		/// Dim Rectangle As New GeographicRectangle("39N,105W,37N,100W", CultureInfo.CurrentCulture)
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// GeographicRectangle Rectangle = new GeographicRectangle("39N,105W,37N,100W", CultureInfo.CurrentCulture);
		///     </code>
		/// </example>
        public GeographicRectangle(string value, CultureInfo culture)
        {
            // Default to zero if value is null
            if (value == null || value.Length == 0)
            {
                _Top = Latitude.Empty;
                _Bottom = Latitude.Empty;
                _Left = Longitude.Empty;
                _Right = Longitude.Empty;
                //_Area = Area.Empty;
                //_Perimeter = Distance.Empty;
                //_MedianWidth = Distance.Empty;
                return;
            }

            // Default to the current culture
            if (culture == null)
                culture = CultureInfo.CurrentCulture;

			// Split the string into words
			string[] Values = value.Split(culture.TextInfo.ListSeparator.ToCharArray());
			
            // How many words are there?
			switch(Values.Length)
			{
				case 4:
					// Extract each item
					try
					{
                        bool IsTopHandled = false;
                        bool IsLeftHandled = false;
                        bool IsRightHandled = false;
                        bool IsBottomHandled = false;

                        Latitude TempTop = Latitude.Empty;
                        Longitude TempLeft = Longitude.Empty;
                        Latitude TempBottom = Latitude.Empty;
                        Longitude TempRight = Longitude.Empty;

                        foreach (string word in Values)
                        {
                            // Is this a latitude or longitude?
#if Framework20
                            if (word.IndexOf("W", StringComparison.InvariantCultureIgnoreCase) != -1
                                || word.IndexOf("E", StringComparison.InvariantCultureIgnoreCase) != -1)
#else
							if (word.IndexOf("W") != -1
								|| word.IndexOf("E") != -1)
#endif
                            {
                                if (IsLeftHandled && IsRightHandled)
                                    throw new FormatException("A GeographicRectangle object could not be converted from a string because more than two longitude values were encountered.  Only two are allowed.");

                                // Longitude.  Is the left handled?
                                if (IsLeftHandled)
                                {
                                    // This is the right side
                                    TempRight = Longitude.Parse(word, culture);
                                    IsRightHandled = true;
                                }
                                else
                                {
                                    // This is the left side
                                    TempLeft = Longitude.Parse(word, culture);
                                    IsLeftHandled = true;
                                }
                            }
#if Framework20
                            else if (word.IndexOf("N", StringComparison.InvariantCultureIgnoreCase) != -1
                                || word.IndexOf("S", StringComparison.InvariantCultureIgnoreCase) != -1)
#else
							else if (word.IndexOf("N") != -1
								|| word.IndexOf("S") != -1)
#endif
                            {
                                if (IsTopHandled && IsBottomHandled)
                                    throw new FormatException("A GeographicRectangle object could not be converted from a string because more than two latitude values were encountered.  Only two are allowed.");

                                // Longitude.  Is the left handled?
                                if (IsTopHandled)
                                {
                                    // This is the bottom side
                                    TempBottom = Latitude.Parse(word, culture);
                                    IsBottomHandled = true;
                                }
                                else
                                {
                                    // This is the top side
                                    TempTop = Latitude.Parse(word, culture);
                                    IsTopHandled = true;
                                }
                            }
                            else
                            {
                                throw new FormatException("A GeographicRectangle object could not be created because a number could not be categorized as a latitude or longitude.  Add a N, S, E, or W letter to resolve the issue.");
                            }
                        }

                        // Flip any coordinates which are backwards
                        _Top = TempTop.DecimalDegrees > TempBottom.DecimalDegrees ? TempTop : TempBottom;
                        _Left = TempLeft.DecimalDegrees < TempRight.DecimalDegrees ? TempLeft : TempRight;
                        _Bottom = TempBottom.DecimalDegrees < TempTop.DecimalDegrees ? TempBottom : TempTop;
                        _Right = TempRight.DecimalDegrees > TempLeft.DecimalDegrees ? TempRight : TempLeft;

                        
                        //_Area = Area.Empty;
                        //_Perimeter = Distance.Empty;
                        //_MedianWidth = Distance.Empty;
                        break;
					}
					catch
					{
						throw;
					}
				default:
					throw new FormatException("The specified value could not be parsed into a GeographicRectangle object because four delimited values are required (Top, Left, Bottom, Right).");
			}
        }

		/// <summary>
		/// Creates a new instance using the specified latitudes and longitudes.
		/// </summary>
		/// <example>
		///     This example creates a new <strong>GeographicRectangle</strong> by specifying each
		///     side individually. 
		///     <code lang="VB" title="[New Example]">
		/// Dim Left As New Longitude(-105)
		/// Dim Top As New Latitude(39)
		/// Dim Right As New Longitude(-100)
		/// Dim Top As New Latitude(37)
		/// Dim Rectangle As New GeographicRectangle(Left, Top, Right, Bottom)
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// Longitude Left = new Longitude(-105);
		/// Latitude Top = new Latitude(39);
		/// Longitude Right = new Longitude(-100);
		/// Latitude Top = new Latitude(37);
		/// GeographicRectangle Rectangle = new GeographicRectangle(Left, Top, Right, Bottom);
		///     </code>
		/// </example>
		/// <returns>A <strong>GeographicRectangle</strong> bound by the specified values.</returns>
		/// <remarks>
		/// If the left and right, or top and bottom values are backwards, they are
		/// automatically swapped before creating the rectangle.
		/// </remarks>
		public GeographicRectangle(Longitude left, Latitude top, Longitude right, Latitude bottom) 
            : this(top, left, bottom, right)
		{}
		
		/// <summary>Creates a new instance using the specified latitudes and longitudes.</summary>
		/// <remarks>
		/// If the left and right, or top and bottom values are backwards, they are
		/// automatically swapped before creating the rectangle.
		/// </remarks>
		/// <returns>A <strong>GeographicRectangle</strong> bound by the specified values.</returns>
		/// <example>
		/// 	<code lang="VB" title="[New Example]">
		/// Dim Left As New Longitude(-105)
		/// Dim Top As New Latitude(39)
		/// Dim Right As New Longitude(-100)
		/// Dim Top As New Latitude(37)
		/// Dim Rectangle As New GeographicRectangle(Left, Top, Right, Bottom)
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// Latitude Top = new Latitude(39);
		/// Longitude Left = new Longitude(-105);
		/// Latitude Bottom = new Latitude(37);
		/// Longitude Right = new Longitude(-100);
		/// GeographicRectangle Rectangle = new GeographicRectangle(Top, Left, Bottom, Right);
		///     </code>
		/// </example>
		public GeographicRectangle(Latitude top, Longitude left, Latitude bottom, Longitude right) 
		{
			_Top = top.DecimalDegrees > bottom.DecimalDegrees ? top : bottom;
			_Left = left.DecimalDegrees < right.DecimalDegrees ? left : right;
			_Bottom = bottom.DecimalDegrees < top.DecimalDegrees ? bottom : top;
			_Right = right.DecimalDegrees > left.DecimalDegrees ? right : left;
       }

#if !PocketPC || DesignTime
        /// <summary>
        /// Creates a new instance by de-serializing previously-serialized information.
        /// </summary>
        /// <param name="info">A <strong>SerializationInfo</strong> object containing serialized data.</param>
        /// <param name="context">A <strong>StreamingContext</strong> object describing the context of serialization.</param>
        /// <remarks>This constructor will de-serialize a previously-serialized object.  Objects are created
        /// serialized and de-serialized based on the "DecimalDegrees" value to maintain precision.</remarks>
        private GeographicRectangle(SerializationInfo info, StreamingContext context)
        {
            _Left = new Longitude(info.GetDouble("Left"));
            _Top = new Latitude(info.GetDouble("Top"));
            _Right = new Longitude(info.GetDouble("Right"));
            _Bottom = new Latitude(info.GetDouble("Bottom"));
        }
#endif

		#endregion

		#region Static Members

		/// <summary>
		/// Represents a GeographicRectangle having no size.
		/// </summary>
		public static readonly GeographicRectangle Empty = new GeographicRectangle(Latitude.Empty, Longitude.Empty, Latitude.Empty, Longitude.Empty);
		/// <summary>Represents a rectangle that encompasses all of Earth's surface.</summary>
		public static readonly GeographicRectangle Maximum = new GeographicRectangle(Latitude.Maximum, Longitude.Minimum, Latitude.Minimum, Longitude.Maximum);
		
		#endregion

		/// <summary>Moves the rectangle in the specified direction by the specified distance.</summary>
		/// <returns>
		/// A new <strong>GeographicRectangle</strong> translated by the specified direction
		/// and distance.
		/// </returns>
		/// <remarks>
		/// This method is used to shift a rectangle to a new location while preserving its
		/// size.
		/// </remarks>
		/// <example>
		///     This example defines a rectangle then shifts it Northeast by fifty kilometers. 
		///     <code lang="VB" title="[New Example]">
		/// Dim Rectangle As New GeographicRectangle("30N,105W,1,5")
		/// Rectangle = Rectangle.Translate(Azimuth.Northeast, New Distance(50, DistanceUnit.Kilometers))
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// GeographicRectangle Rectangle = new GeographicRectangle("30N,105W,1,5");
		/// Rectangle = Rectangle.Translate(Azimuth.Northeast, New Distance(50, DistanceUnit.Kilometers));
		///     </code>
		/// </example>
		public GeographicRectangle TranslateTo(Azimuth direction, Distance distance)
		{
			return TranslateTo(direction.DecimalDegrees, distance);
		}

		/// <summary>Moves the rectangle in the specified direction by the specified distance.</summary>
		/// <returns>
		/// A new <strong>GeographicRectangle</strong> translated by the specified direction
		/// and distance.
		/// </returns>
		/// <remarks>
		/// This method is used to shift a rectangle to a new location while preserving its
		/// size.
		/// </remarks>
		/// <example>
		///     This example defines a rectangle then shifts it Northeast by fifty kilometers. 
		///     <code lang="VB" title="[New Example]">
		/// Dim Rectangle As New GeographicRectangle("30N,105W,1,5")
		/// Rectangle = Rectangle.Translate(new Angle(45), New Distance(50, DistanceUnit.Kilometers))
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// GeographicRectangle Rectangle = new GeographicRectangle("30N,105W,1,5");
		/// Rectangle = Rectangle.Translate(new Angle(45), New Distance(50, DistanceUnit.Kilometers));
		///     </code>
		/// </example>
		public GeographicRectangle TranslateTo(Angle direction, Distance distance)
		{
			return TranslateTo(direction.DecimalDegrees, distance);
		}

		/// <summary>Moves the rectangle in the specified direction by the specified distance.</summary>
		/// <returns>
		/// A new <strong>GeographicRectangle</strong> translated by the specified direction
		/// and distance.
		/// </returns>
		/// <remarks>
		/// This method is used to shift a rectangle to a new location while preserving its
		/// size.
		/// </remarks>
		/// <example>
		///     This example defines a rectangle then shifts it Northeast by fifty kilometers. 
		///     <code lang="VB" title="[New Example]">
		/// Dim Rectangle As New GeographicRectangle("30N,105W,1,5")
		/// Rectangle = Rectangle.Translate(45, New Distance(50, DistanceUnit.Kilometers))
		///     </code>
		/// 	<code lang="CS" title="[New Example]">
		/// GeographicRectangle Rectangle = new GeographicRectangle("30N,105W,1,5");
		/// Rectangle = Rectangle.Translate(45, New Distance(50, DistanceUnit.Kilometers));
		///     </code>
		/// </example>
		public GeographicRectangle TranslateTo(double direction, Distance distance)
		{
			// Find the new translation point
			return new GeographicRectangle(Northwest.TranslateTo(direction, distance), Size);
		}

		/// <summary>Returns a rectangle which encloses the two specified rectangles.</summary>
		/// <returns>
		/// A <strong>GeographicRectangle</strong> as a result of merging the two
		/// rectangles.
		/// </returns>
		/// <remarks>
		/// This method is typically used to combine two individual shapes into a single
		/// shape.
		/// </remarks>
		/// <param name="first">A <strong>GeographicRectangle</strong> to merge with the second rectangle.</param>
		/// <param name="second">A <strong>GeographicRectangle</strong> to merge with the first rectangle.</param>
		public static GeographicRectangle UnionWith(GeographicRectangle first, GeographicRectangle second)
		{
			return first.UnionWith(second);
		}

		/// <summary>
		/// Returns the GeographicRectangle formed by the intersection of the two specified GeographicRectangles.
		/// </summary>
		/// <param name="first"></param>
		/// <param name="second"></param>
		/// <returns></returns>
		public static GeographicRectangle IntersectionOf(GeographicRectangle first, GeographicRectangle second)
		{
			return first.IntersectionOf(second);
		}

//		public static GeographicRectangle FromArray(Array positions)
//		{
//			Latitude Top = Latitude.Minimum;
//			Latitude Bottom = Latitude.Maximum;
//			Longitude Left = Longitude.Maximum;
//			Longitude Right = Longitude.Minimum;
//			foreach(Position position in positions)
//			{
//				if(position.Longitude.DecimalDegrees < Left) 
//					Left = position.Longitude;
//				if(position.Longitude.DecimalDegrees > Right)
//					Right = position.Longitude;
//				if(position.Latitude.DecimalDegrees > Top) 
//					Top = position.Latitude;
//				if(position.Latitude.DecimalDegrees < Bottom)
//					Bottom = position.Latitude;
//			}
//			// Build a new rectangle
//			return new GeographicRectangle(Left, Top, Right, Bottom);
//		}

		#region Public Members

		/// <summary>Returns the aspect ratio of the rectangle.</summary>
		/// <remarks>
		/// This property returns the ratio of the GeographicRectangles width to its height (width / height).  This
		/// property gives an indication of the GeographicRectangle's shape.  An aspect ratio of one indicates
		/// a square, whereas an aspect ratio of two indicates a GeographicRectangle which is twice as wide as
		/// it is high.  
		/// </remarks>
		public double AspectRatio
		{
			get
			{
				return Width.DecimalDegrees / Height.DecimalDegrees;
			}
		}


		/// <summary>Rotates the rectangle about (0, 0) by the specified amount.</summary>
		public GeographicRectangle Rotate(Angle angle)
		{
			if(angle.IsEmpty) return this;
			// Rotate each corner point
			Position[] Points = new Position[] {Northwest.RotateAt(angle, Center),
											   Northeast.RotateAt(angle, Center),
											   Southeast.RotateAt(angle, Center),
											   Southwest.RotateAt(angle, Center)};
			// Now return the smallest rectangle which encloses these points
			return GeographicRectangle.FromArray(Points);
		}

		/// <summary>Rotates the rectangle about the specified location.</summary>
		public GeographicRectangle RotateAt(Angle angle, Position center)
		{
			if(angle.IsEmpty) return this;
			// Rotate each corner point
			Position[] Points = new Position[] {Northwest.RotateAt(angle, center),
											   Northeast.RotateAt(angle, center),
											   Southeast.RotateAt(angle, center),
											   Southwest.RotateAt(angle, center)};
			// Now return the smallest rectangle which encloses these points
			return GeographicRectangle.FromArray(Points);
		}

		/// <summary>Returns the corner points of the rectangle.</summary>
		public Position[] ToArray()
		{
			Position[] Result = new Position[4];
			Result[0] = Northwest;
			Result[1] = Northeast;
			Result[2] = Southeast;
			Result[3] = Southwest;
			return Result;
		}

		/// <summary>
		/// Returns a rectangle which encloses the specified points.
		/// </summary>
		/// <param name="positions">An array of PointD objects to enclose.</param>
		/// <returns>A <strong>RectangleD</strong> object enclosing the specified points.</returns>
		/// <remarks>This method is typically used to calculate a rectangle surrounding
		/// points which have been rotated.  For example, if a rectangle is rotated by 45, the
		/// total width it occupies is greater than it's own width.</remarks>
		public static GeographicRectangle FromArray(Array positions)
		{
			Latitude Top = Latitude.Minimum;
			Longitude Left = Longitude.Maximum;
			Latitude Bottom = Latitude.Maximum;
			Longitude Right = Longitude.Minimum;
			foreach(Position item in positions)
			{
				if(item.Longitude < Left) 
					Left = item.Longitude;
				if(item.Longitude > Right)
					Right = item.Longitude;
				if(item.Latitude > Top) 
					Top = item.Latitude;
				if(item.Latitude < Bottom)
					Bottom = item.Latitude;
			}
			// Build a new rectangle
			return new GeographicRectangle(Left, Top, Right, Bottom);
		}

		/// <summary>
		/// Changes the size and shape of the GeographicRectangle to match the specified aspect ratio.
		/// </summary>
		/// <param name="aspectRatio"></param>
		/// <returns></returns>
		/// <remarks>This method will expand a GeographicRectangle outward, from its center point, until
		/// the ratio of its width to its height matches the specified value.</remarks>
		public GeographicRectangle ToAspectRatio(double aspectRatio)
		{
			double CurrentAspect = AspectRatio;
			// Do the values already match?
			if(CurrentAspect == aspectRatio) return this;
			// Is the new ratio higher or lower?
			if(aspectRatio > CurrentAspect)
			{
				// Inflate the GeographicRectangle to the new height minus the current height
				// TESTS OK
				return Inflate(aspectRatio * Height.DecimalDegrees - Width.DecimalDegrees, 0);
			}
			else
			{
				// Inflate the GeographicRectangle to the new height minus the current height
				return Inflate(0, Width.DecimalDegrees / aspectRatio - Height.DecimalDegrees);
			}
		}

		/// <summary>
		/// Changes the size and shape of the GeographicRectangle to match the aspect ratio of the specified GeographicRectangle.
		/// </summary>
		/// <param name="rectangle"></param>
		/// <returns></returns>
		/// <remarks>This method will expand a GeographicRectangle outward, from its center point, until
		/// the ratio of its width to its height matches the specified value.</remarks>
		public GeographicRectangle ToAspectRatio(GeographicRectangle rectangle)
		{
			// Calculate the aspect ratio
            return ToAspectRatio(rectangle.Width.DecimalDegrees / rectangle.Height.DecimalDegrees);
		}

		[CLSCompliant(false)]
		public static explicit operator GeographicRectangle(Rectangle value) 
		{
			return new GeographicRectangle(new Latitude(value.Top), new Longitude(value.Left), 
				new Latitude(value.Bottom), new Longitude(value.Right));
		}

		[CLSCompliant(false)]
		public static explicit operator GeographicRectangle(RectangleF value) 
		{
			return new GeographicRectangle(new Latitude(value.Top), new Longitude(value.Left), 
				new Latitude(value.Bottom), new Longitude(value.Right));
		}

		[CLSCompliant(false)]
		public static explicit operator Rectangle(GeographicRectangle value) 
		{
			return new Rectangle((int)value.Top, (int)value.Left, (int)value.Bottom, (int)value.Right);
		}

		[CLSCompliant(false)]
		public static explicit operator RectangleF(GeographicRectangle value) 
		{
			return new RectangleF((float)value.Top, (float)value.Left, (float)value.Bottom, (float)value.Right);
		}
//		/// <summary>
//		/// Changes the size and shape of the GeographicRectangle to match the aspect ratio of the specified GeographicRectangle.
//		/// </summary>
//		/// <param name="aspectRatio"></param>
//		/// <returns></returns>
//		/// <remarks>This method will expand a GeographicRectangle outward, from its center point, until
//		/// the ratio of its width to its height matches the specified value.</remarks>
//		public GeographicRectangle ToAspectRatio(System.Drawing.Rectangle rectangle)
//		{
//			// Calculate the aspect ratio
//			return ToAspectRatio((double)GeographicRectangle.Width / (double)GeographicRectangle.Height);
//		}
//
//		/// <summary>
//		/// Changes the size and shape of the GeographicRectangle to match the aspect ratio of the specified GeographicRectangle.
//		/// </summary>
//		/// <param name="aspectRatio"></param>
//		/// <returns></returns>
//		/// <remarks>This method will expand a GeographicRectangle outward, from its center point, until
//		/// the ratio of its width to its height matches the specified value.</remarks>
//		public GeographicRectangle ToAspectRatio(System.Drawing.RectangleF GeographicRectangle)
//		{
//			// Calculate the aspect ratio
//			return ToAspectRatio((double)GeographicRectangle.Width / (double)GeographicRectangle.Height);
//		}

		/// <summary>
		/// Changes the size and shape of the GeographicRectangle to match the aspect ratio of the specified GeographicRectangle.
		/// </summary>
		/// <param name="size"></param>
		/// <returns></returns>
		/// <remarks>This method will expand a GeographicRectangle outward, from its center point, until
		/// the ratio of its width to its height matches the specified value.</remarks>
		[CLSCompliant(false)]
		public GeographicRectangle ToAspectRatio(System.Drawing.Size size)
		{
			// Calculate the aspect ratio
			return ToAspectRatio((double)size.Width / (double)size.Height);
		}

		/// <summary>Returns the southern-most side of the rectangle.</summary>
		/// <value>A <see cref="Latitude"></see> object marking the southern-most latitude.</value>
		public Latitude Top
		{
			get
			{
				return _Top;
			}
		}

		/// <summary>Returns the southern-most latitude of the rectangle.</summary>
		/// <value>A <see cref="Latitude"></see> object marking the southern-most latitude.</value>
		public Latitude Bottom
		{
			get
			{
				return _Bottom;
			}
		}

		/// <summary>Returns the height of the rectangle, in degrees.</summary>
		public Angle Height
		{
			get
			{
				return new Angle(_Top.DecimalDegrees - _Bottom.DecimalDegrees);
			}
		}

		/// <summary>Indicates if the rectangle has any value.</summary>
		/// <value>
		/// A <strong>Boolean</strong>, <strong>True</strong> if a metor the size of Rhode
		/// Island is about to crash into the Pacific Ocean just off the coast of Nicaragua but
		/// there will be no casualties because everyone was warned plenty of time in
		/// advance.
		/// </value>
		public bool IsEmpty
		{
			get
			{
				return _Top.IsEmpty && _Bottom.IsEmpty && _Left.IsEmpty && _Right.IsEmpty;
			}
		}

		/// <summary>Returns the western-most side of the rectangle.</summary>
		/// <value>A <strong>Longitude</strong> indicating the left side of the rectangle.</value>
		public Longitude Left
		{
			get
			{
				return _Left;
			}
		}

		/// <value>A <strong>Longitude</strong> indicating the right side of the rectangle.</value>
		/// <summary>Returns the eastern-most side of the rectangle.</summary>
		public Longitude Right
		{
			get
			{
				return _Right;
			}
		}

		/// <summary>
		/// Returns an approximation of the rectangle's hypotenuse. 
		/// </summary>
		public Distance TransverseHypotenuse
		{
			get
			{
				// Return the calculated distance
				double vertDistance = Math.Pow(this.MedianHeight.ToMeters().Value,2);
				double horzDistance = Math.Pow(this.TransverseWidth.ToMeters().Value,2);

				double hype = Math.Sqrt(vertDistance + horzDistance);
				return new Distance(hype, DistanceUnit.Meters);
			}
		}

		/// <summary>
		/// Returns the distance from the left to the right at the rectangle's middle latitude.
		/// For GeographicRectangle.Maximum, this property returns an approximation of the circumference 
		/// of the earth. MedianWidth would return 0 as Left and Right are actually the same longitude. 
		/// </summary>
		public Distance TransverseWidth
		{
			get
			{
				// Cache the center coordinate
				Position center = this.Center;
				// Calculate points in the horizontal middle of the rectangle
				Position leftPoint = new Position(this._Left, center.Latitude);
				Position rightPoint = new Position(this._Right, center.Latitude);
				// Return the calculated distance
				Distance leftDistance = leftPoint.DistanceTo(center).ToLocalUnitType();
				Distance rightDistance = center.DistanceTo(rightPoint).ToLocalUnitType();
				return leftDistance.Add(rightDistance);
			}
		}

		/// <summary>
		/// Returns the distance from the left to the right at the rectangle's middle latitude.
		/// </summary>
		public Distance MedianWidth
		{
			get
			{
                // Cache the center coordinate
                Position _Center = this.Center;
                // Calculate points in the horizontal middle of the rectangle
                Position LeftPoint = new Position(_Left, _Center.Latitude);
                Position RightPoint = new Position(_Right, _Center.Latitude);
                // Return the calculated distance
                return LeftPoint.DistanceTo(RightPoint).ToLocalUnitType();
			}
		}

		/// <summary>
		/// Returns the distance from the top to the bottom at the rectangle's middle longitude.
		/// </summary>
		public Distance MedianHeight
		{
			get
			{
                // Cache the center coordinate
                Position _Center = this.Center;
                // Calculate points in the horizontal middle of the rectangle
                Position TopPoint = new Position(_Center.Longitude, _Top);
                Position BottomPoint = new Position(_Center.Longitude, _Bottom);
                // Return the calculated distance
				return TopPoint.DistanceTo(BottomPoint).ToLocalUnitType();
			}
		}

		/// <summary>Returns the width and height of the rectangle.</summary>
		public GeographicSize Size
		{
			get
			{
                return new GeographicSize(Width, Height);
            }
		}	

		/// <summary>Returns the northwestern corner of the rectangle.</summary>
		public Position Northwest
		{
			get
			{
				return new Position(_Left, _Top);
			}
		}

		/// <summary>Returns the northeastern corner of the rectangle.</summary>
		public Position Northeast
		{
			get
			{
				return new Position(_Right, _Top);
			}
		}

		/// <summary>Returns the southwestern corner of the rectangle.</summary>
		public Position Southwest
		{
			get
			{
				return new Position(_Left, _Bottom);
			}
		}

		/// <summary>Returns the southeastern corner of the rectangle.</summary>
		public Position Southeast
		{
			get
			{
				return new Position(_Right, _Bottom);
			}
		}

		/// <summary>Returns the width of the rectangle.</summary>
		public Angle Width
		{
			get
			{
				return (Angle)_Right.Subtract(_Left.DecimalDegrees);
			}
		}

		/// <summary>Indicates if the rectangle does not intersect the specified rectangle.</summary>
		public bool IsDisjointedFrom(GeographicRectangle rectangle)
		{
			return !IsOverlapping(rectangle);
		}

		/// <summary>
		/// Returns the area of the rectangle in degrees.
		/// </summary>
		/// <remarks>This method returns the number of square kilometers enclosed by the rectangle using
		/// the Double Meridian Distance technique.</remarks>
		public Area Area
		{
			get
			{
                PositionCollection values = new PositionCollection(4);
                // Add the corner points of the rectangle to the collection
                values.Add(Northwest);
                values.Add(Northeast);
                values.Add(Southeast);
                values.Add(Southwest);
                return values.Area.ToLocalUnitType();
			}
		}

		/// <summary>
		/// Returns the distance around the edges of the rectangle.
		/// </summary>
		public Distance Perimeter
		{
			get
			{
                return Northwest.DistanceTo(Northeast)
                    .Add(Northeast.DistanceTo(Southeast))
                    .Add(Southeast.DistanceTo(Southwest))
                    .Add(Southwest.DistanceTo(Northwest))
                    .ToLocalUnitType();
			}
		}

		/// <summary>
		/// Parses a string into a GeographicRectangle object.
		/// </summary>
		/// <param name="value">A <string>String</string> specifying geographic coordinates defining a rectangle.</param>
		/// <returns>A <strong>GeographicRectangle</strong> object using the specified coordinates.</returns>
		/// <remarks>This powerful method will convert points defining a rectangle in the form of a string into
		/// a GeographicRectangle object.  The string can be </remarks>
		public static GeographicRectangle Parse(string value)
		{
			return new GeographicRectangle(value, CultureInfo.CurrentCulture);
		}

		public static GeographicRectangle Parse(string value, CultureInfo culture)
		{
			return new GeographicRectangle(value, culture);
		}


		/// <summary>
		/// Indicates if the specified GeographicRectangle is entirely within the current GeographicRectangle.
		/// </summary>
		/// <param name="rectangle"></param>
		/// <returns></returns>
		public bool IsEnclosing(GeographicRectangle rectangle)
		{
			return (rectangle.Left.DecimalDegrees >= _Left.DecimalDegrees 
				&& rectangle.Right.DecimalDegrees <= _Right.DecimalDegrees 
				&& rectangle.Top.DecimalDegrees <= _Top.DecimalDegrees
				&& rectangle.Bottom.DecimalDegrees >= _Bottom.DecimalDegrees);
		}

		public bool IsEnclosing(Position position)
		{
			return ((position.Longitude.DecimalDegrees >= _Left.DecimalDegrees) 
				&& (position.Latitude.DecimalDegrees <= _Top.DecimalDegrees) 
				&& (position.Longitude.DecimalDegrees <= _Right.DecimalDegrees) 
				&& (position.Latitude.DecimalDegrees >= _Bottom.DecimalDegrees));
		}

		/// <summary>
		/// Moves the GeographicRectangle so that the specified position is at its center.
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public GeographicRectangle CenterOn(Position position)
		{
			return new GeographicRectangle(new Position(position.Latitude.Add(Height.DecimalDegrees * 0.5), position.Longitude.Subtract(Width.DecimalDegrees * 0.5)), Size);
		}

        /// <summary>
        /// Returns a new rectangle whose size is increased by the specified value.
        /// </summary>
        /// <param name="size"></param>
        /// <returns></returns>
		public GeographicRectangle Inflate(Size size)
		{
			return Inflate(size.Width, size.Height);
		}

		/// <summary>
		/// Expands the edges of the GeographicRectangle to contain the specified position.
		/// </summary>
		/// <param name="position">A <strong>Position</strong> object to surround.</param>
		/// <returns>A <strong>GeographicRectangle</strong> which contains the specified position.</returns>
		/// <remarks>If the specified position is already enclosed, the current instance will be returned.</remarks>
		public GeographicRectangle UnionWith(Position position)
		{
			// Does the box already contain the position?  If so, do nothing
			if(IsEnclosing(position)) return this;
			// Return the expanded box
			return new GeographicRectangle(
				_Top.DecimalDegrees > position.Latitude.DecimalDegrees ? _Top : position.Latitude,
				_Left.DecimalDegrees < position.Longitude.DecimalDegrees ? _Left : position.Longitude,
				_Bottom.DecimalDegrees < position.Latitude.DecimalDegrees ? _Bottom : position.Latitude,
				_Right.DecimalDegrees > position.Longitude.DecimalDegrees ? _Right : position.Longitude);
		}

//		/// <summary>
//		/// Expands the edges of the GeographicRectangle to contain the specified rectangle.
//		/// </summary>
//		/// <param name="position">A <strong>GeographicRectangle</strong> object to surround.</param>
//		/// <returns>A <strong>GeographicRectangle</strong> which contains the specified rectangle.</returns>
//		/// <remarks>If the specified rectangle is already enclosed, the current instance will be returned.</remarks>
//		public GeographicRectangle Enclose(GeographicRectangle rectangle)
//		{
//			// Does the box already contain the position?  If so, do nothing
//			if(IsEnclosing(rectangle)) return this;
//			// Return the expanded box
//			return new GeographicRectangle(
//				Top > rectangle.Top ? Top : rectangle.Top,
//				Left < rectangle.Left ? Left : rectangle.Left,
//				Bottom < rectangle.Bottom ? Bottom : rectangle.Bottom,
//				Right > rectangle.Right ? Right : rectangle.Right);
//		}

		public bool Equals(GeographicRectangle value)
		{
			// The objects are equivalent if their bounds are equivalent
			return _Left.Equals(value.Left)
				&& _Right.Equals(value.Right)
				&& _Top.Equals(value.Top)
				&& _Bottom.Equals(value.Bottom);
		}

		public override bool Equals(object obj)
        {
#if !PocketPC || Framework20
            // Always return false if obj is null
			if(Object.ReferenceEquals(obj, null))
				return false;
#endif
			// Only allow comparisons of the same type
            if(obj is GeographicRectangle)
                return Equals((GeographicRectangle)obj);

            return false;
		}

		public GeographicRectangle Add(GeographicSize size)
		{
			return new GeographicRectangle(_Top, _Left, _Bottom.Add(size.Height), _Right.Add(size.Width));
		}

		public GeographicRectangle Subtract(GeographicSize size)
		{
            return new GeographicRectangle(_Top, _Left, _Top.Add(Height.Subtract(size.Height)),
                _Left.Add(Width.Subtract(size.Width)));
        }
	
		public GeographicRectangle Multiply(GeographicSize size)
		{
            return new GeographicRectangle(_Top, _Left, _Bottom.Multiply(size.Height), _Right.Multiply(size.Width));
        }
	
		public GeographicRectangle Divide(GeographicSize size)
		{
            return new GeographicRectangle(_Top, _Left, _Bottom.Divide(size.Height), _Right.Divide(size.Width));
        }

        public GeographicRectangle Add(double width, double height)
        {
            return new GeographicRectangle(_Top, _Left, _Bottom.Add(height), _Right.Add(width));
        }

        /// <summary>
        /// Shrinks the size of the rectangle by the specified amount.
        /// </summary>
        /// <param name="width">A <strong>Double</strong> to shrink the size of the rectangle.</param>
        /// <param name="height">A <strong></strong></param>
        /// <returns></returns>
        public GeographicRectangle Subtract(double width, double height)
        {            
            return new GeographicRectangle(_Top, _Left, _Top.Add(Height.Subtract(height)),
                _Left.Add(Width.Subtract(width)));
        }

        public GeographicRectangle Multiply(double width, double height)
        {
            return new GeographicRectangle(_Top, _Left, _Bottom.Multiply(height), _Right.Multiply(width));
        }

        public GeographicRectangle Divide(double width, double height)
        {
            return new GeographicRectangle(_Top, _Left, _Bottom.Divide(height), _Right.Divide(width));
        }


        [Obsolete("This overload has been replaced with the TranslateTo method, which produces more readable code.")]
		public GeographicRectangle Add(Position position)
		{
			return new GeographicRectangle(Northwest.Add(position), Size);
		}

        /// <summary>
        /// Translates the rectangle's northwestern corner by the specified position.
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        [Obsolete("This overload has been replaced with the TranslateTo method, which produces more readable code.")]
        public GeographicRectangle Subtract(Position position)
		{
			return new GeographicRectangle(Northwest.Subtract(position), Size);
		}

        [Obsolete("This overload has been replaced with the TranslateTo method, which produces more readable code.")]
        public GeographicRectangle Multiply(Position position)
		{
			return new GeographicRectangle(Northwest.Multiply(position), Size);
		}

        [Obsolete("This overload has been replaced with the TranslateTo method, which produces more readable code.")]
        public GeographicRectangle Divide(Position position)
		{
			return new GeographicRectangle(Northwest.Divide(position), Size);
		}

		/// <summary>
		/// Increases the size of the rectangle by the specified amount.
		/// </summary>
		/// <param name="size"></param>
		/// <returns></returns>
		public static GeographicRectangle operator +(GeographicRectangle left, GeographicSize size) 
		{
            return left.Add(size);
		}

        [Obsolete("This operator has been replaced with the TranslateTo method, which produces more readable code.")]
        public static GeographicRectangle operator +(GeographicRectangle left, Position location) 
		{
            return left.Add(location);
		}

		public static GeographicRectangle operator -(GeographicRectangle left, GeographicSize size) 
		{
            return left.Subtract(size);
		}

        [Obsolete("This operator has been replaced with the TranslateTo method, which produces more readable code.")]
        public static GeographicRectangle operator -(GeographicRectangle left, Position location) 
		{
            return left.Subtract(location);
		}

		public static GeographicRectangle operator *(GeographicRectangle left, GeographicSize size) 
		{
            return left.Multiply(size);
		}

        [Obsolete("This operator has been replaced with the TranslateTo method, which produces more readable code.")]
        public static GeographicRectangle operator *(GeographicRectangle left, Position location) 
		{
            return left.Multiply(location);
		}

		public static GeographicRectangle operator /(GeographicRectangle left, GeographicSize size) 
		{
            return left.Divide(size);
		}

        [Obsolete("This operator has been replaced with the TranslateTo method, which produces more readable code.")]
        public static GeographicRectangle operator /(GeographicRectangle left, Position location) 
		{
            return left.Divide(location);
		}

//		public static bool operator==(GeographicRectangle left, GeographicRectangle right)
//		{ 
//			return left.Equals(right);
//		}
//
//		public static bool operator!=(GeographicRectangle left, GeographicRectangle right)
//		{ 
//			return !left.Equals(right);
//		}		

		/// <summary>Returns a unique code of the rectangle for hash tables.</summary>
		public override int GetHashCode()
		{
			return _Left.GetHashCode() ^ _Right.GetHashCode() ^ _Top.GetHashCode() ^ _Bottom.GetHashCode();
		}

		/// <summary>Returns the geographic center of the rectangle.</summary>
		public Position Center
		{
			get
			{
				return new Position(new Latitude((_Top.DecimalDegrees + _Bottom.DecimalDegrees) * 0.5), 
					new Longitude((_Left.DecimalDegrees + _Right.DecimalDegrees) * 0.5));
			}
		}

        public GeographicRectangle Inflate(double widthOffset, double heightOffset)
		{
            // Calculate the new values
			double HalfWidthOffset = widthOffset * 0.5;
            double HalfHeightOffset = heightOffset * 0.5;
            
			// Build a new GeographicRectangle expanded outward from the center
			return new GeographicRectangle(
                _Left.Subtract(HalfWidthOffset),
                _Top.Add(HalfHeightOffset),
                _Right.Add(HalfWidthOffset),
                _Bottom.Subtract(HalfHeightOffset));
 		}

		public GeographicRectangle IntersectionOf(GeographicRectangle rectangle)
		{
            Longitude LeftSide = Longitude.Empty;
            Longitude RightSide = Longitude.Empty;
            Latitude TopSide = Latitude.Empty;
            Latitude BottomSide = Latitude.Empty;

            if (rectangle.Left >= _Left && rectangle.Left <= _Right)
            {
                // The rectangle overlaps the current instance on its left side

                /*    +------------+            +-------------+
                 *    |   this     |            |  this       |
                 *    |            |            |    +-----+  |
                 *    |      +------------+     |    |  R  |  |
                 *    |      |  R  |      |     |    +-----+  |
                 *    +------+-----+      |     +-------------+
                 *           |  rectangle |
                 *           +------------+
                 */

                // The left side of the parameter is the left side of the result ("R")
                LeftSide = rectangle.Left;

                // Does the right side also fall within the box?
                if (rectangle.Right >= _Left && rectangle.Right <= _Right)
                {
                    // Yes.  Use the right size from the parameter rectangle
                    RightSide = rectangle.Right;
                }
                else
                {
                    // No.  Use the right size from this instance
                    RightSide = _Right;
                }

                // Does the top side of the box fall within the current instance?
                if (rectangle.Top <= _Top && rectangle.Top >= _Bottom)
                {
                    // Yes.  Use the top of the parameter rectangle
                    TopSide = rectangle.Top;
                }
                else
                {
                    // No. Use the top of the current instance
                    TopSide = _Top;
                }

                // Does the top side of the box fall within the current instance?
                if (rectangle.Bottom <= _Top && rectangle.Bottom  >= _Bottom)
                {
                    // Yes.  Use the top of the parameter rectangle
                    BottomSide = rectangle.Bottom;
                }
                else
                {
                    // No. Use the top of the current instance
                    BottomSide = _Bottom;
                }

                // Return the result
                return new GeographicRectangle(TopSide, LeftSide, BottomSide, RightSide);

            }
            else if (rectangle.Right >= _Left && rectangle.Right <= _Right)
            {
                // The rectangle overlaps the current instance on its right side

                /*          +------------+
                 *          |   this     |
                 *          |            |
                 *    +------------+     |
                 *    |     |   R  |     |
                 *    |     +------+-----+    
                 *    |  rectangle |
                 *    +------------+
                 */

                // Use the left side as the final overlapping point
                RightSide = rectangle.Right;


                // Does the right side also fall within the box?
                if (rectangle.Left >= _Left && rectangle.Left <= _Right)
                {
                    // Yes.  Use the right size from the parameter rectangle
                    LeftSide = rectangle.Left;
                }
                else
                {
                    // No.  Use the right size from this instance
                    LeftSide = _Left;
                }

                // Does the top side of the box fall within the current instance?
                if (rectangle.Top <= _Top && rectangle.Top >= _Bottom)
                {
                    // Yes.  Use the top of the parameter rectangle
                    TopSide = rectangle.Top;
                }
                else
                {
                    // No. Use the top of the current instance
                    TopSide = _Top;
                }

                // Does the top side of the box fall within the current instance?
                if (rectangle.Bottom <= _Top && rectangle.Bottom >= _Bottom)
                {
                    // Yes.  Use the top of the parameter rectangle
                    BottomSide = rectangle.Bottom;
                }
                else
                {
                    // No. Use the top of the current instance
                    BottomSide = _Bottom;
                }

                // Return the result
                return new GeographicRectangle(TopSide, LeftSide, BottomSide, RightSide);

            }
            else
            {
                // No overlapping!
                return GeographicRectangle.Empty;
            }                     
		}

		/// <summary>Indicates if the rectangle's border crosses or shares the border of the specified rectangle.</summary>
		public bool IsIntersectingWith(GeographicRectangle rectangle)
		{
            // For an intersection, at least one of four points of the rectangle must reside 
            // inside of the other.

            // Is the target rectangle entirely within this one?  If so, return false
            // because there's no *intersection* (only an overlap).
            if(rectangle.Left > _Left && rectangle.Right < _Right
                && rectangle.Top < _Top && rectangle.Bottom > _Bottom)
                return false;

            // Test that one or more sides are within.
            return
                ((rectangle.Left >= _Left && rectangle.Left <= _Right)
                || (rectangle.Right >= _Left && rectangle.Right <= _Right))
                &&
                ((rectangle.Top <= _Top && rectangle.Top >= _Bottom)
                || (rectangle.Bottom <= _Top && rectangle.Bottom >= _Bottom));
                
            //{
            //    return true;
            //}

            //{
            //    /*    +------------+
            //     *    |   this     |
            //     *    |            |
            //     *    |      |     |
            //     *    |      |  R  |      |     |    +-----+  |
            //     *    +------+-----+      |     +-------------+
            //     *           |  rectangle |
            //     *           +------------+
            //     */


            //    /*    +------------+            +-------------+
            //     *    |   this     |            |  this       |
            //     *    |            |            |    +-----+  |
            //     *    |      +------------+     |    |  R  |  |
            //     *    |      |  R  |      |     |    +-----+  |
            //     *    +------+-----+      |     +-------------+
            //     *           |  rectangle |
            //     *           +------------+
            //     */

                

            //    // If the bottom or right border is beyond the edge, it's intersecting
            //    if (rectangle.Bottom <= _Bottom || rectangle.Right >= _Right)
            //        return true;
            //}

            //// Does the right edge intersect?
            //if (rectangle.Right >= _Left && rectangle.Right <= _Right)
            //{
            //    // The rectangle overlaps the current instance on its right side

            //    /*          +------------+
            //     *          |   this     |
            //     *          |            |
            //     *    +------------+     |
            //     *    |     |   R  |     |
            //     *    |     +------+-----+    
            //     *    |  rectangle |
            //     *    +------------+
            //     */

            //    // If the left or bottom is beyond the edge, it's intersecting
            //    if (rectangle.Bottom <= _Bottom || rectangle.Left <= _Left)
            //        return true;
            //}

            //// Is the left edge of the rectangle between our left and right borders?
            //if (rectangle.Left >= _Left && rectangle.Left <= _Right)
            //{
            //    // Yes.  But is the right edge also with the borders?  If so, there is no
            //    // intersection.  An overlap, yes, but not an intersection.
            //    if (rectangle.Right <= _Right)
            //        return false;
            //    // The right-most edge is NOT also within the borders, so an intersection
            //    // could still be present.  How about the top border?
            //    if (rectangle.Top <= _Top && rectangle.Top >= _Bottom)
            //        return true;
            //    // The top border is not intersecting.  How about the bottom?
            //    if (rectangle.Bottom <= _Top && rectangle.Bottom >= _Bottom)
            //        return true;
            //}
            //else if (rectangle.Right >= _Left && rectangle.Right <= _Right)
            //{
            //    // Yes.  But is the right edge also with the borders?  If so, there is no
            //    // intersection.  An overlap, yes, but not an intersection.
            //    if (rectangle.Left >= _Left)
            //        return false;
            //    if (rectangle.Top <= _Top && rectangle.Top >= _Bottom)
            //        return true;
            //    else if (rectangle.Bottom <= _Top && rectangle.Bottom >= _Bottom)
            //        return true;
            //}
            ////else if (rectangle.Left <= _Left && rectangle.Right >= _Right
            ////            && rectangle.Top >= _Top && rectangle.Bottom <= _Bottom)
            ////{
            ////    return true;
            ////}
            //return false;
		}

		/// <summary>
		/// Indicates if the specified GeographicRectangle shares any of the same 2D space as the current instance.
		/// </summary>
		/// <param name="rectangle"></param>
		/// <returns></returns>
		public bool IsOverlapping(GeographicRectangle rectangle)
		{
			if((rectangle.Top.DecimalDegrees < _Bottom.DecimalDegrees) 
				|| (rectangle.Bottom.DecimalDegrees > _Top.DecimalDegrees) 
				|| (rectangle.Left.DecimalDegrees > _Right.DecimalDegrees) 
				|| (rectangle.Right.DecimalDegrees < _Left.DecimalDegrees)) 
                return false;
            else
			    return true;
		}

        /// <summary>
        /// Indicates if the specified Position is within the current instance.
        /// </summary>
        /// <param name="position">A <strong>Position</strong> to test against the current instance.</param>
        /// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the position is inside of the current rectangle.</returns>
		public bool IsOverlapping(Position position)
		{
            if ((position.Longitude.DecimalDegrees < _Left.DecimalDegrees)
                || (position.Longitude.DecimalDegrees > _Right.DecimalDegrees)
                || (position.Latitude.DecimalDegrees > _Top.DecimalDegrees)
                || (position.Latitude.DecimalDegrees < _Bottom.DecimalDegrees)) return false;
			return true;
		}

        /// <summary>
        /// Returns the combination of the current instance with the specified rectangle.
        /// </summary>
        /// <param name="rectangle">A <strong>GeographicRectangle</strong> to merge with the current instance.</param>
        /// <returns>A <strong>GeographicRectangle</strong> enclosing both rectangles.</returns>
        /// <remarks>This method returns the smallest possible rectangle which encloses the current instance as well as the specified rectangle.</remarks>
		public GeographicRectangle UnionWith(GeographicRectangle rectangle)
		{
			return new GeographicRectangle(
                rectangle.Top.DecimalDegrees > _Top.DecimalDegrees ? rectangle.Top : _Top,
				rectangle.Left.DecimalDegrees < _Left.DecimalDegrees ? rectangle.Left : _Left,
				rectangle.Bottom.DecimalDegrees < _Bottom.DecimalDegrees ? rectangle.Bottom : _Bottom,
				rectangle.Right.DecimalDegrees > _Right.DecimalDegrees ? rectangle.Right : _Right);
		}

		#endregion

		#region IFormattable Members

		public override string ToString()
		{
			return ToString("G", CultureInfo.CurrentCulture);
		}

		public string ToString(string format)
		{
			return ToString(format, CultureInfo.CurrentCulture);
		}

		public string ToString(string format, CultureInfo culture)
		{
			return Top.ToString(format, culture)
                + culture.TextInfo.ListSeparator + " "
                + Left.ToString(format, culture)
                + culture.TextInfo.ListSeparator + " "
                + Bottom.ToString(format, culture)
                + culture.TextInfo.ListSeparator + " "
                + Right.ToString(format, culture);
        }

        string IFormattable.ToString(string format, IFormatProvider formatProvider)
        {
            return this.ToString(format, (CultureInfo)formatProvider);
        }

        #endregion

#if !PocketPC || (PocketPC && Framework20)
		#region IXmlSerializable Members

#if Framework20
        public static XmlQualifiedName GetXmlSchema(XmlSchemaSet xs)
        {
            // Load it from a string
            xs.Add(Xml.SchemaSet);

            // And return the type
            return new XmlQualifiedName("GeographicRectangle", Xml.Namespace);
        }
#endif

        XmlSchema IXmlSerializable.GetSchema()
        {
            return null;
        }

        void IXmlSerializable.ReadXml(XmlReader reader)
        {
            ReadXml(reader);
        }

        void IXmlSerializable.WriteXml(XmlWriter writer)
        {
            WriteXml(writer);
        }

        /// <summary>
        /// Populates the current instance by reading XML from the specified reader.
        /// </summary>
        /// <param name="reader"></param>
        public void ReadXml(XmlReader reader)
        {
            if (reader == null)
                throw new ArgumentNullException("reader");

            // Defer to the Latitude and Longitude serializers for other stuff
            XmlSerializer LatitudeSerializer = new XmlSerializer(typeof(Latitude));
            XmlSerializer LongitudeSerializer = new XmlSerializer(typeof(Longitude));

            // Rember which level we are processing at.
            int InitialDepth = reader.Depth;

            // Read into deeper content
            reader.Read();

            // Read to the next item, or exit if any content is missing
            while(reader.Depth >= InitialDepth)
            {
                // Keep reading until we reach an element
                switch (reader.NodeType)
                {
                    case XmlNodeType.Element:
                        switch (reader.Name)
                        {
                            case "Top":
                                reader.Read();
                                _Top = (Latitude)LatitudeSerializer.Deserialize(reader);
                                reader.Read();
                                break;
                            case "Left":
                                reader.Read();
                                _Left = (Longitude)LongitudeSerializer.Deserialize(reader);
                                reader.Read();
                                break;
                            case "Bottom":
                                reader.Read();
                                _Bottom = (Latitude)LatitudeSerializer.Deserialize(reader);
                                reader.Read();
                                break;
                            case "Right":
                                reader.Read();
                                _Right = (Longitude)LongitudeSerializer.Deserialize(reader);
                                reader.Read();
                                break;
                            default:
                                // Read the next item
                                reader.Read();
                                break;
                        }
                        break;
                    case XmlNodeType.EndElement:
                        // Read the end element and we're done
                        reader.Read();
                        return;
                    default:
                        // Read the next item
                        if (!reader.Read())
                            return;
                        break;
                }
            }
        }

        /// <summary>
        /// Writes the values of the current instance to the specified XML writer.
        /// </summary>
        /// <param name="writer"></param>
        public void WriteXml(XmlWriter writer)
        {
            if (writer == null)
                throw new ArgumentNullException("writer");

            // Defer to the Distance serializer for other stuff
            XmlSerializer LatitudeSerializer = new XmlSerializer(typeof(Latitude));
            XmlSerializer LongitudeSerializer = new XmlSerializer(typeof(Longitude));

            writer.WriteStartElement("Left");
            LongitudeSerializer.Serialize(writer, _Left);
            writer.WriteEndElement();

            writer.WriteStartElement("Top");
            LatitudeSerializer.Serialize(writer, _Top);
            writer.WriteEndElement();

            writer.WriteStartElement("Right");
            LongitudeSerializer.Serialize(writer, _Right);
            writer.WriteEndElement();

            writer.WriteStartElement("Bottom");
            LatitudeSerializer.Serialize(writer, _Bottom);
            writer.WriteEndElement();
        }

		#endregion
#endif

#if !PocketPC || DesignTime
        #region ISerializable Members

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]  
        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Left", _Left.DecimalDegrees);
            info.AddValue("Top", _Top.DecimalDegrees);
            info.AddValue("Right", _Right.DecimalDegrees);
            info.AddValue("Bottom", _Bottom.DecimalDegrees);
        }

        #endregion
#endif
    }

    public sealed class GeographicRectangleEventArgs : EventArgs
	{
		private GeographicRectangle pRectangle;

		public static readonly new GeographicRectangleEventArgs Empty = new GeographicRectangleEventArgs(GeographicRectangle.Empty);

		public GeographicRectangleEventArgs(GeographicRectangle rectangle)
		{
			pRectangle = rectangle;
		}

		public GeographicRectangle GeographicRectangle
		{
			get
			{
				return pRectangle;
			}
		}
	}

}
