using System;
using System.IO;
using System.Drawing;
using System.Globalization;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Security.Permissions;
#if !PocketPC || DesignTime
using System.Runtime.Serialization;
using System.ComponentModel;
#endif
using System.Reflection;

namespace GeoFramework
{
	/// <summary>Represents a highly-precise pixel coordinate.</summary>
	/// <remarks>
	/// 	<para>This class behaves similar to the <strong>PointF</strong> structure in the
	///     <strong>System.Drawing</strong> namespace, except that it supports double-precision
	///     values and can be converted into a geographic coordinate. This structure is also
	///     supported on the Compact Framework version of the <strong>GeoFramework</strong>,
	///     whereas <strong>PointF</strong> is not.</para>
	/// 	<para>Instances of this class are guaranteed to be thread-safe because the class is
	///     immutable (its properties can only be changed via constructors).</para>
	/// </remarks>
#if !PocketPC || DesignTime
#if Framework30
    [TypeConverter("GeoFramework.Design.PointDConverter, GeoFramework.Design, Culture=neutral, Version=1.0.0.0, PublicKeyToken=3ed3cdf4fdda3400")]
#elif Framework20
    [TypeConverter("GeoFramework.Design.PointDConverter, GeoFramework.Design, Culture=neutral, Version=1.0.0.0, PublicKeyToken=3ed3cdf4fdda3400")]
#else
	[TypeConverter(typeof(Design.PointDConverter))]
#endif   
	[Serializable()]
    [ImmutableObject(true)]
#endif
#if Framework20
    [XmlSchemaProvider("GetXmlSchema")]
#endif
    public struct PointD : IFormattable
#if !PocketPC || (PocketPC && Framework20)
	, IXmlSerializable
#endif  
#if !PocketPC || DesignTime
, ISerializable
#endif
	{
       
        private double pX;
		private double pY;

		/// <summary>Returns a point with no value.</summary>
		public static readonly PointD Empty = new PointD(0, 0);
		/// <summary>Represents an invalid coordinate.</summary>
		public static readonly PointD Invalid = new PointD(double.NaN, double.NaN);

		#region Constructors

		/// <summary>Creates a new instance for the specified coordinates.</summary>
		public PointD(double x, double y)
		{
			pX = x;
			pY = y;
		}

        /// <summary>
        /// Creates a new instance by converting the specified size.
        /// </summary>
        /// <param name="size"><A <strong>Size</strong> object to convert.</param>
        [CLSCompliant(false)]
		public PointD(Size size)
		{
			pX = size.Width;
			pY = size.Height;
		}

#if !PocketPC || DesignTime
        /// <summary>
        /// Creates a new instance by de-serializing previously-serialized information.
        /// </summary>
        /// <param name="info">A <strong>SerializationInfo</strong> object containing serialized data.</param>
        /// <param name="context">A <strong>StreamingContext</strong> object describing the context of serialization.</param>
        /// <remarks>This constructor will de-serialize a previously-serialized object.</remarks>
        private PointD(SerializationInfo info, StreamingContext context)
        {
            pX = info.GetDouble("X");
            pY = info.GetDouble("Y");
        }
#endif

		#endregion

		#region Shared Members

		public static PointD Parse(string value)
		{
			return Parse(value, CultureInfo.CurrentCulture);
		}

		public static PointD Parse(string value, CultureInfo culture)
		{
			// Trim the string and remove double spaces
			value = value.Replace("  ", " ").Trim();
			// And separate it via the list separator
			string[] Values = value.Split(Convert.ToChar(culture.TextInfo.ListSeparator, culture));
			// Return the converted values
			return new PointD(double.Parse(Values[0], NumberStyles.Any, culture), 
				double.Parse(Values[1], NumberStyles.Any, culture));
		}        

		#endregion

		#region Public Methods

		public bool Equals(PointD value)
		{
			return (pX == value.X) && (pY == value.Y);
		}

		public bool Equals(PointD value, int precision)
		{
			return ((Math.Round(pX, precision) == Math.Round(value.X, precision)) 
				&& (Math.Round(pY, precision) == Math.Round(value.Y, precision)));
		}

		public override bool Equals(object obj)
		{
			if(obj is PointD)
				return Equals((PointD)obj);
			else 
				return base.Equals(obj);
		}

		/// <summary>Returns a unique code used for hash tables.</summary>
		public override int GetHashCode()
		{
			return pX.GetHashCode() ^ pY.GetHashCode();
		}

		public static bool operator==(PointD left, PointD right)
		{ 
			return left.Equals(right);
		}

		public static bool operator!=(PointD left, PointD right)
		{ 
			return !left.Equals(right);
		}	
		#endregion

		/// <summary>Returns whether the current instance has no value.</summary>
		public bool IsEmpty
		{
			get
			{
				return (pX == 0 && pY == 0);
			}
		}

		/// <summary>Returns whether the current instance has an invalid value.</summary>
		public bool IsInvalid
		{
			get
			{
				return(double.IsNaN(pX) || double.IsNaN(pY));
			}
		}

		/// <summary>Returns the horizontal portion of the coordinate.</summary>
		public double X
		{
			get
			{
				return pX;
			}
		}

		/// <summary>Returns the vertical portion of the coordinate.</summary>
		public double Y
		{
			get
			{
				return pY;
			}
		}

#if !PocketPC
		/// <summary>
		/// Converts the specified array of <strong>PointD</strong> objects into an array of
		/// <strong>PointF</strong> objects.
		/// </summary>
		public static PointF[] ToPointFArray(PointD[] points)
		{
			int Len = points.Length;
			PointF[] Result = new PointF[Len];
			for(int index = 0; index < Len; index++)
				Result[index] = (PointF)points[index];
			return Result;
		}
#endif

		/// <summary>
		/// Converts the specified array of <strong>PointD</strong> objects into an array of
		/// <strong>Point</strong> objects.
		/// </summary>
		[CLSCompliant(false)]
		public static Point[] ToPointArray(PointD[] points)
		{
			int Len = points.Length;
			Point[] Result = new Point[Len];
			for(int index = 0; index < Len; index++)
				Result[index] = (Point)points[index];
			return Result;
		}

		/// <summary>Converts the current instance into a polar coordinate.</summary>
		public PolarCoordinate ToPolarCoordinate()
		{
//			double Value;
//			double TempY = -pY;
//			if(pX == 0)
//				Value = 0;
//			else
//				Value = TempY / pX;
			// Calculate the coordinate using the default: 0 = East
			// and counter-clockwise movement
			Radian Result = new Radian(Math.Atan2(-pY, pX));
			PolarCoordinate Result2 = new PolarCoordinate((float)Math.Sqrt(pX * pX + pY * pY), Result.ToAngle(), Azimuth.East, PolarCoordinateOrientation.Counterclockwise);
			// And re-orient it at North and Clockwise
			return Result2.ToOrientation(Azimuth.North, PolarCoordinateOrientation.Clockwise);
		}

		/// <summary>Returns the current instance rotated about (0, 0).</summary>
        public PointD Rotate(Angle angle)
        {
            return Rotate(angle.DecimalDegrees);
        }

        /// <summary>Returns the current instance rotated about (0, 0).</summary>
        public PointD Rotate(double angle)
		{
			// Convert the angle to radians
			double AngleRadians = Radian.FromDegrees(angle).Value;
			double AngleCos = Math.Cos(AngleRadians);
			double AngleSin = Math.Sin(AngleRadians);
			// Yes.  Rotate the point about 0,0
			return new PointD(AngleCos * pX - AngleSin * pY, AngleSin * pX + AngleCos * pY);
		}

		/// <summary>Returns the current instance rotated about the specified point.</summary>
		public PointD RotateAt(Angle angle, PointD center)
		{
            return RotateAt(angle.DecimalDegrees, center);
		}

        public PointD RotateAt(double angle, PointD center)
        {
            if (angle == 0) 
                return this;

            // Shift the point by its center, rotate, then add the center back in
            return Subtract(center).Rotate(angle).Add(center);
        }

		/// <summary>Calculates the nearest integers larger than the current X and Y values.</summary>
		[CLSCompliant(false)]
		public static Point Ceiling(PointD value)
		{
			return new Point((int)Math.Ceiling(value.X), (int)Math.Ceiling(value.Y));
		}

		/// <summary>Calculates the nearest integers smaller than the current X and Y values.</summary>
		[CLSCompliant(false)]
		public static Point Floor(PointD value)
		{
			return new Point((int)Math.Floor(value.X), (int)Math.Floor(value.Y));
		}

		/// <summary>
		/// Indicates if the current instance is closer to the top of the monitor than the
		/// specified value.
		/// </summary>
		public bool IsAbove(PointD value)
		{
			return pY < value.Y;
		}

		/// <summary>
		/// Indicates if the current instance is closer to the bottom of the monitor than the
		/// specified value.
		/// </summary>
		public bool IsBelow(PointD value)
		{
			return pY > value.Y;
		}

		/// <summary>
		/// Indicates if the current instance is closer to the left of the monitor than the
		/// specified value.
		/// </summary>
		public bool IsLeftOf(PointD value)
		{
			return pX < value.X;
		}

		/// <summary>
		/// Indicates if the current instance is closer to the right of the monitor than the
		/// specified value.
		/// </summary>
		public bool IsRightOf(PointD value)
		{
			return pX > value.X;
		}

		/// <summary>Returns the current instance with its signs switched.</summary>
		/// <remarks>This method returns a new point where the signs of X and Y are flipped.  For example, if
		/// a point, represents (20, 40), this function will return (-20, -40).</remarks>
		public PointD Mirror()
		{
			return new PointD(-pX, -pY);
		}

		/// <summary>
		/// Returns the sum of two points by adding X and Y values together.
		/// </summary>
		/// <param name="offset"></param>
		/// <returns></returns>
		/// <remarks>This method adds the X and Y coordinates and returns a new point at that location.</remarks>
		public PointD Add(PointD offset)
		{
			return new PointD(pX + offset.X, pY + offset.Y);
			//return Offset(offset.X, offset.Y);
		}

		/// <summary>
		/// Returns the sum of two points by adding X and Y values together.
		/// </summary>
		/// <returns></returns>
		/// <remarks>This method adds the X and Y coordinates and returns a new point at that location.</remarks>
		public PointD Add(double offsetX, double offsetY)
		{
			return new PointD(pX + offsetX, pY + offsetY);
		}

		/// <summary>
		/// Returns the difference of two points by subtracting the specified X and Y values.
		/// </summary>
		/// <param name="offset"></param>
		/// <returns></returns>
		/// <remarks>This method subtracts the X and Y coordinates and returns a new point at that location.</remarks>
		public PointD Subtract(PointD offset)
		{
			return new PointD(pX - offset.X, pY - offset.Y);
			//return Offset(offset.X, offset.Y);
		}

		/// <summary>
		/// Returns the difference of two points by subtracting the specified X and Y values.
		/// </summary>
		/// <param name="offsetX"></param>
		/// <param name="offsetY"></param>
		/// <returns></returns>
		/// <remarks>This method subtracts the X and Y coordinates and returns a new point at that location.</remarks>
		public PointD Subtract(double offsetX, double offsetY)
		{
			return new PointD(pX - offsetX, pY - offsetY);
		}

		/// <summary>
		/// Returns the product of two points by multiplying X and Y values together.
		/// </summary>
		/// <param name="offset"></param>
		/// <returns></returns>
		/// <remarks>This method multiplies the X and Y coordinates together and returns a new point at that location.  This
		/// is typically used to scale a point from one coordinate system to another.</remarks>
		public PointD Multiply(PointD offset)
		{
			return new PointD(pX * offset.X, pY * offset.Y);
		}

		public PointD Multiply(double offsetX, double offsetY)
		{
			return new PointD(pX * offsetX, pY * offsetY);
		}

		public PointD Divide(PointD offset)
		{
			return new PointD(pX / offset.X, pY / offset.Y);
		}

		public PointD Divide(double offsetX, double offsetY)
		{
			return new PointD(pX / offsetX, pY / offsetY);
		}

		/// <summary>Calculates the distance to another pixel.</summary>
		public double DistanceTo(PointD value)
		{
			return Math.Sqrt(Math.Abs(value.X - pX) * Math.Abs(value.X - pX) 
				+ Math.Abs(value.Y - pY) * Math.Abs(value.Y - pY));
		}

		/// <summary>Calculates the direction from one point to another.</summary>
		public Azimuth BearingTo(PointD value)
		{
            double Result = value.Subtract(this).ToPolarCoordinate().Theta.DecimalDegrees;
            return new Azimuth(Result).Normalize();
		}

//		public void Translate(double dx, double dy)
//		{
//			X += dx;
//			Y += dy;
//		}
//
//		public Position ToPosition(ProjectionType projection, params object[] parameters)
//		{
//			Projector Projector = new Projector();
//			return Projector.ToPosition(projection, this, parameters);
//		}

        #if !PocketPC
		[CLSCompliant(false)]
		public static explicit operator PointF(PointD value)
		{
			return new PointF((float)value.X, (float)value.Y);
		}
        #endif

		[CLSCompliant(false)]
		public static explicit operator Point(PointD value)
		{
            return new Point((int)Math.Round(value.X, 0), (int)Math.Round(value.Y, 0));
        }

		public static explicit operator PolarCoordinate(PointD value)
		{
			return value.ToPolarCoordinate();
		}

        #if !PocketPC
		[CLSCompliant(false)]
		public static explicit operator PointD(PointF value)
		{
			return new PointD((double)value.X, (double)value.Y);
		}
        #endif

		[CLSCompliant(false)]
		public static explicit operator PointD(Point value)
		{
			return new PointD((double)value.X, (double)value.Y);
		}

		public static explicit operator string(PointD value)
		{
			return value.ToString();
		}


		#region IFormattable Members

		public override string ToString()
		{
			return ToString("G", CultureInfo.CurrentCulture);
		}

		public string ToString(string format)
		{
			return ToString(format, CultureInfo.CurrentCulture);
		}

		public string ToString(string format, IFormatProvider formatProvider)
		{
            // TODO:  Add PointD.ToString implementation
            return pX.ToString(format, formatProvider)
                + CultureInfo.CurrentCulture.TextInfo.ListSeparator
                + pY.ToString(format, formatProvider);
        }

		#endregion

        #region IFormattable Members

        string IFormattable.ToString(string format, IFormatProvider formatProvider)
        {
            return this.ToString(format, formatProvider);
        }

        #endregion

#if !PocketPC || (PocketPC && Framework20)
		#region IXmlSerializable Members

#if Framework20
        public static XmlQualifiedName GetXmlSchema(XmlSchemaSet xs)
        {
            // Load it from a string
            xs.Add(Xml.SchemaSet);

            // And return the type
            return new XmlQualifiedName("PointD", Xml.Namespace);
        }
#endif

        XmlSchema IXmlSerializable.GetSchema()
        {
            return null;
        }

        void IXmlSerializable.ReadXml(XmlReader reader)
        {
            ReadXml(reader);
        }

        void IXmlSerializable.WriteXml(XmlWriter writer)
        {
            WriteXml(writer);
        }

        /// <summary>
        /// Populates the current instance by reading XML from the specified reader.
        /// </summary>
        /// <param name="reader"></param>
        public void ReadXml(XmlReader reader)
        {
            if (reader == null)
                throw new ArgumentNullException("reader");

            // Now read in the elements
            while (!reader.EOF)
            {
                // Keep reading until we reach an element
                switch (reader.NodeType)
                {
                    case XmlNodeType.Element:
                        switch (reader.Name)
                        {
                            case "X":
                                reader.Read();
                                pX = double.Parse(reader.Value, CultureInfo.InvariantCulture);
                                break;
                            case "Y":
                                reader.Read();
                                pY = double.Parse(reader.Value, CultureInfo.InvariantCulture);
                                break;
                            default:
                                // Read the next item
                                reader.Read();
                                break;
                        }
                        break;
                    case XmlNodeType.EndElement:
                        if (String.Compare(reader.Name, "PointD", true, CultureInfo.InvariantCulture) == 0)
                        {
                            // Done reading!
                            return;
                        }
                        else
                        {
                            // Keep going
                            reader.Read();
                            continue;
                        }
                    default:
                        // Read the next item
                        reader.Read();
                        break;
                }
            }
        }

        /// <summary>
        /// Writes the values of the current instance to the specified XML writer.
        /// </summary>
        /// <param name="writer"></param>
        public void WriteXml(XmlWriter writer)
        {
            if (writer == null)
                throw new ArgumentNullException("writer");

            writer.WriteStartElement("X");
            writer.WriteString(pX.ToString("G15", CultureInfo.InvariantCulture));
            writer.WriteEndElement();

            writer.WriteStartElement("Y");
            writer.WriteString(pY.ToString("G15", CultureInfo.InvariantCulture));
            writer.WriteEndElement();
        }

		#endregion
#endif

#if !PocketPC || DesignTime
        #region ISerializable Members

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]  
        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("X", pX);
            info.AddValue("Y", pY);
        }

        #endregion
#endif
	}
}
