using System;
using System.IO;
using System.Text;
using System.Threading;
using System.Globalization;
using System.Security.Permissions;
using System.Reflection;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
#if !PocketPC || DesignTime
using System.ComponentModel;
    using System.Runtime.Serialization;
    using System.ComponentModel.Design.Serialization;
#endif

namespace GeoFramework
{
	/// <summary>Represents a specific location on Earth's surface.</summary>
	/// <remarks>
	/// 	<para>Instances of this class are guaranteed to be thread-safe because the class is
	///     immutable (its properties can only be changed via constructors).</para>
	/// </remarks>
#if !PocketPC || DesignTime
#if Framework30
    [TypeConverter("GeoFramework.Design.PositionConverter, GeoFramework.Design, Culture=neutral, Version=1.0.0.0, PublicKeyToken=3ed3cdf4fdda3400")]
#elif Framework20
    [TypeConverter("GeoFramework.Design.PositionConverter, GeoFramework.Design, Culture=neutral, Version=1.0.0.0, PublicKeyToken=3ed3cdf4fdda3400")]
#else
	[TypeConverter(typeof(Design.PositionConverter))]
#endif   
	[Serializable()]
    [ImmutableObject(true)]
#endif
#if Framework20
    [XmlSchemaProvider("GetXmlSchema")]
#endif
    public class Position : IFormattable
#if !PocketPC || (PocketPC && Framework20)
	, IXmlSerializable
#endif  
#if !PocketPC || DesignTime
, ISerializable
#endif
    {

        #region  Private Variables
        //private Distance pAltitude;
		private Latitude pLatitude;
		private Longitude pLongitude;
		private Distance pNorthing;
		private Distance pEasting;
        //private int pZoneNumber;
        //private ZoneLetter pZoneLetter; 
		private Datum pDatum;
		//private CoordinateSystem pCoordinateSystem;
//		private bool NeedUtmCoordinate;
//		private bool NeedLatLong;
		#endregion

        // Accuracy is set to 1.0E-12, the smallest value allowed by a Latitude or Longitude
        private const double TargetAccuracy = 1.0E-12;

		/// <summary>Represents the location at 0, 0.</summary>
		public static readonly Position Empty = new Position(Latitude.Empty, Longitude.Empty);
		/// <summary>Represents the smallest possible location of 90S, 180W.</summary>
		public static readonly Position Minimum = new Position(Latitude.Minimum, Longitude.Minimum);
        /// <summary>Represents the largest possible location of 90N, 180E.</summary>
        public static readonly Position Maximum = new Position(Latitude.Maximum, Longitude.Maximum);
        /// <summary>Represents the single point at the top of Earth: 90N, 0E.</summary>
        public static readonly Position NorthPole = new Position(Latitude.Maximum, Longitude.Empty);
        /// <summary>Represents the single point at the bottom of Earth: 90S, 0E.</summary>
        public static readonly Position SouthPole = new Position(Latitude.Minimum, Longitude.Empty);


        #region Static members for popular places on Earth
        /*
        /// <summary>Represents the location of GeoFrameworks, where this software was created.</summary>
        public static readonly Position GeoFrameworks = new Position(new Latitude(39.66), new Longitude(-105.11));


        */
        #endregion

        /// <summary>
        /// Returns a random location on Earth.
        /// </summary>
        /// <returns></returns>
		public static Position Random()
		{
            return new Position(Longitude.Random(DateTime.Now.Millisecond), Latitude.Random(DateTime.Now.Millisecond));
		}

        public static Position Random(int seed)
        {
            return new Position(Longitude.Random(seed), Latitude.Random(seed));
        }

        /// <summary>
        /// Returns a random location within the specified geographic rectangle.
        /// </summary>
        /// <param name="boundary"></param>
        /// <returns></returns>
        public static Position Random(GeographicRectangle boundary)
        {
            return new Position(Longitude.Random(boundary.Left, boundary.Right),
                                Latitude.Random(boundary.Top, boundary.Bottom));
        }

		#region Conversions

		public static explicit operator Position(string value) 
		{
			return Position.Parse(value, CultureInfo.CurrentCulture);
		}

		public static explicit operator string(Position value)
		{
			return value.ToString("g", CultureInfo.CurrentCulture);
		}
		#endregion

		#region  Constructors 

        /// <summary>
        /// Creates a new instance using default latitude and longitude of 0.
        /// </summary>
        public Position()
            : this(Latitude.Empty, Longitude.Empty)
        {}

		public Position(Longitude longitude, Latitude latitude) 			
		{
			pLatitude = latitude;
			pLongitude = longitude;
		}

		/// <overloads>Creates a new instance.</overloads>
		public Position(Latitude latitude, Longitude longitude)
		{			
			//pZoneLetter = ZoneLetter.Unknown;
			//pEasting = Distance.Empty;
			//pNorthing = Distance.Empty;
			// Set the coordinate system bias
			//pCoordinateSystem = CoordinateSystem.LatitudeLongitude;
			// Set the new values
			pLatitude = latitude;
			pLongitude = longitude;
			// We'll need the UTM coordinate calculated
			//NeedUtmCoordinate = true;
			//SetUtmCoordinate();
        }

		/// <summary>
		/// Creates a new instance using the specified latitude, longitude and
		/// Datum.
		/// </summary>
		/// <remarks>
		/// This constructor is commonly used when a new position must be created using an
		/// Datum other than the default of WGS1984.
		/// </remarks>
		/// <param name="latitude">An angle from the equator between -90 and +90.</param>
		/// <param name="longitude">A Longitude describing an angle from the Prime Meridian between -90 and +90.</param>
		/// <param name="datum">A Datum object to associate with the given latitude and 
		/// longitude.</param>
		public Position(Latitude latitude, Longitude longitude, Datum datum) 
		{
			// Set the new values
			pLatitude = latitude;
			pLongitude = longitude;
			pDatum = datum;
			//SetUtmCoordinate();
		}

		/// <summary>
		/// Creates a new instance by parsing latitude and longitude from a single string.
		/// </summary>
		/// <param name="value">A <strong>String</strong> containing both a latitude and longitude to parse.</param>
		public Position(string value) 
			: this(value, CultureInfo.CurrentCulture)
		{}

        /// <summary>
        /// Creates a new instance by interpreting the specified latitude and longitude.
        /// </summary>
        /// <param name="latitude">A <strong>String</strong> describing a latitude in the current culture.</param>
        /// <param name="longitude">A <strong>String</strong> describing a longitude in the current culture.</param>
        /// <remarks>Latitude and longitude values are parsed using the current local culture.  For better support
        /// of international cultures, add a CultureInfo parameter.</remarks>
		public Position(string latitude, string longitude) 
			: this(latitude, longitude, CultureInfo.CurrentCulture)
		{}

        /// <summary>
        /// Creates a new instance by interpreting the specified latitude and longitude.
        /// </summary>
        /// <param name="latitude">A <strong>String</strong> describing a latitude in the current culture.</param>
        /// <param name="longitude">A <strong>String</strong> describing a longitude in the current culture.</param>
        /// <remarks>Latitude and longitude values are parsed using the current local culture.  For better support
        /// of international cultures, a CultureInfo parameter should be specified to indicate how numbers should
        /// be parsed.</remarks>
        public Position(string latitude, string longitude, CultureInfo culture)
            : this(Latitude.Parse(latitude, culture),
            Longitude.Parse(longitude, culture))
        { }

        /// <summary>
        /// Creates a new instance by interpreting the specified string.
        /// </summary>
        /// <param name="value">A <strong>String</strong></param>
        /// <param name="datum"></param>
		public Position(string value, Datum datum) 
			: this(value, CultureInfo.CurrentCulture)
		{
			pDatum = datum;
		}

#if !PocketPC || DesignTime
        /// <summary>
        /// Creates a new instance by de-serializing previously-serialized information.
        /// </summary>
        /// <param name="info">A <strong>SerializationInfo</strong> object containing serialized data.</param>
        /// <param name="context">A <strong>StreamingContext</strong> object describing the context of serialization.</param>
        /// <remarks>This constructor will de-serialize a previously-serialized object.</remarks>
        protected Position(SerializationInfo info, StreamingContext context)
        {
            pLatitude = new Latitude(info.GetDouble("Latitude"));
            pLongitude = new Longitude(info.GetDouble("Longitude"));
        }
#endif


        /// <summary>
        /// Creates a new instance by converting the specified string using the specific culture.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="culture"></param>
        public Position(string value, CultureInfo culture)
        {
            Position Result = null;


            // Try to parse the value as UTM
            if (value.IndexOf("") == 0)
            {
                Result = ParseAsUtm(value, culture);
                if (Result != null)
                {
                    pNorthing = Result.Northing;
                    pEasting = Result.Easting;
                    CalculateLatitudeLongitude(Result.ZoneLetter, Result.ZoneNumber);
                    return;
                }
            }

            // Try to parse the value as latitude/longitude
            Result = ParseAsLatLong(value, culture);
            if (Result != null) 
            {
                pLatitude = Result.Latitude.Normalize();
                pLongitude = Result.Longitude.Normalize();
                CalculateNorthingEasting();
                return;
            }

            // Raise an exception
#if PocketPC
                throw new ArgumentException(Resources.Position_InvalidFormat);
#else
            throw new ArgumentException(Resources.Position_InvalidFormat, "value");
#endif

        }


		public Position(ZoneLetter zoneLetter, int zoneNumber, Distance easting, Distance northing)
			: this(zoneLetter, zoneNumber, easting, northing, null)
		{
			pNorthing = northing.ToMeters();
			pEasting = easting.ToMeters();

			CalculateLatitudeLongitude(zoneLetter, zoneNumber);
		}

		public Position(ZoneLetter zoneLetter, int zoneNumber, Distance easting, Distance northing, Datum datum) 
		{
			pDatum = datum;
			pNorthing = northing.ToMeters();
			pEasting = easting.ToMeters();

			CalculateLatitudeLongitude(zoneLetter, zoneNumber);
		}

        /// <summary>
        /// Creates a copy of the specified object.
        /// </summary>
        /// <param name="position"></param>
        public Position(Position position)
            : this(position.Latitude, position.Longitude)
        {}

		#endregion

		#region  Private members 

		#endregion

		#region  Public Members 

		/// <summary>Indicates if the position has no value.</summary>
		public bool IsEmpty
		{
			get
			{
				return pLatitude.IsEmpty && pLongitude.IsEmpty;
			}
		}

        /// <summary>
        /// Indicates wether the current instance is inside of the specified geographic rectangle.
        /// </summary>
        /// <param name="rectangle"></param>
        /// <returns></returns>
        public bool IsWithin(GeographicRectangle rectangle)
        {
            return rectangle.IsEnclosing(this);
        }

		// Returns the speed (as the crow flies) required to reach the given destination in the given time
		/// <summary>
		/// Returns the minimum speed required to travel from the current location to the
		/// specified destination within the specified period of time.
		/// </summary>
		public Speed SpeedTo(Position destination, TimeSpan time)
		{
			try
			{
				// Make sure the position is in the same datum as we are
				//Dim AdjustedDestination As Position = destination.ToDatum(Datum)
				// Now calculate the distance
				double TravelDistance = DistanceTo(destination).ToMeters().Value;
				// Perform the calculation
				return new Speed(TravelDistance / time.TotalSeconds, SpeedUnit.MetersPerSecond);
			}
			catch
			{
				throw;
			}
		}

		public Position Rotate(Angle angle)
		{
			// Normalize the angle
			Angle Offset = angle.Normalize();
			// Anything to do?
            if (Offset.IsEmpty) 
                return this;
			// Convert the angle to radians
            double AngleRadians = Offset.ToRadians().Value;
			// Yes.  Rotate the point about 0,0
			return new Position(
                new Longitude(Math.Cos(AngleRadians) * pLongitude.DecimalDegrees  
                        - Math.Sin(AngleRadians) * pLatitude.DecimalDegrees),
                new Latitude(Math.Sin(AngleRadians) * pLongitude.DecimalDegrees 
                        + Math.Cos(AngleRadians) * pLatitude.DecimalDegrees));
		}

		public Position RotateAt(Angle angle, Position center)
		{
			// Shift the point by its center, rotate, then add the center back in
			return Subtract(center).Rotate(angle).Add(center);
		}


#if !PocketPC || Framework20
        /// <summary>
        /// Indicates whether the current instance is the same reference as the specified object.
        /// </summary>
        /// <param name="obj">An <strong>Angle</strong> to compare with.</param>
        /// <returns>A <strong>Boolean</strong>, True if the current instance is the same instance as the specified object.</returns>
        public bool ReferenceEquals(Position obj)
        {
            return Object.ReferenceEquals(this, obj);
        }

        /// <summary>
        /// Indicates whether the specified instance has the same value as the current instance.
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public bool ValueEquals(Position obj)
        {
            return Equals(obj);
        }

        /// <summary>
        /// Indicates whether the specified instance has the same value as the current instance.
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static bool ValueEquals(Position obj1, Position obj2)
        {
            return obj1.ValueEquals(obj2);
        }
#endif

//		/// <summary>
//		/// Indicates if any value is specified.
//		/// </summary>
//		public bool IsEmpty
//		{
//			get
//			{
//				return (Latitude.IsEmpty && Longitude.IsEmpty);
//			}
//		}

		/// <summary>
		/// Indicates if the current instance is North of the specified position.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> object to examine.</param>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the current instance is more North than the specified instance.</returns>
		public bool IsNorthOf(Position value)
		{
			return Latitude.IsGreaterThan(value.Latitude.DecimalDegrees);
		}

		/// <summary>
		/// Indicates if the current instance is South of the specified position.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> object to examine.</param>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the current instance is more South than the specified instance.</returns>
		public bool IsSouthOf(Position value)
		{
            return Latitude.IsLessThan(value.Latitude.DecimalDegrees);
		}

		/// <summary>
		/// Indicates if the current instance is East of the specified position.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> object to examine.</param>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the current instance is more East than the specified instance.</returns>
		public bool IsEastOf(Position value)
		{
            return Longitude.IsGreaterThan(value.Longitude.DecimalDegrees);
		}

		/// <summary>
		/// Indicates if the current instance is West of the specified position.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> object to examine.</param>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the current instance is more West than the specified instance.</returns>
		public bool IsWestOf(Position value)
		{
            return Longitude.IsLessThan(value.Longitude.DecimalDegrees);
		}

		/// <overloads>
		/// Calculates the minimum speed required to arrive at a destination in the given
		/// time.
		/// </overloads>
		/// <summary>
		/// Returns the minimum speed required to travel over land from the given starting
		/// point to the specified destination within the specified period of time.
		/// </summary>
		/// <returns>
		/// A <strong>Speed</strong> object containing the required minimum travel
		/// speed.
		/// </returns>
		/// <remarks>
		/// This method is typically used to compare the current speed with the minimum
		/// required speed. For example, if the current rate of travel is 30MPH and the minimum
		/// speed is 60MPH, it can be derived that the speed must be doubled to arrive at the
		/// destination on time. Of course, care must be taken when making any suggestion to
		/// increase driving speed.
		/// </remarks>
		/// <param name="start">The beginning point from which calculations are based.</param>
		/// <param name="destination">The ending point to which speed is calculated.</param>
		/// <param name="time">The amount of time allowed to reach the destination.</param>
		public static Speed SpeedTo(Position start, Position destination, TimeSpan time)
		{
			try
			{
				return start.SpeedTo(destination, time);
			}
			catch
			{
				throw;
			}
		}

		// Returns the time (as the crow flies) to the given destination using the specified speed
		/// <summary>
		/// Returns the minimum time required to travel to the given destination at the
		/// specified constant speed.
		/// </summary>
        /// <exception cref="ArgumentOutOfRangeException">The TimeTo method expects a value for Speed greater than zero.</exception>
		public TimeSpan TimeTo(Position destination, Speed speed)
		{
            if (speed.Value <= 0)
                throw new ArgumentOutOfRangeException("The TimeTo method expects a value for Speed greater than zero.");

			// Perform the calculation
            return TimeSpan.FromSeconds(DistanceTo(destination).ToMeters().Value 
                / speed.ToMetersPerSecond().Value);
		}

		/// <overloads>
		/// Calculates the time required to arrive at a destination when traveling at the
		/// specified speed.
		/// </overloads>
		/// <summary>
		/// Returns the minimum amount of time required to reach the specified destination at
		/// the specified speed.
		/// </summary>
		public static TimeSpan TimeTo(Position start, Position destination, Speed speed)
		{
		    return start.TimeTo(destination, speed);
		}

        /// <overloads>Calculates the great circle distance between any two points on 
        /// Earth.</overloads>
        /// <summary>
        /// Returns the distance over land from the given starting point to the specified
        /// destination.
        /// </summary>
        /// <returns>
        /// A <strong>Distance</strong> object containing the calculated distance in
        /// kilometers.
        /// </returns>
        /// <param name="destination">The ending point of a segment.</param>
        /// <remarks>
        /// This method uses trigonometry to calculate the Great Circle (over Earth's curved
        /// surface) distance between any two points on Earth. The distance is returned in
        /// kilometers but can be converted to any other unit type using methods in the
        /// <see cref="Distance">Distance</see>
        /// class.
        /// </remarks>
        public Distance DistanceTo(Position destination)
        {
            return DistanceTo(destination, this.Datum.Ellipsoid);
        }

        /// <overloads>Calculates the great circle distance between any two points on 
        /// Earth.</overloads>
        /// <summary>
        /// Returns the distance over land from the given starting point to the specified
        /// destination.
        /// </summary>
        /// <returns>
        /// A <strong>Distance</strong> object containing the calculated distance in
        /// kilometers.
        /// </returns>
        /// <param name="destination">The ending point of a segment.</param>
        /// <remarks>
        /// This method uses a high-speed formula to determine the Great Circle distance from one
        /// point to another.  This method is typically used in situations where hundreds of distance
        /// measurements must be made in a short period of time.  The <strong>DistanceTo</strong> method
        /// produces accuracy to one millimeter, but its formula is about a hundred times slower than this 
        /// method.  
        /// <see cref="Distance">Distance</see>
        /// class.
        /// </remarks>
        public Distance DistanceTo(Position destination, bool isApproximated)
        {
            if (!isApproximated)
                return DistanceTo(destination);

            //Dim AdjustedDestination As Position = destination.ToDatum(Datum)
            // USING THE FORMULA FROM: 
            //$lat1 = deg2rad(28.5333);
            double lat1 = Latitude.ToRadians().Value;
            //$lat2 = deg2rad(31.1000);
            double lat2 = destination.Latitude.ToRadians().Value;
            //$long1 = deg2rad(-81.3667);
            double long1 = Longitude.ToRadians().Value;
            //$long2 = deg2rad(121.3667);
            double long2 = destination.Longitude.ToRadians().Value;
            //$dlat = abs($lat2 - $lat1);
            double dlat = Math.Abs(lat2 - lat1);
            //$dlong = abs($long2 - $long1);
            double dlong = Math.Abs(long2 - long1);
            //$l = ($lat1 + $lat2) / 2;
            double l = (lat1 + lat2) * 0.5;
            //$a = 6378;
            double a = Datum.Ellipsoid.EquatorialRadius.ToKilometers().Value;
            //$b = 6357;
            double b = Datum.Ellipsoid.PolarRadius.ToKilometers().Value;
            //$e = sqrt(1 - ($b * $b)/($a * $a));
            double e = Math.Sqrt(1 - (b * b) / (a * a));
            //$r1 = ($a * (1 - ($e * $e))) / pow((1 - ($e * $e) * (sin($l) * sin($l))), 3/2);
            double r1 = (a * (1 - (e * e))) / Math.Pow((1 - (e * e) * (Math.Sin(l) * Math.Sin(l))), 3 * 0.5);
            //$r2 = $a / sqrt(1 - ($e * $e) * (sin($l) * sin($l)));
            double r2 = a / Math.Sqrt(1 - (e * e) * (Math.Sin(l) * Math.Sin(l)));
            //$ravg = ($r1 * ($dlat / ($dlat + $dlong))) + ($r2 * ($dlong / ($dlat + $dlong)));
            double ravg = (r1 * (dlat / (dlat + dlong))) + (r2 * (dlong / (dlat + dlong)));
            //$sinlat = sin($dlat / 2);
            double sinlat = Math.Sin(dlat * 0.5);
            //$sinlon = sin($dlong / 2);
            double sinlon = Math.Sin(dlong * 0.5);
            //$a = pow($sinlat, 2) + cos($lat1) * cos($lat2) * pow($sinlon, 2);
            a = Math.Pow(sinlat, 2) + Math.Cos(lat1) * Math.Cos(lat2) * Math.Pow(sinlon, 2);
            //$c = 2 * asin(min(1, sqrt($a)));
            double c = 2 * Math.Asin(Math.Min(1, Math.Sqrt(a)));
            //$d = $ravg * $c; 
            double d = ravg * c;
            // If it's NaN, return zero
            if (double.IsNaN(d))
            {
                d = 0.0;
            }
            // Return a new distance
            return new Distance(d, DistanceUnit.Kilometers).ToLocalUnitType();
        }

		/// <overloads>Calculates the great circle distance between any two points on 
		/// Earth using a specific model of Earth's shape.</overloads>
		/// <summary>
		/// Returns the distance over land from the given starting point to the specified
		/// destination.
		/// </summary>
		/// <returns>
		/// A <strong>Distance</strong> object containing the calculated distance in
		/// kilometers.
		/// </returns>
        /// <param name="destination">The ending point of a segment.</param>
        /// <param name="ellipsoid">The model of the Earth to use for the distance calculation.</param>
        /// <remarks>
		/// This method uses trigonometry to calculate the Great Circle (over Earth's curved
		/// surface) distance between any two points on Earth. The distance is returned in
		/// kilometers but can be converted to any other unit type using methods in the
		/// <see cref="Distance">Distance</see>
		/// class.
		/// </remarks>
		//[CLSCompliant(false)]
		public Distance DistanceTo(Position destination, Ellipsoid ellipsoid)
		{
            // From: http://www.mathworks.com/matlabcentral/files/8607/vdist.m

            /*
             function varargout = vdist(lat1,lon1,lat2,lon2)
% VDIST - Using the WGS-84 Earth ellipsoid, compute the distance between
%         two points within a few millimeters of accuracy, compute forward
%         azimuth, and compute backward azimuth, all using a vectorized
%         version of Vincenty's algorithm.
%
% s = vdist(lat1,lon1,lat2,lon2)
% [s,a12] = vdist(lat1,lon1,lat2,lon2)
% [s,a12,a21] = vdist(lat1,lon1,lat2,lon2)
%
% s = distance in meters (inputs may be scalars, vectors, or matrices)
% a12 = azimuth in degrees from first point to second point (forward)
% a21 = azimuth in degrees from second point to first point (backward)
%       (Azimuths are in degrees clockwise from north.)
% lat1 = GEODETIC latitude of first point (degrees)
% lon1 = longitude of first point (degrees)
% lat2, lon2 = second point (degrees)
%
%  Original algorithm source:
%  T. Vincenty, "Direct and Inverse Solutions of Geodesics on the Ellipsoid
%  with Application of Nested Equations", Survey Review, vol. 23, no. 176,
%  April 1975, pp 88-93.
%  Available at: http://www.ngs.noaa.gov/PUBS_LIB/inverse.pdf
%
% Notes: (1) lat1,lon1,lat2,lon2 can be any (identical) size/shape. Outputs
%            will have the same size and shape.
%        (2) Error correcting code, convergence failure traps, antipodal
%            corrections, polar error corrections, WGS84 ellipsoid
%            parameters, testing, and comments: Michael Kleder, 2004.
%        (3) Azimuth implementation (including quadrant abiguity
%            resolution) and code vectorization, Michael Kleder, Sep 2005.
%        (4) Vectorization is convergence sensitive; that is, quantities
%            which have already converged to within tolerance are not
%            recomputed during subsequent iterations (while other
%            quantities are still converging).
%        (5) Vincenty describes his distance algorithm as precise to within
%            0.01 millimeters, subject to the ellipsoidal model.
%        (6) For distance calculations, essentially antipodal points are
%            treated as exactly antipodal, potentially reducing accuracy
%            slightly.
%        (7) Distance failures for points exactly at the poles are
%            eliminated by moving the points by 0.6 millimeters.
%        (8) The Vincenty distance algorithm was transcribed verbatim by
%            Peter Cederholm, August 12, 2003. It was modified and
%            translated to English by Michael Kleder.
%            Mr. Cederholm's website is http://www.plan.aau.dk/~pce/
%        (9) Distances agree with the Mapping Toolbox, version 2.2 (R14SP3)
%            with a max relative difference of about 5e-9, except when the
%            two points are nearly antipodal, and except when one point is
%            near the equator and the two longitudes are nearly 180 degrees
%            apart. This function (vdist) is more accurate in such cases.
%            For example, note this difference (as of this writing):
%            >>vdist(0.2,305,15,125)
%            18322827.0131551
%            >>distance(0.2,305,15,125,[6378137 0.08181919])
%            0
%       (10) Azimuths FROM the north pole (either forward starting at the
%            north pole or backward when ending at the north pole) are set
%            to 180 degrees by convention. Azimuths FROM the south pole are
%            set to 0 degrees by convention.
%       (11) Azimuths agree with the Mapping Toolbox, version 2.2 (R14SP3)
%            to within about a hundred-thousandth of a degree, except when
%            traversing to or from a pole, where the convention for this
%            function is described in (10), and except in the cases noted
%            above in (9).
%       (12) No warranties; use at your own risk.

% reshape inputs
keepsize = size(lat1);
lat1=lat1(:);
lon1=lon1(:);
lat2=lat2(:);
lon2=lon2(:);
% Input check:
if any(abs(lat1)>90 | abs(lat2)>90)
    error('Input latitudes must be between -90 and 90 degrees, inclusive.')
end
% Supply WGS84 earth ellipsoid axis lengths in meters:
a = 6378137; % definitionally
b = 6356752.31424518; % computed from WGS84 earth flattening coefficient
% preserve true input latitudes:
lat1tr = lat1;
lat2tr = lat2;
% convert inputs in degrees to radians:
lat1 = lat1 * 0.0174532925199433;
lon1 = lon1 * 0.0174532925199433;
lat2 = lat2 * 0.0174532925199433;
lon2 = lon2 * 0.0174532925199433;
% correct for errors at exact poles by adjusting 0.6 millimeters:
kidx = abs(pi/2-abs(lat1)) < 1e-10;
if any(kidx);
    lat1(kidx) = sign(lat1(kidx))*(pi/2-(1e-10));
end
kidx = abs(pi/2-abs(lat2)) < 1e-10;
if any(kidx)
    lat2(kidx) = sign(lat2(kidx))*(pi/2-(1e-10));
end
f = (a-b)/a;
U1 = atan((1-f)*tan(lat1));
U2 = atan((1-f)*tan(lat2));
lon1 = mod(lon1,2*pi);
lon2 = mod(lon2,2*pi);
L = abs(lon2-lon1);
kidx = L > pi;
if any(kidx)
    L(kidx) = 2*pi - L(kidx);
end
lambda = L;
lambdaold = 0*lat1;
itercount = 0;
notdone = logical(1+0*lat1);
alpha = 0*lat1;
sigma = 0*lat1;
cos2sigmam = 0*lat1;
C = 0*lat1;
warninggiven = logical(0);
while any(notdone)  % force at least one execution
    %disp(['lambda(21752) = ' num2str(lambda(21752),20)]);
    itercount = itercount+1;
    if itercount > 50
        if ~warninggiven
            warning(['Essentially antipodal points encountered. ' ...
                'Precision may be reduced slightly.']);
        end
        lambda(notdone) = pi;
        break
    end
    lambdaold(notdone) = lambda(notdone);
    sinsigma(notdone) = sqrt((cos(U2(notdone)).*sin(lambda(notdone)))...
        .^2+(cos(U1(notdone)).*sin(U2(notdone))-sin(U1(notdone)).*...
        cos(U2(notdone)).*cos(lambda(notdone))).^2);
    cossigma(notdone) = sin(U1(notdone)).*sin(U2(notdone))+...
        cos(U1(notdone)).*cos(U2(notdone)).*cos(lambda(notdone));
    % eliminate rare imaginary portions at limit of numerical precision:
    sinsigma(notdone)=real(sinsigma(notdone));
    cossigma(notdone)=real(cossigma(notdone));
    sigma(notdone) = atan2(sinsigma(notdone),cossigma(notdone));
    alpha(notdone) = asin(cos(U1(notdone)).*cos(U2(notdone)).*...
        sin(lambda(notdone))./sin(sigma(notdone)));
    cos2sigmam(notdone) = cos(sigma(notdone))-2*sin(U1(notdone)).*...
        sin(U2(notdone))./cos(alpha(notdone)).^2;
    C(notdone) = f/16*cos(alpha(notdone)).^2.*(4+f*(4-3*...
        cos(alpha(notdone)).^2));
    lambda(notdone) = L(notdone)+(1-C(notdone)).*f.*sin(alpha(notdone))...
        .*(sigma(notdone)+C(notdone).*sin(sigma(notdone)).*...
        (cos2sigmam(notdone)+C(notdone).*cos(sigma(notdone)).*...
        (-1+2.*cos2sigmam(notdone).^2)));
    %disp(['then, lambda(21752) = ' num2str(lambda(21752),20)]);
    % correct for convergence failure in the case of essentially antipodal
    % points
    if any(lambda(notdone) > pi)
        warning(['Essentially antipodal points encountered. ' ...
            'Precision may be reduced slightly.']);
        warninggiven = logical(1);
        lambdaold(lambda>pi) = pi;
        lambda(lambda>pi) = pi;
    end
    notdone = abs(lambda-lambdaold) > 1e-12;
end
u2 = cos(alpha).^2.*(a^2-b^2)/b^2;
A = 1+u2./16384.*(4096+u2.*(-768+u2.*(320-175.*u2)));
B = u2./1024.*(256+u2.*(-128+u2.*(74-47.*u2)));
deltasigma = B.*sin(sigma).*(cos2sigmam+B./4.*(cos(sigma).*(-1+2.*...
    cos2sigmam.^2)-B./6.*cos2sigmam.*(-3+4.*sin(sigma).^2).*(-3+4*...
    cos2sigmam.^2)));
varargout{1} = reshape(b.*A.*(sigma-deltasigma),keepsize);
if nargout > 1
    % From point #1 to point #2
    % correct sign of lambda for azimuth calcs:
    lambda = abs(lambda);
    kidx=sign(sin(lon2-lon1)) .* sign(sin(lambda)) < 0;
    lambda(kidx) = -lambda(kidx);
    numer = cos(U2).*sin(lambda);
    denom = cos(U1).*sin(U2)-sin(U1).*cos(U2).*cos(lambda);
    a12 = atan2(numer,denom);
    kidx = a12<0;
    a12(kidx)=a12(kidx)+2*pi;
    % from poles:
    a12(lat1tr <= -90) = 0;
    a12(lat1tr >= 90 ) = pi;
    varargout{2} = reshape(a12 * 57.2957795130823,keepsize); % to degrees
end
if nargout > 2
    a21=NaN*lat1;
    % From point #2 to point #1
    % correct sign of lambda for azimuth calcs:
    lambda = abs(lambda);
    kidx=sign(sin(lon1-lon2)) .* sign(sin(lambda)) < 0;
    lambda(kidx)=-lambda(kidx);
    numer = cos(U1).*sin(lambda);
    denom = sin(U1).*cos(U2)-cos(U1).*sin(U2).*cos(lambda);
    a21 = atan2(numer,denom);
    kidx=a21<0;
    a21(kidx)= a21(kidx)+2*pi;
    % backwards from poles:
    a21(lat2tr >= 90) = pi;
    a21(lat2tr <= -90) = 0;
    varargout{3} = reshape(a21 * 57.2957795130823,keepsize); % to degrees
end
return

             * 
             * 
             */

            // If positions are equivalent, return zero
            if (Equals(destination))
                return Distance.Empty;

            #region Newer code

            double goodalpha = 0;
            double goodsigma = 0;
            double goodcos2sigmam = 0;

//            % reshape inputs
//keepsize = size(lat1);
//lat1=lat1(:);
//lon1=lon1(:);
//lat2=lat2(:);
//lon2=lon2(:);

            // ?

//% Input check:
//if any(abs(lat1)>90 | abs(lat2)>90)
//    error('Input latitudes must be between -90 and 90 degrees, inclusive.')
//end

            // The -90 to 90 check is handled by Normalize

//% Supply WGS84 earth ellipsoid axis lengths in meters:
//a = 6378137; % definitionally
//b = 6356752.31424518; % computed from WGS84 earth flattening coefficient

            double a = ellipsoid.EquatorialRadius.ToMeters().Value;
            double b = ellipsoid.PolarRadius.ToMeters().Value;

//% preserve true input latitudes:
//lat1tr = lat1;
//lat2tr = lat2;

            //double lat1tr = pLatitude.DecimalDegrees;
            //double lat2tr = destination.Latitude.DecimalDegrees;

//% convert inputs in degrees to radians:
//lat1 = lat1 * 0.0174532925199433;
//lon1 = lon1 * 0.0174532925199433;
//lat2 = lat2 * 0.0174532925199433;
//lon2 = lon2 * 0.0174532925199433;

            // Convert inputs into radians
            double lat1 = this.Latitude.Normalize().ToRadians().Value;
            double lon1 = this.Longitude.Normalize().ToRadians().Value;
            double lat2 = destination.Latitude.Normalize().ToRadians().Value;
            double lon2 = destination.Longitude.Normalize().ToRadians().Value;

//% correct for errors at exact poles by adjusting 0.6 millimeters:
//kidx = abs(pi/2-abs(lat1)) < 1e-10;
//if any(kidx);
//    lat1(kidx) = sign(lat1(kidx))*(pi/2-(1e-10));
//end

            // Correct for errors at exact poles by adjusting 0.6mm
            if (Math.Abs(Math.PI * 0.5 - Math.Abs(lat1)) < 1E-10)
            {
                lat1 = Math.Sign(lat1) * (Math.PI * 0.5 - 1E-10);
            }

//kidx = abs(pi/2-abs(lat2)) < 1e-10;
//if any(kidx)
//    lat2(kidx) = sign(lat2(kidx))*(pi/2-(1e-10));
//end

            if (Math.Abs(Math.PI * 0.5 - Math.Abs(lat2)) < 1E-10)
            {
                lat2 = Math.Sign(lat2) * (Math.PI * 0.5 - 1E-10);
            }


//f = (a-b)/a;

            double f = ellipsoid.Flattening; 

//U1 = atan((1-f)*tan(lat1));

            double U1 = Math.Atan((1 - f) * Math.Tan(lat1));

//U2 = atan((1-f)*tan(lat2));

            double U2 = Math.Atan((1 - f) * Math.Tan(lat2));

//lon1 = mod(lon1,2*pi);

            lon1 = lon1 % (2 * Math.PI);

//lon2 = mod(lon2,2*pi);

            lon2 = lon2 % (2 * Math.PI);

//L = abs(lon2-lon1);

            double L = Math.Abs(lon2 - lon1);

//kidx = L > pi;
//if any(kidx)
//    L(kidx) = 2*pi - L(kidx);
//end

            if (L > Math.PI)
            {
                L = 2.0 * Math.PI - L;
            }

//lambda = L;

            double lambda = L;

//lambdaold = 0*lat1;

            double lambdaold = 0;

//itercount = 0;

            int itercount = 0;

//notdone = logical(1+0*lat1);

            bool notdone = true;

//alpha = 0*lat1;

            double alpha = 0;

//sigma = 0*lat1;

            double sigma = 0;

//cos2sigmam = 0*lat1;

            double cos2sigmam = 0;

//C = 0*lat1;

            double C = 0;

//warninggiven = logical(0);

            //bool warninggiven = false;

//while any(notdone)  % force at least one execution

            while (notdone)
            {

                //    %disp(['lambda(21752) = ' num2str(lambda(21752),20)]);
                //    itercount = itercount+1;

                itercount++;

                //    if itercount > 50

                if (itercount > 50)
                {

                    //        if ~warninggiven

                    //if (!warninggiven)
                    //{
                    //    //            warning(['Essentially antipodal points encountered. ' ...
                    //    //                'Precision may be reduced slightly.']);

                    //    warninggiven = true;
                    //    throw new WarningException("Distance calculation accuracy may be reduced because the two endpoints are antipodal.");
                    //}

                    //        end
                    //        lambda(notdone) = pi;

                    lambda = Math.PI;

                    //        break

                    break;

                    //    end

                }

                //    lambdaold(notdone) = lambda(notdone);

                lambdaold = lambda;

                //    sinsigma(notdone) = sqrt((cos(U2(notdone)).*sin(lambda(notdone)))...
                //        .^2+(cos(U1(notdone)).*sin(U2(notdone))-sin(U1(notdone)).*...
                //        cos(U2(notdone)).*cos(lambda(notdone))).^2);

                double sinsigma = Math.Sqrt(Math.Pow((Math.Cos(U2) * Math.Sin(lambda))
                        , 2) + Math.Pow((Math.Cos(U1) * Math.Sin(U2) - Math.Sin(U1) *
                        Math.Cos(U2) * Math.Cos(lambda)), 2));

                //    cossigma(notdone) = sin(U1(notdone)).*sin(U2(notdone))+...
                //        cos(U1(notdone)).*cos(U2(notdone)).*cos(lambda(notdone));

                double cossigma = Math.Sin(U1) * Math.Sin(U2) +
                    Math.Cos(U1) * Math.Cos(U2) * Math.Cos(lambda);

                //    % eliminate rare imaginary portions at limit of numerical precision:
                //    sinsigma(notdone)=real(sinsigma(notdone));
                //    cossigma(notdone)=real(cossigma(notdone));

                // Eliminate rare imaginary portions at limit of numerical precision:
                // ?

                //    sigma(notdone) = atan2(sinsigma(notdone),cossigma(notdone));

                sigma = Math.Atan2(sinsigma, cossigma);

                //    alpha(notdone) = asin(cos(U1(notdone)).*cos(U2(notdone)).*...
                //        sin(lambda(notdone))./sin(sigma(notdone)));

                alpha = Math.Asin(Math.Cos(U1) * Math.Cos(U2) *
                    Math.Sin(lambda) / Math.Sin(sigma));

                //    cos2sigmam(notdone) = cos(sigma(notdone))-2*sin(U1(notdone)).*...
                //        sin(U2(notdone))./cos(alpha(notdone)).^2;

                cos2sigmam = Math.Cos(sigma) - 2.0 * Math.Sin(U1) *
                    Math.Sin(U2) / Math.Pow(Math.Cos(alpha) , 2);

                //    C(notdone) = f/16*cos(alpha(notdone)).^2.*(4+f*(4-3*...
                //        cos(alpha(notdone)).^2));

                C = f / 16 * Math.Pow(Math.Cos(alpha) , 2) * (4 + f * (4 - 3 *
                    Math.Pow(Math.Cos(alpha) , 2)));

                //    lambda(notdone) = L(notdone)+(1-C(notdone)).*f.*sin(alpha(notdone))...
                //        .*(sigma(notdone)+C(notdone).*sin(sigma(notdone)).*...
                //        (cos2sigmam(notdone)+C(notdone).*cos(sigma(notdone)).*...
                //        (-1+2.*cos2sigmam(notdone).^2)));

                lambda = L + (1 - C) * f * Math.Sin(alpha)
                            * (sigma + C * Math.Sin(sigma) *
                            (cos2sigmam + C * Math.Cos(sigma) *
                            (-1 + 2 * Math.Pow(cos2sigmam, 2))));

                //    %disp(['then, lambda(21752) = ' num2str(lambda(21752),20)]);
                //    % correct for convergence failure in the case of essentially antipodal
                //    % points

                // Correct for convergence failure in the case of essentially antipodal points

                //    if any(lambda(notdone) > pi)

                if (lambda > Math.PI)
                {

                    //        if ~warninggiven

                    //if (!warninggiven)
                    //{
                    //    //            warning(['Essentially antipodal points encountered. ' ...
                    //    //                'Precision may be reduced slightly.']);

                    //    warninggiven = true;
                    //    throw new WarningException("Distance calculation accuracy may be reduced because the two endpoints are antipodal.");
                    //}

                    //        end

                    //        lambdaold(lambda>pi) = pi;

                    lambdaold = Math.PI;

                    //        lambda(lambda>pi) = pi;

                    lambda = Math.PI;

                    //    end

                }

                //    notdone = abs(lambda-lambdaold) > 1e-12;

                notdone = Math.Abs(lambda - lambdaold) > TargetAccuracy;

                //end

                // NOTE: In some cases "alpha" would return a "NaN".  If values are healthy, 
                // remember them so we get a good distance calc.
                if (!double.IsNaN(alpha))
                {
                    goodalpha = alpha;
                    goodsigma = sigma;
                    goodcos2sigmam = cos2sigmam;
                }

                // Allow other threads some breathing room
                Thread.Sleep(0);
            }


//u2 = cos(alpha).^2.*(a^2-b^2)/b^2;

            double u2 = Math.Pow(Math.Cos(goodalpha), 2) * (Math.Pow(a, 2) - Math.Pow(b, 2)) / Math.Pow(b, 2);

//A = 1+u2./16384.*(4096+u2.*(-768+u2.*(320-175.*u2)));

            double A = 1 + u2 / 16384 * (4096 + u2 *(-768 + u2 * (320 - 175 * u2)));

//B = u2./1024.*(256+u2.*(-128+u2.*(74-47.*u2)));

            double B = u2 / 1024 * (256 + u2 * (-128 + u2 * (74 - 47 * u2)));

//deltasigma = B.*sin(sigma).*(cos2sigmam+B./4.*(cos(sigma).*(-1+2.*...
//    cos2sigmam.^2)-B./6.*cos2sigmam.*(-3+4.*sin(sigma).^2).*(-3+4*...
//    cos2sigmam.^2)));

            double deltasigma = B * Math.Sin(goodsigma) * (goodcos2sigmam + B / 4 * (Math.Cos(goodsigma) * (-1 + 2 *
                Math.Pow(goodcos2sigmam, 2)) - B / 6 * goodcos2sigmam * (-3 + 4 * Math.Pow(Math.Sin(goodsigma), 2)) * (-3 + 4 *
                Math.Pow(goodcos2sigmam, 2))));

//varargout{1} = reshape(b.*A.*(sigma-deltasigma),keepsize);
            double s = b * A * (goodsigma - deltasigma);

            // Return the Distance in meters
            return new Distance(s, DistanceUnit.Meters).ToLocalUnitType();

//if nargout > 1
//    % From point #1 to point #2
//    % correct sign of lambda for azimuth calcs:
//    lambda = abs(lambda);
//    kidx=sign(sin(lon2-lon1)) .* sign(sin(lambda)) < 0;
//    lambda(kidx) = -lambda(kidx);
//    numer = cos(U2).*sin(lambda);
//    denom = cos(U1).*sin(U2)-sin(U1).*cos(U2).*cos(lambda);
//    a12 = atan2(numer,denom);
//    kidx = a12<0;
//    a12(kidx)=a12(kidx)+2*pi;
//    % from poles:
//    a12(lat1tr <= -90) = 0;
//    a12(lat1tr >= 90 ) = pi;
//    varargout{2} = reshape(a12 * 57.2957795130823,keepsize); % to degrees
//end
//if nargout > 2
//    a21=NaN*lat1;
//    % From point #2 to point #1
//    % correct sign of lambda for azimuth calcs:
//    lambda = abs(lambda);
//    kidx=sign(sin(lon1-lon2)) .* sign(sin(lambda)) < 0;
//    lambda(kidx)=-lambda(kidx);
//    numer = cos(U1).*sin(lambda);
//    denom = sin(U1).*cos(U2)-cos(U1).*sin(U2).*cos(lambda);
//    a21 = atan2(numer,denom);
//    kidx=a21<0;
//    a21(kidx)= a21(kidx)+2*pi;
//    % backwards from poles:
//    a21(lat2tr >= 90) = pi;
//    a21(lat2tr <= -90) = 0;
//    varargout{3} = reshape(a21 * 57.2957795130823,keepsize); % to degrees
//end
//return

            #endregion


            #region Old Code

            /*
            //Dim AdjustedDestination As Position = destination.ToDatum(Datum)
            // USING THE FORMULA FROM: 
            //$lat1 = deg2rad(28.5333);
            double lat1 = Latitude.ToRadians().Value;
            //$lat2 = deg2rad(31.1000);
            double lat2 = destination.Latitude.ToRadians().Value;
            //$long1 = deg2rad(-81.3667);
            double long1 = Longitude.ToRadians().Value;
            //$long2 = deg2rad(121.3667);
            double long2 = destination.Longitude.ToRadians().Value;
            //$dlat = abs($lat2 - $lat1);
            double dlat = Math.Abs(lat2 - lat1);
            //$dlong = abs($long2 - $long1);
            double dlong = Math.Abs(long2 - long1);
            //$l = ($lat1 + $lat2) / 2;
            double l = (lat1 + lat2) * 0.5;
            //$a = 6378;
            double a = ellipsoid.EquatorialRadius.ToKilometers().Value;
            //$b = 6357;
            double b = ellipsoid.PolarRadius.ToKilometers().Value;
            //$e = sqrt(1 - ($b * $b)/($a * $a));
            double e = Math.Sqrt(1 - (b * b) / (a * a));
            //$r1 = ($a * (1 - ($e * $e))) / pow((1 - ($e * $e) * (sin($l) * sin($l))), 3/2);
            double r1 = (a * (1 - (e * e))) / Math.Pow((1 - (e * e) * (Math.Sin(l) * Math.Sin(l))), 3 * 0.5);
            //$r2 = $a / sqrt(1 - ($e * $e) * (sin($l) * sin($l)));
            double r2 = a / Math.Sqrt(1 - (e * e) * (Math.Sin(l) * Math.Sin(l)));
            //$ravg = ($r1 * ($dlat / ($dlat + $dlong))) + ($r2 * ($dlong / ($dlat + $dlong)));
            double ravg = (r1 * (dlat / (dlat + dlong))) + (r2 * (dlong / (dlat + dlong)));
            //$sinlat = sin($dlat / 2);
            double sinlat = Math.Sin(dlat * 0.5);
            //$sinlon = sin($dlong / 2);
            double sinlon = Math.Sin(dlong * 0.5);
            //$a = pow($sinlat, 2) + cos($lat1) * cos($lat2) * pow($sinlon, 2);
            a = Math.Pow(sinlat, 2) + Math.Cos(lat1) * Math.Cos(lat2) * Math.Pow(sinlon, 2);
            //$c = 2 * asin(min(1, sqrt($a)));
            double c = 2 * Math.Asin(Math.Min(1, Math.Sqrt(a)));
            //$d = $ravg * $c; 
            double d = ravg * c;
            // If it's NaN, return zero
            if (double.IsNaN(d))
            {
                d = 0.0;
            }
            // Return a new distance
            return new Distance(d, DistanceUnit.Kilometers).ToLocalUnitType();
             **/

            #endregion
        }

		/// <overloads>Calculates the great circle distance between any two points on 
		/// Earth.</overloads>
		/// <summary>
		/// Returns the distance over land from the given starting point to the specified
		/// destination.
		/// </summary>
		/// <returns>
		/// A <strong>Distance</strong> object containing the calculated distance in
		/// kilometers.
		/// </returns>
		/// <param name="start">A beginning point from which to calculate distance.</param>
		/// <param name="destination">The ending point of a segment.</param>
		/// <remarks>
		/// This method uses trigonometry to calculate the Great Circle (over Earth's curved
		/// surface) distance between any two points on Earth. The distance is returned in
		/// kilometers but can be converted to any other unit type using methods in the
		/// <see cref="Distance">Distance</see>
		/// class.
		/// </remarks>
		//[CLSCompliant(false)]
		public static Distance DistanceTo(Position start, Position destination)
		{
			return start.DistanceTo(destination);
		}

		/// <summary>
		/// Returns the remaining travel distance if traveling for a certain speed for a certain period of time.
		/// </summary>
		/// <param name="destination">A <strong>Position</strong> marking the destination location.</param>
		/// <param name="speed">A <strong>Speed</strong> travelled from the current instance.</param>
		/// <param name="time">A <strong>TimeSpan</strong> representing the time already elapsed during transit to the destination.</param>
		/// <returns>A <strong>Distance</strong> measuring the remaining distance to travel.</returns>
		//[CLSCompliant(false)]
		public Distance DistanceTo(Position destination, Speed speed, TimeSpan time)
		{
			double TraveledDistance = speed.ToMetersPerSecond().Value * time.TotalSeconds;
			// Get the total distance (without travelling)
			double TotalDistance = DistanceTo(destination).ToMeters().Value;
			// Return the distance travelled
			return new Distance(TotalDistance - TraveledDistance, DistanceUnit.Meters).ToLocalUnitType();
		}

		/// <summary>
		/// Returns the remaining travel distance if traveling for a certain speed for a certain period of time.
		/// </summary>
		/// <param name="start">A <strong>Position</strong> marking the starting location from which to calculate.</param>
		/// <param name="destination">A <strong>Position</strong> marking the destination location.</param>
		/// <param name="speed">A <strong>Speed</strong> travelled from the current instance.</param>
		/// <param name="time">A <strong>TimeSpan</strong> representing the time already elapsed during transit to the destination.</param>
		/// <returns>A <strong>Distance</strong> measuring the remaining distance to travel.</returns>
		//[CLSCompliant(false)]
		public static Distance DistanceTo(Position start, Position destination, Speed speed, TimeSpan time)
		{
			return start.DistanceTo(destination, speed, time);
		}

		/// <summary>
		/// Calculates the intersection of two lines created by the current instance, another point, and a direction of travel from each point.
		/// </summary>
		/// <param name="firstBearing">An <strong>Angle</strong> specifying a travel direction from the current instance.</param>
		/// <param name="secondPosition">A <strong>Position</strong> specifying the start of the second line of intersection.</param>
		/// <param name="secondBearing">An <strong>Angle</strong> specifying a travel direction from the second position.</param>
		/// <returns>A <strong>Position</strong> representing the point of intersection, if one exists.</returns>
		/// <remarks>This method is typically used to determine the point where two objects in motion would meet.  </remarks>
        public Position IntersectionOf(Azimuth firstBearing, Position secondPosition, Azimuth secondBearing)
        {
            return IntersectionOf(firstBearing.DecimalDegrees, secondPosition, secondBearing.DecimalDegrees);
        }

        public Position IntersectionOf(Angle firstBearing, Position secondPosition, Angle secondBearing)
        {
            return IntersectionOf(firstBearing.DecimalDegrees, secondPosition, secondBearing.DecimalDegrees);
        }

        public Position IntersectionOf(double firstBearing, Position secondPosition, double secondBearing)
		{
			//Dim AdjustedDestination As Position = secondPosition.ToDatum(Datum)

			double tol = 0.000000000000001;
			//			  // in1 - line 1, point 1 latitude index
			//  // in2 - line 1, point 1 longitude index
			//  // in3 - line 1, bearing index
			//  // in4 - line 2, point 1 latitude index 
			//  // in5 - line 2, point 1 longitude index
			//  // in6 - line 2, bearing index
			//  // in7 - intersection point latitude index
			//  // in8 - intersection point longitude index

			//var j = form.units.selectedIndex;
			//var units = form.units.options[j].value;

			//double latddd1 = Latitude.DecimalDegrees;
			double latrad1 = pLatitude.ToRadians().Value;
			//double londdd1 = Longitude.DecimalDegrees;
			double lonrad1 = pLongitude.ToRadians().Value;

			//double latddd2 = secondPosition.Latitude.DecimalDegrees;
			double latrad2 = secondPosition.Latitude.ToRadians().Value;
			//double londdd2 = secondPosition.Longitude.DecimalDegrees;
			double lonrad2 = secondPosition.Longitude.ToRadians().Value;

			double crs13 = Radian.FromDegrees(firstBearing).Value;
			double crs23 = Radian.FromDegrees(secondBearing).Value;

			//  var latddd1 = dms2ddd(form.degreevalue[in1].value, form.minutevalue[in1].value, form.secondvalue[in1].value);  // convert latitude of point to decimal degrees then radians
			//  var latrad1 = deg2rad(latddd1);
			//  var londdd1 = dms2ddd(form.degreevalue[in2].value, form.minutevalue[in2].value, form.secondvalue[in2].value);  // likewise for longitude
			//  var lonrad1 = deg2rad(londdd1);
			//  var latddd2 = dms2ddd(form.degreevalue[in4].value, form.minutevalue[in4].value, form.secondvalue[in4].value);  // convert latitude of point to decimal degrees then radians
			//  var latrad2 = deg2rad(latddd2);
			//  var londdd2 = dms2ddd(form.degreevalue[in5].value, form.minutevalue[in5].value, form.secondvalue[in5].value);  // likewise for longitude
			//  var lonrad2 = deg2rad(londdd2);

			//  var crs13 = dms2ddd(form.degreevalue[in3].value, form.minutevalue[in3].value, form.secondvalue[in3].value);  //convert bearing line 1 to decimal degrees
			//  crs13 = deg2rad(crs13);			 //convert to radians
			//  var crs23 = dms2ddd(form.degreevalue[in6].value, form.minutevalue[in6].value, form.secondvalue[in6].value);  //convert bearing line 2 to decimal degrees
			//  crs23 = deg2rad(crs23);			 //convert to radians

			double w = lonrad2 - lonrad1;
			double v = latrad1 - latrad2;
			double s = 2 * Math.Asin(Math.Sqrt((Math.Sin(v* 0.5) * Math.Sin(v * 0.5)) + (Math.Cos(latrad1) * Math.Cos(latrad2) * Math.Sin(w * 0.5) * Math.Sin(w * 0.5)))); // //distance between start points

			//  var w = lonrad2 - lonrad1; 
			//  var v = latrad1 - latrad2;
			//  var s = 2 * Math.asin(Math.sqrt((Math.sin(v * 0.5) * Math.sin(v * 0.5)) + (Math.cos(latrad1) * Math.cos(latrad2) * Math.sin(w * 0.5) * Math.sin(w * 0.5))));	 //distance between start points

			double crs12 = 0;
			if (Math.Sin(lonrad1 - lonrad2) < 0)
			{
				crs12 = Math.Acos((Math.Sin(latrad2) - Math.Sin(latrad1) * Math.Cos(s)) / (Math.Sin(s) * Math.Cos(latrad1)));
			}
			else
			{
				crs12 = 2 * Math.PI - Math.Acos((Math.Sin(latrad2) - Math.Sin(latrad1) * Math.Cos(s)) / (Math.Sin(s) * Math.Cos(latrad1)));
			}

			//// calculate course 1 to 2
			//  if (Math.sin(lonrad1 - lonrad2) < 0){
			//	  var crs12 = Math.acos((Math.sin(latrad2) - Math.sin(latrad1) * Math.cos(s)) / (Math.sin(s) * Math.cos(latrad1)));
			//	}
			//   else {
			//	  var crs12 = 2 * pi - Math.acos((Math.sin(latrad2) - Math.sin(latrad1) * Math.cos(s)) / (Math.sin(s) * Math.cos(latrad1)));
			//	}

			double crs21 = 0;
			if (Math.Sin(lonrad2 - lonrad1) < 0)
			{
				crs21 = Math.Acos((Math.Sin(latrad1) - Math.Sin(latrad2) * Math.Cos(s)) / (Math.Sin(s) * Math.Cos(latrad2)));
			}
			else
			{
				crs21 = 2 * Math.PI - Math.Acos((Math.Sin(latrad1) - Math.Sin(latrad2) * Math.Cos(s)) / (Math.Sin(s) * Math.Cos(latrad2)));
			}

			//// calculate course 2 to 1
			//   if (Math.sin(lonrad2 - lonrad1) < 0){
			//	  var crs21 = Math.acos((Math.sin(latrad1) - Math.sin(latrad2) * Math.cos(s)) / (Math.sin(s) * Math.cos(latrad2)));
			//	}
			//   else {
			//	  var crs21 = 2 * pi - Math.acos((Math.sin(latrad1) - Math.sin(latrad2) * Math.cos(s)) / (Math.sin(s) * Math.cos(latrad2)));
			//	}

			double ang1 = (crs13 - crs12 + Math.PI) % (2 * Math.PI) - Math.PI;
			double ang2 = (crs21 - crs23 + Math.PI) % (2 * Math.PI) - Math.PI;

			// var ang1 = mod(crs13 - crs12 + pi, 2 * pi) - pi;
			// var ang2 = mod(crs21 - crs23 + pi, 2 * pi) - pi;

			if (Math.Sin(ang1) * Math.Sin(ang2) <= Math.Sqrt(tol))
			{
				// NO EXCEPTION IS THROWN.  RETURN NULL
				return Position.Empty;
				//throw new GeoException("No intersection exists between these two points and the given bearings.");
			}
			else
			{
				ang1 = Math.Abs(ang1);
				ang2 = Math.Abs(ang2);
				double ang3 = Math.Acos(Math.Sin(ang1) * Math.Sin(ang2) * Math.Cos(s) - Math.Cos(ang1) * Math.Cos(ang2));
				double dst13 = Math.Asin(Math.Sin(ang2) * Math.Sin(s) / Math.Sin(ang3));
				double latrad3 = Math.Asin(Math.Sin(latrad1) * Math.Cos(dst13) + Math.Cos(latrad1) * Math.Sin(dst13) * Math.Cos(crs13));
				double lonrad3 = lonrad1 + Math.Asin(Math.Sin(crs13) * Math.Sin(dst13) / Math.Cos(latrad3));
				lonrad3 = ((lonrad3 + Math.PI) % (2 * Math.PI)) - Math.PI;

				Latitude NewLatitude = new Latitude(latrad3 * 180 / Math.PI);
				Longitude NewLongitude = new Longitude(lonrad3 * 180 / Math.PI);
				// Return the new position
				return new Position(NewLatitude, NewLongitude);
			}

			// if (Math.sin(ang1) * Math.sin(ang2) <= Math.sqrt(tol)) {
			//		  alert('No Intersection Exists');
			//		  return true;
			//	 }
			//else {
			//var	ang1 = Math.abs(ang1);
			//var	ang2 = Math.abs(ang2);
			//var	ang3 = Math.acos(Math.sin(ang1) * Math.sin(ang2) * Math.cos(s) - Math.cos(ang1) * Math.cos(ang2));
			//var	dst13 = Math.asin(Math.sin(ang2) * Math.sin(s) / Math.sin(ang3));
			//var	latrad3 = Math.asin(Math.sin(latrad1) * Math.cos(dst13) + Math.cos(latrad1) * Math.sin(dst13) * Math.cos(crs13));
			//var	lonrad3 = lonrad1 + Math.asin(Math.sin(crs13) * Math.sin(dst13) / Math.cos(latrad3));
			//	   lonrad3 = mod(lonrad3 + pi, 2 * pi) - pi;
			//  ddd2dms(form, in8, rad2deg(lonrad3));
			//  ddd2dms(form, in7, rad2deg(latrad3));
			//}
		}

		/// <overloads>Calculates a position which marks the intersection of two 
		/// vectors.</overloads>
		/// <summary>Calculates the point (if any) at which two imaginary lines 
		/// intersect.</summary>
		/// <returns>A <strong>Position</strong> object specifying the intersection 
		/// point.</returns>
		/// <remarks>
		/// This method uses trigonometry to calculate the point at which two lines intersect
		/// on Earth's surface. This method is typically used to see where two objects in motion
		/// would meet given their current directions of travel.  This method does not take the speed 
		/// of each object into account.
		/// <img src="IntersectionOf.jpg"/>
		/// </remarks>
		/// <param name="firstPosition">A <strong>Position</strong> specifying a position which marks the start of a line.</param>
		/// <param name="firstBearing">An <strong>Angle</strong> specifying a direction from the first Position.</param>
		/// <param name="secondPosition">A <strong>Position</strong> specifying the second position, marking the start of a second line.</param>
		/// <param name="secondBearing">An <strong>Angle</strong> specifying a direction from the second Position.</param>
		public static Position IntersectionOf(Position firstPosition, Angle firstBearing, Position secondPosition, Angle secondBearing)
		{
			try
			{
				return firstPosition.IntersectionOf(firstBearing, secondPosition, secondBearing);
			}
			catch
			{
				throw;
			}
		}

        /// <summary>
        /// Calculates a position relative to the current instance based upon the given bearing and distance.
        /// </summary>
        /// <param name="bearing">An <strong>Angle</strong> object specifying a direction to shift.</param>
        /// <param name="distance">A <strong>Distance</strong> object specifying the distance to shift.</param>
        /// <returns></returns>
        public Position TranslateTo(Angle bearing, Distance distance)
        {
            return TranslateTo(bearing.DecimalDegrees, distance, Datum.Ellipsoid);
        }

        public Position TranslateTo(Angle bearing, Distance distance, Ellipsoid ellipsoid)
        {
            return TranslateTo(bearing.DecimalDegrees, distance, ellipsoid);
        }

        /// <summary>
        /// Calculates a position relative to the current instance based upon the given bearing and distance.
        /// </summary>
        /// <param name="bearing">An <strong>Azimuth</strong> object specifying a direction to shift.</param>
        /// <param name="distance">A <strong>Distance</strong> object specifying the distance to shift.</param>
        /// <returns>A <strong>Position</strong> representing the calculated position.</returns>
        /// <remarks>This function is designed to calculate positions for any location on Earth, with
        /// the exception of coordinates which lie at the poles (e.g. 90N or 90S).</remarks>
        public Position TranslateTo(Azimuth bearing, Distance distance)
        {
            return TranslateTo(bearing.DecimalDegrees, distance, Datum.Ellipsoid);
        }


        /// <summary>
        /// Calculates a position relative to the current instance based upon the given bearing and distance.
        /// </summary>
        /// <param name="bearing">An <strong>Azimuth</strong> object specifying a direction to shift.</param>
        /// <param name="distance">A <strong>Distance</strong> object specifying the distance to shift.</param>
        /// <returns>A <strong>Position</strong> representing the calculated position.</returns>
        /// <remarks>This function is designed to calculate positions for any location on Earth, with
        /// the exception of coordinates which lie at the poles (e.g. 90N or 90S).</remarks>
        public Position TranslateTo(Azimuth bearing, Distance distance, Ellipsoid ellipsoid)
        {
            return TranslateTo(bearing.DecimalDegrees, distance, ellipsoid);
        }

		/// <summary>
		/// Calculates a position relative to the current instance based upon the given bearing and distance.
		/// </summary>
		/// <param name="bearing">A <strong>Double</strong> specifying a direction to shift.</param>
		/// <param name="distance">A <strong>Distance</strong> object specifying the distance to shift.</param>
        /// <returns>A <strong>Position</strong> representing the calculated position.</returns>
        /// <remarks>This function is designed to calculate positions for any location on Earth, with
        /// the exception of coordinates which lie at the poles (e.g. 90N or 90S).</remarks>
        public Position TranslateTo(double bearing, Distance distance)
        {
            return TranslateTo(bearing, distance, Datum.Ellipsoid);
        }

		/// <summary>
		/// Calculates a position relative to the current instance based upon the given bearing and distance.
		/// </summary>
		/// <param name="bearing">A <strong>Double</strong> specifying a direction to shift.</param>
		/// <param name="distance">A <strong>Distance</strong> object specifying the distance to shift.</param>
		/// <returns>A <strong>Position</strong> representing the calculated position.</returns>
        /// <remarks>This function is designed to calculate positions for any location on Earth, with
        /// the exception of coordinates which lie at the poles (e.g. 90N or 90S).</remarks>
        public Position TranslateTo(double bearing, Distance distance, Ellipsoid ellipsoid)
        {
            // Taken from: http://www.koders.com/java/fid72A4E7D3DA8195D118CF926431263DDB8C45C5A1.aspx
            /*
             * Vincenty's Inverse Algorythm.
             * 
             * NOTE: MATLAB had no formula which took the ellipsoid into accout -- only a
             * formula which used Earth's average radius.  This is why the formulae have
             * two separate sources.  -- Jon
             * 
             *      
             *  
             * // Flattening
                    f  = (semiMajorAxis-semiMinorAxis) / semiMajorAxis;
             * 
             * // 1 - flattening
                    fo = 1.0 - f;
             * 
             * // Flattening squared
                    f2 = f*f;
             * // Flattening cubed
                    f3 = f*f2;
             * // Flattening ^ 4
                    f4 = f*f3;
                    eccentricitySquared = f * (2.0-f);

             *          * Solution of the geodetic direct problem after T.Vincenty.
                     * Modified Rainsford's method with Helmert's elliptical terms.
                     * Effective in any azimuth and at any distance short of antipodal.
                     *
                     * Latitudes and longitudes in radians positive North and East.
                     * Forward azimuths at both points returned in radians from North.
                     *
                     * Programmed for CDC-6600 by LCDR L.Pfeifer NGS ROCKVILLE MD 18FEB75
                     * Modified for IBM SYSTEM 360 by John G.Gergen NGS ROCKVILLE MD 7507
                     * Ported from Fortran to Java by Daniele Franzoni.
                     *
                     * Source: ftp://ftp.ngs.noaa.gov/pub/pcsoft/for_inv.3d/source/forward.for
                     *         subroutine DIRECT1
            // Protect internal variables from changes
                    final double lat1     = this.lat1;
                    final double long1    = this.long1;
                    final double azimuth  = this.azimuth;
                    final double distance = this.distance;
                    double TU  = fo*Math.sin(lat1) / Math.cos(lat1);
                    double SF  = Math.sin(azimuth);
                    double CF  = Math.cos(azimuth);
                    double BAZ = (CF!=0) ? Math.atan2(TU,CF)*2.0 : 0;
                    double CU  = 1/Math.sqrt(TU*TU + 1.0);
                    double SU  = TU*CU;
                    double SA  = CU*SF;
                    double C2A = 1.0 - SA*SA;
                    double X   = Math.sqrt((1.0/fo/fo-1)*C2A+1.0) + 1.0;
                    X   = (X-2.0)/X;
                    double C   = 1.0-X;
                    C   = (X*X/4.0+1.0)/C;
                    double D   = (0.375*X*X-1.0)*X;
                    TU   = distance / fo / semiMajorAxis / C;
                    double Y   = TU;
                    double SY, CY, CZ, E;
                    do {
                        SY = Math.sin(Y);
                        CY = Math.cos(Y);
                        CZ = Math.cos(BAZ+Y);
                        E  = CZ*CZ*2.0-1.0;
                        C  = Y;
                        X  = E*CY;
                        Y  = E+E-1.0;
                        Y  = (((SY*SY*4.0-3.0)*Y*CZ*D/6.0+X)*D/4.0-CZ)*SY*D+TU;
                    } while (Math.abs(Y-C) > TOLERANCE_1);
                    BAZ  = CU*CY*CF - SU*SY;
                    C    = fo*Math.sqrt(SA*SA+BAZ*BAZ);
                    D    = SU*CY + CU*SY*CF;
                    lat2 = Math.atan2(D,C);
                    C    = CU*CY-SU*SY*CF;
                    X    = Math.atan2(SY*SF,C);
                    C    = ((-3.0*C2A+4.0)*f+4.0)*C2A*f/16.0;
                    D    = ((E*CY*C+CZ)*SY*C+Y)*SA;
                    long2 = long1+X - (1.0-C)*D*f;
                    long2 = castToAngleRange(long2);
                    destinationValid = true;

             */

            double f = ellipsoid.Flattening;
            double fo = 1.0 - f;
            double semiMajorAxis = ellipsoid.SemiMajorAxis.ToMeters().Value;

            //final double lat1     = this.lat1;
            double lat1 = pLatitude.ToRadians().Value;

            //final double long1    = this.long1;
            double long1 = pLongitude.ToRadians().Value;                        

            //final double azimuth  = this.azimuth;
            double azimuth = bearing / 180.0 * Math.PI;

            //final double distance = this.distance;
            double dist = distance.ToMeters().Value;

            //double TU  = fo*Math.sin(lat1) / Math.cos(lat1);
            double TU = fo * Math.Sin(lat1) / Math.Cos(lat1);

            //double SF  = Math.sin(azimuth);
            double SF = Math.Sin(azimuth);

            //double CF  = Math.cos(azimuth);
            double CF = Math.Cos(azimuth);

            //double BAZ = (CF!=0) ? Math.atan2(TU,CF)*2.0 : 0;
            double BAZ = (CF != 0) ? Math.Atan2(TU, CF) * 2.0 : 0;

            //double CU  = 1/Math.sqrt(TU*TU + 1.0);
            double CU = 1.0 / Math.Sqrt(TU * TU + 1.0);

            //double SU  = TU*CU;
            double SU = TU * CU;

            //double SA  = CU*SF;
            double SA = CU * SF;

            //double C2A = 1.0 - SA*SA;
            double C2A = 1.0 - SA * SA;

            //double X   = Math.sqrt((1.0/fo/fo-1)*C2A+1.0) + 1.0;
            double X = Math.Sqrt((1.0 / fo / fo - 1) * C2A + 1.0) + 1.0;

            //X   = (X-2.0)/X;
            X = (X - 2.0) / X;

            //double C   = 1.0-X;
            double C = 1.0 - X;

            //C   = (X*X/4.0+1.0)/C;
            C = (X * X / 4.0 + 1.0) / C;

            //double D   = (0.375*X*X-1.0)*X;
            double D = (0.375 * X * X - 1.0) * X;

            //TU   = distance / fo / semiMajorAxis / C;
            TU = dist / fo / semiMajorAxis / C;

            //double Y   = TU;
            double Y = TU;

            //double SY, CY, CZ, E;
            double SY, CY, CZ, E;
            int Iterations = 0;

            //do {
            do {
            //    SY = Math.sin(Y);
                SY = Math.Sin(Y);
            //    CY = Math.cos(Y);
                CY = Math.Cos(Y);
            //    CZ = Math.cos(BAZ+Y);
                CZ = Math.Cos(BAZ+Y);
            //    E  = CZ*CZ*2.0-1.0;
                E  = CZ*CZ*2.0-1.0;
            //    C  = Y;
                C  = Y;
            //    X  = E*CY;
                X  = E*CY;
            //    Y  = E+E-1.0;
                Y  = E+E-1.0;
            //    Y  = (((SY*SY*4.0-3.0)*Y*CZ*D/6.0+X)*D/4.0-CZ)*SY*D+TU;
                Y  = (((SY*SY*4.0-3.0)*Y*CZ*D/6.0+X)*D/4.0-CZ)*SY*D+TU;
            //} while (Math.abs(Y-C) > TOLERANCE_1);
                Iterations++;

            } while (Iterations < 30 && Math.Abs(Y - C) > TargetAccuracy);
            
            //BAZ  = CU*CY*CF - SU*SY;
            BAZ = CU * CY * CF - SU * SY;

            //C    = fo*Math.sqrt(SA*SA+BAZ*BAZ);
            C = fo * Math.Sqrt(SA * SA + BAZ * BAZ);
            
            //D    = SU*CY + CU*SY*CF;
            D = SU * CY + CU * SY * CF;

            //lat2 = Math.atan2(D,C);
            double lat2 = Math.Atan2(D, C);

            //C    = CU*CY-SU*SY*CF;
            C = CU * CY - SU * SY * CF;

            //X    = Math.atan2(SY*SF,C);
            X = Math.Atan2(SY * SF, C);

            //C    = ((-3.0*C2A+4.0)*f+4.0)*C2A*f/16.0;
            C = ((-3.0 * C2A + 4.0) * f + 4.0) * C2A * f / 16.0;

            //D    = ((E*CY*C+CZ)*SY*C+Y)*SA;
            D = ((E * CY * C + CZ) * SY * C + Y) * SA;
            
            //long2 = long1+X - (1.0-C)*D*f;
            double long2 = long1 + X - (1.0 - C) * D * f;

            //long2 = castToAngleRange(long2);
            long2 = long2 - (2 * Math.PI) * Math.Floor(long2 / (2 * Math.PI) + 0.5);

            //destinationValid = true;

            // Return a new position, rounded to 10 digits of accuracy

            return new Position(
                Latitude.FromRadians(lat2).Round(10),
                Longitude.FromRadians(long2).Round(10));
        }

        //public Position TranslateTo(double bearing, Distance distance)
        //{
        //    try
        //    {
        //        //lat = asin(sin(lat1) * cos(d) + cos(lat1) * sin(d) * cos(tc))
        //        //dlon = atan2(sin(tc) * sin(d) * cos(lat1), cos(d) - sin(lat1) * sin(lat))
        //        //lon=mod(lon1-dlon +pi,2*pi)-pi
        //        double d = distance.ToMeters().Value / Distance.EarthsAverageRadius.Value; // .EarthsArithmeticMeanRadiusWgs84.Value
        //        double tc = Radian.FromDegrees(bearing).Value;
        //        double lat1 = Latitude.ToRadians().Value;
        //        double lon1 = Longitude.ToRadians().Value;
        //        double lat = Math.Asin(Math.Sin(lat1) * Math.Cos(d) + Math.Cos(lat1) * Math.Sin(d) * Math.Cos(tc));
        //        double dlon = Math.Atan2(Math.Sin(tc) * Math.Sin(d) * Math.Cos(lat1), Math.Cos(d) - Math.Sin(lat1) * Math.Sin(lat));
        //        double lon = ((lon1 + dlon + Math.PI) % (2 * Math.PI)) - Math.PI;
        //        // Return the new position
        //        return new Position(Latitude.FromRadians(lat).Normalize(), Longitude.FromRadians(lon).Normalize());
        //        //'  var latddd1 = dms2ddd(form.degreevalue[in1].value, form.minutevalue[in1].value, form.secondvalue[in1].value);  // convert latitude of point to decimal degrees then radians
        //        //Dim latddd1 As Double = Latitude.DecimalDegrees
        //        //'  var latrad1 = deg2rad(latddd1);
        //        //Dim latrad1 As Double = Latitude.ToRadians().Value
        //        //'  var londdd1 = dms2ddd(form.degreevalue[in2].value, form.minutevalue[in2].value, form.secondvalue[in2].value);  // likewise for longitude
        //        //Dim londdd1 As Double = Longitude.DecimalDegrees
        //        //'  var lonrad1 = deg2rad(londdd1);
        //        //Dim lonrad1 As Double = Longitude.ToRadians().Value
        //        //'  var x12 = dms2ddd(form.degreevalue[in6].value, form.minutevalue[in6].value, form.secondvalue[in6].value);  //convert to decimal degrees
        //        //Dim x12 As Double = bearing.DecimalDegrees
        //        //'  x12 = deg2rad(x12);			 //convert to radians
        //        //x12 = x12 * Math.PI / 180
        //        //'  var s = (parseFloat(form.distvalue[in5].value) * units);  //in meters
        //        //Dim s As Double = distance.ToMeters().Value
        //        //'  s = s * pi /(180 * 60 * 1852);  //convert to radians 
        //        //s = s * Math.PI / (180 * 60 * 1852)
        //        //'  var latrad2 = Math.asin(Math.sin(latrad1) * Math.cos(s) + Math.cos(latrad1) * Math.sin(s) * Math.cos(x12));
        //        //Dim latrad2 As Double = Math.Asin(Math.Sin(latrad1) * Math.Cos(s) + Math.Cos(latrad1) * Math.Sin(s) * Math.Cos(x12))
        //        //'  var w = Math.atan2(Math.sin(x12) * Math.sin(s) * Math.cos(latrad1), Math.cos(s) - Math.sin(latrad1) * Math.sin(latrad2));
        //        //Dim w As Double = Math.Atan2(Math.Sin(x12) * Math.Sin(s) * Math.Cos(latrad1), Math.Cos(s) - Math.Sin(latrad1) * Math.Sin(latrad2))
        //        //' Now calculate the position
        //        //'  var lonrad2 = modlon(lonrad1 + w);
        //        //Dim lonrad2 As Double = lonrad1 + w
        //        //'  latrad2 = modlat(latrad2);
        //        //'latrad2 = latrad2
        //        //'  ddd2dms(form, in4, rad2deg(lonrad2));
        //        //Dim NewLatitude As New Latitude(latrad2 * 180 / Math.PI)
        //        //'  ddd2dms(form, in3, rad2deg(latrad2));
        //        //Dim NewLongitude As New Longitude(lonrad2 * 180 / Math.PI)
        //        //' Return the new position
        //        //Return New Position(NewLatitude, NewLongitude)
        //    }
        //    catch
        //    {
        //        throw;
        //    }
        //    //  // in1 - point 1 latitude index
        //    //  // in2 - point 1 longitude index
        //    //  // in3 - point 2 latitude index 
        //    //  // in4 - point 2 longitude index
        //    //  // in5 - distance index
        //    //  // in6 - bearing index
        //    //var j = form.units.selectedIndex;
        //    //var units = form.units.options[j].value;
        //}

		/// <overloads>
		/// Returns the position shifted by the specified bearing and distance as new
		/// Position object.
		/// </overloads>
		/// <summary>Returns a new instance shifted by the specified direction and 
		/// distance.</summary>
		/// <returns>A new <strong>Position</strong> object adjusted by the specified 
		/// amount.</returns>
		/// <remarks>
		/// 	<para>This method is typically used to create an destination point relative to an
		///  existing location. For example, this method could be used to create a point ten
		///  miles northeast of the current location.</para>
		/// 	<para><em>NOTE: The trigonometric formula used for this method is subject to errors
		///  when the distance to translate falls below a quarter mile (approximately 433
		///  meters).</em></para>
		/// </remarks>
		/// <example>
		/// 	<code lang="VB" title="[New Example]" description="This example creates a destination point ten miles northwest of the 
		///                current location.">
		/// ' Create a distance of ten miles
		/// Dim TravelDistance As New Distance(10, DistanceUnit.StatuteMiles)
		/// ' Calculate the point
		/// Dim DestinationPoint As Position
		/// DestinationPoint = Position.CurrentPosition.TranslateTo(Azimuth.Northwest, 
		/// TravelDistance)
		///     </code>
		/// </example>
		//[CLSCompliant(false)]
		public static Position TranslateTo(Position start, Angle bearing, Distance distance)
		{
			try
			{
				return start.TranslateTo(bearing, distance);
			}
			catch
			{
				throw;
			}
		}

        #if !PocketPC && MOLODENSKYSUPPORT
		public Position TranslateTo(MolodenskyTransformType sourceTransformType, MolodenskyTransformType targetTransformType)
		{
			return TranslateTo(Geodesy.GetMolodenskyTransform(sourceTransformType), Geodesy.GetMolodenskyTransform(targetTransformType));
		}

		public static Position TranslateTo(Position value, MolodenskyTransformType sourceTransformType, MolodenskyTransformType targetTransformType)
		{
			return value.TranslateTo(Geodesy.GetMolodenskyTransform(sourceTransformType), Geodesy.GetMolodenskyTransform(targetTransformType));
		}

		public static Position TranslateTo(Position value, MolodenskyTransform sourceTransform, MolodenskyTransform targetTransform)
		{
			return value.TranslateTo(sourceTransform, targetTransform);
		}

		public Position TranslateTo(MolodenskyTransform sourceTransform, MolodenskyTransform targetTransform)
		{
			// Declare required variables
			try
			{
				//Dim NewDatum As Datum = Geodesy.GetDatum(targetTransform.Datum.Ellipsoid.Type)
				// Convert to the target system
				double from_h = 0; //SourcePosition.Altitude.Value
				double from_a = sourceTransform.Datum.Ellipsoid.SemiMajorAxis.Value;
				double from_f = sourceTransform.Datum.Ellipsoid.Flattening;
				double from_esq = 2 * from_f - from_f * from_f; //Math.Pow(SourcePosition.Datum.Ellipsoid.Datum.Eccentricity, 2)
				double da = targetTransform.Datum.Ellipsoid.SemiMajorAxis.Value - from_a;
				double df = targetTransform.Datum.Ellipsoid.Flattening - from_f;

				double dx = -(targetTransform.DX - sourceTransform.DX);
				double dy = -(targetTransform.DY - sourceTransform.DY);
				double dz = -(targetTransform.DZ - sourceTransform.DZ);
				// Perform the calculation!
				double slat = Math.Sin(Latitude.ToRadians().Value);
				double clat = Math.Cos(Latitude.ToRadians().Value);

				double slon = Math.Sin(Longitude.ToRadians().Value);
				double clon = Math.Cos(Longitude.ToRadians().Value);

				double ssqlat = slat * slat;
				double adb = 1.0 / (1.0 - from_f); // "a divided by b"

				double rn = from_a / Math.Sqrt(1.0 - from_esq * ssqlat);
				double rm = from_a * (1.0 - from_esq) / Math.Pow((1.0 - from_esq * ssqlat), 1.5);
				double dlat = (((((-dx * slat * clon - dy * slat * slon) + dz * clat) + (da * ((rn * from_esq * slat * clat) / from_a))) + (df * (rm * adb + rn / adb) * slat * clat))) / (rm + from_h);
				double dlon = (-dx * slon + dy * clon) / ((rn + from_h) * clat);
				//Dim dh As Double = (dx * clat * clon) + (dy * clat * slon) + (dz * slat) _
				//- (da * (from_a / rn)) + ((df * rn * ssqlat) / adb)

				dlat += Latitude.ToRadians().Value;
				dlon += Longitude.ToRadians().Value;

				double NewLatitude = Radian.ToAngle(dlat).DecimalDegrees;
				double NewLongitude = Radian.ToAngle(dlon).DecimalDegrees;

				// Return the adjusted position
				return new Position(new Latitude(NewLatitude), new Longitude(NewLongitude));
			}
			catch
			{
				throw;
			}

			//http://home.hiwaay.net/.taylorc/bookshelf/math-science/geodesy/datum/transform/molodensky/
			///*
			//	* transform
			//	*
			//	* Parameters:
			//	*	 from:	 The geodetic position to be translated.
			//	*	 from_a:   The semi-major axis of the "from" Datum.
			//	*	 from_f:   Flattening of the "from" Datum.
			//	*	 from_esq: Eccentricity-squared of the "from" Datum.
			//	*	 da:	   Change in semi-major axis length (meters); "to" minus "from"	
			//		 *	 df:	   Change in flattening; "to" minus "from"
			//	*	 dx:	   Change in x between "from" and "to" datum.
			//	*	 dy:	   Change in y between "from" and "to" datum.
			//	*	 dz:	   Change in z between "from" and "to" datum.
			//	*/
			//	public GeodeticPosition transform (GeodeticPosition from,
			//									   double from_a, double from_f,

			//									   double from_esq, double da, double df,
			//									   double dx, double dy, double dz)
			//	{
			//		double slat = Math.sin (from.lat);
			//		double clat = Math.cos (from.lat);
			//		double slon = Math.sin (from.lon);
			//		double clon = Math.cos (from.lon);
			//		double ssqlat = slat * slat;
			//		double adb = 1.0 / (1.0 - from_f);  // "a divided by b"
			//		double dlat, dlon, dh;

			//		double rn = from_a / Math.sqrt (1.0 - from_esq * ssqlat);
			//		double rm = from_a * (1. - from_esq) / Math.pow ((1.0 - from_esq * ssqlat), 1.5);

			//		dlat = (((((-dx * slat * clon - dy * slat * slon) + dz * clat)
			//					+ (da * ((rn * from_esq * slat * clat) / from_a)))
			//				+ (df * (rm * adb + rn / adb) * slat * clat)))
			//			/ (rm + from.h);

			//		dlon = (-dx * slon + dy * clon) / ((rn + from.h) * clat);

			//		dh = (dx * clat * clon) + (dy * clat * slon) + (dz * slat)
			//			 - (da * (from_a / rn)) + ((df * rn * ssqlat) / adb);

			//		return new GeodeticPosition (from.lon + dlon, from.lat + dlat, from.h + dh);
			//	}


			// SECOND METHOD:  From GPSBabel  Jeeps.c

			//Dim dx As Double = -(TargetDatum.DX - SourceDatum.DX)
			//Dim dy As Double = -(TargetDatum.DY - SourceDatum.DY)
			//Dim dz As Double = -(TargetDatum.DZ - SourceDatum.DZ)

			//Dim Sphi As Double = Latitude.DecimalDegrees
			//Dim Slam As Double = Longitude.DecimalDegrees
			//Dim SH As Double = 0				' Height
			//Dim Sa As Double = Sourcedatum.Ellipsoid.Ellipsoid.SemiMajorAxis.ToMeters().Value
			//Dim Sif As Double = Sourcedatum.Ellipsoid.Ellipsoid.InverseFlattening

			//Dim Dphi1 As Double
			//Dim Dlam As Double
			//Dim Da1 As Double = Targetdatum.Ellipsoid.Ellipsoid.SemiMajorAxis.ToMeters().Value
			//Dim Dif As Double = Targetdatum.Ellipsoid.Ellipsoid.InverseFlattening

			//'Sf = (double)1.0 / Sif;
			//Dim Sf As Double
			//'   double Df;
			//Dim Df1 As Double
			//'   double esq;
			//Dim esq As Double
			//'   double bda;
			//Dim bda As Double
			//'   double da;
			//Dim da2 As Double
			//'   double df;
			//Dim df2 As Double
			//'   double N;
			//Dim N As Double
			//'   double M;
			//Dim M As Double
			//'   double tmp;
			//Dim tmp As Double
			//'   double tmp2;
			//Dim tmp2 As Double
			//'   double dphi;
			//Dim dphi2 As Double
			//'   double dlambda;
			//Dim dlambda As Double
			//'   double dheight;
			//Dim dheight As Double
			//'   double phis;
			//Dim phis As Double
			//'   double phic;
			//Dim phic As Double
			//'   double lams;
			//Dim lams As Double
			//'   double lamc;
			//Dim lamc As Double

			//'   Sf = (double)1.0 / Sif;
			//Sf = 1.0 / Sif
			//'   Df = (double)1.0 / Dif;
			//Df1 = 1.0 / Dif

			//'   esq = (double)2.0*Sf - pow(Sf,(double)2.0);
			//esq = 2.0 * Sf - Math.Pow(Sf, 2.0)
			//'   bda = (double)1.0 - Sf;
			//bda = 1.0 - Sf
			//'   Sphi = GPS_Math_Deg_To_Rad(Sphi);
			//Sphi = Latitude.ToRadians().Value
			//'   Slam = GPS_Math_Deg_To_Rad(Slam);
			//Sphi = Longitude.ToRadians().Value

			//'   da = Da - Sa;
			//da2 = Da1 - Sa
			//'   df = Df - Sf;
			//df2 = Df1 - Sf

			//'   phis = sin(Sphi);
			//phis = Math.Sin(Sphi)
			//'   phic = cos(Sphi);
			//phic = Math.Cos(Sphi)
			//'   lams = sin(Slam);
			//lams = Math.Sin(Slam)
			//'   lamc = cos(Slam);
			//lamc = Math.Cos(Slam)

			//'   N = Sa /  sqrt((double)1.0 - esq*pow(phis,(double)2.0));
			//N = Sa / Math.Sqrt(1.0 - esq * Math.Pow(phis, 2.0))

			//'   tmp = ((double)1.0-esq) /pow(((double)1.0-esq*pow(phis,(double)2.0)),1.5);
			//tmp = (1.0 - esq) / Math.Pow((1.0 - esq * Math.Pow(phis, 2.0)), 1.5)
			//'   M   = Sa * tmp;
			//M = Sa * tmp

			//'   tmp  = df * ((M/bda)+N*bda) * phis * phic;
			//tmp = df2 * ((M / bda) + N * bda) * phis * phic
			//'   tmp2 = da * N * esq * phis * phic / Sa;
			//tmp2 = da2 * N * esq * phis * phic / Sa
			//'   tmp2 += ((-dx*phis*lamc-dy*phis*lams) + dz*phic);
			//tmp2 += ((-dx * phis * lamc - dy * phis * lams) + dz * phic)
			//'   dphi = (tmp2 + tmp) / (M + SH);
			//dphi2 = (tmp2 + tmp) / (M + SH)

			//'   dlambda = (-dx*lams+dy*lamc) / ((N+SH)*phic);
			//dlambda = (-dx * lams + dy * lamc) / ((N + SH) * phic)

			//'   dheight = dx*phic*lamc + dy*phic*lams + dz*phis - da*(Sa/N) +
			//'df*bda*N*phis*phis;

			//'   *Dphi = Sphi + dphi;
			//Dphi1 = Sphi + dphi2
			//'   *Dlam = Slam + dlambda;
			//Dlam = Slam + dlambda
			//'   *DH   = SH   + dheight;


			//'   *Dphi = GPS_Math_Rad_To_Deg(*Dphi);
			//Dphi1 = Dphi1 / Math.PI * 180
			//'   *Dlam = GPS_Math_Rad_To_Deg(*Dlam);
			//Dlam = Dlam / Math.PI * 180

			//Return New Position(Dphi1, Dlam)
			//'   return;
		}
        #endif
//
//        public DateTime Sunrise()
//        {
//            return Sunrise(DateTime.Now);
//        }
//
//        public DateTime Sunrise(DateTime day)
//        {
//            day.j
//            var t = calcTimeJulianCent(JD);
//
//            // *** First pass to approximate sunrise
//
//            var eqTime = calcEquationOfTime(t);
//            var solarDec = calcSunDeclination(t);
//            var hourAngle = calcHourAngleSunrise(latitude, solarDec);
//
//            var delta = longitude - radToDeg(hourAngle);
//            var timeDiff = 4 * delta;	// in minutes of time
//            var timeUTC = 720 + timeDiff - eqTime;	// in minutes
//
//            // alert("eqTime = " + eqTime + "\nsolarDec = " + solarDec + "\ntimeUTC = " + timeUTC);
//
//            // *** Second pass includes fractional jday in gamma calc
//
//            var newt = calcTimeJulianCent(calcJDFromJulianCent(t) + timeUTC / 1440.0);
//            eqTime = calcEquationOfTime(newt);
//            solarDec = calcSunDeclination(newt);
//            hourAngle = calcHourAngleSunrise(latitude, solarDec);
//            delta = longitude - radToDeg(hourAngle);
//            timeDiff = 4 * delta;
//            timeUTC = 720 + timeDiff - eqTime; // in minutes
//
//            // alert("eqTime = " + eqTime + "\nsolarDec = " + solarDec + "\ntimeUTC = " + timeUTC);
//
//            return timeUTC;
//        }
//
//        public DateTime Sunset()
//        {
//            return Sunset(DateTime.Now);
//        }
//
//        public DateTime Sunrise(DateTime day)
//        {
//        }

		/// <summary>Returns the latitude expressed as UTM northing.</summary>
		//[CLSCompliant(false)]
		public Distance Northing
		{
			get
			{
				// Return the easting
                if(pNorthing.IsEmpty)
                    CalculateNorthingEasting();
				return pNorthing;
			}
        }

		/// <summary>Returns the longitude expressed as UTM northing.</summary>
		//[CLSCompliant(false)]
		public Distance Easting
		{
			get
			{
                // Perform calculation JIT style
                if (pEasting.IsEmpty)
                    CalculateNorthingEasting();
				// Return the easting
				return pEasting;
			}
        }

		public int ZoneNumber
		{
			get
			{
				// Return the easting
				return pLongitude.UtmZoneNumber;
			}
        }

		public string Zone
		{
			get
			{
				// Return the easting
                return ZoneNumber.ToString(CultureInfo.InvariantCulture) + ZoneLetter.ToString();
			}
//			set
//			{
//				// Force the value to upper-case
//				value = value.ToUpper(CultureInfo.InvariantCulture).Trim();
//				// Blank
//				if (value.Length == 0)
//				{
//					throw new ArgumentNullException("Zone", "The Zone of a UTM Position cannot be a blank value.");
//				}
//				// Parse out the zone number and character
//				int Count = 0;
//				bool IsZoneLetterIdentified = false;
//				string NewZoneLetter = "";
//				System.Text.StringBuilder ZoneNumberStringBuilder = new System.Text.StringBuilder();
//				int NewZoneNumber = 0;
//				// OK --- INSTANT C# NOTE: The ending condition of VB 'For' loops is tested only on entry to the loop. Instant C# has created a temporary variable in order to use the initial value of Value.Length - 1 for every iteration:
//				for (Count = 0; Count <= value.Length - 1; Count++)
//				{
//					if (char.IsLetter(value[Count]))
//					{
//						if (IsZoneLetterIdentified)
//						{
//							throw new FormatException("Duplicate zone identifiers are not allowed.");
//						}
//						else
//						{
//							IsZoneLetterIdentified = true;
//							NewZoneLetter = value.Substring(Count, 1).ToUpper(CultureInfo.InvariantCulture);
//							//ZoneNumberString = ZoneNumberString & NewZoneLetter
//						}
//					}
//					else if (char.IsNumber(value, Count))
//					{
//						ZoneNumberStringBuilder.Append(value.Substring(Count, 1));
//					}
//					else if(value == "0?")
//					{
//						// No zone at all!
//						ZoneNumberStringBuilder.Append("0");
//					}
//					else
//					{
//						throw new ArgumentException("Invalid UTM zone characters were encountered.  Only one character is allowed for a zone letter.");
//					}
//				}
//				// Set the zone number
//				string ZoneNumberString = ZoneNumberStringBuilder.ToString();
//				if (ZoneNumberString.Length == 0 & NewZoneLetter.Length == 0)
//				{
//					throw new ArgumentException("No zone number could be extracted during parsing.  A UtmPosition cannot be created.");
//				}
//				else
//				{
//					NewZoneNumber = Convert.ToInt32(ZoneNumberString, CultureInfo.InvariantCulture);
//				}
//				// Commit the final values
//				switch (NewZoneLetter)
//				{
//					case "C":
//						pZoneLetter = ZoneLetter.C;
//						break;
//					case "D":
//						pZoneLetter = ZoneLetter.D;
//						break;
//					case "E":
//						pZoneLetter = ZoneLetter.E;
//						break;
//					case "F":
//						pZoneLetter = ZoneLetter.F;
//						break;
//					case "G":
//						pZoneLetter = ZoneLetter.G;
//						break;
//					case "H":
//						pZoneLetter = ZoneLetter.H;
//						break;
//					case "J":
//						pZoneLetter = ZoneLetter.J;
//						break;
//					case "K":
//						pZoneLetter = ZoneLetter.K;
//						break;
//					case "L":
//						pZoneLetter = ZoneLetter.L;
//						break;
//					case "M":
//						pZoneLetter = ZoneLetter.M;
//						break;
//					case "N":
//						pZoneLetter = ZoneLetter.N;
//						break;
//					case "P":
//						pZoneLetter = ZoneLetter.P;
//						break;
//					case "Q":
//						pZoneLetter = ZoneLetter.Q;
//						break;
//					case "R":
//						pZoneLetter = ZoneLetter.R;
//						break;
//					case "S":
//						pZoneLetter = ZoneLetter.S;
//						break;
//					case "T":
//						pZoneLetter = ZoneLetter.T;
//						break;
//					case "U":
//						pZoneLetter = ZoneLetter.U;
//						break;
//					case "V":
//						pZoneLetter = ZoneLetter.V;
//						break;
//					case "W":
//						pZoneLetter = ZoneLetter.W;
//						break;
//					case "X":
//						pZoneLetter = ZoneLetter.X;
//						break;
//				}
//				//pZoneLetter = CType(System.Enum.Parse(ZoneLetter.GetType, NewZoneLetter), UtmZone)
//				pZoneNumber = NewZoneNumber;
//				pZone = pZoneNumber + NewZoneLetter;
//				OnZoneChanged();
//                // Recalculate latitude and longitude
//                SetLatitudeLongitudeCoordinate();
//            }
		}

		public ZoneLetter ZoneLetter
		{
			get
			{
                // Return the easting
                return pLatitude.UtmZoneLetter;
			}
        }

//		/// <overloads>Compares the equivalence of the current instance with the specified 
//		/// value.</overloads>
//		public bool Equals(Angle latitude, Angle longitude)
//		{
//			return (Latitude.Equals(latitude, DefaultPrecisionDigits) 
//				&& Longitude.Equals(longitude, DefaultPrecisionDigits));
//		}

//		/// <summary>
//		/// Compares the current instance to the specified position using the specified numeric precision.
//		/// </summary>
//		/// <param name="latitude">A <strong>Latitude</strong> object to compare with.</param>
//		/// <param name="longitude">A <strong>Longitude</strong> object to compare with.</param>
//		/// <param name="precision">An <strong>Integer</strong> specifying the number of fractional digits to round to before comparison.</param>
//		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the values are identical.</returns>
//		/// <remarks>The two objects are compared at up to four digits of precision.</remarks>
//		public bool Equals(Angle latitude, Angle longitude, int decimals)
//		{
//			return (Latitude.Equals(latitude, decimals) 
//				&& Longitude.Equals(longitude, decimals));
//		}

		/// <summary>
		/// Compares the current instance to the specified position.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> object to compare with.</param>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the values are identical.</returns>
		/// <remarks>The two objects are compared at up to four digits of precision.</remarks>
		public bool Equals(Position value)
		{
			if(value == null) return false;
			return pLatitude.Equals(value.Latitude)
				&& pLongitude.Equals(value.Longitude);
		}

		/// <summary>
		/// Compares the current instance to the specified position using the specified numeric precision.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> object to compare with.</param>
		/// <param name="decimals">An <strong>Integer</strong> specifying the number of fractional digits to compare.</param>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the values are identical.</returns>
		/// <remarks>This method is typically used when positions do not mark the same location unless they are
		/// extremely close to one another.  Conversely, a low or even negative value for <strong>Precision</strong>
		/// allows positions to be considered equal even when they do not precisely match.</remarks>
		public bool Equals(Position value, int decimals)
		{
			// Compare latitude and longitude
			if(value == null) return false;
			return (pLatitude.Equals(value.Latitude, decimals)
                && pLongitude.Equals(value.Longitude, decimals));
		}

		public override bool Equals(object obj)
        {
#if !PocketPC || Framework20
            // Always return false if obj is null
			if(Object.ReferenceEquals(obj, null))
				return false;
#endif
            Position PositionValue = obj as Position;
            if (PositionValue != null)
                return Equals(PositionValue);

            return base.Equals(obj);
		}

		public override int GetHashCode()
		{
			return Latitude.GetHashCode() ^ Longitude.GetHashCode();
		}

		/// <summary>
		/// Subtracts the specified latitude and longitude from the current latitude and longitude.
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public Position Subtract(Position position)
		{
            return new Position(pLatitude.Subtract(position.Latitude.DecimalDegrees),
                pLongitude.Subtract(position.Longitude.DecimalDegrees));
		}

		/// <summary>
		/// Adds the specified latitude and longitude from the current latitude and longitude.
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public Position Add(Position position)
		{
            return new Position(pLatitude.Add(position.Latitude.DecimalDegrees),
                pLongitude.Add(position.Longitude.DecimalDegrees));
		}

		/// <summary>
		/// Multiplies the specified latitude and longitude from the current latitude and longitude.
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public Position Multiply(Position position)
		{
            return new Position(pLatitude.Multiply(position.Latitude.DecimalDegrees),
                pLongitude.Multiply(position.Longitude.DecimalDegrees));
		}

		public Position Normalize()
		{
			return new Position(pLatitude.Normalize(), pLongitude.Normalize(), pDatum);
		}

		/// <summary>
		/// Divides the specified latitude and longitude from the current latitude and longitude.
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public Position Divide(Position position)
		{
            return new Position(pLatitude.Divide(position.Latitude.DecimalDegrees), pLongitude.Divide(position.Longitude.DecimalDegrees));
		}

		public Position Clone()
		{
			return new Position(pLongitude, pLatitude);
		}

        private void CalculateNorthingEasting()
        {
            //double a = Datum[Datum].EquatorialRadius;
            double a1 = Datum.Ellipsoid.EquatorialRadius.Value;
            //double eccSquared = Datum[Datum].eccentricitySquared;
            double eccSquared = Datum.Ellipsoid.EccentricitySquared;
            //double k0 = 0.9996
            const double k0 = 0.9996;
            //double LongitudeOrigin;
            Longitude LongitudeOrigin; // = new Longitude();
            //double eccPrimeSquared;
            double eccPrimeSquared = 0;
            //double N, T, C, A, M;
            double N = 0;
            double T = 0;
            double C = 0;
            double A2 = 0;
            double M = 0;
            ////Make sure the longitude is between -180.00 .. 179.9
            //double LongitudeTemp = (Longitude+180)-int((Longitude+180)/360)*360-180; // -180.00 .. 179.9;
//#if Framework20 && !PocketPC
//            double LongitudeTemp = (Longitude.DecimalDegrees + 180) - (int)Math.Truncate((Longitude.DecimalDegrees + 180) / 360) * 360 - 180;
//#else
//            double LongitudeTemp = (Longitude.DecimalDegrees + 180) - Angle.Truncate((Longitude.DecimalDegrees + 180) / 360) * 360 - 180;
//#endif
            //double LatRad = Lat*deg2rad;
            double LatRad = Latitude.ToRadians().Value;
            //double LongitudeRad = LongitudeTemp*deg2rad;
            double LongitudeRad = Longitude.ToRadians().Value;
            //double LongitudeOriginRad;
            double LongitudeOriginRad = 0;

            //LongitudeOrigin = (ZoneNumber - 1)*6 - 180 + 3;  //+3 puts origin in middle of zone
            LongitudeOrigin = new Longitude((ZoneNumber - 1) * 6 - 180 + 3); // +3 puts origin in middle of zone
            //LongitudeOriginRad = LongitudeOrigin * deg2rad;
            LongitudeOriginRad = LongitudeOrigin.ToRadians().Value;

            // Compute the UTM Zone from the latitude and Longitudeitude
            //sprintf(UTMZone, "%d%c", ZoneNumber, UTMLetterDesignator(Lat));
            //Debug.WriteLine(ZoneNumber & UtmLetterDesignator(Latitude.DecimalDegrees))

            //eccPrimeSquared = (eccSquared)/(1-eccSquared);
            eccPrimeSquared = (eccSquared) / (1 - eccSquared);

            //N = a/sqrt(1-eccSquared*sin(LatRad)*sin(LatRad));
            N = a1 / Math.Sqrt(1 - eccSquared * Math.Sin(LatRad) * Math.Sin(LatRad));
            //T = tan(LatRad)*tan(LatRad);
            T = Math.Tan(LatRad) * Math.Tan(LatRad);
            //C = eccPrimeSquared*cos(LatRad)*cos(LatRad);
            C = eccPrimeSquared * Math.Cos(LatRad) * Math.Cos(LatRad);
            //A = cos(LatRad)*(LongitudeRad-LongitudeOriginRad);
            A2 = Math.Cos(LatRad) * (LongitudeRad - LongitudeOriginRad);


            //M = a*((1	- eccSquared/4		- 3*eccSquared*eccSquared/64	- 5*eccSquared*eccSquared*eccSquared/256)*LatRad 
            //			- (3*eccSquared/8	+ 3*eccSquared*eccSquared/32	+ 45*eccSquared*eccSquared*eccSquared/1024)*sin(2*LatRad)
            //								+ (15*eccSquared*eccSquared/256 + 45*eccSquared*eccSquared*eccSquared/1024)*sin(4*LatRad) 
            //								- (35*eccSquared*eccSquared*eccSquared/3072)*sin(6*LatRad));
            M = a1 * ((1 - eccSquared / 4 - 3 * eccSquared * eccSquared / 64 - 5 * eccSquared * eccSquared * eccSquared / 256) * LatRad - (3 * eccSquared / 8 + 3 * eccSquared * eccSquared / 32 + 45 * eccSquared * eccSquared * eccSquared / 1024) * Math.Sin(2 * LatRad) + (15 * eccSquared * eccSquared / 256 + 45 * eccSquared * eccSquared * eccSquared / 1024) * Math.Sin(4 * LatRad) - (35 * eccSquared * eccSquared * eccSquared / 3072) * Math.Sin(6 * LatRad));

            //UTMEasting = (double)(k0*N*(A+(1-T+C)*A*A*A/6
            //				+ (5-18*T+T*T+72*C-58*eccPrimeSquared)*A*A*A*A*A/120)
            //				+ 500000.0);

            double UTMEasting = (k0 * N * (A2 + (1 - T + C) * A2 * A2 * A2 / 6 + (5 - 18 * T + T * T + 72 * C - 58 * eccPrimeSquared) * A2 * A2 * A2 * A2 * A2 / 120) + 500000.0);

            //UTMNorthing = (double)(k0*(M+N*tan(LatRad)*(A*A/2+(5-T+9*C+4*C*C)*A*A*A*A/24
            //			 + (61-58*T+T*T+600*C-330*eccPrimeSquared)*A*A*A*A*A*A/720)));
            double UTMNorthing = (k0 * (M + N * Math.Tan(LatRad) * (A2 * A2 * 0.5 + (5 - T + 9 * C + 4 * C * C) * A2 * A2 * A2 * A2 / 24 + (61 - 58 * T + T * T + 600 * C - 330 * eccPrimeSquared) * A2 * A2 * A2 * A2 * A2 * A2 / 720)));

            // Adjust for southern hemisphere
            //If (Lat < 0) Then
            //UTMNorthing += 10000000.0; //10000000 meter offset for southern hemisphere

            if (Latitude.DecimalDegrees < 0)
            {
                UTMNorthing += 10000000.0; // 10000000 meter offset for southern hemisphere
            }

            // Update the northing/easting values
            pNorthing = new Distance(UTMNorthing, DistanceUnit.Meters);
            pEasting = new Distance(UTMEasting, DistanceUnit.Meters);

            //pZoneNumber = ZoneNumber;
            //pZoneLetter = Latitude.UtmZoneLetter;
            //string pZone = pZoneNumber + pZoneLetter.ToString();
            // Set the new values
            //pNorthing 
            //return new UtmPosition(pZone, pEasting, pNorthing);
			//NeedUtmCoordinate = false;
        }

        // Converts UTM information to latitude/longitude
        private void CalculateLatitudeLongitude(ZoneLetter zoneLetter, int zoneNumber)
        {
            try
            {
                //converts UTM coords to lat/long.  Equations from USGS Bulletin 1532 
                //East Longitudes are positive, West longitudes are negative. 
                //North latitudes are positive, South latitudes are negative
                //Lat and Long are in decimal degrees. 
                //Written by Chuck Gantz- chuck.gantz@globalstar.com

                double k0 = 0.9996;
                double a = Datum.Ellipsoid.EquatorialRadius.Value; // ellipsoid[ReferenceEllipsoid].EquatorialRadius;
                double eccSquared = Datum.Ellipsoid.EccentricitySquared; //.vellipsoid[ReferenceEllipsoid].eccentricitySquared;
                double eccPrimeSquared;
                double e1 = (1 - Math.Sqrt(1 - eccSquared)) / (1 + Math.Sqrt(1 - eccSquared));
                double N1, T1, C1, R1, D, M;
                double LongOrigin;
                double mu, phi1Rad;
                double x, y;
                //int ZoneNumber;
                //char* ZoneLetter;
                //int NorthernHemisphere; //1 for northern hemispher, 0 for southern

                x = pEasting.Value;// - 500000.0; //remove 500,000 meter offset for longitude
                y = pNorthing.Value;

                // Is the zone unknown?


                // Remove the 10-million meter offset used for the southern hemisphere
                if (zoneLetter < ZoneLetter.N && zoneLetter != ZoneLetter.Unknown)
                    y -= 10000000.0;

				if (zoneNumber != 0)
					x-= 500000.0;
                ////			ZoneNumber = strtoul(UTMZone, &ZoneLetter, 10);
                ////			if((*ZoneLetter - 'N') >= 0)
                ////				NorthernHemisphere = 1;//point is in northern hemisphere
                ////			else
                ////			{
                ////				NorthernHemisphere = 0;//point is in southern hemisphere
                ////				y -= 10000000.0;//remove 10,000,000 meter offset used for southern hemisphere
                ////			}

				LongOrigin = (zoneNumber - 1) * 6 - 180 + 3;  //+3 puts origin in middle of zone

                eccPrimeSquared = (eccSquared) / (1.0 - eccSquared);

                M = y / k0;
                mu = M / (a * (1 - eccSquared / 4 - 3 * eccSquared * eccSquared / 64 - 5 * eccSquared * eccSquared * eccSquared / 256));

                phi1Rad = mu + (3 * e1 * 0.5 - 27 * e1 * e1 * e1 / 32) * Math.Sin(2 * mu)
                    + (21 * e1 * e1 / 16 - 55 * e1 * e1 * e1 * e1 / 32) * Math.Sin(4 * mu)
                    + (151 * e1 * e1 * e1 / 96) * Math.Sin(6 * mu);
                //phi1 = Angle.FromRadians(phi1Rad); //*rad2deg;

                N1 = a / Math.Sqrt(1 - eccSquared * Math.Sin(phi1Rad) * Math.Sin(phi1Rad));
                T1 = Math.Tan(phi1Rad) * Math.Tan(phi1Rad);
                C1 = eccPrimeSquared * Math.Cos(phi1Rad) * Math.Cos(phi1Rad);
                R1 = a * (1 - eccSquared) / Math.Pow(1 - eccSquared * Math.Sin(phi1Rad) * Math.Sin(phi1Rad), 1.5);
                D = x / (N1 * k0);

                double LatRad = phi1Rad - (N1 * Math.Tan(phi1Rad) / R1) * (D * D * 0.5 - (5 + 3 * T1 + 10 * C1 - 4 * C1 * C1 - 9 * eccPrimeSquared) * D * D * D * D / 24
                    + (61 + 90 * T1 + 298 * C1 + 45 * T1 * T1 - 252 * eccPrimeSquared - 3 * C1 * C1) * D * D * D * D * D * D / 720);
                pLatitude = Latitude.FromRadians(LatRad);

                double LongRad = (D - (1 + 2 * T1 + C1) * D * D * D / 6 + (5 - 2 * C1 + 28 * T1 - 3 * C1 * C1 + 8 * eccPrimeSquared + 24 * T1 * T1)
                    * D * D * D * D * D / 120) / Math.Cos(phi1Rad);
                if (zoneNumber != 0)
                    pLongitude = Longitude.FromRadians(LongRad).Add(LongOrigin);
                else
                    pLongitude = Longitude.FromRadians(LongRad);
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Calculates the direction of travel to the specified destination.
        /// </summary>
        /// <param name="destination">A <strong>Position</strong> object to which the bearing is calculated.</param>
        /// <returns>An <strong>Azimuth</strong> object representing the calculated distance.</returns>
        public Azimuth BearingTo(Position destination)
        {
            return BearingTo(destination, this.Datum.Ellipsoid);
        }

        /// <summary>
        /// Calculates the direction of travel to the specified destination using the specified interpretation of Earth's shape.
        /// </summary>
        /// <param name="destination">A <strong>Position</strong> object to which the bearing is calculated.</param>
        /// <param name="ellipsoid">An <strong>Ellipsoid</strong> object used to fine-tune bearing calculations.</param>
        /// <returns>An <strong>Azimuth</strong> object representing the calculated distance.</returns>
        public Azimuth BearingTo(Position destination, Ellipsoid ellipsoid)
        {
            // From: http://www.mathworks.com/matlabcentral/files/8607/vdist.m

            /*
             function varargout = vdist(lat1,lon1,lat2,lon2)
% VDIST - Using the WGS-84 Earth ellipsoid, compute the distance between
%         two points within a few millimeters of accuracy, compute forward
%         azimuth, and compute backward azimuth, all using a vectorized
%         version of Vincenty's algorithm.
%
% s = vdist(lat1,lon1,lat2,lon2)
% [s,a12] = vdist(lat1,lon1,lat2,lon2)
% [s,a12,a21] = vdist(lat1,lon1,lat2,lon2)
%
% s = distance in meters (inputs may be scalars, vectors, or matrices)
% a12 = azimuth in degrees from first point to second point (forward)
% a21 = azimuth in degrees from second point to first point (backward)
%       (Azimuths are in degrees clockwise from north.)
% lat1 = GEODETIC latitude of first point (degrees)
% lon1 = longitude of first point (degrees)
% lat2, lon2 = second point (degrees)
%
%  Original algorithm source:
%  T. Vincenty, "Direct and Inverse Solutions of Geodesics on the Ellipsoid
%  with Application of Nested Equations", Survey Review, vol. 23, no. 176,
%  April 1975, pp 88-93.
%  Available at: http://www.ngs.noaa.gov/PUBS_LIB/inverse.pdf
%
% Notes: (1) lat1,lon1,lat2,lon2 can be any (identical) size/shape. Outputs
%            will have the same size and shape.
%        (2) Error correcting code, convergence failure traps, antipodal
%            corrections, polar error corrections, WGS84 ellipsoid
%            parameters, testing, and comments: Michael Kleder, 2004.
%        (3) Azimuth implementation (including quadrant abiguity
%            resolution) and code vectorization, Michael Kleder, Sep 2005.
%        (4) Vectorization is convergence sensitive; that is, quantities
%            which have already converged to within tolerance are not
%            recomputed during subsequent iterations (while other
%            quantities are still converging).
%        (5) Vincenty describes his distance algorithm as precise to within
%            0.01 millimeters, subject to the ellipsoidal model.
%        (6) For distance calculations, essentially antipodal points are
%            treated as exactly antipodal, potentially reducing accuracy
%            slightly.
%        (7) Distance failures for points exactly at the poles are
%            eliminated by moving the points by 0.6 millimeters.
%        (8) The Vincenty distance algorithm was transcribed verbatim by
%            Peter Cederholm, August 12, 2003. It was modified and
%            translated to English by Michael Kleder.
%            Mr. Cederholm's website is http://www.plan.aau.dk/~pce/
%        (9) Distances agree with the Mapping Toolbox, version 2.2 (R14SP3)
%            with a max relative difference of about 5e-9, except when the
%            two points are nearly antipodal, and except when one point is
%            near the equator and the two longitudes are nearly 180 degrees
%            apart. This function (vdist) is more accurate in such cases.
%            For example, note this difference (as of this writing):
%            >>vdist(0.2,305,15,125)
%            18322827.0131551
%            >>distance(0.2,305,15,125,[6378137 0.08181919])
%            0
%       (10) Azimuths FROM the north pole (either forward starting at the
%            north pole or backward when ending at the north pole) are set
%            to 180 degrees by convention. Azimuths FROM the south pole are
%            set to 0 degrees by convention.
%       (11) Azimuths agree with the Mapping Toolbox, version 2.2 (R14SP3)
%            to within about a hundred-thousandth of a degree, except when
%            traversing to or from a pole, where the convention for this
%            function is described in (10), and except in the cases noted
%            above in (9).
%       (12) No warranties; use at your own risk.

% reshape inputs
keepsize = size(lat1);
lat1=lat1(:);
lon1=lon1(:);
lat2=lat2(:);
lon2=lon2(:);
% Input check:
if any(abs(lat1)>90 | abs(lat2)>90)
    error('Input latitudes must be between -90 and 90 degrees, inclusive.')
end
% Supply WGS84 earth ellipsoid axis lengths in meters:
a = 6378137; % definitionally
b = 6356752.31424518; % computed from WGS84 earth flattening coefficient
% preserve true input latitudes:
lat1tr = lat1;
lat2tr = lat2;
% convert inputs in degrees to radians:
lat1 = lat1 * 0.0174532925199433;
lon1 = lon1 * 0.0174532925199433;
lat2 = lat2 * 0.0174532925199433;
lon2 = lon2 * 0.0174532925199433;
% correct for errors at exact poles by adjusting 0.6 millimeters:
kidx = abs(pi/2-abs(lat1)) < 1e-10;
if any(kidx);
    lat1(kidx) = sign(lat1(kidx))*(pi/2-(1e-10));
end
kidx = abs(pi/2-abs(lat2)) < 1e-10;
if any(kidx)
    lat2(kidx) = sign(lat2(kidx))*(pi/2-(1e-10));
end
f = (a-b)/a;
U1 = atan((1-f)*tan(lat1));
U2 = atan((1-f)*tan(lat2));
lon1 = mod(lon1,2*pi);
lon2 = mod(lon2,2*pi);
L = abs(lon2-lon1);
kidx = L > pi;
if any(kidx)
    L(kidx) = 2*pi - L(kidx);
end
lambda = L;
lambdaold = 0*lat1;
itercount = 0;
notdone = logical(1+0*lat1);
alpha = 0*lat1;
sigma = 0*lat1;
cos2sigmam = 0*lat1;
C = 0*lat1;
warninggiven = logical(0);
while any(notdone)  % force at least one execution
    %disp(['lambda(21752) = ' num2str(lambda(21752),20)]);
    itercount = itercount+1;
    if itercount > 50
        if ~warninggiven
            warning(['Essentially antipodal points encountered. ' ...
                'Precision may be reduced slightly.']);
        end
        lambda(notdone) = pi;
        break
    end
    lambdaold(notdone) = lambda(notdone);
    sinsigma(notdone) = sqrt((cos(U2(notdone)).*sin(lambda(notdone)))...
        .^2+(cos(U1(notdone)).*sin(U2(notdone))-sin(U1(notdone)).*...
        cos(U2(notdone)).*cos(lambda(notdone))).^2);
    cossigma(notdone) = sin(U1(notdone)).*sin(U2(notdone))+...
        cos(U1(notdone)).*cos(U2(notdone)).*cos(lambda(notdone));
    % eliminate rare imaginary portions at limit of numerical precision:
    sinsigma(notdone)=real(sinsigma(notdone));
    cossigma(notdone)=real(cossigma(notdone));
    sigma(notdone) = atan2(sinsigma(notdone),cossigma(notdone));
    alpha(notdone) = asin(cos(U1(notdone)).*cos(U2(notdone)).*...
        sin(lambda(notdone))./sin(sigma(notdone)));
    cos2sigmam(notdone) = cos(sigma(notdone))-2*sin(U1(notdone)).*...
        sin(U2(notdone))./cos(alpha(notdone)).^2;
    C(notdone) = f/16*cos(alpha(notdone)).^2.*(4+f*(4-3*...
        cos(alpha(notdone)).^2));
    lambda(notdone) = L(notdone)+(1-C(notdone)).*f.*sin(alpha(notdone))...
        .*(sigma(notdone)+C(notdone).*sin(sigma(notdone)).*...
        (cos2sigmam(notdone)+C(notdone).*cos(sigma(notdone)).*...
        (-1+2.*cos2sigmam(notdone).^2)));
    %disp(['then, lambda(21752) = ' num2str(lambda(21752),20)]);
    % correct for convergence failure in the case of essentially antipodal
    % points
    if any(lambda(notdone) > pi)
        warning(['Essentially antipodal points encountered. ' ...
            'Precision may be reduced slightly.']);
        warninggiven = logical(1);
        lambdaold(lambda>pi) = pi;
        lambda(lambda>pi) = pi;
    end
    notdone = abs(lambda-lambdaold) > 1e-12;
end
u2 = cos(alpha).^2.*(a^2-b^2)/b^2;
A = 1+u2./16384.*(4096+u2.*(-768+u2.*(320-175.*u2)));
B = u2./1024.*(256+u2.*(-128+u2.*(74-47.*u2)));
deltasigma = B.*sin(sigma).*(cos2sigmam+B./4.*(cos(sigma).*(-1+2.*...
    cos2sigmam.^2)-B./6.*cos2sigmam.*(-3+4.*sin(sigma).^2).*(-3+4*...
    cos2sigmam.^2)));
varargout{1} = reshape(b.*A.*(sigma-deltasigma),keepsize);
if nargout > 1
    % From point #1 to point #2
    % correct sign of lambda for azimuth calcs:
    lambda = abs(lambda);
    kidx=sign(sin(lon2-lon1)) .* sign(sin(lambda)) < 0;
    lambda(kidx) = -lambda(kidx);
    numer = cos(U2).*sin(lambda);
    denom = cos(U1).*sin(U2)-sin(U1).*cos(U2).*cos(lambda);
    a12 = atan2(numer,denom);
    kidx = a12<0;
    a12(kidx)=a12(kidx)+2*pi;
    % from poles:
    a12(lat1tr <= -90) = 0;
    a12(lat1tr >= 90 ) = pi;
    varargout{2} = reshape(a12 * 57.2957795130823,keepsize); % to degrees
end
if nargout > 2
    a21=NaN*lat1;
    % From point #2 to point #1
    % correct sign of lambda for azimuth calcs:
    lambda = abs(lambda);
    kidx=sign(sin(lon1-lon2)) .* sign(sin(lambda)) < 0;
    lambda(kidx)=-lambda(kidx);
    numer = cos(U1).*sin(lambda);
    denom = sin(U1).*cos(U2)-cos(U1).*sin(U2).*cos(lambda);
    a21 = atan2(numer,denom);
    kidx=a21<0;
    a21(kidx)= a21(kidx)+2*pi;
    % backwards from poles:
    a21(lat2tr >= 90) = pi;
    a21(lat2tr <= -90) = 0;
    varargout{3} = reshape(a21 * 57.2957795130823,keepsize); % to degrees
end
return

             * 
             * 
             */

            // If positions are equivalent, return zero
            if (Equals(destination))
                return Azimuth.Empty;

            #region Newer code

            double goodlambda = 0;
            double goodalpha = 0;
            double goodsigma = 0;
            double goodcos2sigmam = 0;

            //            % reshape inputs
            //keepsize = size(lat1);
            //lat1=lat1(:);
            //lon1=lon1(:);
            //lat2=lat2(:);
            //lon2=lon2(:);

            // ?

            //% Input check:
            //if any(abs(lat1)>90 | abs(lat2)>90)
            //    error('Input latitudes must be between -90 and 90 degrees, inclusive.')
            //end

            // The -90 to 90 check is handled by Normalize

            //% Supply WGS84 earth ellipsoid axis lengths in meters:
            //a = 6378137; % definitionally
            //b = 6356752.31424518; % computed from WGS84 earth flattening coefficient

            double a = ellipsoid.EquatorialRadius.ToMeters().Value;
            double b = ellipsoid.PolarRadius.ToMeters().Value;

            //% preserve true input latitudes:
            //lat1tr = lat1;
            //lat2tr = lat2;

            double lat1tr = pLatitude.DecimalDegrees;
            double lat2tr = destination.Latitude.DecimalDegrees;

            //% convert inputs in degrees to radians:
            //lat1 = lat1 * 0.0174532925199433;
            //lon1 = lon1 * 0.0174532925199433;
            //lat2 = lat2 * 0.0174532925199433;
            //lon2 = lon2 * 0.0174532925199433;

            // Convert inputs into radians
            double lat1 = this.Latitude.Normalize().ToRadians().Value;
            double lon1 = this.Longitude.Normalize().ToRadians().Value;
            double lat2 = destination.Latitude.Normalize().ToRadians().Value;
            double lon2 = destination.Longitude.Normalize().ToRadians().Value;

            //% correct for errors at exact poles by adjusting 0.6 millimeters:
            //kidx = abs(pi/2-abs(lat1)) < 1e-10;
            //if any(kidx);
            //    lat1(kidx) = sign(lat1(kidx))*(pi/2-(1e-10));
            //end

            // Correct for errors at exact poles by adjusting 0.6mm
            if (Math.Abs(Math.PI * 0.5 - Math.Abs(lat1)) < 1E-10)
            {
                lat1 = Math.Sign(lat1) * (Math.PI * 0.5 - 1E-10);
            }

            //kidx = abs(pi/2-abs(lat2)) < 1e-10;
            //if any(kidx)
            //    lat2(kidx) = sign(lat2(kidx))*(pi/2-(1e-10));
            //end

            if (Math.Abs(Math.PI * 0.5 - Math.Abs(lat2)) < 1E-10)
            {
                lat2 = Math.Sign(lat2) * (Math.PI * 0.5 - 1E-10);
            }


            //f = (a-b)/a;

            double f = ellipsoid.Flattening;

            //U1 = atan((1-f)*tan(lat1));

            double U1 = Math.Atan((1 - f) * Math.Tan(lat1));

            //U2 = atan((1-f)*tan(lat2));

            double U2 = Math.Atan((1 - f) * Math.Tan(lat2));

            //lon1 = mod(lon1,2*pi);

            lon1 = lon1 % (2 * Math.PI);

            //lon2 = mod(lon2,2*pi);

            lon2 = lon2 % (2 * Math.PI);

            //L = abs(lon2-lon1);

            double L = Math.Abs(lon2 - lon1);

            //kidx = L > pi;
            //if any(kidx)
            //    L(kidx) = 2*pi - L(kidx);
            //end

            if (L > Math.PI)
            {
                L = 2.0 * Math.PI - L;
            }

            //lambda = L;

            double lambda = L;

            //lambdaold = 0*lat1;

            double lambdaold = 0;

            //itercount = 0;

            int itercount = 0;

            //notdone = logical(1+0*lat1);

            bool notdone = true;

            //alpha = 0*lat1;

            double alpha = 0;

            //sigma = 0*lat1;

            double sigma = 0;

            //cos2sigmam = 0*lat1;

            double cos2sigmam = 0;

            //C = 0*lat1;

            double C = 0;

            //warninggiven = logical(0);

            //bool warninggiven = false;

            //while any(notdone)  % force at least one execution

            while (notdone)
            {

                //    %disp(['lambda(21752) = ' num2str(lambda(21752),20)]);
                //    itercount = itercount+1;

                itercount++;

                //    if itercount > 50

                if (itercount > 50)
                {

                    //        if ~warninggiven

                    //if (!warninggiven)
                    //{
                    //    //            warning(['Essentially antipodal points encountered. ' ...
                    //    //                'Precision may be reduced slightly.']);

                    //    warninggiven = true;
                    //    throw new WarningException("Distance calculation accuracy may be reduced because the two endpoints are antipodal.");
                    //}

                    //        end
                    //        lambda(notdone) = pi;

                    lambda = Math.PI;

                    //        break

                    break;

                    //    end

                }

                //    lambdaold(notdone) = lambda(notdone);

                lambdaold = lambda;

                //    sinsigma(notdone) = sqrt((cos(U2(notdone)).*sin(lambda(notdone)))...
                //        .^2+(cos(U1(notdone)).*sin(U2(notdone))-sin(U1(notdone)).*...
                //        cos(U2(notdone)).*cos(lambda(notdone))).^2);

                double sinsigma = Math.Sqrt(Math.Pow((Math.Cos(U2) * Math.Sin(lambda))
                        , 2) + Math.Pow((Math.Cos(U1) * Math.Sin(U2) - Math.Sin(U1) *
                        Math.Cos(U2) * Math.Cos(lambda)), 2));

                //    cossigma(notdone) = sin(U1(notdone)).*sin(U2(notdone))+...
                //        cos(U1(notdone)).*cos(U2(notdone)).*cos(lambda(notdone));

                double cossigma = Math.Sin(U1) * Math.Sin(U2) +
                    Math.Cos(U1) * Math.Cos(U2) * Math.Cos(lambda);

                //    % eliminate rare imaginary portions at limit of numerical precision:
                //    sinsigma(notdone)=real(sinsigma(notdone));
                //    cossigma(notdone)=real(cossigma(notdone));

                // Eliminate rare imaginary portions at limit of numerical precision:
                // ?

                //    sigma(notdone) = atan2(sinsigma(notdone),cossigma(notdone));

                sigma = Math.Atan2(sinsigma, cossigma);

                //    alpha(notdone) = asin(cos(U1(notdone)).*cos(U2(notdone)).*...
                //        sin(lambda(notdone))./sin(sigma(notdone)));

                alpha = Math.Asin(Math.Cos(U1) * Math.Cos(U2) *
                    Math.Sin(lambda) / Math.Sin(sigma));

                //    cos2sigmam(notdone) = cos(sigma(notdone))-2*sin(U1(notdone)).*...
                //        sin(U2(notdone))./cos(alpha(notdone)).^2;

                cos2sigmam = Math.Cos(sigma) - 2.0 * Math.Sin(U1) *
                    Math.Sin(U2) / Math.Pow(Math.Cos(alpha), 2);

                //    C(notdone) = f/16*cos(alpha(notdone)).^2.*(4+f*(4-3*...
                //        cos(alpha(notdone)).^2));

                C = f / 16 * Math.Pow(Math.Cos(alpha), 2) * (4 + f * (4 - 3 *
                    Math.Pow(Math.Cos(alpha), 2)));

                //    lambda(notdone) = L(notdone)+(1-C(notdone)).*f.*sin(alpha(notdone))...
                //        .*(sigma(notdone)+C(notdone).*sin(sigma(notdone)).*...
                //        (cos2sigmam(notdone)+C(notdone).*cos(sigma(notdone)).*...
                //        (-1+2.*cos2sigmam(notdone).^2)));

                lambda = L + (1 - C) * f * Math.Sin(alpha)
                            * (sigma + C * Math.Sin(sigma) *
                            (cos2sigmam + C * Math.Cos(sigma) *
                            (-1 + 2 * Math.Pow(cos2sigmam, 2))));

                //    %disp(['then, lambda(21752) = ' num2str(lambda(21752),20)]);
                //    % correct for convergence failure in the case of essentially antipodal
                //    % points

                // Correct for convergence failure in the case of essentially antipodal points

                //    if any(lambda(notdone) > pi)

                if (lambda > Math.PI)
                {

                    //        if ~warninggiven

                    //if (!warninggiven)
                    //{
                    //    //            warning(['Essentially antipodal points encountered. ' ...
                    //    //                'Precision may be reduced slightly.']);

                    //    warninggiven = true;
                    //    throw new WarningException("Distance calculation accuracy may be reduced because the two endpoints are antipodal.");
                    //}

                    //        end

                    //        lambdaold(lambda>pi) = pi;

                    lambdaold = Math.PI;

                    //        lambda(lambda>pi) = pi;

                    lambda = Math.PI;

                    //    end

                }

                //    notdone = abs(lambda-lambdaold) > 1e-12;

                notdone = Math.Abs(lambda - lambdaold) > TargetAccuracy;

                //end

                // NOTE: In some cases "alpha" would return a "NaN".  If values are healthy, 
                // remember them so we get a good distance calc.
                if (!double.IsNaN(alpha))
                {
                    goodlambda = lambda;
                    goodalpha = alpha;
                    goodsigma = sigma;
                    goodcos2sigmam = cos2sigmam;
                }
            }


            //u2 = cos(alpha).^2.*(a^2-b^2)/b^2;

            double u2 = Math.Pow(Math.Cos(goodalpha), 2) * (Math.Pow(a, 2) - Math.Pow(b, 2)) / Math.Pow(b, 2);

            //A = 1+u2./16384.*(4096+u2.*(-768+u2.*(320-175.*u2)));

            double A = 1 + u2 / 16384 * (4096 + u2 * (-768 + u2 * (320 - 175 * u2)));

            //B = u2./1024.*(256+u2.*(-128+u2.*(74-47.*u2)));

            double B = u2 / 1024 * (256 + u2 * (-128 + u2 * (74 - 47 * u2)));

            //deltasigma = B.*sin(sigma).*(cos2sigmam+B./4.*(cos(sigma).*(-1+2.*...
            //    cos2sigmam.^2)-B./6.*cos2sigmam.*(-3+4.*sin(sigma).^2).*(-3+4*...
            //    cos2sigmam.^2)));

            double deltasigma = B * Math.Sin(goodsigma) * (goodcos2sigmam + B / 4 * (Math.Cos(goodsigma) * (-1 + 2 *
                Math.Pow(goodcos2sigmam, 2)) - B / 6 * goodcos2sigmam * (-3 + 4 * Math.Pow(Math.Sin(goodsigma), 2)) * (-3 + 4 *
                Math.Pow(goodcos2sigmam, 2))));

            //varargout{1} = reshape(b.*A.*(sigma-deltasigma),keepsize);
            double s = b * A * (goodsigma - deltasigma);

            // Return the Distance in meters
            //return new Distance(s, DistanceUnit.Meters).ToLocalUnitType();

            //if nargout > 1
            //    % From point #1 to point #2
            //    % correct sign of lambda for azimuth calcs:

            //    lambda = abs(lambda);

            goodlambda = Math.Abs(goodlambda);

            //    kidx=sign(sin(lon2-lon1)) .* sign(sin(lambda)) < 0;

            bool kidx = Math.Sign(Math.Sin(lon2 - lon1)) * Math.Sign(Math.Sin(goodlambda)) < 0;

            //    lambda(kidx) = -lambda(kidx);

            if (kidx)
                goodlambda = -goodlambda;

            //    numer = cos(U2).*sin(lambda);

            double numer = Math.Cos(U2) * Math.Sin(goodlambda);

            //    denom = cos(U1).*sin(U2)-sin(U1).*cos(U2).*cos(lambda);

            double denom = Math.Cos(U1) * Math.Sin(U2) - Math.Sin(U1) * Math.Cos(U2) * Math.Cos(goodlambda);

            //    a12 = atan2(numer,denom);

            double a12 = Math.Atan2(numer, denom);

            //    kidx = a12<0;

            kidx = a12 < 0;

            //    a12(kidx)=a12(kidx)+2*pi;

            if (kidx)
                a12 = a12 + 2 * Math.PI;

            //    % from poles:
            //    a12(lat1tr <= -90) = 0;

            if (lat1tr <= -90.0)
                a12 = 0;

            //    a12(lat1tr >= 90 ) = pi;

            if (lat1tr >= 90)
                a12 = Math.PI;

            //    varargout{2} = reshape(a12 * 57.2957795130823,keepsize); % to degrees

            // Convert to degrees
            return Azimuth.FromRadians(a12);

            //end
            //if nargout > 2
            //    a21=NaN*lat1;
            //    % From point #2 to point #1
            //    % correct sign of lambda for azimuth calcs:
            //    lambda = abs(lambda);
            //    kidx=sign(sin(lon1-lon2)) .* sign(sin(lambda)) < 0;
            //    lambda(kidx)=-lambda(kidx);
            //    numer = cos(U1).*sin(lambda);
            //    denom = sin(U1).*cos(U2)-cos(U1).*sin(U2).*cos(lambda);
            //    a21 = atan2(numer,denom);
            //    kidx=a21<0;
            //    a21(kidx)= a21(kidx)+2*pi;
            //    % backwards from poles:
            //    a21(lat2tr >= 90) = pi;
            //    a21(lat2tr <= -90) = 0;
            //    varargout{3} = reshape(a21 * 57.2957795130823,keepsize); % to degrees
            //end
            //return
 
            #endregion


            #region Old Code

            /*
            double lonrad = pLongitude.ToRadians().Value;
			double latrad = pLatitude.ToRadians().Value;
			double destlonrad = destination.Longitude.ToRadians().Value;
			double destlatrad = destination.Latitude.ToRadians().Value;
			
			double y = Math.Sin(lonrad - destlonrad) * Math.Cos(destlatrad);
			double x = Math.Cos(latrad) * Math.Sin(destlatrad) 
				     - Math.Sin(latrad) * Math.Cos(destlatrad) * Math.Cos(lonrad - destlonrad);

			double rad = Math.Atan2(-y, x);
			return Azimuth.FromRadians(rad).Normalize();
             */

//			try
////			{
//				//Dim AdjustedDestination As Position = destination.ToEllipsoid(Ellipsoid.Type)
//
//				double y = -Math.Sin(Longitude.ToRadians().Value - destination.Longitude.ToRadians().Value) 
//					* Math.Cos(destination.Latitude.ToRadians().Value);
//				double x = Math.Cos(Latitude.ToRadians().Value) * Math.Sin(destination.Latitude.ToRadians().Value) 
//					- Math.Sin(Latitude.ToRadians().Value) * Math.Cos(destination.Latitude.ToRadians().Value) 
//					* Math.Cos(Longitude.ToRadians().Value - destination.Longitude.ToRadians().Value);
//
//			//Console.WriteLine(String.Format("X: {0}, Y: {1}
//
////			atan2( -sin(long1-long2).cos(lat2),
////cos(lat1).sin(lat2) - sin(lat1).cos(lat2).cos(long1-long2) ) 
//
//
//				return new Azimuth(((Math.Atan2(y, x) * 180.0 / Math.PI) + 360) % 360); //+ 1 / 7200.0)
//			}
//			catch
//			{
//				throw new GpsException("Error while calculating initial bearing.");
//			}
			// Test Data
			//
			// Name: Denver Oklahoma City 
			// Latitude: 39  45 ' 0.00000 '' 35  26 ' 0.00000 '' 
			// Longitude: 105  0 ' 0.00000 '' 97  28 ' 0.00000 '' 
			// Forward Azimuth:  236  35 ' 21.15 ''  
			// Reverse Azimuth:  51  59 ' 10.32 '' 
			// Datumal Distance:  819373.914 meters  

			//			'' Converted from JavaScript: http://www.movable-type.co.uk/scripts/LatLong.html
			//			''
			//			''	LatLong.bearing = function(p1, p2) {
			//			''  var y = Math.sin(p1.long-p2.long) * Math.cos(p2.lat);
			//			''  var x = Math.cos(p1.lat)*Math.sin(p2.lat) -
			//			''		  Math.sin(p1.lat)*Math.cos(p2.lat)*Math.cos(p1.long-p2.long);
			//			''  return(Math.atan2(-y, x)); // -y 'cos Williams treats W as +ve!
			//			'Try
			//			'	Dim AdjustedDestination As Position = destination				' destination.ToDatum(Datum.Type)

			//			'	Dim StartLatRad As Double = Latitude.ToRadians().Value
			//			'	Dim StartLonRad As Double = Longitude.ToRadians().Value
			//			'	Dim DestLatRad As Double = AdjustedDestination.Latitude.ToRadians().Value
			//			'	Dim DestLonRad As Double = AdjustedDestination.Longitude.ToRadians().Value

			//			'	Dim y As Double = -Math.Sin(StartLonRad - DestLonRad) * Math.Cos(DestLatRad)
			//			'	Dim x As Double = Math.Cos(StartLatRad) * Math.Sin(DestLatRad) _
			//			'		 - Math.Sin(StartLatRad) * Math.Cos(DestLatRad) * Math.Cos(StartLonRad - DestLonRad)
			//			'	Dim NewBearingRads As New Radian(Math.Atan2(y, x))
			//			'	'tc1=mod(atan2(sin(lon1-lon2)*cos(lat2),
			//			'	'		   cos(lat1)*sin(lat2)-sin(lat1)*cos(lat2)*cos(lon1-lon2)), 2*pi)

			//			'	Dim NewBearing As Double = (NewBearingRads.ToAngle.DecimalDegrees + 360) Mod 360				 '* 180.0 / Math.PI '+ 360) Mod 360 + 1 / 7200.0

			//			'	Return New Azimuth(NewBearing)				' Azimuth(double.Parse()
			//			'Catch ex As Exception
			//			'	Throw New GeoException("Error while calculating initial bearing.", ex)
			//			'End Try
			//			'' Test Data
			//			''
			//			'' Name: Denver Oklahoma City 
			//			'' Latitude: 39  45 ' 0.00000 '' 35  26 ' 0.00000 '' 
			//			'' Longitude: 105  0 ' 0.00000 '' 97  28 ' 0.00000 '' 
			//			'' Forward Azimuth:  236  35 ' 21.15 ''  
			//			'' Reverse Azimuth:  51  59 ' 10.32 '' 
            //		   '' Datumal Distance:  819373.914 meters  
            #endregion
        }

		/// <overloads>Returns the direction of travel from one position to another.</overloads>
		public static Azimuth BearingTo(Position start, Position destination)
		{
			return start.BearingTo(destination);
		}

		/// <summary>Represents the vertical North/South portion of the location.</summary>
		public Latitude Latitude
		{
			get
			{
				// Perform calculation JIT style
//				if(pLatitude.IsEmpty) 
//					SetLatitudeLongitudeCoordinate();
				return pLatitude;
			}
		}

		/// <summary>Represents the horizontal East/West portion of the location.</summary>
		public Longitude Longitude
		{
			get
			{
				// Perform calculation JIT style
//				if(pLongitude.IsEmpty) 
//					SetLatitudeLongitudeCoordinate();
				return pLongitude;
			}
		}

		

		/// <summary>
		/// Outputs the current instance as a string using the default format.
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return ToString("g", CultureInfo.CurrentCulture); // Always support "g" as a default format
		}

		/// <overloads>Outputs the current instance as a formatted string.</overloads>
		/// <summary>
		/// Outputs the current instance as a string using the specified format.
		/// </summary>
		/// <returns></returns>
		public virtual string ToString(string format)
		{
			return ToString(format, CultureInfo.CurrentCulture);
		}

		/// <summary>
		/// Outputs the current instance as a string using the specified format and culture information.
		/// </summary>
		/// <returns></returns>
		public virtual string ToString(string format, CultureInfo culture)
		{
            if (culture == null) 
				culture = CultureInfo.CurrentCulture;
            if (format == null)
                format = "G";
			// Is this UTM or lat/long?
			string LowerFormat = format.ToLower(culture);
			if(LowerFormat.IndexOf("z") != -1 || LowerFormat.IndexOf("n") != -1 || LowerFormat.IndexOf("e") != -1)
			{
				// Output as a UTM coordinate
				string[] FormatWords = LowerFormat.Split(' ');
				StringBuilder Output = new StringBuilder();
				foreach(string SubFormat in FormatWords)
				{
					if(SubFormat.IndexOf("z") != -1)
					{
						Output.Append(Zone);
					}
					else if(SubFormat.IndexOf("n") != -1)
					{
						// Replace N with "V" for the Distance ToString method
                        Output.Append(Northing.ToString(SubFormat.Replace("n", "v"), culture).Substring(0, SubFormat.Length) + "N");
					}
					else if(SubFormat.IndexOf("e") != -1)
					{
						// Replace N with "V" for the Distance ToString method
                        Output.Append(Easting.ToString(SubFormat.Replace("e", "v"), culture).Substring(0, SubFormat.Length) + "E");
					}
					Output.Append(" ");
				}
				// Return the result
				return Output.ToString().Trim();
			}
			else
			{
				// Output as latitude and longitude
				return Latitude.ToString(format, culture) + culture.TextInfo.ListSeparator + Longitude.ToString(format, culture);
			}
        }

		/// <summary>Returns the coordinate system used for this location.</summary>
		public Datum Datum
		{
			get
			{
				// Use WGS84 as a default if nothing is given
				if(pDatum == null)
					return Datum.Default;
				return pDatum;
			}
		}

		/// <summary>Converts a string-based positional measurement into a Position 
		/// object.</summary>
		/// <param name="value">A <strong>String</strong> containing both latitude and longitude in the form of a string.</param>
		/// <remarks>
		/// This powerful method will analyze a string containing latitude and longitude and
		/// create a Position object matching the specified values.  The latitude and longitude
		/// must be separated by a non-space delimiter such as a comma.</remarks>
		public static Position Parse(string value)
		{
			return new Position(value, CultureInfo.CurrentCulture);
		}
	
		public static Position Parse(string value, CultureInfo culture)
		{
			return new Position(value, culture);
		}

		/// <summary>
		/// Converts the current instance into an Earth-centered, Earth-fixed (ECEF) Cartesian point.
		/// </summary>
		/// <returns></returns>
		public CartesianPoint ToCartesianPoint()
		{
			return ToCartesianPoint(Ellipsoid.Wgs1984);
		}

		public CartesianPoint ToCartesianPoint(Ellipsoid ellipsoid)
        {
            
            //	% LLA2ECEF - convert latitude, longitude, and altitude to
			//	%            earth-centered, earth-fixed (ECEF) cartesian
			//	% 
			//	% USAGE:
			//	% [x,y,z] = lla2ecef(lat,lon,alt)
			//	% 
			//	% x = ECEF X-coordinate (m)
			//	% y = ECEF Y-coordinate (m)
			//	% z = ECEF Z-coordinate (m)
			//	% lat = geodetic latitude (radians)
			//	% lon = longitude (radians)
			//	% alt = height above WGS84 ellipsoid (m)
			//	% 
			//	% Notes: This function assumes the WGS84 model.
			//	%        Latitude is customary geodetic (not geocentric).
			//	% 
			//	% Source: "Department of Defense World Geodetic System 1984"
			//	%         Page 4-4
			//	%         National Imagery and Mapping Agency
			//	%         Last updated June, 2004
			//	%         NIMA TR8350.2
			//	% 
			//	% Michael Kleder, July 2005
			//
			//	function [x,y,z]=lla2ecef(lat,lon,alt)
			double lat = Latitude.ToRadians().Value;
			double lon = Longitude.ToRadians().Value;
			// Altitude is assumed at 100 meters
			double alt = 100;

			//
			//	% WGS84 ellipsoid constants:
			//	a = 6378137;
			double a = ellipsoid.EquatorialRadius.ToMeters().Value;
			//	e = 8.1819190842622e-2;
			double e = ellipsoid.Eccentricity;
			//
			//	% intermediate calculation
			//	% (prime vertical radius of curvature)
			//	N = a ./ sqrt(1 - e^2 .* sin(lat).^2);
			double N = a / Math.Sqrt(1.0 - Math.Pow(e, 2) * Math.Pow(Math.Sin(lat), 2));
			//
			//	% results:
			//	x = (N+alt) .* cos(lat) .* cos(lon);
			double x = (N + alt) * Math.Cos(lat) * Math.Cos(lon);
			//	y = (N+alt) .* cos(lat) .* sin(lon);
			double y = (N + alt) * Math.Cos(lat) * Math.Sin(lon);
			//	z = ((1-e^2) .* N + alt) .* sin(lat);
			double z = ((1.0 - Math.Pow(e, 2)) * N + alt) * Math.Sin(lat);
			//
			//	return

			return new CartesianPoint(
				new Distance(x, DistanceUnit.Meters),
				new Distance(y, DistanceUnit.Meters),
				new Distance(z, DistanceUnit.Meters));
                          
        }


//		// Sets the coordinate system to the specified value
//		protected void SetDatum(Datum value)
//		{
//			pDatum = value;
//		}

		//'/* @func GPS_Math_Molodensky *******************************************
		//'**
		//'** Transform one datum to another
		//'**
		//'** @param [r] Sphi [double] source latitude (deg)
		//'** @param [r] Slam [double] source longitude (deg)
		//'** @param [r] SH   [double] source height  (metres)
		//'** @param [r] Sa   [double] source semi-major axis (metres)
		//'** @param [r] Sif  [double] source inverse flattening
		//'** @param [w] Dphi [double *] dest latitude (deg)
		//'** @param [w] Dlam [double *] dest longitude (deg)
		//'** @param [w] DH   [double *] dest height  (metres)
		//'** @param [r] Da   [double]   dest semi-major axis (metres)
		//'** @param [r] Dif  [double]   dest inverse flattening
		//'** @param [r] dx  [double]   dx
		//'** @param [r] dy  [double]   dy
		//'** @param [r] dz  [double]   dz
		//'**
		//'** @return [void]
		//'************************************************************************/

		//Public Overloads Function ToDatum(ByVal DatumType As DatumType) As Position
		//	Dim NewDatum As Datum = Geodesy.GetDatum(DatumType)
		//	'void GPS_Math_Molodensky(double Sphi, double Slam, double SH, double Sa,
		//	'			 double Sif, double *Dphi, double *Dlam,
		//'			 double *DH, double Da, double Dif, double dx,
		//'			 double dy, double dz)
		//	'{
		//	Dim Sphi As Double = Latitude.DecimalDegrees
		//	Dim Slam As Double = Longitude.DecimalDegrees
		//	Dim SH As Double = 0
		//	Dim Sa As Double = datum.Ellipsoid.Ellipsoid.SemiMajorAxis.ToMeters().Value
		//	Dim Sif As Double = datum.Ellipsoid.Ellipsoid.InverseFlattening
		//	Dim Dphi As Double		' Output latitude
		//	Dim Dlam As Double		' Output longitude 
		//	Dim DH As Double		 ' Output height 
		//	Dim Da As Double = Newdatum.Ellipsoid.Ellipsoid.SemiMajorAxis.ToMeters().Value
		//	Dim Dif As Double = Newdatum.Ellipsoid.Ellipsoid.InverseFlattening
		//	Dim dx As Double = NewDatum.DX
		//	Dim dy As Double = NewDatum.DY
		//	Dim dz As Double = NewDatum.DZ
		//	'	double Sf;
		//	Dim Sf As Double
		//	'	double Df;
		//	Dim Df As Double
		//	'	double esq;
		//	Dim esq As Double
		//	'	double bda;
		//	Dim bda As Double
		//	'	double da2;
		//	Dim da2 As Double
		//	'	double df2;
		//	Dim df2 As Double
		//	'	double N;
		//	Dim N As Double
		//	'	double M;
		//	Dim M As Double
		//	'	double tmp;
		//	Dim tmp As Double
		//	'	double tmp2;
		//	Dim tmp2 As Double
		//	'	double dphi;
		//	Dim dphi2 As Double
		//	'	double dlambda2;
		//	Dim dlambda2 As Double
		//	'	double dheight;
		//	Dim dheight As Double
		//	'	double phis;
		//	Dim phis As Double
		//	'	double phic;
		//	Dim phic As Double
		//	'	double lams;
		//	Dim lams As Double
		//	'	double lamc;
		//	Dim lamc As Double

		//	'	Sf = 1.0 / Sif;
		//	Sf = 1.0 / Sif
		//	'	Df = 1.0 / Dif;
		//	Df = 1.0 / Dif

		//	'	esq = 2.0*Sf - pow(Sf,2.0);
		//	esq = 2.0 * Sf - Math.Pow(Sf, 20)
		//	'	bda = 1.0 - Sf;
		//	bda = 1.0 - Sf
		//	'	Sphi = GPS_Math_Deg_To_Rad(Sphi);
		//	Sphi = Latitude.ToRadians().Value
		//	'	Slam = GPS_Math_Deg_To_Rad(Slam);
		//	Slam = Longitude.ToRadians().Value

		//	'	da2 = Da - Sa;
		//	da2 = Da - Sa
		//	'	df2 = Df - Sf;
		//	df2 = Df - Sf

		//	'	phis = sin(Sphi);
		//	phis = Math.Sin(Sphi)
		//	'	phic = cos(Sphi);
		//	phic = Math.Cos(Sphi)
		//	'	lams = sin(Slam);
		//	lams = Math.Sin(Slam)
		//	'	lamc = cos(Slam);
		//	lamc = Math.Cos(Slam)

		//	'	N = Sa /  sqrt(1.0 - esq*pow(phis,2.0));
		//	N = Sa / Math.Sqrt(1.0 - esq * Math.Pow(phis, 2.0))
		//	'	tmp = (1.0-esq) /pow((1.0-esq*pow(phis,2.0)),1.5);
		//	tmp = (1.0 - esq) / Math.Pow((1.0 - esq * Math.Pow(phis, 2.0)), 1.5)
		//	'	M   = Sa * tmp;
		//	M = Sa * tmp
		//	'	tmp  = df2 * ((M/bda)+N*bda) * phis * phic;
		//	tmp = df2 * ((M / bda) + N * bda) * phis * phic
		//	'	tmp2 = da2 * N * esq * phis * phic / Sa;
		//	tmp2 = da2 * N * esq * phis * phic / Sa
		//	'	tmp2 += ((-dx*phis*lamc-dy*phis*lams) + dz*phic);
		//	tmp2 += ((-dx * phis * lamc - dy * phis * lams) + dz * phic)
		//	'	dphi2 = (tmp2 + tmp) / (M + SH);
		//	dphi2 = (tmp2 + tmp) / (M + SH)
		//	'	dlambda2 = (-dx*lams+dy*lamc) / ((N+SH)*phic);
		//	dlambda2 = (-dx * lams + dy * lamc) / ((N + SH) * phic)
		//	'	dheight = dx*phic*lamc + dy*phic*lams + dz*phis - da2*(Sa/N) +	df2*bda*N*phis*phis;
		//	dheight = dx * phic * lamc + dy * phic * lams + dz * phis - da2 * (Sa / N) + df2 * bda * N * phis * phis
		//	'	*Dphi = Sphi + dphi2;
		//	Dphi = Sphi + dphi2
		//	'	*Dlam = Slam + dlambda2;
		//	Dlam = Slam + dlambda2
		//	'	*DH   = SH   + dheight;
		//	DH = SH + dheight

		//	'	*Dphi = GPS_Math_Rad_To_Deg(*Dphi);
		//	Dphi = Radian.ToAngle(Dphi).DecimalDegrees
		//'	*Dlam = GPS_Math_Rad_To_Deg(*Dlam);
		//	Dlam = Radian.ToAngle(Dlam).DecimalDegrees
		//	'	return;
		//	' Return a new position
		//	Return New Position(Dphi, Dlam, DatumType)
		//'}
		//End Function

		//Public Overloads Function ToSphericalCoordinate(ByVal altitude As Distance) As SphericalCoordinate
		//End Function

		///* @func GPS_Math_LatLonH_To_XYZ **********************************
		//**
		//** Convert latitude and longitude and Datumal height to
		//** X, Y & Z coordinates
		//Public Overloads Function ToSphericalCoordinate() As SphericalCoordinate

		//**
		//** @param [r] phi [double] latitude (deg)
		//** @param [r] lambda [double] longitude (deg)
		//** @param [r] H [double] Datumal height (metres)
		//** @param [w] x [double *] X
		//** @param [w] y [double *] Y
		//** @param [w] z [double *] Z
		//** @param [r] a [double] semi-major axis (metres)
		//** @param [r] b [double] semi-minor axis (metres)
		//**
		//** @return [void]
		//************************************************************************/
		//void GPS_Math_LatLonH_To_XYZ(double phi, double lambda, double H,
		//				 double *x, double *y, double *z,
		//				 double a, double b)
		//{
		//	double esq;
		//	double nu;

		//	phi	= GPS_Math_Deg_To_Rad(phi);
		//	lambda = GPS_Math_Deg_To_Rad(lambda);

		//	esq   = ((a*a)-(b*b)) / (a*a);

		//	nu	= a / pow(((double)1.0-esq*sin(phi)*sin(phi)),(double)0.5);
		//	*x	= (nu+H) * cos(phi) * cos(lambda);
		//	*y	= (nu+H) * cos(phi) * sin(lambda);
		//	*z	= (((double)1.0-esq)*nu+H) * sin(phi);

		//	return;
		//}
		//End Function

        ///// <summary>
        ///// Returns the coordinate system used during a Position object's construction.
        ///// </summary>
        ///// <value>A value from the <strong>CoordinateSystem</strong> enumeration.</value>
        ///// <remarks>The Position class supports both latitude, longitude and Universal Transverse Mercator (UTM) 
        ///// coordinates within the same instance.  Conversion to and from coordinate systems is automatic yet
        ///// can result in a small loss of precision.  In order to preserve precision, a bias is given to a coordinate
        ///// system depending on which one is used to create the object.  This bias is then considered when
        ///// making copies of the object.  For most applications, the precision error during conversion is so
        ///// small that it is unnoticed, making this property important only to applications demanding the highest precision.</remarks>
        //public CoordinateSystem NativeCoordinateSystem
        //{
        //    get
        //    {
        //        return pCoordinateSystem;
        //    }
        //}

        //		/// <summary>Returns the current location reported by the GPS device.</summary>
		//		public static Position CurrentPosition
		//		{
		//			get
		//			{
		//				return Receiver.CurrentPosition;
		//			}
		//		}

		//' Converts the position from the current coordinate system to WGS84, to the target coordinate system
		//Public Overloads Function ToDatum(ByVal Datum As Datum) As Position
		//	Try
		//		Return ToDatum(Datum.Type)
		//	Catch ex As GeoException
		//		Throw ex
		//	End Try
		//End Function

		//Public Overloads Shared Function ToDatum(ByVal value As Position, ByVal DatumType As DatumType) As Position
		//	Try
		//		Return value.ToDatum(DatumType)
		//	Catch ex As GeoException
		//		Throw ex
		//	End Try
		//End Function

		//Public Overloads Shared Function ToDatum(ByVal value As Position, ByVal Datum As Datum) As Position
		//	Try
		//		Return value.ToDatum(Datum)
		//	Catch ex As GeoException
		//		Throw ex
		//	End Try
		//End Function
		#endregion

        public static Position ParseAsUtm(string value, CultureInfo culture)
        {
            // Try to interpreter the value as a UTM coordinate
            // Get new variables t ostore parsed info
            try
            {
                Distance pNorthing = Distance.Empty;
                Distance pEasting = Distance.Empty;
                int pZoneNumber = 0;
                ZoneLetter pZoneLetter = ZoneLetter.Unknown;
                bool IsZoneLetterIdentified = false;
                //bool IsZoneNumberIdentified = false;
                bool IsZoneHandled = false;
                bool IsEastingHandled = false;
                bool IsNorthingHandled = false;
                // First, clean up the string
                value = value.ToUpper(culture).Replace(culture.TextInfo.ListSeparator, "").Replace("M", "").Replace("METERS", "").Replace("METRES", "").Replace("ZONE", "").Replace("  ", " ").Trim();
                //									// Replace any zone letter separated by a space with just the letter in an attempt to make the zone information one word
                //									.Replace("A ", "A").Replace("B ", "B").Replace("C ", "C").Replace("D ", "D")
                //									.Replace("E ", "E").Replace("F ", "F").Replace("G ", "G").Replace("H ", "H")
                //									.Replace("I ", "I").Replace("J ", "J").Replace("K ", "K").Replace("L ", "L")
                //									.Replace("M ", "M").Replace("N ", "N").Replace("O ", "O").Replace("P ", "P")
                //									.Replace("Q ", "Q").Replace("R ", "R").Replace("S ", "S").Replace("T ", "T")
                //									.Replace("U ", "U").Replace("V ", "V").Replace("W ", "W").Replace("X ", "X")
                //									.Replace("Y ", "Y").Replace("Z ", "Z");
                // Try to find a zone
                string[] Words = value.Split(' ');
                if (Words.Length > 4)
                {
                    return null;
                    //throw new ArgumentException("Only three individual words are allowed during UTM parsing.  Use a three-word format such as \"S12 012345 012345\".");
                }
                //Dim IsHandled(Words.Length - 1) As Boolean
                // Look for a word starting with a letter that is 2 or 3 characters long.  This is the zone
                int Count = 0;
                StringBuilder ZoneNumberStringBuilder = new StringBuilder();

                foreach (string Word in Words)
                {
                    switch (Word.Length)
                    {
                        case 1:
                            break;
                        case 2:
                        case 3:
                            if (IsZoneHandled)
                            {
                                break;
                                //throw new ArgumentException(Resources.Position_DuplicateZoneInformation), "value");
                            }
                            for (Count = 0; Count < Word.Length; Count++)
                            {
                                if (char.IsLetter(Word[Count]))
                                {
                                    if (IsZoneLetterIdentified)
                                    {
                                        return null;
                                        //throw new ArgumentException(Resources.Position_InvalidZoneLetter), "value");
                                    }
                                    else
                                    {
                                        IsZoneLetterIdentified = true;
#if !PocketPC
                                        pZoneLetter = (ZoneLetter)Enum.Parse(typeof(ZoneLetter), Word.Substring(Count, 1), true);
#else
										//pZoneLetter = (ZoneLetter)Enum.ToObject(typeof(ZoneLetter), Word.Substring(Count, 1));

                                                        switch(Word.Substring(Count, 1).ToUpper())
                                                        {
                                                            case "C":
                                                                pZoneLetter = ZoneLetter.C;
                                                                break;
                                                            case "D":
                                                                pZoneLetter = ZoneLetter.D;
                                                                break;
                                                            case "E":
                                                                pZoneLetter = ZoneLetter.E;
                                                                break;
                                                            case "F":
                                                                pZoneLetter = ZoneLetter.F;
                                                                break;
                                                            case "G":
                                                                pZoneLetter = ZoneLetter.G;
                                                                break;
                                                            case "H":
                                                                pZoneLetter = ZoneLetter.H;
                                                                break;
                                                            case "J":
                                                                pZoneLetter = ZoneLetter.J;
                                                                break;
                                                            case "K":
                                                                pZoneLetter = ZoneLetter.K;
                                                                break;
                                                            case "L":
                                                                pZoneLetter = ZoneLetter.L;
                                                                break;
                                                            case "M":
                                                                pZoneLetter = ZoneLetter.M;
                                                                break;
                                                            case "N":
                                                                pZoneLetter = ZoneLetter.N;
                                                                break;
                                                            case "P":
                                                                pZoneLetter = ZoneLetter.P;
                                                                break;
                                                            case "Q":
                                                                pZoneLetter = ZoneLetter.Q;
                                                                break;
                                                            case "R":
                                                                pZoneLetter = ZoneLetter.R;
                                                                break;
                                                            case "S":
                                                                pZoneLetter = ZoneLetter.S;
                                                                break;
                                                            case "T":
                                                                pZoneLetter = ZoneLetter.T;
                                                                break;
                                                            case "U":
                                                                pZoneLetter = ZoneLetter.U;
                                                                break;
                                                            case "V":
                                                                pZoneLetter = ZoneLetter.V;
                                                                break;
                                                            case "W":
                                                                pZoneLetter = ZoneLetter.W;
                                                                break;
                                                            case "X":
                                                                pZoneLetter = ZoneLetter.X;
                                                                break;
                                                            default:
                                                                pZoneLetter = ZoneLetter.Unknown;
                                                                break;
                                                        }
#endif
                                    }
                                }
                                else if (char.IsNumber(Word, Count))
                                {
                                    ZoneNumberStringBuilder.Append(Word.Substring(Count, 1));
                                }
                                else
                                {
                                    return null;
                                    //throw new ArgumentException(Resources.Position_InvalidZoneNumber), "value");
                                }
                            }
                            // Set the zone number
                            string ZoneNumberString = ZoneNumberStringBuilder.ToString();
                            if (ZoneNumberString.Length == 0)
                            {
                                return null;
                                //throw new ArgumentException(Resources.Position_NoUTMZoneNumber), "value");
                            }
                            else
                            {
                                pZoneNumber = Convert.ToInt32(ZoneNumberString, culture);
                            }
                            // Make the new zone
                            //NewZone = pZoneLetter + pZoneNumber;
                            // The zone is handled
                            IsZoneHandled = true;
                            continue;
                        default:
                            // Fall through to easting/northing parsing
                            break;
                    }

                    bool IsEasting = false;
                    // Try to determine if this is an easting or a northing value
                    if (Word.IndexOf("E") != -1)
                        // The value is an easting
                        IsEasting = true;
                    else if (Word.IndexOf("N") != -1)
                        // The value is a northing
                        IsEasting = false;
                    else
                        // Is the eastig already handled
                        if (IsEastingHandled)
                            // Yes. This must be the northing
                            IsEasting = false;
                        else
                            // No.  The easting always comes first!
                            IsEasting = true;
                    // Now remove any letters and extract the value
                    string CleanedWord = Word.Replace("N", "").Replace("E", "");
                    if (IsEasting)
                    {
                        pEasting = new Distance(double.Parse(CleanedWord, culture), DistanceUnit.Meters);
                        IsEastingHandled = true;
                    }
                    else
                    {
                        pNorthing = new Distance(double.Parse(CleanedWord, culture), DistanceUnit.Meters);
                        IsNorthingHandled = true;
                    }


                }

                // If we have values, return the coordinate!
                if (IsZoneHandled && IsNorthingHandled && IsEastingHandled)
                {
                    return new Position(pZoneLetter, pZoneNumber, pEasting, pNorthing);
                }
                else
                {
                    return null;
                }
            }
            catch //(Exception ex)
            {
                return null;
                //#if PocketPC
                //                                throw new ArgumentException(Resources.Position_InvalidUtmCoordinate), ex);
                //#else
                //                throw new ArgumentException(Resources.Position_InvalidUtmCoordinate), "value", ex);
                //#endif
            }
        }

        public static Position ParseAsLatLong(string value, CultureInfo culture)
        {
            try
            {
                // Take the value and add spaces after N, S, E, or W
                value = value.Replace("N", "N ").Replace("S", "S ").Replace("E", "E ").Replace("W", "W ")
                    .Replace("n", "N ").Replace("s", "S ").Replace("e", "E ").Replace("w", "W ")
                    // And remove duplicate spaces
                    .Replace("  ", " ").Replace(" , ", ",").Trim();

                // How many words are there?
                string[] Values = value.Split(culture.TextInfo.ListSeparator.ToCharArray());

                // Just one?
                if (Values.Length == 1)
                {
                    // Yep.  Try to split using a space
                    Values = value.Split(' ');
                }

                // Parse out latitude and longitude
                bool IsLatitudeHandled = false;
                bool IsLongitudeHandled = false;
                Latitude pLatitude = Latitude.Empty;
                Longitude pLongitude = Longitude.Empty;

                foreach (string Word in Values)
                {
                    // Is this a latitude?
                    if (Word.IndexOf("N") != -1 || Word.IndexOf("n") != -1 || Word.IndexOf("S") != -1 || Word.IndexOf("s") != -1)
                    {
                        // Do we already have a latitude?
                        if (IsLatitudeHandled)
                        {
                            return null;
                        }
                        pLatitude = Latitude.Parse(Word, culture);
                        IsLatitudeHandled = true;
                    }
                    // Is this a longitude?
                    else if (Word.IndexOf("E") != -1 || Word.IndexOf("e") != -1 || Word.IndexOf("W") != -1 || Word.IndexOf("w") != -1)
                    {
                        // Do we already have a longitude?
                        if (IsLongitudeHandled)
                        {
                            return null;
                        }
                        pLongitude = Longitude.Parse(Word, culture);
                        IsLongitudeHandled = true;
                    }
                    else
                    {
                        // Take a guess
                        if (IsLongitudeHandled)
                        {
                            pLatitude = Latitude.Parse(Word, culture);
                            IsLatitudeHandled = true;
                        }
                        else if (IsLatitudeHandled)
                        {
                            pLongitude = Longitude.Parse(Word, culture);
                            IsLongitudeHandled = true;
                        }
                    }
                }

                if (IsLongitudeHandled && IsLatitudeHandled)
                {
                    return new Position(pLatitude, pLongitude);
                }
                else
                {
                    // Zero?
                    if (Values[0].Trim() == Values[1].Trim())
                    {
                        return new Position(Latitude.Parse(Values[0]), Longitude.Parse(Values[1]));
                    }

                    // Try to interpret as doubles (?)
                    // No.  The coordinate information is ambiguous!  Which is the lat
                    // and which is the long?
                    return null;
                }
            }
            catch
            {
                return null;
            }
        }


        #region IFormattable Members

        string IFormattable.ToString(string format, IFormatProvider formatProvider)
        {
            return this.ToString(format, (CultureInfo)formatProvider);
        }

        #endregion

#if !PocketPC || (PocketPC && Framework20)
		#region IXmlSerializable Members

#if Framework20
        public static XmlQualifiedName GetXmlSchema(XmlSchemaSet xs)
        {
            // Load it from a string
            xs.Add(Xml.SchemaSet);

            // And return the type
            return new XmlQualifiedName("Position", Xml.Namespace);
        }
#endif

        XmlSchema IXmlSerializable.GetSchema()
        {
            return null;
        }

        void IXmlSerializable.ReadXml(XmlReader reader)
        {
            ReadXml(reader);
        }

        void IXmlSerializable.WriteXml(XmlWriter writer)
        {
            WriteXml(writer);
        }

        /// <summary>
        /// Populates the current instance by reading XML from the specified reader.
        /// </summary>
        /// <param name="reader"></param>
        public void ReadXml(XmlReader reader)
        {
            if (reader == null)
                throw new ArgumentNullException("reader");

            // Create serializers used for child nodes
            XmlSerializer LatitudeSerializer = new XmlSerializer(typeof(Latitude));
            XmlSerializer LongitudeSerializer = new XmlSerializer(typeof(Longitude));

            // Rember which level we are processing at.
            int InitialDepth = reader.Depth;

            // Read into deeper content
            reader.Read();

            // Read in the datum if it is specified
            if (reader.HasAttributes)
            {                
                reader.MoveToAttribute("Datum");
                if(reader.Value.Length > 0)
                    pDatum = Datum.FromName(reader.Value);
            }

            // Read to the next item, or exit if any content is missing
            while(reader.Depth >= InitialDepth)
            {
                // If the depth is equal or less than the initial depth, stop
                switch (reader.NodeType)
                {
                    case XmlNodeType.Element:
                        switch (reader.Name.ToUpper(CultureInfo.InvariantCulture))
                        {
                            case "LATITUDE":
                                pLatitude = (Latitude)LatitudeSerializer.Deserialize(reader);
                                break;
                            case "LONGITUDE":
                                pLongitude = (Longitude)LongitudeSerializer.Deserialize(reader);
                                break;
                            case "ZONELETTER":
                            case "ZONENUMBER":
                            case "NORTHING":
                            case "EASTING":
                                // Rember which level we are processing at.
                                int NodeDepth = reader.Depth;
                                // Read into the next node
                                reader.Read();
                                // Keep reading until we're back out again
                                while (reader.Depth > NodeDepth)
                                {
                                    reader.Read();
                                }
                                // Read the end element
                                reader.Read();
                                break;
                            default:
                                // Read the next item
                                reader.Read();
                                break;
                        }
                        break;
                    case XmlNodeType.EndElement:
                        reader.Read();
                        return;
                    default:
                        // Read the next item
                        reader.Read();
                        break;
                }
            }
        }

        /// <summary>
        /// Writes the values of the current instance to the specified XML writer.
        /// </summary>
        /// <param name="writer"></param>
        public void WriteXml(XmlWriter writer)
        {
            if (writer == null)
                throw new ArgumentNullException("writer");

            // Defer to latitude and longitude serializers
            XmlSerializer LongitudeSerializer = new XmlSerializer(typeof(Longitude));
            XmlSerializer LatitudeSerializer = new XmlSerializer(typeof(Latitude));
            XmlSerializer DistanceSerializer = new XmlSerializer(typeof(Distance));

            writer.WriteStartAttribute("Datum", Xml.Namespace);
            writer.WriteString(Datum.ToString());
            writer.WriteEndAttribute();

            LatitudeSerializer.Serialize(writer, pLatitude);
            LongitudeSerializer.Serialize(writer, pLongitude);

            // Is the zone valid?  If not, skip it
            if (ZoneLetter != ZoneLetter.Unknown)
            {
                writer.WriteStartElement("ZoneLetter");
                writer.WriteString(ZoneLetter.ToString());
                writer.WriteEndElement();

                writer.WriteStartElement("ZoneNumber");
                writer.WriteString(ZoneNumber.ToString(CultureInfo.InvariantCulture));
                writer.WriteEndElement();

                writer.WriteStartElement("Easting");
                DistanceSerializer.Serialize(writer, Easting);
                writer.WriteEndElement();

                writer.WriteStartElement("Northing");
                DistanceSerializer.Serialize(writer, Northing);
                writer.WriteEndElement();
            }
        }


		#endregion
#endif

#if !PocketPC || DesignTime
        #region ISerializable Members

        [SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter=true)]  
        protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Latitude", pLatitude.DecimalDegrees);
            info.AddValue("Longitude", pLongitude.DecimalDegrees);
        }

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]  
        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);
        }

        #endregion
#endif
    }

    public sealed class PositionEventArgs : EventArgs
	{
		private Position oPosition;
		private DateTime oPositionTimeStamp;

		public static readonly new PositionEventArgs Empty = new PositionEventArgs(Position.Empty);

		public PositionEventArgs(Position position) 
            : base()
		{
			oPosition = position;
			oPositionTimeStamp = DateTime.MinValue;
		}

		public PositionEventArgs(Position position, DateTime timestamp)
		{
			oPosition = position;
			oPositionTimeStamp = timestamp;
		}

		public Position Position
		{
			get
			{
				return oPosition;
			}
		}

		public DateTime PositionTimeStamp
		{
			get
			{
				return oPositionTimeStamp;
			}
		}
	}

    /// <summary>Indicates a vertical slice of the Earth used as a starting point for UTM positions.</summary>
	public enum ZoneLetter
	{
		Unknown = 0,
		C,
		D,
		E,
		F,
		G,
		H,
		J,
		K,
		L,
		M,
		N,
		P,
		Q,
		R,
		S,
		T,
		U,
		V,
		W,
		X
	}

}