using System;
using System.Globalization;
using System.Security.Permissions;
using System.Reflection;
using System.Xml.Serialization;
#if !PocketPC || DesignTime
using System.ComponentModel;
using System.Runtime.Serialization;
using System.ComponentModel.Design.Serialization;
#endif

namespace GeoFramework
{
	/// <summary>
	/// Represents a position on Earth marked by latitude, longitude, and altitude.
	/// </summary>
	/// <remarks>
	/// 	<para>Instances of this class are guaranteed to be thread-safe because the class is
	///     immutable (its properties can only be changed via constructors).</para>
	/// </remarks>
#if !PocketPC || DesignTime
#if Framework20
    [Obfuscation(Feature = "renaming", Exclude = false, ApplyToMembers = true)]
    [Obfuscation(Feature = "controlflow", Exclude = true, ApplyToMembers = true)]
    [Obfuscation(Feature = "stringencryption", Exclude = false, ApplyToMembers = true)]
#endif
    [ImmutableObject(true)]
    [Serializable()]
#endif
    public class Position3D : Position
	{
		private Distance pAltitude;

		#region  Constructors

		public Position3D(Longitude longitude, Latitude latitude, Distance altitude) 
            : base(longitude, latitude)
		{   
			pAltitude = altitude;
		}

		/// <overloads>Creates a new instance.</overloads>
		public Position3D(Latitude latitude, Longitude longitude, Distance altitude) 
            : base(latitude, longitude)
		{
            pAltitude = altitude;
		}

		/// <summary>
		/// Creates a new instance using the specified latitude, longitude and
		/// Datum.
		/// </summary>
		/// <remarks>
		/// This constructor is commonly used when a new position must be created using an
		/// Datum other than the default of WGS1984.
		/// </remarks>
		/// <param name="latitude">An angle from the equator between -90 and +90.</param>
		/// <param name="longitude">An angle from the Prime Meridian between -90 and +90.</param>
		/// <param name="datum">An Datum to associate with the given latitude and 
		/// longitude.</param>
		public Position3D(Latitude latitude, Longitude longitude, Datum datum, Distance altitude) 
            : base(latitude, longitude, datum)
		{
			pAltitude = altitude;
		}

		/// <summary>
		/// Creates a new instance by parsing latitude and longitude from a single string.
		/// </summary>
		/// <param name="value">A <strong>String</strong> containing both a latitude and longitude to parse.</param>
		public Position3D(string value, string altitude) 
            : base(value, CultureInfo.CurrentCulture)
		{
            pAltitude = Distance.Parse(altitude, CultureInfo.CurrentCulture);
		}

        /// <summary>
        /// Creates a new instance by parsing latitude and longitude from a single string.
        /// </summary>
        /// <param name="value">A <strong>String</strong> containing both a latitude and longitude to parse.</param>
        public Position3D(string value, string altitude, CultureInfo culture)
            : base(value, culture)
        {
            pAltitude = Distance.Parse(altitude, culture);
        }

		//[CLSCompliant(false)]
		public Position3D(ZoneLetter zoneLetter, int zoneNumber, Distance easting, Distance northing, Distance altitude) : base(zoneLetter, zoneNumber, easting, northing)
		{
			pAltitude = altitude;
		}

		//[CLSCompliant(false)]
		public Position3D(ZoneLetter zoneLetter, int zoneNumber, Distance easting, Distance northing, Datum datum, Distance altitude) : base(zoneLetter, zoneNumber, northing, easting, datum)
		{
			pAltitude = altitude;
		}

#if !PocketPC || DesignTime
        /// <summary>
        /// Creates a new instance by de-serializing previously-serialized information.
        /// </summary>
        /// <param name="info">A <strong>SerializationInfo</strong> object containing serialized data.</param>
        /// <param name="context">A <strong>StreamingContext</strong> object describing the context of serialization.</param>
        /// <remarks>This constructor will de-serialize a previously-serialized object.</remarks>
        protected Position3D(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            pAltitude = (Distance)info.GetValue("Altitude", typeof(Distance));
        }

        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
        protected override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            // Set the altitude
            info.AddValue("Altitude", pAltitude);

            // And any other items
            base.GetObjectData(info, context);
        }
#endif

		#endregion

		/// <summary>Returns the location's distance above sea level.</summary>
		public Distance Altitude
		{
			get
			{
				return pAltitude;
			}
		}

		/// <summary>
		/// Converts the current instance into an Earth-centered, Earth-fixed (ECEF) Cartesian point.
		/// </summary>
		/// <returns></returns>
		public new CartesianPoint ToCartesianPoint()
		{
			return ToCartesianPoint(Ellipsoid.Wgs1984);
		}

		public new CartesianPoint ToCartesianPoint(Ellipsoid ellipsoid)
		{
			//	% LLA2ECEF - convert latitude, longitude, and altitude to
			//	%            earth-centered, earth-fixed (ECEF) cartesian
			//	% 
			//	% USAGE:
			//	% [x,y,z] = lla2ecef(lat,lon,alt)
			//	% 
			//	% x = ECEF X-coordinate (m)
			//	% y = ECEF Y-coordinate (m)
			//	% z = ECEF Z-coordinate (m)
			//	% lat = geodetic latitude (radians)
			//	% lon = longitude (radians)
			//	% alt = height above WGS84 ellipsoid (m)
			//	% 
			//	% Notes: This function assumes the WGS84 model.
			//	%        Latitude is customary geodetic (not geocentric).
			//	% 
			//	% Source: "Department of Defense World Geodetic System 1984"
			//	%         Page 4-4
			//	%         National Imagery and Mapping Agency
			//	%         Last updated June, 2004
			//	%         NIMA TR8350.2
			//	% 
			//	% Michael Kleder, July 2005
			//
			//	function [x,y,z]=lla2ecef(lat,lon,alt)
			double lat = Latitude.ToRadians().Value;
			double lon = Longitude.ToRadians().Value;
			//
			//	% WGS84 ellipsoid constants:
			//	a = 6378137;
			double a = ellipsoid.EquatorialRadius.ToMeters().Value;
			//	e = 8.1819190842622e-2;
			double e = ellipsoid.Eccentricity;
			//
			//	% intermediate calculation
			//	% (prime vertical radius of curvature)
			//	N = a ./ sqrt(1 - e^2 .* sin(lat).^2);
			double N = a / Math.Sqrt(1.0 - Math.Pow(e, 2) * Math.Pow(Math.Sin(lat), 2));
			//
			//	% results:
			//	x = (N+alt) .* cos(lat) .* cos(lon);
			double alt = pAltitude.ToMeters().Value;
			double x = (N + alt) * Math.Cos(lat) * Math.Cos(lon);
			//	y = (N+alt) .* cos(lat) .* sin(lon);
			double y = (N + alt) * Math.Cos(lat) * Math.Sin(lon);
			//	z = ((1-e^2) .* N + alt) .* sin(lat);
			double z = ((1.0 - Math.Pow(e, 2)) * N + alt) * Math.Sin(lat);
			//
			//	return

			return new CartesianPoint(
				new Distance(x, DistanceUnit.Meters),
				new Distance(y, DistanceUnit.Meters),
				new Distance(z, DistanceUnit.Meters));
		}
	}
}
