using System;
using System.IO;
using System.Globalization;
using System.Collections;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Security.Permissions;
#if !PocketPC || DesignTime
using System.Runtime.Serialization;
using System.ComponentModel;
#endif
using System.Reflection;

namespace GeoFramework
{

	/// <summary>Represents an arbitrary collection of locations on the Earth's 
	/// surface.</summary>
	/// <remarks>
	/// 	<para>This collection provides methods which work with a series of locations,
	///     typically a travelled route. Features are provided for calculating distance, area,
	///     perimeter, travel time, minimum travel speed, and nearest relative items.</para>
	/// 	<para>This class is also used as a base class for other collections such as the
	///     <strong>Shape</strong> object in <strong>GIS.NET</strong>, and the
	///     <strong>WaypointCollection</strong> class for Garmin waypoints.</para>
	/// 	<para>Instances of this class are likely to be thread-safe because the class uses
	///     thread synchronization before modifying the underlying collection. Lock the
	///     <strong>SyncRoot</strong> property before enumerating through this
	///     collection.</para>
	/// </remarks>
#if !PocketPC || DesignTime
#if Framework20
    [Obfuscation(Feature = "renaming", Exclude = false, ApplyToMembers = true)]
    [Obfuscation(Feature = "controlflow", Exclude = true, ApplyToMembers = true)]
    [Obfuscation(Feature = "stringencryption", Exclude = false, ApplyToMembers = true)]
#endif
    [TypeConverter(typeof(ExpandableObjectConverter))]
    [Serializable()]
#endif
#if Framework20
    [XmlSchemaProvider("GetXmlSchema")]
#endif
    public class PositionCollection : IEnumerable, IList, ICollection, IFormattable
#if !PocketPC || (PocketPC && Framework20)
	, IXmlSerializable
#endif  
#if !PocketPC || DesignTime
, ISerializable
#endif
    {
        #region Private Variables
        private ArrayList pItems;
		private GeographicRectangle pExtent = GeographicRectangle.Empty;
		private bool pIsExtentCalculated;
        private bool pIsPoint = false;

#if !PocketPC
		private Area pArea = Area.Empty;
        private bool pIsAreaCalculated;
        private Distance pPerimeter = Distance.Empty;
        private bool pIsPerimeterCalculated;
#endif
 
		#endregion

		#region Constructors

		/// <summary>
		/// Creates a new instance.
		/// </summary>
		public PositionCollection() 
            : this(8)
		{
#if !PocketPC
            // Flag that we don't need recalculation
            pIsAreaCalculated = true;
            pIsExtentCalculated = true;
            pIsPerimeterCalculated = true;
#endif
        }

		public PositionCollection(int capacity) 
		{
			pItems = new ArrayList(capacity);

#if !PocketPC
            // Flag that we don't need recalculation
            pIsAreaCalculated = true;
            pIsExtentCalculated = true;
            pIsPerimeterCalculated = true;
#endif
        }

        public PositionCollection(Position[] items)
        {
            if (items == null)
                throw new ArgumentNullException("items");

            // Create a new arraylist using the capacity of the "items" array
            pItems = new ArrayList(items.Length);

            // Add all positions to the collection.
            this.AddRange(items);
        }

        #if !PocketPC || DesignTime
        /// <summary>
        /// Creates a new instance by de-serializing previously-serialized information.
        /// </summary>
        /// <param name="info">A <strong>SerializationInfo</strong> object containing serialized data.</param>
        /// <param name="context">A <strong>StreamingContext</strong> object describing the context of serialization.</param>
        /// <remarks>This constructor will de-serialize a previously-serialized object.</remarks>
        protected PositionCollection(SerializationInfo info, StreamingContext context)
        {
            // Initialize the array
            pItems = new ArrayList(info.MemberCount);

            // Now deserialize each item
            for (int i = 0; i < info.MemberCount; i++)
            {
                Position x = (Position)info.GetValue("Item" + i.ToString(), typeof(Position));
                pItems.Add(x);
            }
        }
#endif

        #endregion

        /// <summary>Returns the underlying list of items.</summary>
		public ArrayList InnerList
		{
			get
			{
				return pItems;
			}
		}


		#region Static Members
		
		//public static readonly PositionCollection Empty = new PositionCollection(0);

		#endregion

		#region Events

        /// <summary>
        /// Occurs when a Position object has been added to the collection.
        /// </summary>
		public event PositionEventHandler ItemAdded;
        /// <summary>
        /// Occurs when a Position object has been removed from the collection.
        /// </summary>
        public event PositionEventHandler ItemRemoved;
        /// <summary>
        /// Occurs when a Position object in the collection has been replaced with a new Position object.
        /// </summary>
        public event PositionEventHandler ItemReplaced;
        /// <summary>
        /// Occurs when the entire collection has been emptied of all Position objects.
        /// </summary>
        public event EventHandler ClearCompleted;

		#endregion


		/// <summary>Returns the number of items in the collection.</summary>
		public int Count
		{
			get
			{
				return pItems.Count;
			}
		}

		/// <summary>Creates a new collection from the specified array.</summary>
        public static PositionCollection FromArray(Position[] array)
        {
            return new PositionCollection(array);
        }

        public override string ToString()
        {
            return ToString("G", CultureInfo.CurrentCulture);
        }

        public virtual string ToString(string format)
        {
            return ToString(format, CultureInfo.CurrentCulture);
        }

		public virtual string ToString(string format, CultureInfo culture)
        {
            return "PositionCollection (" + pItems.Count.ToString(format, culture) + " items)";
        }


		/// <summary>Converts the collection to an array of <strong>Position</strong> objects.</summary>
        public Position[] ToArray()
        {
            return (Position[])pItems.ToArray(typeof(Position));
        }

		/// <summary>
		/// Indicates if the collection can be safely accessed from multiple threads at
		/// once.
		/// </summary>
		public bool IsSynchronized
		{
			get
			{
				return true;
			}
		}

        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        public bool IsFixedSize
        {
            get
            {
                return false;
            }
        }

		/// <summary>
		/// Returns an object used to lock the collection before enumeration or
		/// modification.
		/// </summary>
		public object SyncRoot
		{
			get
			{
				return pItems.SyncRoot;
			}
		}

		#region Public Properties

        /// <summary>Returns the first item in the collection, if one exists.</summary>
		/// <remarks>
		/// This property is typically used to access the first item in the collection during
		/// a calculation which involves the entire collection.
		/// </remarks>
		/// <value>
		/// The first <strong>Position</strong> in the collection or <strong>null</strong>
		/// (<strong>Nothing</strong> in Visual Basic) if the collection is empty.
		/// </value>
		public Position First
		{
			get
			{
                return this.pItems.Count == 0 ? null : pItems[0] as Position;
			}
		}

		/// <summary>Returns a <strong>Position</strong> object at the specified index.</summary>
		public virtual Position this[int index]
		{
			get
			{
				return (Position)pItems[index];
			}
			set
			{
                // Get the old value which will be replaced
                Position OldValue = pItems[index] as Position;
                
                // Replace the item with the new value
                lock (pItems.SyncRoot)
                {
                    pItems[index] = value;
                }

                // Flag that other items needs to be recalculated
                pIsExtentCalculated = false;

#if !PocketPC
                pIsAreaCalculated = false;
                pIsPerimeterCalculated = false;
#endif

				// Broadcast that the item has been replaced
				OnItemReplaced(index, OldValue, value);
			}
		}

//		/// <summary>Returns an enumerable list of all Positions in the collection.</summary>
//		public ArrayList Items
//		{
//			get
//			{
//				return InnerList;
//			}
//		}

		/// <summary>Returns the last item in the collection.</summary>
		public Position Last
		{
			get
			{
				return this.pItems.Count == 0 ? null : pItems[pItems.Count - 1] as Position;
			}
		}

        protected bool IsExtentCalculated
        {
            get
            {
                return pIsExtentCalculated;
            }
            set
            {
                pIsExtentCalculated = value;
            }
        }

#if !PocketPC
        protected bool IsAreaCalculated
        {
            get
            {
                return pIsAreaCalculated;
            }
            set
            {
                pIsAreaCalculated = value;
            }
        }

        protected bool IsPerimeterCalculated
        {
            get
            {
                return pIsPerimeterCalculated;
            }
            set
            {
                pIsPerimeterCalculated = value;
            }
        }
#endif

		/// <summary>Returns a rectangle which encloses all points in the collection.</summary>
		public virtual GeographicRectangle Extent
		{
			get
			{
                // Has the extent been recalculated lately?
                if (pIsExtentCalculated)
                    return pExtent;

                    if (this.Count == 0)
                        return GeographicRectangle.Empty;
                
                    // Start with the largest possible values
                    double Left = 181.0;
                    double Top = -91.0;
                    double Bottom = 91.0;
                    double Right = -181.0;

                    // Now find the smallest bounding box
                    lock (pItems.SyncRoot)
                    {
                        foreach (Position item in pItems)
                        {
                            // Cache the latitude and longitude
                            double TestLatitude = item.Latitude.DecimalDegrees;
                            double TestLongitude = item.Longitude.DecimalDegrees;

                            // Compare all possible values to shrink the bounding box
                            if (TestLatitude > Top)
                                Top = item.Latitude.DecimalDegrees;
                            if (TestLatitude < Bottom)
                                Bottom = item.Latitude.DecimalDegrees;
                            if (TestLongitude < Left)
                                Left = item.Longitude.DecimalDegrees;
                            if (TestLongitude > Right)
                                Right = item.Longitude.DecimalDegrees;
                        }
                    }

                    // Expand the box to enclose the coordinate
                    pExtent = new GeographicRectangle(new Latitude(Top), new Longitude(Left), new Latitude(Bottom), new Longitude(Right));

                    // We no longer need to calculate it
                    pIsExtentCalculated = true;
                
                // Return the calculated extent
				return pExtent;
			}
		}

		#endregion

		#region Protected Methods

        protected void SetExtent(GeographicRectangle value)
        {
            pExtent = value;
        }

#if !PocketPC
        protected void SetArea(Area value)
        {
            pArea = value;
        }

        protected void SetPerimeter(Distance value)
        {
            pPerimeter = value;
        }
#endif

		#endregion

		#region Public Methods

		/// <summary>Adds a <strong>Position</strong> object to the end of the collection.</summary>
		/// <remarks>
		/// <para>This method is used to add a single Position object to the end of the collection.  Calling
		/// this method causes the current extent, area and perimeter calculations for the collection
		/// to become invalid.</para>
		/// </remarks>
		public virtual void Add(Position value)
		{
            // Add the new item to the collection
            lock (pItems.SyncRoot)
            {
                pItems.Add(value);
            }

            // Flag that other items needs to be recalculated
            pIsExtentCalculated = false;

#if !PocketPC
                pIsAreaCalculated = false;
                pIsPerimeterCalculated = false;
#endif

			// Broadcast that the item has been added
			OnItemAdded(value);
		}

		/// <summary>Returns the allocated size of the underlying array.</summary>
		public int Capacity
		{
			get
			{
				return pItems.Capacity;
			}
			set
			{
                lock (pItems.SyncRoot)
                {
                    pItems.Capacity = value;
                }
			}
		}

		/// <summary>Removes all items from the collection.</summary>
		public void Clear()
		{
            // Remove all items in the collection
            lock (pItems.SyncRoot)
            {
                pItems.Clear();
            }

            // Reset all values
            pExtent = GeographicRectangle.Empty;
            pIsExtentCalculated = true;

#if !PocketPC
            pArea = Area.Empty;
            pPerimeter = Distance.Empty;

            // Flag that other items don't need to be calculated
            pIsAreaCalculated = true;
            pIsPerimeterCalculated = true;
#endif

            // Broadcast of the change to the collection
            OnClearCompleted();
		}

        /// <summary>Adds a collection of Position objects to the current collection.</summary>
		/// <remarks>
		///	 <para>This method is typically used to combine a separate collection of Position
		///	 objects into the collection.</para>
		/// </remarks>
		/// <param name="collection">A collection of <strong>Position</strong> objects to add.</param>
		public virtual void AddRange(ICollection collection)
		{
			// Add each item to the collection
			lock(collection.SyncRoot)
			{
				foreach(Position position in collection)
					Add(position);
			}
		}

        public virtual void AddRange(Position[] array)
        {
            for (int count = 0; count < array.Length; count++)
                Add(array[count]);
        }

        /// <summary>
		/// Removes a particular instance of a <strong>Position</strong> object from the
		/// collection.
		/// </summary>
		public virtual void Remove(Position value)
		{
            // Remove the item from the collection
            lock (pItems.SyncRoot)
            {
                pItems.Remove(value);
            }

            // Flag that other items needs to be recalculated
            pIsExtentCalculated = false;

#if !PocketPC
            pIsAreaCalculated = false;
            pIsPerimeterCalculated = false;
#endif

			// Broadcast of the change to the collection
			OnItemRemoved(value);
		}

		/// <summary>
		/// Indicates if the specified position is surrounded by all points in the
		/// collection.
		/// </summary>
		/// <returns>A <strong>Boolean</strong>, <strong>True</strong> if the position is inside of the shape created by the collection.</returns>
		/// <remarks><para>Assuming that the PositionCollection defines a shape, this method will
		/// indicate if the specified point is inside of the shape or outside of it.  This method works with
		/// complicated shapes, including polygons containing holes.</para>
		/// </remarks>
		public bool IsEnclosing(Position position)
		{
			if(IsLine || !Extent.IsOverlapping(position))
				return false;

			// First, let's see if the position is at all inside the bounding box
//			if(!Extent.IsEnclosing(position)
//				|| IsLine) 
//				return false;		

			//			// cn_PnPoly(): crossing number test for a point in a polygon
			//			//      Input:   P = a point,
			//			//               V[] = vertex points of a polygon V[n+1] with V[n]=V[0]
			//			//      Return:  0 = outside, 1 = inside
			//			// This code is patterned after [Franklin, 2000]
			//			int
			//				cn_PnPoly( Point P, Point* V, int n )
			//			{
			//				int    cn = 0;    // the crossing number counter
			//
			//				// loop through all edges of the polygon
			//				for (int i=0; i<n; i++) 
			//				{    // edge from V[i] to V[i+1]
			//					if (((V[i].y <= P.y) && (V[i+1].y > P.y))    // an upward crossing
			//						|| ((V[i].y > P.y) && (V[i+1].y <= P.y))) 
			//					{ // a downward crossing
			//						// compute the actual edge-ray intersect x-coordinate
			//						float vt = (float)(P.y - V[i].y) / (V[i+1].y - V[i].y);
			//						if (P.x < V[i].x + vt * (V[i+1].x - V[i].x)) // P.x < intersect
			//							++cn;   // a valid crossing of y=P.y right of P.x
			//					}
			//				}
			//				return (cn&1);    // 0 if even (out), and 1 if odd (in)
			//
			//			}

			// cn_PnPoly(): crossing number test for a point in a polygon
			//      Input:   P = a point,
			//               V[] = vertex points of a polygon V[n+1] with V[n]=V[0]
			//      Return:  0 = outside, 1 = inside
			// This code is patterned after [Franklin, 2000]
			//			int
			//				cn_PnPoly( Point P, Point* V, int n )
			//			{
			int    cn = 0;    // the crossing number counter

			// loop through all edges of the polygon
			for (int i = 0; i < Count - 1; i++) 
			{    
				// edge from this[i] to this[i+1]
				if (((this[i].Latitude.DecimalDegrees >= position.Latitude.DecimalDegrees) 
					&& (this[i+1].Latitude.DecimalDegrees < position.Latitude.DecimalDegrees))    // an upward crossing
					|| ((this[i].Latitude.DecimalDegrees < position.Latitude.DecimalDegrees) 
					&& (this[i+1].Latitude.DecimalDegrees >= position.Latitude.DecimalDegrees))) 
				{ // a downward crossing
					// compute the actual edge-ray intersect x-coordinate
					double vt = (position.Latitude.DecimalDegrees - this[i].Latitude.DecimalDegrees) / (this[i+1].Latitude.DecimalDegrees - this[i].Latitude.DecimalDegrees);
					if (position.Longitude.DecimalDegrees < this[i].Longitude.DecimalDegrees + vt * (this[i+1].Longitude.DecimalDegrees - this[i].Longitude.DecimalDegrees)) // position.Longitude.DecimalDegrees < intersect
						++cn;   // a valid crossing of y=position.Latitude.DecimalDegrees right of position.Longitude.DecimalDegrees
				}
			}

			return (cn & 1) == 1;    // 0 if even (out), and 1 if odd (in)

//			// The position is within this area.  So, determine how many intersections take place
//			Position p1 = First;
//			double xinters;
//			int counter = 0;
//			lock(pItems.SyncRoot)
//			{
//				for(int i = 1; i < Count; i++)
//				{
//					Position p2 = this[i];
//					if(position.Latitude.DecimalDegrees > Math.Min(p1.Latitude.DecimalDegrees, p2.Latitude.DecimalDegrees)
//						&& position.Latitude.DecimalDegrees <= Math.Max(p1.Latitude.DecimalDegrees, p2.Latitude.DecimalDegrees)
//						&& position.Longitude.DecimalDegrees <= Math.Max(p1.Longitude.DecimalDegrees, p2.Longitude.DecimalDegrees)
//						&& p1.Latitude.DecimalDegrees != p2.Latitude.DecimalDegrees)
//					{
//						xinters = (position.Latitude.DecimalDegrees - p1.Latitude.DecimalDegrees) 
//							* (p2.Longitude.DecimalDegrees - p1.Longitude.DecimalDegrees) 
//							/ (p2.Latitude.DecimalDegrees - p1.Latitude.DecimalDegrees) 
//							+ p1.Longitude.DecimalDegrees;
//						if(p1.Longitude.DecimalDegrees == p2.Longitude.DecimalDegrees 
//							|| position.Longitude <= xinters)
//							counter++;
//					}
//					p1 = p2;
//				}
//			}
//			return (counter % 2) != 0;
		}

		/// <summary>Indicates if the last point in the series does not match the first point.</summary>
		/// <remarks>
		/// This method is used to tell if a polygon is "closed," where the first and last points are the same.
		/// A polygon must be closed in order for a fill brush to be applied.  "Open" polygons are typically
		/// objects such as lines (i.e. railroads, roads, trails).</remarks>
		public bool IsLine
		{
            get 
            {
                if (this.pItems.Count == 0)
                    return false;
                else
                    return !this.Last.Equals(this.First) && (this.First != null) && !this.pIsPoint; 
            }
        }

		/// <summary>Indicates if the last point in the series is the same as the first point.</summary>
        public bool IsPolygon
        {
            get
            {
                if (this.pItems.Count == 0)
                    return false;
                else
                    return this.Last.Equals(this.First) && (this.First != null) && !this.pIsPoint;
            }
        }

        public bool IsPoint
        {
            get { return this.pIsPoint; }
            set { this.pIsPoint = value; }
        }

//			int counter = 0;
//			int i;
//			double xinters;
//			Point p1,p2;
//
//			p1 = polygon[0];
//			for (i=1;i<=N;i++) 
//			{
//				p2 = polygon[i % N];
//				if (p.Latitude > MIN(p1.Latitude,p2.Latitude)) 
//				{
//					if (p.Latitude <= MAX(p1.Latitude,p2.Latitude)) 
//					{
//						if (p.Longitude  <= MAX(p1.Longitude ,p2.Longitude )) 
//						{
//							if (p1.Latitude != p2.Latitude) 
//							{
//								xinters = (p.Latitude-p1.Latitude)*(p2.Longitude -p1.Longitude )/(p2.Latitude-p1.Latitude)+p1.Longitude ;
//								if (p1.Longitude  == p2.Longitude  || p.Longitude  <= xinters)
//									counter++;
//							}
//						}
//					}
//				}
//				p1 = p2;
//			}
//
//			if (counter % 2 == 0)
//				return(OUTSIDE);
//			else
//				return(INSIDE);
//
//		}
//
//#define MIN(x,y) (x < y ? x : y)
//#define MAX(x,y) (x > y ? x : y)
//#define INSIDE 0
//#define OUTSIDE 1
//
//		typedef struct 
//				{
//					double x,y;
//				} Point;
//
//		int InsidePolygon(Point *polygon,int N,Point p)
//		{
//		}

		/// <summary>
		/// Removes an item from the collection which is located at the specified
		/// index.
		/// </summary>
		/// <param name="index">
		/// A zero-based <strong>Integer</strong> indicating the position of the item to
		/// remove.
		/// </param>
		public virtual void RemoveAt(int index)
		{
			// Remove it
			Remove(this[index]);
		}

		public virtual void RemoveRange(ICollection collection)
		{
			lock(collection.SyncRoot)
			{
				foreach (Position position in collection)
					Remove(position);
			}
		}

		/// <summary>
		/// Removes a series of objects from the collection starting at the specified
		/// index.
		/// </summary>
		public void RemoveRange(int index, int count)
		{
			// Loop through the collection and remove items
			for(int itemIndex = index; itemIndex <= index + count; itemIndex++)
			{
				// And remove it
				Remove(this[itemIndex]);
			}
		}

		#region Protected Methods

		/// <summary>Occurs when a new item has been added to the collection.</summary>
		protected virtual void OnItemAdded(Position value)
		{
            // Raise an event if one is hooked in
			if(ItemAdded != null)
#if PocketPC && !Framework20
				ItemAdded(this, new PositionEventArgs(value));
#else
				DynamicEvent.BeginInvoke(ItemAdded, this, new PositionEventArgs(value));
#endif
        }

		/// <summary>Occurs when an item has been removed from the collection.</summary>
        protected virtual void OnItemRemoved(Position value)
        {
			if(ItemRemoved != null)
#if PocketPC && !Framework20
				ItemRemoved(this, new PositionEventArgs(value));
#else
                DynamicEvent.BeginInvoke(ItemRemoved, this, new PositionEventArgs(value));
#endif
        }

		/// <summary>Occurs when all items have been removed from the collection.</summary>
		protected virtual void OnClearCompleted()
		{			
			if(ClearCompleted != null)
#if PocketPC && !Framework20
				ClearCompleted(this, EventArgs.Empty);
#else
                DynamicEvent.BeginInvoke(ClearCompleted, this);
#endif
		}

		/// <summary>Occurs when an item in the collection has been replaced with another item.</summary>
        protected virtual void OnItemReplaced(int index, Position oldValue, Position newValue)
        {
            if (ItemReplaced != null)
#if PocketPC && !Framework20
				ItemReplaced(this, new PositionEventArgs(oldValue));
#else
                DynamicEvent.BeginInvoke(ItemReplaced, this, new PositionEventArgs(oldValue));
#endif
        }

		#endregion

		/// <summary>Sorts the contents of the collection.</summary>
		public virtual void Sort()
		{
			lock(pItems.SyncRoot)
			{
				pItems.Sort();
			}
		}

		/// <summary>Sorts the collection using the specified custom comparison method.</summary>
		/// <param name="comparer">
		/// An <strong>IComparer</strong> object which specifies how one
		/// <strong>Position</strong> ranks compared to another <strong>Position</strong>.
		/// </param>
		public virtual void Sort(IComparer comparer)
		{
			lock(pItems.SyncRoot)
			{
				pItems.Sort(comparer);
			}
		}

		/// <summary>
		/// Sorts a subset of the collection using the specified custom comparison
		/// method.
		/// </summary>
		/// <param name="index">
		/// An <strong>Integer</strong> indicating the location from which to start
		/// sorting.
		/// </param>
		/// <param name="count">
		/// An <strong>Integer</strong> specifying the number of items to include in the
		/// sort.
		/// </param>
		/// <param name="comparer">
		/// An <strong>IComparer</strong> object which specifies how one
		/// <strong>Position</strong> ranks compared to another <strong>Position</strong>.
		/// </param>
		public virtual void Sort(int index, int count, IComparer comparer)
		{
			lock(pItems.SyncRoot)
			{
				pItems.Sort(index, count, comparer);
			}
		}

		// Returns the total area of the points, if closed into a polygon
		//Public ReadOnly Property TotalArea() As Double
		//	Get

		//	End Get
		//End Property

//		// Translates the entire collection by the specified bearing and distance
//		/// <summary>Shifts the entire collection by the specified bearing and distance.</summary>
//		/// <remarks>
//		/// This method is typically used to make adjustments to an entire collection of
//		/// positions at one time. For example, if each position is located 10 kilometers too far
//		/// to the north, this method can correct the problem by translating each item 10
//		/// kilometers to the south.
//		/// </remarks>
//		/// <param name="bearing">
//		/// An <strong>Angle</strong> indicating the direction in which to shift all
//		/// positions.
//		/// </param>
//		/// <param name="distance">A <strong>Distance</strong> indicating the amount of shift 
//		/// applied.</param>
//		public void Translate(Angle direction, Distance distance)
//		{
//			// Move all items in the collection
//			foreach (Position item in List)
//			{
//				// Translate the item
//				item.Translate(direction, distance);
//			}
//			// Translate the bounding box as well
//			pExtent = pExtent.TranslateTo(direction, distance);
//		}

		// Returns the time required to travel all points at the given speed
		/// <summary>
		/// Returns the minimum time required to travel through each point in the collection
		/// at the given speed.
		/// </summary>
		/// <returns>A <strong>TimeSpan</strong> containing the length of time required.</returns>
		/// <remarks>
		///	 <para>This method is typically used to see if an entire PositionCollection can be travelled in
		///	 time at the current speed. The MinimumSpeed method can be used to calculate what
		///	 minimum speed is required to reach a destination within a set period of
		///	 time.</para>
		/// 
		///	 <para><img src="BestPractice.jpg"/></para><para><strong>Avoid suggesting that the
		///	 user increase or decrease speed.</strong></para>
		/// 
		///	 <para>The MinimumTime method might yield a result which suggests that the user can
		///	 make a destination on time by increasing (or decreasing) their current speed. Avoid
		///	 making recommendations which might cause the user to disobey a maximum (or minimum)
		///	 speed limit.</para>
		/// </remarks>
		/// <param name="speed">
		/// A <strong>Speed</strong> indicating how fast to travel through all points in the
		/// collection.
		/// </param>
		public TimeSpan GetMinimumTravelTime(Speed speed)
		{
			//Dim AdjustedDestination As Position = destination.ToEllipsoid(Ellipsoid)
			double TravelDistance = Perimeter.ToMeters().Value;
			double TravelSpeed = speed.ToMetersPerSecond().Value;
			// Perform the calculation
			return new TimeSpan((long)(TravelDistance / TravelSpeed * TimeSpan.TicksPerSecond));
		}

		// Returns the speed required to travel from start to finish in the given time
		/// <summary>
		/// Returns the minimum speed required to travel through each point in the collection
		/// within the given time.
		/// </summary>
		/// <remarks>
		///	 <para>This method is typically used to see if an entire PositionCollection can be travelled at
		///	 the current speed within the specified length of time. An application could also
		///	 use this method to calculate how early (or late) arrival time would be if
		///	 modifications to the speed were made.</para>
		/// 
		///	 <para><img src="BestPractice.jpg"/></para><para><strong>Avoid suggesting that the
		///	 user increase or decrease speed.</strong></para>
		/// 
		///	 <para>The MinimumSpeed method does not take into account any local traffic laws. As
		///	 a result, it might return a result of 90MPH for a PositionCollection that goes through
		///	 neighborhoods. Recommendations to slow down should also be avoided for roads that
		///	 require a minimum speed, such as an interstate highway.</para>
		/// </remarks>
		public Speed GetMinimumTravelSpeed(TimeSpan time)
		{
			// Now calculate the distance
			double TravelDistance = Perimeter.ToMeters().Value;
			// Perform the calculation
			return new Speed(TravelDistance / time.TotalSeconds, SpeedUnit.MetersPerSecond);
		}	

        ///// <summary>
        ///// Forces the extent of the collection to be recalculated.
        ///// </summary>
        ///// <remarks>In some special situations, the extent of the collection may need to be recalculated.  This
        ///// method is used to force the extent to be recalculated.</remarks>
        //protected void InvalidateExtent()
        //{
        //    pExtentIsNeeded = true;
        //}

        //protected void InvalidateArea()
        //{
        //    pAreaIsNeeded = true;
        //}

        //protected void InvalidatePerimeter()
        //{
        //    pPerimeterIsNeeded = true;
        //}

		/// <summary>
		/// Returns the item in the collection which is nearest to the specified position.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> to compare with the collection.</param>
		/// <returns>An <strong>ArrayList</strong> containing all items within the specified range.</returns>
		//[CLSCompliant(false)]
		public PositionCollection ItemsWithinRange(Position value, Distance range)
		{
            // Make sure the distance is greater than zero
            if(range.Value < 0)
				throw new ArgumentException(Resources.PositionCollection_RangeMustBeAboveZero, "value");

            // Make a list to store results
            PositionCollection Results = new PositionCollection();

			// Convert the range to kilometers for fastest comparison
			double MaximumRange = range.ToMeters().Value;

			// Compare each item in the collection
			lock(pItems.SyncRoot)
			{
				foreach (Position CollectionMember in pItems)
				{
					// Is the distance within range?
                    if (value.DistanceTo(CollectionMember).ToMeters().Value <= MaximumRange)
						// Add the result to the collection
						Results.Add(CollectionMember);
				}
			}

			// Return the nearest items
			return Results;
		}

		/// <summary>
		/// Returns the specified number of points nearest to the specified position.
		/// </summary>
		/// <param name="value"></param>
		/// <param name="count"></param>
		/// <returns></returns>
		public PositionCollection NearestPoints(Position value, int count)
		{
			ArrayList SortedList = new ArrayList(pItems.Count);
			// Make a collection of all points
			lock(pItems.SyncRoot)
			{
				foreach(Position item in pItems)
				{
					RelativePosition NewItem = new RelativePosition(item, item.DistanceTo(value));
					SortedList.Add(NewItem);
				}
			}
			// Now sort the collection
			SortedList.Sort();
			// And add the requested number of items
			PositionCollection Result = new PositionCollection(count);
			for(int index = 0; index < count; index++)
			{
				RelativePosition item = (RelativePosition)SortedList[index];
				Result.Add(item.Position);
			}
			return Result;
		}

		internal class RelativePosition : IComparable
		{
			public Position Position;
			public Distance Distance;

			public RelativePosition(Position position, Distance distance)
			{
				Position = position; 
				Distance = distance;
			}

//			public int CompareTo(RelativePosition obj)
//			{
//				return Distance.CompareTo(obj.Distance);
//			}

			#region IComparable Members

			int IComparable.CompareTo(object obj)
			{
				return Distance.CompareTo(((RelativePosition)obj).Distance);
			}

			#endregion
		}

		/// <summary>
		/// Returns the two points nearest each other in the current and specified collections.
		/// </summary>
		/// <param name="collection"></param>
		/// <returns></returns>
		public PositionCollection NearestPoints(PositionCollection collection)
		{
			PositionCollection Result = new PositionCollection(2);
			// Given two shapes, where are they in relation to each other?
			// Where are the centroids of the two collections in relation to each other?
			Distance SmallestDistance = Distance.Maximum;
			Position NearestLeft = null;
			Position NearestRight = null;
			lock(pItems.SyncRoot)
			{
				lock(collection.SyncRoot)
				{
					foreach(Position item in pItems)
					{
						foreach(Position item2 in collection)
						{
							Distance Dist = item.DistanceTo(item2);
							if(Dist < SmallestDistance)
							{
								SmallestDistance = Dist;
								NearestLeft = item;
								NearestRight = item2;
							}
						}
					}
				}
            }
			// Return the result
			Result.Add(NearestLeft);
			Result.Add(NearestRight);
			return Result;
		}

		

		/// <summary>
		/// Returns the point in the collection which is nearest to the specified position and also within the specified distance.
		/// </summary>
		/// <param name="value">A <strong>Position</strong> specifying a location to search around.</param>
		/// <returns>A <strong>Position</strong> from the collection which is located closer to the specified position than any other point.</returns>
		/// <remarks>This method is useful for determining which of several points is nearest to the current location.
        /// This method is typically used to locate an upcoming intersection, waypoint, or other point of interest.</remarks>
		public Position Nearest(Position value)
		{
            int limit = pItems.Count;

            // Anything?
            if (limit == 0)
                return null;

            double shortestDistance = double.MaxValue;
            Position shortestItem = Position.Empty;

            // Go through each location in the list
            for (int index = 0; index < limit; index++)
            {
                // If the distance is shorter, remember the index
                Position item = (Position)pItems[index];
                Distance distance = item.DistanceTo(value).ToMeters();
                if (distance.Value < shortestDistance)
                {
                    shortestItem = item;
                    shortestDistance = distance.Value;
                }
            }

            // Return the item at the selected index
            return shortestItem;
		}

		//		Private Function SignedPolygonArea() As Single
		//												 Dim pt As Integer
		//															Dim area As Single
		//
		//																		 ' Add the first point to the end.
		//		ReDim Preserve m_Points(1 To m_NumPoints + 1)
		//		m_Points(m_NumPoints + 1) = m_Points(1)
		//
		//		' Get the areas.
		//		For pt = 1 To m_NumPoints
		//					   area = area + _
		//										 (m_Points(pt + 1).Longitude  - m_Points(pt).Longitude ) * _
		//																					 (m_Points(pt + 1).Latitude + m_Points(pt).Latitude) / 2
		//		Next pt
		//
		//			' Return the result.
		//		SignedPolygonArea = area
		//								End Function
		//
		//										' Find the polygon's area.
		//		Public Function PolygonArea() As Single
		//										  ' Return the absolute value of the signed area.
		//		PolygonArea = Abs(SignedPolygonArea())
		//		End Function


		/// <summary>
		/// Returns the area enclosed by all points in the collection.
		/// </summary>
		/// <remarks>
		/// <para>This method uses the Double Meridian Distance method to determine the total area enclosed
		/// by all points in the collection.  If the collection represents an "open" polygon, where the first and
		/// last points are not equivalent, the polygon is temporarily closed to perform this calculation.</para>
		/// <para>Area values are returned in square kilometers, but can be converted to other area units using
		/// methods in the <see cref="GeoFramework.Area">Area</see> structure.</para></remarks>
		public virtual Area Area
		{
			get
			{
#if !PocketPC
                // Has the area been recalculated lately?
                if (!pIsAreaCalculated)
                {
#endif
                    try
                    {
                        double totalArea = 0.0;
                        double previousMeridianDistance = 0.0;
                        double previousDeparture = 0.0;
                        double departure = 0.0;
                        double doubleMeridianDistance = 0.0;
                        Position PreviousPosition = Position.Empty;

                        // Make a collection of points
                        PositionCollection AreaItems = Clone();
                        // Is the shape closed?
                        if (!AreaItems.IsPolygon)
                            // No.  Add the first point to the end to close it for this calculation
                            AreaItems.Add(AreaItems.First);
                        // Calculate individual area
                        //lock (AreaItems.SyncRoot)
                        {
                            foreach (Position item in AreaItems)
                            {
                                // Is this the first position?
                                if (!PreviousPosition.IsEmpty)
                                {
                                    // No.  Continue with area calculations.
                                    Radian h = Radian.FromDegrees(90.0 - PreviousPosition.Latitude.DecimalDegrees);
                                    Radian a = Radian.FromDegrees(90.0 - item.Latitude.DecimalDegrees);
                                    Radian T = item.Longitude.Subtract(PreviousPosition.Longitude.DecimalDegrees).ToRadians();
                                    Radian cosc = new Radian(((Math.Cos(a.Value)) * (Math.Cos(h.Value)) + ((Math.Sin(a.Value)) * (Math.Sin(h.Value)) * (Math.Cos(T.Value)))));
                                    Radian Sinc = new Radian(((Math.Sin(a.Value)) * (Math.Sin(h.Value)) + ((Math.Cos(a.Value)) * (Math.Cos(h.Value)) * (Math.Sin(T.Value)))));
                                    // Calculate area in square nautical miles
                                    double areaSqNaut = Math.Acos(cosc.Value) * 3438.12;
                                    // Convert the area to square feet :-P
                                    double distance = areaSqNaut * 6076.76;
                                    // Calculate the bearing to the current position
                                    // If there is no change in latitude, ignore the bearing
                                    double bearing;
                                    if (item.Longitude.Equals(PreviousPosition.Longitude))
                                        bearing = 0;
                                    else
                                    {
                                        bearing = PreviousPosition.BearingTo(item).DecimalDegrees;
                                        if (bearing >= 180)
                                            bearing = item.BearingTo(PreviousPosition).DecimalDegrees;
                                    }
                                    //Direction indicator;
                                    double PolarBearing = bearing;
                                    // Get a general cardinal direction of the bearing
                                    if (bearing > 0)
                                    {
                                        if (item.Latitude > PreviousPosition.Latitude)
                                        {
                                            if (item.Longitude > PreviousPosition.Longitude)
                                            {
                                                //indicator = Direction.Northeast;
                                                PolarBearing = 180.0;
                                            }
                                            else
                                            {
                                                //indicator = Direction.Northwest;
                                                PolarBearing = 360.0 - bearing;
                                            }
                                        }
                                        else
                                        {
                                            if (item.Longitude > PreviousPosition.Longitude)
                                            {
                                                //indicator = Direction.Southeast;
                                                PolarBearing = 180.0 - bearing;
                                            }
                                            else
                                            {
                                                //indicator = Direction.Southwest;
                                                PolarBearing = 180.0 + bearing;
                                            }
                                        }
                                    }
                                    // 
                                    //
                                    //=IF(G18=0,(0),(ROUND(IF(Calculations!M69="NE",(G18),IF(Calculations!M69="SE",(180-G18),IF(Calculations!M69="NW",(360-G18),180+G18))),3)))
                                    //
                                    //	if("NE") G18
                                    //				 if(SE) 180 - G18
                                    //							if(nw)   360-g18)
                                    //												if(sw) 180+g18

                                    //						PolarBearing = Math.Round(PolarBearing, 3);
                                    //						distance = Math.Round(distance, 3);

                                    double lat = (distance * (Math.Cos(Radian.FromDegrees(PolarBearing).Value)));
                                    departure = (distance * (Math.Sin(Radian.FromDegrees(PolarBearing).Value)));
                                    doubleMeridianDistance = previousMeridianDistance + previousDeparture + departure;
                                    double doubleArea = lat * doubleMeridianDistance;
                                    //						Console.WriteLine(", Bearing    : " + bearing);
                                    //						Console.WriteLine(", Indicator  : " + indicator);
                                    //						Console.WriteLine(", Pol. Bear. : " + PolarBearing);
                                    //						Console.WriteLine(", Lat (N/S)  : " + lat);
                                    //						Console.WriteLine(", Departure  : " + departure);
                                    //						Console.WriteLine(", Double MD  : " + doubleMeridianDistance);
                                    //						Console.WriteLine(", Double Area: " + doubleArea);
                                    //						Console.WriteLine();
                                    // Add this to the total
                                    totalArea += doubleArea;
                                    // Remember this for the next iteration
                                    previousMeridianDistance = doubleMeridianDistance;
                                    previousDeparture = departure;
                                }
                                PreviousPosition = item;
                            }
                        }

#if PocketPC
                        // Return the area, converted to square feet
                        return new Area(Math.Abs(totalArea * 0.5), AreaUnit.SquareFeet).ToLocalUnitType();
#else
                        // Return the area, converted to square feet
                        pArea = new Area(Math.Abs(totalArea * 0.5), AreaUnit.SquareFeet).ToLocalUnitType();
#endif
                    }
                    catch (Exception ex)
                    {
                        throw new ArithmeticException(Resources.PositionCollection_AreaError, ex);
                    }
                    //catch
                    //{
                    //    throw new ArithmeticException(Resources.PositionCollection_AreaError));
                    //}

#if !PocketPC
                    finally
                    {
                        // We no longer need to calculate it
                        pIsAreaCalculated = true;
                    }
                }				
				// Nope, return the already-calculated area
				return pArea;
#endif

			}
		}

        /// <summary>
        /// Returns the avarage of all coordinates in the collection.
        /// </summary>
        /// <remarks>This method is typically used for position-averaging, a technique used
        /// to reduce noise in position reports from GPS devices when the GPS device is at rest.  All points in the collection
        /// are evaluated, then a single Position object is returned representing the average of all items.</remarks>
        public Position Average
        {
            get
            {
                lock (pItems.SyncRoot)
                {
                    double Latitude = 0.0;
                    double Longitude = 0.0;
                    foreach (Position item in pItems)
                    {
                        Latitude += item.Latitude.DecimalDegrees;
                        Longitude += item.Longitude.DecimalDegrees;
                    }
                    return new Position(
                        new Latitude(Latitude / pItems.Count),
                        new Longitude(Longitude / pItems.Count));
                }
            }
        }

		/// <summary>Returns the weighted mean of all coordinates.</summary>
		public Position Centroid
		{
			get
			{
//                %% POLYGON_CENTROID_2D computes the centroid of a polygon in 2D.
//%
//%  Discussion:
//%
//%    Denoting the centroid coordinates by CENTROID, then
//%
//%      CENTROID(1) = Integral ( Polygon interior ) x dx dy / Area ( Polygon )
//%      CENTROID(2) = Integral ( Polygon interior ) y dx dy / Area ( Polygon ).
//%
//%    Green's theorem states that for continuously differentiable functions
//%    M(x,y) and N(x,y),
//%
//%      Integral ( Polygon boundary ) ( M dx + N dy ) =
//%      Integral ( Polygon interior ) ( dN/dx - dM/dy ) dx dy.
//%
//%    Using M(x,y) = 0 and N(x,y) = x**2/2, we get:
//%
//%      CENTROID(1) = 0.5 * Integral ( Polygon boundary ) x**2 dy 
//%                  / Area ( Polygon ),
//%
//%    which becomes
//%
//%      CENTROID(1) = 1/6 sum ( 1 <= I <= N )
//%        ( X(I+1) + X(I) ) * ( X(I) * Y(I+1) - X(I+1) * Y(I))
//%        / Area ( Polygon )
//%
//%    where, when I = N, the index "I+1" is replaced by 1.
//%
//%    A similar calculation gives us a formula for CENTROID(2).
//%
//%  Modified:
//%
//%    22 May 2005
//%
//%  Author:
//%
//%    John Burkardt
//%
//%  Reference:
//%
//%    Gerard Bashein and Paul Detmer,
//%    Centroid of a Polygon,
//%    Graphics Gems IV, 
//%    edited by Paul Heckbert,
//%    AP Professional, 1994.
//%
//%  Parameters:
//%
//%    Input, integer N, the number of sides of the polygon.
//%
//%    Input, real V(2,N), the coordinates of the vertices.
//%
//%    Output, real CENTROID(2), the coordinates of the centroid.
//%
//  dim_num = 2;

//  area = 0.0;
//  centroid(1:dim_num) = 0.0;

//  for i = 1 : n

//    if ( i < n )
//      ip1 = i + 1;
//    else
//      ip1 = 1;
//    end

//    temp = ( v(1,i) * v(2,ip1) - v(1,ip1) * v(2,i) );

//    area = area + temp;

//    centroid(1:dim_num) = centroid(1:dim_num) + ( v(1:dim_num,ip1) + v(1:dim_num,i) )' * temp;

//  end

//  area = area / 2.0;

//  if ( area == 0.0 )
//    centroid(1:dim_num) = v(1:dim_num,1);
//  else
//    centroid(1:dim_num) = centroid(1:dim_num) / ( 6.0 * area );
                //  end

                #region New Code

//%% POLYGON_CENTROID_2D computes the centroid of a polygon in 2D.
//%
//%  Discussion:
//%
//%    Denoting the centroid coordinates by CENTROID, then
//%
//%      CENTROID(1) = Integral ( Polygon interior ) x dx dy / Area ( Polygon )
//%      CENTROID(2) = Integral ( Polygon interior ) y dx dy / Area ( Polygon ).
//%
//%    Green's theorem states that for continuously differentiable functions
//%    M(x,y) and N(x,y),
//%
//%      Integral ( Polygon boundary ) ( M dx + N dy ) =
//%      Integral ( Polygon interior ) ( dN/dx - dM/dy ) dx dy.
//%
//%    Using M(x,y) = 0 and N(x,y) = x**2/2, we get:
//%
//%      CENTROID(1) = 0.5 * Integral ( Polygon boundary ) x**2 dy 
//%                  / Area ( Polygon ),
//%
//%    which becomes
//%
//%      CENTROID(1) = 1/6 sum ( 1 <= I <= N )
//%        ( X(I+1) + X(I) ) * ( X(I) * Y(I+1) - X(I+1) * Y(I))
//%        / Area ( Polygon )
//%
//%    where, when I = N, the index "I+1" is replaced by 1.
//%
//%    A similar calculation gives us a formula for CENTROID(2).
//%
//%  Modified:
//%
//%    22 May 2005
//%
//%  Author:
//%
//%    John Burkardt
//%
//%  Reference:
//%
//%    Gerard Bashein and Paul Detmer,
//%    Centroid of a Polygon,
//%    Graphics Gems IV, 
//%    edited by Paul Heckbert,
//%    AP Professional, 1994.
//%
//%  Parameters:
//%
//%    Input, integer N, the number of sides of the polygon.
//%
//%    Input, real V(2,N), the coordinates of the vertices.
//%
//%    Output, real CENTROID(2), the coordinates of the centroid.
//%
//  dim_num = 2;

                //int dim_num = 2;

//  area = 0.0;

                double area = 0.0;

//  centroid(1:dim_num) = 0.0;

                // This stores the result.  No need for it now
                double CentroidLatitude = 0.0;
                double CentroidLongitude = 0.0;

//  for i = 1 : n
                ArrayList items = (ArrayList)pItems.Clone();
                if (!this.Last.Equals(this.First))
                    items.Add(this.First);

                // Search all points in the collection
                lock (items.SyncRoot)
                {
                    for (int i = 0; i < items.Count - 1; i++)
                    {

                        //    if ( i < n )
                        //      ip1 = i + 1;
                        //    else
                        //      ip1 = 1;
                        //    end

                        // This code re-uses the first point in the collection as the
                        // last point.  However, PositionCollection will store the start
                        // point again as an endpoint for any polygon.  So, this code is
                        // not necessary

                        //    temp = ( v(1,i) * v(2,ip1) - v(1,ip1) * v(2,i) );

                        // v(1, _) is Latitude
                        // v(2, _) is Longitude

                        Position Current = (Position)items[i];
                        Position Next = (Position)items[i + 1];

                        double temp = Current.Latitude.DecimalDegrees
                            * Next.Longitude.DecimalDegrees
                            - Next.Latitude.DecimalDegrees 
                            * Current.Longitude.DecimalDegrees;

                        //    area = area + temp;

                        area += temp;

                        //    centroid(1:dim_num) = centroid(1:dim_num) + ( v(1:dim_num,ip1) + v(1:dim_num,i) )' * temp;

                        CentroidLatitude += (Current.Latitude.DecimalDegrees + Next.Latitude.DecimalDegrees) * temp;
                        CentroidLongitude += (Current.Longitude.DecimalDegrees + Next.Longitude.DecimalDegrees) * temp;

                        //  end

                    }


                    //  area = area / 2.0;

                    // Always faster to multiply by 0.5 than divide by 2
                    area *= 0.5;

                    //  if ( area == 0.0 )
                    //    centroid(1:dim_num) = v(1:dim_num,1);
                    //  else
                    //    centroid(1:dim_num) = centroid(1:dim_num) / ( 6.0 * area );
                    //  end

                    // Is there any area?
                    if (area == 0)
                        // No.  Return the first coordinate
                        return (Position)pItems[0];
                    else
                        // Yes!  Return the calculated centroid
                        return new Position(
                            new Latitude(CentroidLatitude / (6.0 * area)),
                            new Longitude(CentroidLongitude / (6.0 * area)));

                }

                #endregion

                #region Old Code

                /*
				double factor;
				double cx = 0;
				double cy = 0;
				lock(pItems.SyncRoot)
				{
                    if (pItems.Count == 0)
                        return Position.Empty;

					for(int index = 0; index < Count; index++)
					{
						Position ThisOne = this[index];
						Position NextOne = this[(index + 1) % Count];
						// Get the index of the point ahead of the current index, 
						// wrapping back to zero if necessary
						factor = ThisOne.Longitude.DecimalDegrees 
                            * NextOne.Latitude.DecimalDegrees 
                            - NextOne.Longitude.DecimalDegrees 
                            * ThisOne.Latitude.DecimalDegrees;
						cx += (ThisOne.Longitude.DecimalDegrees + NextOne.Longitude.DecimalDegrees) * factor;
                        cy += (ThisOne.Latitude.DecimalDegrees + NextOne.Latitude.DecimalDegrees) * factor; //B B B B B B B B B B B 
					}
				}
				double SixArea = Area.Value * 6.0f;
				
				factor = 1.0 / SixArea;
				cx *= factor;
				cy *= factor;

				return new Position(new Latitude(cy), new Longitude(cx));
                 * */

                #endregion
            }
		}

            /// <summary>
            /// Calculates the total distance by traveling through all points in the
            /// collection.
            /// </summary>
            //[CLSCompliant(false)]
            public Distance Perimeter
		{
			get
			{
#if !PocketPC
                // Has the area been recalculated lately?
                if (!pIsPerimeterCalculated)
                {
#endif
                    Position PreviousPosition = Position.Empty;
                    try
                    {
                        Distance SubTotalDistance = new Distance(0, DistanceUnit.Meters);
                        lock (pItems.SyncRoot)
                        {
                            foreach (Position PositionWithinLoop in pItems)
                            {
                                // Are we at the first position?
                                if (PreviousPosition.IsEmpty)
                                {
                                    // Yes. Remember this as the starting point
                                    PreviousPosition = PositionWithinLoop;
                                }
                                else
                                {
                                    // No. Add the distance from the previous point
                                    SubTotalDistance += PreviousPosition.DistanceTo(PositionWithinLoop);
                                    // And update the last position
                                    PreviousPosition = PositionWithinLoop;
                                }
                            }
                        }

#if PocketPC
                        return SubTotalDistance;
#else
                        // Return the distance
                        pPerimeter = SubTotalDistance;
#endif
                    }
                    catch (Exception ex)
                    {
                        throw new ArithmeticException(Resources.PositionCollection_DistanceError, ex);
                    }
                    //catch
                    //{
                    //    throw new ArithmeticException(Resources.PositionCollection_DistanceError);
                    //}
#if !PocketPC
                    finally
                    {
                        // We no longer need to calculate it
                        pIsPerimeterCalculated = true;
                    }
                }

                // Nope, return the already-calculated area
                return pPerimeter;
#endif
			}
		}		

		/// <summary>Copies all or part of the current collection to another array.</summary>
		public void CopyTo(Array array, int index)
		{
			pItems.CopyTo(array, index);
		}

        /// <summary>Copies all or part of the current collection to another array.</summary>
        public void CopyTo(Position[] array, int index)
        {
            pItems.CopyTo(array, index);
        }

		/// <summary>
		/// Indicates if the specified <strong>Position</strong> object exists in the
		/// collection.
		/// </summary>
		public bool Contains(Position item)
		{
			return pItems.Contains(item);
		}

		/// <summary>
		/// Returns the position in the collection of the specified Position object.
		/// </summary>
		/// <param name="value"></param>
		/// <returns></returns>
		public int IndexOf(Position value)
		{
			return pItems.IndexOf(value);
		}

		/// <summary>Adds a <strong>Position</strong> to the collection at the specified index.</summary>
		public void Insert(int index, Position value)
		{
			// Insert the item to the specified location
            lock (pItems.SyncRoot)
            {
                pItems.Insert(index, value);
            }

            // Flag that other items needs to be recalculated
            pIsExtentCalculated = false;

#if !PocketPC
            pIsAreaCalculated = false;
            pIsPerimeterCalculated = false;
#endif
			
            // Broadcast that the item is updated
			OnItemAdded(value);
		}

		/// <summary>
		/// Creates a shallow copy of the current collection.
		/// </summary>
		/// <returns>A "shallow" copy means that the copy contains references to the same objects as the original collection.
		/// Modifications made to the cloned collection also affect the contents of the original.</returns>
		public PositionCollection Clone()
		{
            return new PositionCollection(this.ToArray());
		}

		#endregion

		public IEnumerator GetEnumerator()
		{
			return pItems.GetEnumerator();
		}

#if !PocketPC
		public IEnumerator GetEnumerator(int index, int count)
		{
			return pItems.GetEnumerator(index, count);
		}
#endif

		#region IEnumerable Members

		IEnumerator IEnumerable.GetEnumerator()
		{
			return this.GetEnumerator();
		}

		#endregion

		#region IList Members

		bool IList.IsReadOnly
		{
			get
			{
				return IsReadOnly;
			}
		}

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
                this[index] = (Position)value;
			}
		}

		void IList.Insert(int index, object value)
		{
			Insert(index, (Position)value);
		}

		void IList.Remove(object value)
		{
			Remove((Position)value);
		}

		bool IList.Contains(object value)
		{
			return Contains((Position)value);
		}

		int IList.IndexOf(object value)
		{
			return IndexOf((Position)value);
		}

		int IList.Add(object value)
		{
			Add((Position)value);
            return IndexOf((Position)value);
		}

		bool IList.IsFixedSize
		{
			get
			{
				return IsFixedSize;
			}
		}

		#endregion

		#region ICollection Members

		bool ICollection.IsSynchronized
		{
			get
			{
				return IsSynchronized;
			}
		}

		void ICollection.CopyTo(Array array, int index)
		{
			CopyTo(array, index);
		}

		object ICollection.SyncRoot
		{
			get
			{
				return SyncRoot;
			}
		}

		#endregion

#if !PocketPC || (PocketPC && Framework20)
		#region IXmlSerializable Members

#if Framework20
        public static XmlQualifiedName GetXmlSchema(XmlSchemaSet xs)
        {
            // Load it from a string
            xs.Add(Xml.SchemaSet);

            // And return the type
            return new XmlQualifiedName("PositionCollection", Xml.Namespace);
        }
#endif    

        XmlSchema IXmlSerializable.GetSchema()
        {
            return null;
        }

        void IXmlSerializable.ReadXml(XmlReader reader)
        {
            ReadXml(reader);
        }

        void IXmlSerializable.WriteXml(XmlWriter writer)
        {
            WriteXml(writer);
        }

        /// <summary>
        /// Populates the current instance by reading XML from the specified reader.
        /// </summary>
        /// <param name="reader"></param>
        public void ReadXml(XmlReader reader)
        {
            if (reader == null)
                throw new ArgumentNullException("reader");

            // Defer to latitude and longitude serializers
            XmlSerializer PositionSerializer = new XmlSerializer(typeof(Position));

            // Rember which level we are processing at.
            int InitialDepth = reader.Depth;

            // Read into deeper content
            reader.Read();

            // Read to the next item, or exit if any content is missing
            while (reader.Depth >= InitialDepth)
            {
                // Keep reading until we reach an element
                switch (reader.NodeType)
                {
                    case XmlNodeType.Element:
                        switch (reader.Name.ToUpper(CultureInfo.InvariantCulture))
                        {
                            case "POSITION":
                                pItems.Add((Position)PositionSerializer.Deserialize(reader));
                                break;
                            default:
                                // Read the next item
                                reader.Read();
                                break;
                        }
                        break;
                    case XmlNodeType.EndElement:
                        reader.Read();
                        return;
                    default:
                        // Read the next item
                        reader.Read();
                        break;
                }
            }

#if !PocketPC
            // Flag that calculations are now needed
            pIsAreaCalculated = false;
            pIsPerimeterCalculated = false;
            pIsExtentCalculated = false;
#endif
        }

        /// <summary>
        /// Writes the values of the current instance to the specified XML writer.
        /// </summary>
        /// <param name="writer"></param>
        public void WriteXml(XmlWriter writer)
        {
            if (writer == null)
                throw new ArgumentNullException("writer");

            // Defer to latitude and longitude serializers
            XmlSerializer PositionSerializer = new XmlSerializer(typeof(Position));
            foreach (Position item in pItems)
            {
                PositionSerializer.Serialize(writer, item);
            }
        }

		#endregion
#endif

#if !PocketPC || DesignTime
        #region ISerializable Members

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            lock (pItems.SyncRoot)
            {
                for (int i = 0; i < pItems.Count; i++)
                {
                    // Add the new item to the collection
                    info.AddValue("Item" + i.ToString(), pItems[i]);
                }
            }
        }

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]  
        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);
        }

        #endregion
#endif

        #region IFormattable Members

        string IFormattable.ToString(string format, IFormatProvider formatProvider)
        {
            return ToString(format, formatProvider as CultureInfo);
        }

        #endregion
    }


	public sealed class PositionCollectionEventArgs : EventArgs
	{
		private PositionCollection oPositionCollection;

		public static readonly new PositionCollectionEventArgs Empty = new PositionCollectionEventArgs(new PositionCollection(0));

		public PositionCollectionEventArgs(PositionCollection collection)
		{
			oPositionCollection = collection;
		}

		public PositionCollection PositionCollection
		{
			get
			{
				return oPositionCollection;
			}
		}
	}
}