#if !PocketPC || DesignTime
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System;
using System.Globalization;
using System.Text;
using System.Reflection;
using System.Windows.Forms;

namespace GeoFramework.Design
{
	/// <summary>
	/// Provides functionality to converting values to and from
	/// <strong>PolarCoordinate</strong> objects.
	/// </summary>
	/// <remarks>
	/// 	<para>This class allows any <strong>PolarCoordinate</strong> object to be converted
	///     between other data types, such as <strong>String</strong>. This class is used
	///     primarily during the Windows Forms designer to give detailed information about
	///     properties of type <strong>PolarCoordinate</strong>, and also allows developers to
	///     type in string values such as "2,80" and have them converted to
	///     <strong>PolarCoordinate</strong> objects automatically. Finally, this class
	///     controls design-time serialization of <strong>PolarCoordinate</strong> object
	///     properties.</para>
	/// 	<para>In most situations this class is used by the Visual Studio.NET IDE and is
	///     rarely created at run-time.</para>
	/// </remarks>
#if Framework20
    public sealed class PolarCoordinateConverter : GeoFrameworkObjectConverter 
    {
        protected override string HandledTypeName
        {
            get
            {
                return "GeoFramework.PolarCoordinate";
            }
        }

        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
                // What is the destination type?
                if (destinationType == typeof(string) && value == null)
                {
                        return "0.0, 0";
                    }
                else if (destinationType == typeof(InstanceDescriptor))
                {
                    // Instance descriptor.  Used during Windows Forms serialization.

                    // Get the type of the object (probably an PolarCoordinate)
                    Type PolarCoordinateType = value.GetType();
                    // Is the value null?
                    if (value == null)
                    {
                        // Yes.  Return the "Empty" static value
                        return new InstanceDescriptor(PolarCoordinateType.GetField("Empty"), null);
                    }
                    else
                    {
                        // Get the properties needed to generate a constructor
                        object[] ConstructorParameters = new object[] 
                            { PolarCoordinateType.GetProperty("R").GetValue(value, null),
                              PolarCoordinateType.GetProperty("Theta").GetValue(value, null) };
                        Type[] ConstructorTypes = new Type[] 
                            { typeof(double), 
                              Type.GetType("GeoFramework.Angle") };

                        //#if DEBUG
                        //                        System.Diagnostics.EventLog.WriteEntry("SUCCESS: PolarCoordinateConverter: ConvertTo: \"" + value.ToString() + "\" to New PolarCoordinate(" + DecimalDegrees.ToString() + ").", "");
                        //#endif

                        // Now activate the constructor
                        ConstructorInfo Constructor = PolarCoordinateType.GetConstructor(ConstructorTypes);
                        return new InstanceDescriptor(Constructor, ConstructorParameters);
                    }
                }

                // Defer to the base class
                return base.ConvertTo(context, culture, value, destinationType);
        }

        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }

        public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[] {
		            "Empty",
		            "Center" });
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return false;
        }
    }
#else
	public sealed class PolarCoordinateConverter : ExpandableObjectConverter
	{
		#region ConvertFrom: Converts an object to an PolarCoordinate

		// Indicates if the type can be converted
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType.Equals(typeof(System.String))
				|| sourceType.Equals(typeof(PolarCoordinate))
				|| sourceType.Equals(typeof(InstanceDescriptor)))
				return true;
			// Defer to the base class
			return base.CanConvertFrom(context, sourceType);
		}

		// Converts an object of one type to an PolarCoordinate
		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			Type ValueType = value.GetType();
			if(ValueType.Equals(typeof(System.String)))
				return PolarCoordinate.Parse((string)value, culture);
			else if(ValueType.Equals(typeof(PolarCoordinate)))
				return value;
			// Defer to the base class
			return base.ConvertFrom(context, culture, value);
		}

		#endregion

		#region ConvertTo: Converts an PolarCoordinate to another type

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if(destinationType.Equals(typeof(System.String))
				|| destinationType.Equals(typeof(PolarCoordinate))
				|| destinationType.Equals(typeof(InstanceDescriptor)))
				return true;
            
			// Defer to the base class
			return base.CanConvertTo(context, destinationType);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if(destinationType.Equals(value.GetType()))
				return value;
			else if(destinationType.Equals(typeof(System.String)))
				return value.ToString();
			else if(destinationType == typeof(InstanceDescriptor))
			{
				ConstructorInfo Constructor = typeof(PolarCoordinate).GetConstructor(
					new Type[] {typeof(double), typeof(Angle)});
				PolarCoordinate point = (PolarCoordinate)value;
				return new InstanceDescriptor(Constructor, new object[] { point.R, point.Theta });
			}
			// Defer to the base class
			return base.ConvertTo(context, culture, value, destinationType);
		}

		#endregion
	}
#endif
}
#endif