using System;
using System.Globalization;
using System.Drawing;
using System.Reflection;
using System.Security.Permissions;
#if !PocketPC || DesignTime
using System.Runtime.Serialization;
#endif

namespace GeoFramework
{
	/// <summary>
	/// Represents a geographic coordinate which is associated with a specific pixel coordinate.
	/// </summary>
	/// <remarks>
	/// Georeferenced points are geographic coordinates which are associated with
	/// a specific pixel.  These coordinates are typically used to derive formulas for
	/// converting geographic coordinates to and from pixel coordinates.  For example,
	/// a bitmap image of Florida would use georeferenced points to calculate the scale
	/// of the bitmap relative to the real world.
	/// </remarks>
#if !PocketPC || DesignTime
#if Framework20
    [Obfuscation(Feature = "renaming", Exclude = false, ApplyToMembers = true)]
    [Obfuscation(Feature = "controlflow", Exclude = true, ApplyToMembers = true)]
    [Obfuscation(Feature = "stringencryption", Exclude = false, ApplyToMembers = true)]
#endif
    [Serializable()]
	#endif
	public sealed class Waypoint3D : Position3D
	{
		#region Private Variables

		private bool pIsVisible = true;
		private Color pForeColor = Color.Black;
		private string pText = "(Untitled)";

		#endregion

		#region  Constructors

		public Waypoint3D(Longitude longitude, Latitude latitude, Distance altitude) 
			: base(longitude, latitude, altitude)
		{}

		/// <overloads>Creates a new instance.</overloads>
		public Waypoint3D(Latitude latitude, Longitude longitude, Distance altitude) 
			: base(latitude, longitude, altitude)
		{}

		/// <summary>
		/// Creates a new instance using the specified latitude, longitude and
		/// Datum.
		/// </summary>
		/// <remarks>
		/// This constructor is commonly used when a new position must be created using an
		/// Datum other than the default of WGS1984.
		/// </remarks>
		/// <param name="latitude">An angle from the equator between -90 and +90.</param>
		/// <param name="longitude">An angle from the Prime Meridian between -90 and +90.</param>
		/// <param name="datum">An Datum to associate with the given latitude and 
		/// longitude.</param>
		public Waypoint3D(Latitude latitude, Longitude longitude, Datum datum, Distance altitude) 
			: base(latitude, longitude, datum, altitude)
		{}

		/// <summary>
		/// Creates a new instance by parsing latitude and longitude from a single string.
		/// </summary>
		/// <param name="value">A <strong>String</strong> containing both a latitude and longitude to parse.</param>
		public Waypoint3D(string value, string altitude) 
			: base(value, altitude, CultureInfo.CurrentCulture)
		{}

        /// <summary>
		/// Creates a new instance by parsing latitude and longitude from a single string.
		/// </summary>
		/// <param name="value">A <strong>String</strong> containing both a latitude and longitude to parse.</param>
		public Waypoint3D(string value, string altitude, CultureInfo culture) 
			: base(value, altitude, culture)
		{}

		//[CLSCompliant(false)]
		public Waypoint3D(ZoneLetter zoneLetter, int zoneNumber, Distance easting, Distance northing, Distance altitude) 
			: base(zoneLetter, zoneNumber, easting, northing, altitude)
		{}

		//[CLSCompliant(false)]
		public Waypoint3D(ZoneLetter zoneLetter, int zoneNumber, Distance easting, Distance northing, Datum datum, Distance altitude) 
			: base(zoneLetter, zoneNumber, northing, easting, datum, altitude)
		{}

#if !PocketPC || DesignTime
        /// <summary>
        /// Creates a new instance by de-serializing previously-serialized information.
        /// </summary>
        /// <param name="info">A <strong>SerializationInfo</strong> object containing serialized data.</param>
        /// <param name="context">A <strong>StreamingContext</strong> object describing the context of serialization.</param>
        /// <remarks>This constructor will de-serialize a previously-serialized object.</remarks>
        private Waypoint3D(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            pIsVisible = info.GetBoolean("Visible");
            pForeColor = (Color)info.GetValue("ForeColor", typeof(Color));
            pText = info.GetString("Text");
        }

        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
        protected override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            // Set the altitude
            info.AddValue("Visible", pIsVisible);
            info.AddValue("ForeColor", pForeColor);
            info.AddValue("Text", pText);

            // And any other items
            base.GetObjectData(info, context);
        }
#endif

		#endregion

		public bool IsVisible
		{
			get
			{
				return pIsVisible;
			}
			set
			{
				if (pIsVisible == value) return;
				pIsVisible = value;
			}
		}

		[CLSCompliant(false)]
		public Color ForeColor
		{
			get
			{
				return pForeColor;
			}
			set
			{
				pForeColor = value;
			}
		}

		public string Text
		{
			get
			{
				return pText;
			}
			set
			{
				if (pText == value) return;
				pText = value;
			}
		}
	}
}
